// *****************************************************************************
//
// source:    AC_RICH_evnt.C
//
// type:      source code
//
// created:   03 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    AC_RICH_evnt()
//
// called by: AC_evnt()
//
// hints:     see $HBROOT/ARTE/dev/include/dq/DQ.h for COMP_XYZ and LVL_ABC
//
// changes:
// 03 Nov 1999 (AG) further developments
// 19 Nov 1999 (AG) cna -> ac (alca)
// 22 Nov 1999 (AG) includes in alca
// 02 Feb 2000 (DD) beta=1 histogram, rings/event
// 03 Feb 2000 (AG) fixed function name
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "dq/DQ.h"
#include "farm/farm.hh"
#include "rich/rich.h"
#include "rise/RingSearchARTE.h"
#include "alca/AC_RICH.hh"

// externals
extern int FAGRR;

// prototypes
int AC_RICH_rec_beta1( ArtePointer<RSEG> rich );


//////////////// Fill RICH AC histograms /////////////////////////////
int AC_RICH_evnt() {

  int nrRICHrings=0; 
  for(ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin();
      irseg!=ArteTable<RSEG>::end(); ++irseg) {
    if ((*irseg)->cmp!=Rsegc::rich) continue;
    ArtePointer<RSEG> rich=(*irseg);
    nrRICHrings++;

    float tx=rich->txf;
    float ty=rich->tyf;
    int cr=rhp_hfill(AC_ID(COMP_RICH, LVL_REC, 201),tx,ty,1);
    if (cr!=GRR_OK) cout<<"Error filling histogram "<<AC_ID(COMP_RICH, LVL_REC, 201)<<endl;
    rhp_hf1(AC_ID(COMP_RICH, LVL_REC, 202),rich->time,1);
    if (!AC_RICH_rec_beta1(rich))
      rhp_hf1(AC_ID(COMP_RICH, LVL_REC, 203),rich->time,1);
  }
  rhp_hf1(AC_ID(COMP_RICH,LVL_REC, 200),nrRICHrings,1);

  return 0; // return error code (0 = okay)
}



//////////////  Select RICH Beta=1 w/ ECAL ////////////////////////
int AC_RICH_rec_beta1(ArtePointer<RSEG> rich) {
  static float max_dty=3e-3;  // angle match
  static float min_esum=8;    // minimum cluster energy
  static float max_p=200;     // maximum p
  static float min_p=10;      // minimum p
  for (ArteTable<RCCL>::iterator irccl=ArteTable<RCCL>::begin();
       irccl!=ArteTable<RCCL>::end(); ++irccl) {
    if ((*irccl)->esum<min_esum) continue;
    if (fabs((*irccl)->ty - rich->tyf)>max_dty) continue;
    float p=288/fabs((*irccl)->x - rich->txf*(*irccl)->z);
    if (p<min_p) continue;
    if (p>max_p) continue;
    return 0; // cluster found
  }
  return 1; // cluster not found
}
