
// *****************************************************************************
//
// source:    RC_init.C
//
// type:      source code
//
// created:   23 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    RC_init()
//
// called by: acinit_()
//
// changes:
// 24 Nov 1999 (AG) improve print-out
//  1 Mar 2000 (CP) use PRCMHITMASK to steer HITB-building
// 21 Mar 2000 (AG) polish print-out
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE icludes
#include "runcfg.h"
#include "grr.h"


// defines
#define PRCMHITMASK "HITmask" // preliminary from ONLINE (runcfg.h)


// Arte includes
#include "farm/farm.hh"
#include "dq/DQ.h"

// externals
extern int FADEB;

// reco flags
extern int recoTARGET;
extern int recoVDS;
extern int recoVDSmodule;
extern int recoHPT;
extern int recoOTR;
extern int recoRICH;
extern int recoTRD;
extern int recoECAL;
extern int recoMUON;

//
// function
//
int
RC_init()
{
  // show up
  //cout << "RC_init()" << endl;

  // inits
  int rc = 0;

  // hit production flags
  char *rhitmask = getenv( PRCMHITMASK );
  int hmask = 0;
  if( rhitmask ) hmask = atoi( rhitmask );
  char txt1[80];
  memset( txt1, 0, 80 );
  sprintf( txt1, "RC_init(): HIT env mask=%d", hmask );
  ARTE_error( GRR_SINFO, txt1, 0 );

  // print
  cout << "RC_init(): HIT in run (PRCMHITMASK): Mask=" << hmask << endl;
  if( FADEB ){
    cout << "           SVD= " << faHit( COMP_M_SVD )  << endl;
    cout << "           ITR= " << faHit( COMP_M_ITR )  << endl;
    cout << "           HIPT=" << faHit( COMP_M_HIPT ) << endl;
    cout << "           OTR= " << faHit( COMP_M_OTR )  << endl;
    cout << "           RICH=" << faHit( COMP_M_RICH ) << endl;
    cout << "           TRD= " << faHit( COMP_M_TRD )  << endl;
    cout << "           ECAL=" << faHit( COMP_M_ECAL ) << endl;
    cout << "           MUON=" << faHit( COMP_M_MUON ) << endl;
  }

  // reco flags
  char *rrecomask = getenv( PRCMRECOMASK );
  int mask = 0;
  if( rrecomask ) mask = atoi( rrecomask );
  char txt[80];
  memset( txt, 0, 80 );
  sprintf( txt, "RC_init(): RECO env mask=%d", mask );
  ARTE_error( GRR_SINFO, txt, 0 );

  // print
  cout << "RC_init(): RECO in run (PRCMRECOMASK): Mask=" << mask << endl;
  if( FADEB ){
    cout << "           SVD= " << faReco( COMP_M_SVD )  << endl;
    cout << "           ITR= " << faReco( COMP_M_ITR )  << endl;
    cout << "           HIPT=" << faReco( COMP_M_HIPT ) << endl;
    cout << "           OTR= " << faReco( COMP_M_OTR )  << endl;
    cout << "           RICH=" << faReco( COMP_M_RICH ) << endl;
    cout << "           TRD= " << faReco( COMP_M_TRD )  << endl;
    cout << "           ECAL=" << faReco( COMP_M_ECAL ) << endl;
    cout << "           MUON=" << faReco( COMP_M_MUON ) << endl;
  }

  // print initial reco flags
  if( FADEB ){
    cout << "RC_init(): initial RECO flags:" << endl;
    cout << "           TARGET=   " << recoTARGET    << endl;
    cout << "           VDS=      " << recoVDS       << endl;
    cout << "           VDSmodule=" << recoVDSmodule << endl;
    cout << "           HIPT=     " << recoHPT       << endl;
    cout << "           OTR=      " << recoOTR       << endl;
    cout << "           RICH=     " << recoRICH      << endl;
    cout << "           TRD=      " << recoTRD       << endl;
    cout << "           ECAL=     " << recoECAL      << endl;
    cout << "           MUON=     " << recoMUON      << endl;
  }


  // store inital settings
  static int first = 1;
  static int recoTARGET_init    = 0;
  static int recoVDS_init       = 0;
  static int recoVDSmodule_init = 0;
  static int recoHPT_init       = 0;
  static int recoOTR_init       = 0;
  static int recoRICH_init      = 0;
  static int recoTRD_init       = 0;
  static int recoECAL_init      = 0;
  static int recoMUON_init      = 0;
  if( first ){
    first = 0; 
    recoTARGET_init    = recoTARGET;
    recoVDS_init       = recoVDS;
    recoVDSmodule_init = recoVDSmodule;
    recoHPT_init       = recoHPT;
    recoOTR_init       = recoOTR;
    recoRICH_init      = recoRICH;
    recoTRD_init       = recoTRD;
    recoECAL_init      = recoECAL;
    recoMUON_init      = recoMUON;
  }

  // set flags
  recoTARGET    = recoTARGET_init    & faReco( COMP_M_SVD );
  recoVDS       = recoVDS_init       & faReco( COMP_M_SVD );
  recoVDSmodule = recoVDSmodule_init & faReco( COMP_M_SVD );
  recoHPT       = recoHPT_init       & faReco( COMP_M_HIPT );
  recoOTR       = recoOTR_init       & faReco( COMP_M_OTR );
  recoRICH      = recoRICH_init      & faReco( COMP_M_RICH );
  recoTRD       = recoTRD_init       & faReco( COMP_M_TRD );
  recoECAL      = recoECAL_init      & faReco( COMP_M_ECAL );
  recoMUON      = recoMUON_init      & faReco( COMP_M_MUON );

  // print final reco flags
  if( FADEB ){
    cout << "RC_init(): final RECO flags:" << endl;
    cout << "           TARGET="    << recoTARGET    << endl;
    cout << "           VDS="       << recoVDS       << endl;
    cout << "           VDSmodule=" << recoVDSmodule << endl;
    cout << "           HIPT="      << recoHPT       << endl;
    cout << "           OTR="       << recoOTR       << endl;
    cout << "           RICH="      << recoRICH      << endl;
    cout << "           TRD="       << recoTRD       << endl;
    cout << "           ECAL="      << recoECAL      << endl;
    cout << "           MUON="      << recoMUON      << endl;
  }

  // Modify, at least to the minimum in case it is needed for data quality
  if( DQ_flag( COMP_SVD,  LVL_M_HIT ) && ( recoVDS       == 0 ) ) recoVDS       = faHit( COMP_M_SVD );
  if( DQ_flag( COMP_SVD,  LVL_M_HIT ) && ( recoVDSmodule == 0 ) ) recoVDSmodule = faHit( COMP_M_SVD );
  if( DQ_flag( COMP_HIPT, LVL_M_HIT ) && ( recoHPT       == 0 ) ) recoHPT       = faHit( COMP_M_HIPT );
  if( DQ_flag( COMP_OTR,  LVL_M_HIT ) && ( recoOTR       == 0 ) ) recoOTR       = faHit( COMP_M_OTR );
  if( DQ_flag( COMP_RICH, LVL_M_HIT ) && ( recoRICH      == 0 ) ) recoRICH      = faHit( COMP_M_RICH );
  if( DQ_flag( COMP_ECAL, LVL_M_HIT ) && ( recoECAL      == 0 ) ) recoECAL      = faHit( COMP_M_ECAL );
  if( DQ_flag( COMP_MUON, LVL_M_HIT ) && ( recoMUON      == 0 ) ) recoMUON      = faHit( COMP_M_MUON );

  // print modified
  cout << "RC_init(): RECO flags (RECO kumac && HIT env && comp env):" << endl;
  cout << "           TARGET=   " << recoTARGET    << endl;
  cout << "           VDS=      " << recoVDS       << endl;
  cout << "           VDSmodule=" << recoVDSmodule << endl;
  cout << "           HIPT=     " << recoHPT       << endl;
  cout << "           OTR=      " << recoOTR       << endl;
  cout << "           RICH=     " << recoRICH      << endl;
  cout << "           TRD=      " << recoTRD       << endl;
  cout << "           ECAL=     " << recoECAL      << endl;
  cout << "           MUON=     " << recoMUON      << endl;

  return rc;
}


