//                              -*- Mode: C++ -*-
//: CmdCareECDBMIZ.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 12/6/97
// Last Update ........
//

#include "care/CmdCareECDBMIZ.hh"
#include "arte/arte.hh"
#include "care/care.hh"
#include "arte/ArteMessage.hh"
#include "care/ECHF_status.hh"
#include "care/caredata.hh"
#include "care/MMBank.hh"

#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include <string>
#include <math.h>
#include "care/EcalDB.hh"

/////////////// The function seen by kuip
#ifdef USEKUIP
extern "C" {

  void ecdbmiz_(){
    CmdCareECDBMIZ boardfile;
    boardfile.readAndExe();
    ECalInfo.dbflag = 2;
  }
}

#endif

/////////////// Static data definition

char CmdCareECDBMIZ::filename[256] = "\0";

/////////////// CmdCareECDBMIZ class member function definition

CmdCareECDBMIZ::CmdCareECDBMIZ(){}
CmdCareECDBMIZ::~CmdCareECDBMIZ(){}

void CmdCareECDBMIZ::defValues(){
  strcpy(filename,"ECAL/EcalCalibMiz.db");
  action = none;
  match = 0;
  major = 1;
  minor = 10;
  tbset = TALL;
  info.firstTime = 0;
  info.lastTime = 0;
  info.storageTime = 0;
  info.version = 0;
  strcpy(info.name," ");

}


//--
//   Execute stored command // read or write from the DB
void CmdCareECDBMIZ::execute(){
  switch(action){
  case dbread:
    readDB(tbset, match);
    break;
  case dbwrite:
    writeDB(tbset, match);
    break;
  default:
    break;
  }
}

#ifdef USEKUIP
//--
//   Read acommand from external
void CmdCareECDBMIZ::read(){
  char* name = readString();
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
    filename[i] = name[i];
  filename[i] = '\0';

  // reading the TB set
  int itbset = readInt();
  if( itbset>=0 && itbset<=(int)THOT ) tbset = (CmdCareECDBMIZ::Tbset)itbset;
  
  int act = readInt();
  action = none;
  if( act==1) action = dbread;
  else if( act==2) action = dbwrite;
  //
  // DB version numbers
  //
  major = readInt();
  minor = readInt();

  // 
  // Match parameter 
  //
  char* smatch = readUppercaseString();
  const char tests[] = "EQGTLTGELE";
  for(i=0; i<5; i++){
    if( !strncmp(smatch,tests+i*2,2) ) break;
  }
  if( i<5 ) match = i;
}
#endif

char* CmdCareECDBMIZ::calibfile(){
  if( filename[0]==0) 
    defValues();
  return filename;
}

void CmdCareECDBMIZ::dump(){
  arteMsg<<"Calibration file name :"<<filename;
  arteMsg.allp();
  arteMsg<<" Tables:"<<(int)tbset<<" action:"<<(int)action
         <<" Tags: "<<major<<","<<minor<<"  match:"<<match;
  arteMsg.allp();
}


void CmdCareECDBMIZ::setInfo(int itime1, int itime2,
			  char* iname, int iversion){
  info.firstTime = itime1;
  info.lastTime = itime2;
  info.storageTime = 0;
  info.version = iversion;
  strncpy(info.name,iname,12);
}


int CmdCareECDBMIZ::readDB(Tbset whichtable, int access){
  int rc = 0;
  int mymajor = -1;
  int myminor = -1;

  arteMsg<< "Opening  " << calibfile();
  arteMsg.allp();
  EcalDB & mydb = *EcalDB::instance();
  Tables x = mydb.Open(calibfile(), 0);
  if( x==0 ){
    rc = 1;
    arteMsg<<" Cannot find the ECAL calibration DB "<<calibfile()<<".";
    arteMsg.error();
  } else {
    switch(whichtable){
    case TSLOW:
      arteMsg<<"Reading calibration data (SLOW): ";
      arteMsg.allp();
      rc += readECBO( mydb, access);
      rc += readECHS( mydb, access);
      break;
    case TECBO:
      arteMsg<<"Reading calibration data (ECBO): ";
      arteMsg.allp(); 
      rc += readECBO( mydb, access);
      break;
    case TECHS:
      arteMsg<<"Reading calibration data (ECHS): ";
      arteMsg.allp();
      rc += readECHS( mydb, access);
      break;
    case TFAST:
      rc += readECHF( mydb, access);
      break;
    case TPED:
      rc += readECPD( mydb, access);
      break;
    case TSENS:
      rc += readECSN( mydb, access);
    break;
    case THOT:
      rc += readECHT( mydb, access);
      break;
    case TALL:
      arteMsg<<"Reading calibration data (ALL): ";
      arteMsg.allp();    
      rc += readECBO( mydb, access);
      rc += readECHS( mydb, access);
      rc += readECHF( mydb, access);
      break;
    }
    arteMsg << "Closing " << calibfile();
    arteMsg.allp();
    rc += mydb.Close();
    minor = myminor;
    major = mymajor;
  }
  return rc;
}

int CmdCareECDBMIZ::writeDB(Tbset whichtable, int access){
  int rc = 0;
  arteMsg<<"Opening "<<calibfile();
  arteMsg.allp();

  EcalDB & mydb = *EcalDB::instance();
  
  Tables x = mydb.Open(calibfile(), 0);
  if( x==0 ){
    rc = 1;
    arteMsg<<" Cannot find the ECAL calibration DB "<<calibfile()<<".";
    arteMsg.error();
    exit(1);
  } else {
    switch(whichtable){
    case TSLOW:
      rc += writeECBO(mydb, access);
      rc += writeECHS(mydb, access);
      break;
    case TECBO:
      rc += writeECBO(mydb, access);
      break;
    case TECHS:
      rc += writeECHS(mydb, access);
      break;
    case TFAST:
      rc += writeECHF(mydb, access);
      break;
    case TPED:
      rc += writeECPD( mydb, access);
      break;
    case TSENS:
      rc += writeECSN( mydb, access);
      break;
    case THOT:
      rc += writeECHT( mydb, access);
      break;
    case TALL:
      rc += writeECBO(mydb, access);
      rc += writeECHS(mydb, access);
      rc += writeECHF(mydb, access);
      break;
    }
    arteMsg << "Closing " << calibfile();
    arteMsg.allp();
    rc += mydb.Close();
  }
  return rc;
}

void CmdCareECDBMIZ::setDB( const char *name, int imaj, int imin){
  major = imaj;
  minor = imin;
  int i=0;
  for(; i<256 && name[i]!=' ' && name[i]!='\0'; i++)
      filename[i] = name[i];
  filename[i] = '\0';
}

 
int  CmdCareECDBMIZ::readECBO(EcalDB & mydb, int access){
  int rc = 0;
  mydb.ECBO.Book();
  
  if( mydb.ECBO.Match(major, minor,access)==0 ){
    arteMsg<<"This ECBO table version does not exists";
    arteMsg.warn();
    rc = 1;
  } else {
    int nc = mydb.ECBO.Nrow(major, minor,access);
    if( nc>=0){
      MMBank<ECBO>::Drop();
      MMBank<ECBO>::Book(nc);
      mydb.ECBO.Fetch(&(*(ArtePointer<ECBO>(1))));
      arteMsg<<"ECBO: Tags ("<< mydb.ECBO.Major()<<","<<mydb.ECBO.Minor()
	     <<")   Entries: "<<ArtePointer<ECBO>::nrow();
      arteMsg.allp();
      mydb.ECBO.Unbook();
      info = mydb.ECBO.Info();
    } else 
      rc =1;
  }
  return rc;
}

int  CmdCareECDBMIZ::readECHS(EcalDB & mydb, int access){
  int rc = 0;
  mydb.ECHS.Book();
  if( mydb.ECHS.Match(major, minor,access)==0){
    arteMsg<<"This ECHS table version does not exists";
    arteMsg.warn();
    rc = 1;
  } else {
    int nc = mydb.ECHS.Nrow(major, minor,access);
    if( nc>=0 ){
      MMBank<ECHS>::Drop();
      MMBank<ECHS>::Book(nc);
      mydb.ECHS.Fetch(&(*(ArtePointer<ECHS>(1))));
      arteMsg<<"ECHS: Tags ("<< mydb.ECHS.Major()<<","<<mydb.ECHS.Minor()
	     <<")   Entries: "<<ArtePointer<ECHS>::nrow();
      arteMsg.allp();
      mydb.ECHS.Unbook();
      info = mydb.ECHS.Info();
    } else 
      rc = 1;
  }
  return rc;
}

int  CmdCareECDBMIZ::readECHF(EcalDB & mydb, int access){
  int rc = 0;
  rc += readECPD( mydb, access);
  rc += readECSN( mydb, access);
  rc += readECHT( mydb, access);
  rc += makeECHF();
  return rc;
}


int  CmdCareECDBMIZ::makeECHF(){
  int rc = 0;
  MMBank<ECHF>::Drop();
  MMBank<ECHF>::Book(5956);
  ArteTable<ECHF>::iterator echf=ArteTable<ECHF>::begin();
  ArteTable<ECPD>::iterator echped=ArteTable<ECPD>::begin();
  ArteTable<ECSN>::iterator echsens=ArteTable<ECSN>::begin();
  for(; echf!=ArteTable<ECHF>::end(); ++echf, ++echped, ++echsens){
    //cout<<" Channel "<<echf<<endl;
    echf->stat = (echped->stat & 0xFFFF)+ ((echsens->stat & 0xFFFF)<<16);
    if( (echsens->stat & 1)==0 ) echf->stat =echf->stat & 0xFFFFFFFA; 
    echf->pedf = echped->pedf;
    echf->wid = echped->wid;
    echf->sens = echsens->sens;
    echf->esen = echsens->esen;
  }
  int entries = ArtePointer<ECHF>::nrow();
  for(ArteTable<ECHT>::iterator echotc=ArteTable<ECHT>::begin(); 
      echotc!=ArteTable<ECHT>::end(); ++echotc){
    if( echotc->chan>0 && echotc->chan<=entries && (echotc->stat &1)==1 ){
      ArtePointer<ECHF> echfx(echotc->chan);
      echfx->stat =  echfx->stat & 0xFFFFFFFA;
    }
  }
  return rc;
}

int CmdCareECDBMIZ::readECPD(EcalDB & mydb, int access){
  int rc = 0;

  mydb.ECPD.Book();
  if( mydb.ECPD.Match(major, minor,access)==0){
    arteMsg<<"This ECPD table version does not exists";
    arteMsg.warn();
    rc = 1;
  } else {
    int nc = mydb.ECPD.Nrow(major, minor,access);
    if( nc>=0 ){
      MMBank<ECPD>::Drop();
      MMBank<ECPD>::Book(nc);
      mydb.ECPD.Fetch(&(*(ArtePointer<ECPD>(1))));
      arteMsg<<"ECPD: Tags ("<< mydb.ECPD.Major()<<","<<mydb.ECPD.Minor()
	     <<")   Entries: "<<ArtePointer<ECPD>::nrow();
      arteMsg.allp();
      mydb.ECPD.Unbook();
      info = mydb.ECPD.Info();
    } else
      rc = 1;
  }
  return rc;
}

int CmdCareECDBMIZ::readECSN(EcalDB & mydb, int access){
  int rc = 0;
  mydb.ECSN.Book();
  if( mydb.ECSN.Match(major, minor,access)==0){
    arteMsg<<"This ECSN table version does not exists";
    arteMsg.warn();
    rc = 1;
  } else {
    int nc = mydb.ECSN.Nrow(major, minor,access);
    if( nc>=0 ){
      MMBank<ECSN>::Drop();
      MMBank<ECSN>::Book(nc);
      mydb.ECSN.Fetch(&(*(ArtePointer<ECSN>(1))));
      arteMsg<<"ECSN: Tags ("<< mydb.ECSN.Major()<<","<<mydb.ECSN.Minor()
	     <<")   Entries: "<<ArtePointer<ECSN>::nrow();
      arteMsg.allp();
      mydb.ECSN.Unbook();
      info = mydb.ECSN.Info();
    } else
      rc = 1;
  }
  return rc;
}

int  CmdCareECDBMIZ::readECHT(EcalDB & mydb, int access){
  int rc = 0;
  mydb.ECHT.Book();
  if( mydb.ECHT.Match(major, minor,access)==0){
    arteMsg<<"This ECHT table version does not exists";
    arteMsg.warn();
    rc = 1;
  } else {
    int entries = mydb.ECHT.Nrow(major, minor,access);
    if( entries>=0) {
      MMBank<ECHT>::Drop();
      MMBank<ECHT>::Book(entries);
      mydb.ECHT.Fetch(&(*(ArtePointer<ECHT>(1))));
      arteMsg<<"ECHT: Tags ("<< mydb.ECHT.Major()<<","<<mydb.ECHT.Minor()
	     <<")   Entries: "<<ArtePointer<ECHT>::nrow();
      arteMsg.allp();
      mydb.ECHT.Unbook();
      info = mydb.ECHT.Info();
    } else
      rc = 1;
//    arteMsg<<"ECHT: infos ("<< info.storageTime<<","<<info.name<<")";
//    arteMsg.allp();
  }
  return rc;
}

int CmdCareECDBMIZ::writeECBO(EcalDB & mydb, int access){
  int rc = 1;
  rc *= mydb.ECBO.Book();
  rc *= mydb.ECBO.Store(ArtePointer<ECBO>::nrow(),
		  &(*(ArtePointer<ECBO>(1))),
		  major, minor, info, access);
  rc *= mydb.ECBO.Unbook();
  return rc==0;
}

int CmdCareECDBMIZ::writeECHS(EcalDB & mydb, int access){
  int rc = 1;
  rc *= mydb.ECHS.Book();  
  rc *= mydb.ECHS.Store(ArtePointer<ECHS>::nrow(),
		  &(*(ArtePointer<ECHS>(1))),
		  major, minor, info, access);
  rc *= mydb.ECHS.Unbook();
  return rc==0;
}

int  CmdCareECDBMIZ::writeECHF(EcalDB & mydb, int access){
  int rc = 0;
  rc += writeECPD(mydb, access);
  rc += writeECSN(mydb, access);
  rc += writeECHT(mydb, access);
  return rc;
}

int CmdCareECDBMIZ::writeECPD(EcalDB & mydb, int access){
  int rc = 1;
  rc *= mydb.ECPD.Book();
  rc *= mydb.ECPD.Store(ArtePointer<ECPD>::nrow(),
		  &(*(ArtePointer<ECPD>(1))),
		  major, minor, info, access);
  rc *= mydb.ECPD.Unbook();
  return rc==0;
}

int CmdCareECDBMIZ::writeECSN(EcalDB & mydb, int access){
  int rc = 1;
  rc *= mydb.ECSN.Book();
  rc *= mydb.ECSN.Store(ArtePointer<ECSN>::nrow(),
		  &(*(ArtePointer<ECSN>(1))),
		  major, minor, info, access);
  rc *= mydb.ECSN.Unbook();
  return rc==0;
}

int CmdCareECDBMIZ::writeECHT(EcalDB & mydb, int access){
  int rc = 1;
  rc *= mydb.ECHT.Book();
  rc *= mydb.ECHT.Store(ArtePointer<ECHT>::nrow(),
		  &(*(ArtePointer<ECHT>(1))),
		  major, minor, info, access);
  rc *= mydb.ECHT.Unbook();
  return rc==0;
}

