//                              -*- Mode: C++ -*-
//:LUTsManager.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 9/4/97
// Last Update ........
//

#include "care/ELLUTsManager.hh"
#include "care/ELCellGrid.hh"
#include "care/ELNearNeiCellLUT.hh"
#include "care/ECCellID.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteProto.hh"
#include "care/caredata.hh"
#include "care/MMBank.hh"
#undef WRITE
#include <iostream>

////////////////// Data storage for LUTS

// luts
ELCellGrid ELLUTsManager::cells; // to find  the index of the cell given 
                               // a point on ECAL
                               // or to find the module number or the Cell ID

ELNearNeiCellLUT   ELLUTsManager::nearNei;   // Table of nearest neibourings

/////////////////// ELLUTsManager static data initialization

int ELLUTsManager::initialized = false; // set to false the initialization

/////////////////// ELLUTsManager member function definitions

void ELLUTsManager::instance(){

  if(!initialized){
    // now set to inizialized 
    initialized = true;

    // build all the LookUpTables and static data
    // first step find ECAL in GESL bank
#   ifdef WRITE
    std::cout<<"Searchin for ECAL"<<std::endl;
#   endif 
    // Ecal Pointer to GESL table
    ArteTable<GESL>::iterator ecalp = ArteTable<GESL>::begin();
    for(; ecalp!=ArteTable<GESL>::end(); ++ecalp){
#     ifdef WRITE
      std::cout<<ecalp->name<<std::endl;
#     endif //WRITE
      if(!strncmp(ecalp->name,"CAL",3)){
      // ECAL found. Exit this loop
      break;
      }
    }
    if( ecalp!=ArteTable<GESL>::end() ){
      ECalInfo.ecalp = ecalp;
#     ifdef WRITE
      std::cout<<" Ecal Found! Now building tables"<<std::endl;
      std::cout<<" ECAL"<<std::endl
	  <<" Number of layers : "<<ECalInfo.ecalp->lay<<std::endl
	  <<" Outer dimension X: "<<ECalInfo.ecalp->x1
	  <<" Outer dimension Y: "<<ECalInfo.ecalp->y1<<std::endl
	  <<" Z begin: "<<ECalInfo.ecalp->z1
	  <<" Z end: "<<ECalInfo.ecalp->z2<<std::endl
	  <<" Number of planes : "<<ECalInfo.ecalp->npl<<std::endl;

#     endif
      arteMsg<<" * Inizialization of Look Up Tables for CARE ";
      arteMsg.allp();
      // start initializing the CellID class first plane
      ECCellID::setFirstPlane(ECalInfo.ecalp->fpl);

      // Define static data for the calorimeter: the lower corner and
      // the size of modules
      // the lower corner is found as the minimum of all xmin and ymin 
      Coord_t x0,y0;
      // Define a Module iterator
      ArteTable<GCAL>::iterator 
	modulep(ArtePointer<GCAL>(ECalInfo.ecalp->fpl));
      // Define module sizes
      ECalInfo.xMsize = ((int)((modulep->xmax-modulep->xmin)*1000.+0.5))*0.001;
      ECalInfo.yMsize = ((int)((modulep->ymax-modulep->ymin)*1000.+0.5))*0.001;

      // Loop on Modules to find Ecal xmin and ymin (X0,Y0)
      x0 = modulep->xmax;
      y0 = modulep->ymax;
      int i=0,j,k; 
      int  numberOfModules = ECalInfo.ecalp->npl; // Number of modules

#     ifdef WRITE
      std::cout<<"Searching Xmin and Ymin for ECAL"<<std::endl;
#     endif // WRITE

      for( ArteTable<GCAL>::iterator 
	     modp(ArtePointer<GCAL>(ECalInfo.ecalp->fpl)); 
	   i<numberOfModules; ++modp, i++){
	if( x0 > modp->xmin) x0 = modp->xmin;
	if( y0 > modp->ymin) y0 = modp->ymin;
      }
      // now in X0, Y0 there is the lower corner of ECAL
      ECalInfo.x0 = x0;
      ECalInfo.y0 = y0;
      //
#     ifdef WRITE
      std::cout<<"Set ECAL lower corner to ("<<x0<<","<<y0<<")"<<std::endl;
#     endif

      Coord_t halfSizeX = ECalInfo.xMsize*0.5;
      Coord_t halfSizeY = ECalInfo.yMsize*0.5;
    
      // Below this point all data of the ECal structure are defined
      //
      // Drop all Calorimeter banks that are not void
#     ifndef ONLINE
      if( ArtePointer<GCCE>::getLink().getRef()!=0)
	MMBank<GCCE>::Drop();
      if( ArtePointer<GNEI>::getLink().getRef()!=0)
	MMBank<GNEI>::Drop();
#     endif

      // loop on Modules to find cells
      // Start with the Grid 
      // Initialize to 0 the whole LUT
      //   First find number of X and Y modules
      //   At the moment this is fixed to 56,42 (see definition of ECal)
      //   Fill with blanks
      // Initialize the grid to -1
      cells.reserve(ECalInfo.nModX*ECalInfo.nModY);

      // define a void CellStruct
      GCCE CS = {0,0,0};    
      CellIdx_t ncell = 1; // define the number of already found cells
      // now loop on all modules and cell inside a module to fill LUTs
      int numcells,ndim;
      CellIdx_t cellIdxtemp;
      // Define a Module iterator
#     ifdef WRITE
      std::cout<<" Now Loop on modules and then on cells;"<<std::endl;
      std::cout<<" Ecal first plane:"<<ECalInfo.ecalp->fpl<<std::endl;
#     endif
      ArteTable<GCAL>::iterator 
	modulp(ArtePointer<GCAL>(ECalInfo.ecalp->fpl));
      for(i=0; i<numberOfModules; ++modulp, i++){
	modulp->fcl = ncell.index();
#     ifdef WRITE
	std::cout<<" Cell "<<modulp<<" first link to GCCE:"<<modulp->fcl<<std::endl;
#     endif
	// Initialization for first cell
	int ix,iy;
	ELCellGrid::ErrorTypes ierr;
	// this function fills ix,iy,ierr
	cellIdxtemp = cells.findIXIY(modulp->xmin+halfSizeX, 
				     modulp->ymin+halfSizeY, 
				     ix, iy, ierr);
	// Warning: the preceding function during initialization should
	// ALWAYS return an error code HOLE.
	if(ierr==ELCellGrid::OK || ierr==ELCellGrid::HOLE){
	  // now I know where to put ncell in Cells
	  cells.set(ix,iy,ncell);
	  cells.lookUp(ix,iy);
	  // Get ready to fill the CellDesc LUT
	  // For each cell there is an entry.
	  ndim = modulp->ncl;
	  for(j=1; j<=ndim; j++){
	    for(k=1; k<=ndim; k++){
	      CS.ceid = ECCellID(modulp.index(),j,k,ndim)();
	      ArtePointer<GCCE>::append(CS);
	    }
	  } 
	  numcells = ndim*ndim;
	  ncell = ncell.index() + numcells;
	} else {
	  //
	  // the following lines are executed this means that
	  // there is a rathar bad error in the geometry of ECAL
	  // (table GCAL)
	  //
	  static int LUTerror = 0x0ECA0003;
	  arteMsg<<"ELLUTsManager: Error a module corner is not in ECAL ??";
	  arteMsg.error(LUTerror);
	  arteMsg<<modulp<<" X,Y="<<modulp->xmin+halfSizeX
	      <<", "<<modulp->ymin+halfSizeY<<" IX,IY = "<<ix<<" "<<iy;
	  arteMsg.allp();
        }
      }
      arteMsg<<" * CellGrid           LUT initialized with "
	     <<ArtePointer<GCGD>::nrow()<<" elements";
      arteMsg.allp();
      arteMsg<<" * FirstCellInMod     LUT initialized with "	
	     <<ArtePointer<GCAL>::nrow()<<" elements";
      arteMsg.allp();
      arteMsg<<" * CellDescription    LUT initialized with "
	     <<ArtePointer<GCCE>::nrow()<<" elements";
      arteMsg.allp();
      //
      // at this point the CellGrid and the FirstCellInMod are
      // fully initialized
      // The CellDesc LUT is initialized only for ModuleNumber
      // Now call initialize on NearNei and this will change even the
      // CellDesc LUT
      //
#     ifdef WRITE
      std::cout<<"Now remains to initialize neibourings"<<std::endl;
#     endif
      nearNei.initialize();
      arteMsg<<" * NearestNeibourings LUT initialized with "
          <<nearNei.size()<<" elements";
      arteMsg.allp();
    } else {
      //
      // ECAL not found in the current geometry.
      // it should be switched off
      //
      arteMsg<<" ECAL NOT FOUND !!!!!!!!!";
      arteMsg.error();
      ECalInfo.nModX = 0;
      ECalInfo.nModY = 0;
    }
  }
}





// Rebuild the GCAX table
// This function will be called when after the first initialization of
// CARE the geometry has been read again. 

void ELLUTsManager::RebuildGCAL() {
  

  CellIdx_t ncell = 1; // define the number of already found cells
  // now loop on all modules and cell inside a module to fill LUTs
  int numcells,ndim;
  int numberOfModules = ECalInfo.ecalp->npl; // Number of modules
  ArteTable<GCAL>::iterator
    modulp(ArtePointer<GCAL>(ECalInfo.ecalp->fpl));
  for(int i=0; i<numberOfModules; ++modulp, i++){
    modulp->fcl = ncell.index();
#   ifdef WRITE
    std::cout<<" Cell "<<modulp<<" first link to GCCE:"<<modulp->fcl<<std::endl;
#   endif
    // For each cell there is an entry.
    ndim = modulp->ncl;
    numcells = ndim*ndim;
    ncell = ncell.index() + numcells;
  }
}
