// ********************************************************************
//
//  Routine:      acReadECAL
//  Author:       M.Villa,  March 08, 2000 
//
//  Description:  checks for DB updates and triggers the DB readings
//                
//
//  Modification:
//
// *******
//                - returns <0 error
//                           0 no DB update necessary
//                           1 update done using key
//                           2 update done using date/time or run number
//*************************************************************

#include <stdio.h>

#include "arte/arte.hh"
#include "database/keytable.h"
#include "farm/farm.hh"
#include "grr.h"
#include "care/ECALDetector.hh"

static int ECALserver = -1;
enum Tab_t { Ecbo, Echs, Ecpd, Ecsn, Echt};
static int ECALtable[5]  = {-1,-1,-1,-1,-1};

int acReadECAL( int KTstatus) {

  int rc = 0; // default: no update necessary
  static char  dbname[256];
  char   *dbnamep=0;

  std::cout<<"AC_ReadECAL called with:"<<KTstatus<<std::endl;

  if( KTstatus==2){
    ////////////////////// initialization //////////////////////

    dbnamep=getenv("HBECALDB");
    if (!dbnamep) {
      strcpy(&dbname[0],"/ECAL_CALIB");
    } else {
      strcpy(&dbname[0],&dbnamep[0]);
    }
    ECALserver = KeyFindServer(dbname);
    if (ECALserver==-1) {
      ARTE_error( GRR_SWARNING," acReadECAL: KeyTable server not found", -1); 
      KTstatus = -2;      
    } else {
      ECALtable[0] = KeyFindTable(ECALserver, "ECBO");
      ECALtable[1] = KeyFindTable(ECALserver, "ECHS");
      ECALtable[2] = KeyFindTable(ECALserver, "ECPD");
      ECALtable[3] = KeyFindTable(ECALserver, "ECSN");
      ECALtable[4] = KeyFindTable(ECALserver, "ECHT");
      if (ECALtable[0]==-1 || ECALtable[1]==-1 || ECALtable[2]==-1 ||
	  ECALtable[3]==-1 || ECALtable[4]==-1 ) {
	ECALtable[0] = -1;
	ARTE_error( GRR_SWARNING," $$$ acReadECAL: KeyTable not found", -1);
	KTstatus = -2;
      } else {
	KTstatus = 1;
      }
    }
  }

  if( KTstatus==1){   
    /////////////////////// key assigned /////////////////////
    // Database entry with assigned key
    // take the keytable revision and run
    
    if (ECALserver==-1 || ECALtable[0]==-1) {
      KTstatus = -2;
    } else {
      int minor[5];
      int major[5];
      rc = 0;
      for(unsigned int i=0; i<5; i++){
	if ( KeyTableNew(ECALtable[i]) ){
	  major[i] = KeyTableMajor(ECALtable[i]);
	  minor[i] = KeyTableMinor(ECALtable[i]);
	  std::cout<<" Loading table  "<<i<<" major,minor = "<<major[i]
		   <<","<<minor[i]<<std::endl;
	  ++rc; 
	} else {
	  major[i] = -1;
	  minor[i] = -1; 
	}
      }
      if( rc>0){
	rc = ECALDetector::newDBData(dbname, major, minor);
	if( rc==0) rc = 1;  // update using keytable
	else rc = -1;       // error condition
      }     // else rc=0 ; no update necessary
    }	
  }

  if( KTstatus==-2 && ArteTable<ECHF>::size()==0 ){

    //////////////////// key NOT assigned ////////////////////
    // Database entry without assigned key:
    // take the highest revision of nearest run
    int run = 0;
    if( ArteTable<EVHD>::size()>0){
      run = (*ArteTable<EVHD>::begin())->run;
    }
    rc = ECALDetector::newDBData(run);
    if( rc==0 ) rc = 2; // update using run number
    else rc = -2;       // error condition
  }
  cout<<" Return code for KT status "<<KTstatus<<" : "<<rc<<endl;
  return rc;
}
