#ifndef LOVER_HH
#define LOVER_HH

#include <stdio.h>
#include <iostream.h>
#include <fstream.h>
#include <iomanip.h>
#include <math.h>
#include <arte/arte.hh>
#include <arte/RSEG_cmp.hh>
#include "../include/hbook.h"
#include "../include/higz.h"
//stl library
#include <map>
#include <multimap.h>
#include <vector>
#include <set>
#include <algorithm>
#include <list>

using namespace std;

extern int acLocalToGlobal(float *, float *, int);
extern "C"{
  void CatsExtrapolate(ArtePointer<RSEG>, float, 
		       float& , float& , float& , float& , float *);
}

typedef struct{
  int   index;             // internal tracknumber
  ArtePointer<RSEG> rseg;  // pointer to rseg
  float z;                 // z-coordinate of point on a track
  float x;                 // x-coordinate of point on a track
  float y;                 // y-coordinate of point on a track
  float tx;                // slope dx/dz                     
  float ty;                // slope dy/dz                     
  float p;                 // trackmomentum
  float C[15];             // (triangular) covariance matrix (x,y,tx,ty,p) 
  float w;                 // weight for primary vertexing
  int   used;              // used already for vertexing
} Tracks_t;

typedef map< int, Tracks_t , less<int> >                 Tracks_m;
typedef map< int, Tracks_t , less<int> >::iterator       Tracks_mi;
typedef map< int, Tracks_t , less<int> >::const_iterator Tracks_mci;

typedef map< int,list<int>,less<int> >                     Cluster_m;
typedef map< int,list<int>,less<int> >::iterator           Cluster_mi;


//information about the targets for primary vertices

typedef struct {
                int wnr;
	        int flaghorver;
	        float x;
		float y;
		float z;
		float dx;
		float dy;
                float dz;
                int   n_tr;
                int   n_vx;
} targetstruct ;

typedef map< int, targetstruct, less<int> >           Target_m;
typedef map< int, targetstruct, less<int> >::iterator Target_mi;

typedef multimap<float,Tracks_t,less<float> >  trackmap;
typedef multimap<float,Tracks_t,less<float> >::iterator trackmapit;

typedef map< int, vector<double> , less<int> >         vector_m;

class Lover
{
  private:
          Tracks_m tracks;
          RVER vertex;         //vertexpositions  
          Tracks_mi trackincluster[200];
          int ntrincluster;
	  int mintracksinvertex;
          Target_m targets;
          Tracks_mi trackonwire[9][200];    // pointers to tracks
          float trackweight[9][200];       // track weights on corr. wire
          int ntronwire[9];                 // array size (number of tracks)

          void fillRSEGmap(Tracks_m&);
          void get_targets(void);
          void trackpropagate(Tracks_t&, float);
          int  target_near(Tracks_t&, int, float);
          void vertex_refit(void);
          void four_dim_inv(float [4][4]);
          void graph_ini(void);
          void graph_close(void);
          void draw_event(void);
  // verona:
          void verona_init(void);
          void start_hsto(void);
          void fill_ver_hsto(void);
          void fill_track_hsto(void);
          int  getclosestvertex(vector<double>,vector<double>,vector_m,int&);
          int  closestwire(vector<double>,vector<double>,int&, float&, float&, int&);
 public:
        Lover();
	~Lover();
  	void lover_primaries(void);
        void verona_event(void);
};	 	  
#endif



