/****************************************************************************
*                                                                           *
*                  ROVER                                                    *
*               version 1.01                                                *
*                                                                           *
*            Dmitri Emeliyanov                                              *
*            Ivan Kisel                       Modified: 13-Mar-00 ver. 1.00 *
*            Max-Planck-Institut fuer Physik            21-Mar-00 ver. 1.01 *
*            Muenchen                                                       *
*  Email:    ikisel@mppmu.mpg.de                                            *
*            HERA-B Collaboration                                           *
*                                                                           *
*                                                                           *
* ------------------------------------------------------------------------- *
*       vertex reconstruction package                                       *  
*                                                                           *
****************************************************************************/
 
#include "../cats/include/rover.hh" // all includes are there

static int Black =  1, 
    DarkGray     =  2, 
    Blue         =  3, 
    Green        =  4, 
    Cyan         =  5, 
    Red          =  6, 
    Magenta      =  7, 
    Brown        =  8, 
    LightBlue    =  9, 
    LightGreen   = 10, 
    LightCyan    = 11, 
    LightRed     = 12, 
    LightMagenta = 13, 
    Yellow       = 14, 
    LightGray    = 15, 
    White        =  0;


static Lover* lover = static_cast<Lover *>(0);
int rover_newgeometry = 1;
int ROVER_DRAW;
static float zsta = 0.0;;

extern "C"{
  void prima_love(void);
  
  void prima_love(void){
    static int firstcall = 1;
    
    if(firstcall){
      cout<<"ROVER version 1.01 started"<<endl;
      if(lover == static_cast<Lover*>(0))
	lover = new Lover();
      firstcall = 0;
    } 
    lover->lover_primaries();
    //    lover->verona_event();
  }
}

Lover::Lover(){
  mintracksinvertex = 3;
  if (ROVER_DRAW)
    graph_ini();
}

Lover::~Lover(){
  if (ROVER_DRAW)
    graph_close();

}

void Lover::lover_primaries(void){

  fillRSEGmap(tracks);
  get_targets();

  if (ROVER_DRAW)
    draw_event();

  ArteTable<RVER>::clear();

  for(Target_mi tarit = targets.begin();tarit != targets.end();++tarit){
    int wnr = (tarit->second).wnr;
    int ntr = 0;
    for(Tracks_mi it = tracks.begin();it != tracks.end();++it){
      trackpropagate(it->second,(tarit->second).z);
      if(target_near(it->second,tarit->first,3.5)){
	ntr++;
	trackonwire[wnr][ntr] = it;
	trackweight[wnr][ntr] = 0.0;;
      }
    }
    ntronwire[wnr] = ntr;
  }

  int   vertexfound = 1;
  ntrincluster = 0;

  while(true){   
    //  while(vertexfound){   
    vertexfound = 0;
    
    Tracks_mi it_best = tracks.begin();
    float w_best      = 0.0;
    int max_clu = 0;
    Target_mi   wire_best   = 0;
    Tracks_mi it, jt;
    int tracksonwire;
    for(Target_mi tarit = targets.begin();tarit != targets.end();++tarit){
      int wnr = (tarit->second).wnr;
      tracksonwire = ntronwire[wnr];
      if(tracksonwire < mintracksinvertex) 
	continue;
      int itrnew = 0;

      for (int itr=1; itr <= tracksonwire; itr++){ // remove used tracks
	it = trackonwire[wnr][itr];
	if ((it->second).used)
	  continue;
	itrnew++;
	trackonwire[wnr][itrnew] = trackonwire[wnr][itr];
	trackweight[wnr][itrnew] = trackweight[wnr][itr];
	trackpropagate(it->second,(tarit->second).z);
      }
      tracksonwire = ntronwire[wnr] = itrnew;
      if(tracksonwire < mintracksinvertex) 
	continue;

      //      cout<<"Wire "<<wnr<<" ";
      for (int itr=1; itr <= tracksonwire; itr++){ // calculate weights
	it = trackonwire[wnr][itr];
	(it->second).w = 0.0;
	float x1 = (it->second).x;
	float y1 = (it->second).y;
	int ntr_clu = 0;
	for (int jtr=1; jtr <= tracksonwire; jtr++){ 
	  jt = trackonwire[wnr][jtr];
	  float x2 = (jt->second).x;
	  float y2 = (jt->second).y;
	  float dist2 = (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2);
	  float cut2 = 3.5*3.5*
	    ((it->second).C[0]+(it->second).C[2]+
	     (jt->second).C[0]+(jt->second).C[2]);
	  if(dist2 < cut2){
	    (it->second).w+=(1.0-(dist2/cut2))*(1.0-(dist2/cut2));///
	    //	      (sqrt((jt->second).C[0])+sqrt((jt->second).C[2]));
	    ntr_clu++;
	  }
	}
	//	cout<<(it->second).w<<"/"<<ntr_clu<<" ";
	//	(it->second).w/=((it->second).C[0]+(it->second).C[2]);
	if ((it->second).w>w_best){
	  //if (ntr_clu>max_clu){
	  it_best = it;
	  w_best  = (it->second).w;
	  max_clu = ntr_clu;
	  wire_best = tarit;
	}
      }
      //      cout<<endl<<endl;
    }

    if (w_best==0.0)
      break;

#ifdef XXX
    if (ROVER_DRAW){
      
      ismk(30); ismksc(7.0);	ispmci(Green); isplci(Green);
      
      float xpm[1], ypm[1];
      
	
      xpm[0] = (it_best->second).x;
      ypm[0] = (it_best->second).y;
      ipm(1,xpm,ypm);
	
      cout<<"Best track on wire "<<(wire_best->second).wnr<<" : "
	  <<sqrt((it_best->second).C[0])<<" "
	  <<sqrt((it_best->second).C[2])<<" has "
	  <<max_clu<<" neighbours"<<endl;
    }
#endif

    int wnr = (wire_best->second).wnr;
    tracksonwire = ntronwire[wnr];

    for (int itr=1; itr <= tracksonwire; itr++){ // propagate tracks
      it = trackonwire[wnr][itr];
      trackpropagate(it->second,(wire_best->second).z);
    }

    for (int itr=1; itr <= tracksonwire; itr++){ // calculate weights
      it = trackonwire[wnr][itr];
      (it->second).w = 0.0;
      float x1 = (it->second).x;
      float y1 = (it->second).y;
      for (int jtr=1; jtr <= tracksonwire; jtr++){ 
	jt = trackonwire[wnr][jtr];
	float x2 = (jt->second).x;
	float y2 = (jt->second).y;
	float dist2 = (x1-x2)*(x1-x2)+(y1-y2)*(y1-y2);
	float cut2 = 3.5*3.5*
	  ((it->second).C[0]+(it->second).C[2]+
	   (jt->second).C[0]+(jt->second).C[2]);
	if(dist2 < cut2)
	  (it->second).w+=(1.0-(dist2/cut2))*(1.0-(dist2/cut2));///
	//	      (sqrt((jt->second).C[0])+sqrt((jt->second).C[2]));
      }
      //      (it->second).w/=((it->second).C[0]+(it->second).C[2]);
    }


    float x_best  = (it_best->second).x;
    float y_best  = (it_best->second).y;
    float c0_best = (it_best->second).C[0];
    float c2_best = (it_best->second).C[2];
    float vertx = 0.0, verty = 0.0, vertweight = 0.0;
    ntrincluster = 0;
    for (int itr=1; itr <= tracksonwire; itr++){ // collect cluster
      it = trackonwire[wnr][itr];
      float x = (it->second).x;
      float y = (it->second).y;
      float w = (it->second).w;
      float dist2 = (x-x_best)*(x-x_best)+(y-y_best)*(y-y_best);
      float cut2 = 3.5*3.5*
	((it->second).C[0]+(it->second).C[2]+
	 c0_best+c2_best);
      if(dist2 < cut2){
	vertx += w*x;
	verty += w*y;
	vertweight += w;
	trackincluster[++ntrincluster] = it;
      }
    }

    if (ntrincluster < mintracksinvertex){
      //      cout<<"BREAK with "<<ntrincluster<<" tracks in cluster"<<endl;
      break;
    }

    if(ntrincluster >= mintracksinvertex){
      
      vertex.mass = 0.0;
      vertex.merr = 0.0;
      
      vertx /= vertweight;
      verty /= vertweight;      
      vertex.xv = vertx;
      vertex.yv = verty;
      vertex.zv = (wire_best->second).z;
      
      float cov_x = 0.0;
      float cov_y = 0.0;
      float cov_z = 0.0;
      for(int itr=1; itr<=ntrincluster; itr++){
	it = trackincluster[itr];
	cov_x = ((it->second).x-vertx)*((it->second).x-vertx);
	cov_y = ((it->second).y-verty)*((it->second).y-verty);
      }
      cov_x /= ntrincluster;
      cov_y /= ntrincluster;
      cov_z  = 1.0e-2;
      
      vertex.cv[0] = cov_x; //float(vcov[0]) == Vcov[0][0];     
      vertex.cv[1] = 0.0;   //float(vcov[3]) == Vcov[0][1];
      vertex.cv[2] = cov_y; //float(vcov[4]) == Vcov[1][1];
      vertex.cv[3] = 0.0;   //float(vcov[6]) == Vcov[0][2];     
      vertex.cv[4] = 0.0;   //float(vcov[7]) == Vcov[1][2];
      vertex.cv[5] = cov_z; //float(vcov[8]) == Vcov[2][2];
      vertex.nt = ntrincluster;

#ifdef XXX
      if (ROVER_DRAW){
	ismk(30); ismksc(3.0); ispmci(Blue); isplci(Blue);
	  
	float xpm[1], ypm[1];
	xpm[0]=vertex.xv;
	ypm[0]=vertex.yv;
	ipm(1,xpm,ypm);
	
	float xv = vertex.xv;
	float dx = sqrt(vertex.cv[0]);
	float yv = vertex.yv;
	float dy = sqrt(vertex.cv[2]);
	
	float xpl[5], ypl[5];
	xpl[0] = xv-dx;
	ypl[0] = yv-dy;
	xpl[1] = xv+dx;
	ypl[1] = yv-dy;
	xpl[2] = xv+dx;
	ypl[2] = yv+dy;
	xpl[3] = xv-dx;
	ypl[3] = yv+dy;
	xpl[4] = xv-dx;
	ypl[4] = yv-dy;
	ipl(5,xpl,ypl);
      }
#endif

      vertex_refit();

      if (vertex.nt>=mintracksinvertex){
	vertexfound = 1;
	
	// modify the target wire position
	
	float dx = 0.1, dy = 0.1;
	
	float xt    = (wire_best->second).x;
	float yt    = (wire_best->second).y;
	float zt    = (wire_best->second).z;
	float dxt   = (wire_best->second).dx;
	float dyt   = (wire_best->second).dy;
	float dzt   = (wire_best->second).dz;
	int   nt_tr = (wire_best->second).n_tr;
	int   nt_vx = (wire_best->second).n_vx;
	int   nv  = vertex.nt;
	
	float x = vertex.xv;
	float y = vertex.yv;
	float z = vertex.zv;
	
	
	if ((fabs(x-xt)<dx)&&(fabs(y-yt)<dy)){
	  xt = xt*(nt_tr+nt_vx)+x*3*nv;
	  yt = yt*(nt_tr+nt_vx)+y*3*nv;
	  zt = zt*nt_vx+z*3*nv;
	  nt_vx+= 3*nv;
	  xt /= (nt_tr+nt_vx);
	  yt /= (nt_tr+nt_vx);
	  zt /= nt_vx;
	  
	  dxt = (dxt*(nt_tr+nt_vx-3*nv)+nv*fabs(x-xt))/(nt_tr+nt_vx);
	  dyt = (dyt*(nt_tr+nt_vx-3*nv)+nv*fabs(y-yt))/(nt_tr+nt_vx);
	  dzt = (dzt*(nt_vx-3*nv)+3*nv*fabs(z-zt))/nt_vx;
	}
	
	if (nt_tr+nt_vx > 10000){
	  nt_tr = 100;
	  nt_vx = 100;
	}	

	(wire_best->second).x    = xt;;
	(wire_best->second).y    = yt;
	(wire_best->second).z    = zt;
	(wire_best->second).dx   = dxt;
	(wire_best->second).dy   = dyt;
	(wire_best->second).dz   = dzt;
	(wire_best->second).n_tr = nt_tr;
	(wire_best->second).n_vx = nt_vx;
	

	if (ROVER_DRAW){

	  ismk(3); ismksc(0.5);	ispmci(LightRed); isplci(LightRed);

	  float xpm[1], ypm[1];

	  for(int itr=1; itr<=ntrincluster; itr++){
	    Tracks_mi it = trackincluster[itr];
	    if (!(it->second).used) 
	      continue;

	    xpm[0] = (it->second).x;
	    ypm[0] = (it->second).y;
	    ipm(1,xpm,ypm);
	    
	    float xm = (it->second).x;
	    float ym = (it->second).y;
	    float dx = sqrt((it->second).C[0]);
	    float dy = sqrt((it->second).C[2]);
	    
	    igarc(xm, ym, 0.5*(dx+dy),0.5*(dx+dy), 0.0, 0.0);
	  }

	  ismk(30); ismksc(5.0); ispmci(Red); isplci(Red);
	  
	  xpm[0]=vertex.xv;
	  ypm[0]=vertex.yv;
	  ipm(1,xpm,ypm);
	  
	  float xv = vertex.xv;
	  float dx = sqrt(vertex.cv[0]);
	  float yv = vertex.yv;
	  float dy = sqrt(vertex.cv[2]);
	  
	  float xpl[5], ypl[5];
	  xpl[0] = xv-dx;
	  ypl[0] = yv-dy;
	  xpl[1] = xv+dx;
	  ypl[1] = yv-dy;
	  xpl[2] = xv+dx;
	  ypl[2] = yv+dy;
	  xpl[3] = xv-dx;
	  ypl[3] = yv+dy;
	  xpl[4] = xv-dx;
	  ypl[4] = yv-dy;
	  ipl(5,xpl,ypl);
	}
	
      }    
    }
  }


  if (ROVER_DRAW){

    ismk(3); ismksc(0.5); ispmci(Blue); isplci(Blue);

    float xpm[1], ypm[1];

    for(Target_mi tarit = targets.begin();tarit != targets.end();++tarit){
      int wnr = (tarit->second).wnr;
      int tracksonwire = ntronwire[wnr];

      for (int itr=1; itr <= tracksonwire; itr++){ // draw unused tracks
	Tracks_mi it = trackonwire[wnr][itr];
	if ((it->second).used)
	  continue;

	trackpropagate(it->second,(tarit->second).z);
	
	xpm[0] = (it->second).x;
	ypm[0] = (it->second).y;
	ipm(1,xpm,ypm);
	
	float xm = (it->second).x;
	float ym = (it->second).y;
	float dx = sqrt((it->second).C[0]);
	float dy = sqrt((it->second).C[2]);
	
	igarc(xm, ym, 0.5*(dx+dy),0.5*(dx+dy), 0.0, 0.0);
      }
    }


    iuwk(1,1);
    
  }

  for( ArteTable<RVER>::iterator rverIt = ArteTable<RVER>::begin();
       rverIt != ArteTable<RVER>::end() ; ++rverIt){
    
    // transform to global coordinates !!!
    
    float local[]  = {0,0,0,0,0};
    float global[] = {0,0,0,0,0};
    
    local[0] = (*rverIt)->xv;
    local[1] = (*rverIt)->yv;
    local[2] = (*rverIt)->zv;
    acLocalToGlobal(local, global, 1);
    (*rverIt)->xv = global[0];
    (*rverIt)->yv = global[1];
    (*rverIt)->zv = global[2];
  }

}


void Lover::fillRSEGmap(Tracks_m& tracks){
  
  Tracks_t trackstruct;
  tracks.clear();
  int ntracks = 0;
  
  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin(); 
       rseg != ArteTable<RSEG>::end(); ++rseg ){
    ++ntracks;
    if(  !((*rseg)->cmp & ~(Rsegc::vxd | Rsegc::bit16)) ){
      trackstruct.index = ntracks;
      trackstruct.rseg = *rseg;
      trackstruct.z  = (*rseg)->zf;
      trackstruct.x  = (*rseg)->xf;
      trackstruct.y  = (*rseg)->yf;
      trackstruct.tx = (*rseg)->txf;
      trackstruct.ty = (*rseg)->tyf;
      trackstruct.p  = (*rseg)->pf;
      for(int i=0;i<15;++i)
	trackstruct.C[i] = (*rseg)->cvf[i];
      trackstruct.used  = 0;
      tracks[trackstruct.index] = trackstruct;
    }
  } 
  
}     

void Lover::get_targets(void){
  float dx = 0.1, dy = 0.1, dz = 0.1;

  if (rover_newgeometry){
    rover_newgeometry = 0;

    //	cout<<"Beam "<<BEAM.x<<" "<<BEAM.y<<" "<<BEAM.z<<endl; //???
    
    float beam_x = 0.0, beam_y = 0.0;  // beam center for simu data

    if (!ArteTable<MCIN>::booked()){
      beam_x =  0.30;                  // beam center for real data
      beam_y = -0.05;
    }
    
    cout.setf(ios::fixed);
    cout.setf(ios::showpoint);
    cout.precision(4);
    cout<<endl;

    float zsta1 = 0.0, zsta2 = 0.0;
    int   nsta1 = 0,   nsta2 = 0;
    //initialize target arrays used for allignemt
    for( ArteTable<GTAR>::iterator gtarIt = ArteTable<GTAR>::begin();
	 gtarIt != ArteTable<GTAR>::end(); ++gtarIt ){

      //      if ((*gtarIt)->z<-2.5) 
      //	continue;

      if ((fabs((*gtarIt)->x)>1.0)||(fabs((*gtarIt)->y)>1.0))
      	continue;

      cout<<"ROVER : target wire "<<(*gtarIt)->wnr << " at (";
      if ((*gtarIt)->x>=0) 
	cout<<" ";
      cout<<(*gtarIt)->x<<", ";
      if ((*gtarIt)->y>=0) 
	cout<<" ";
      cout<<(*gtarIt)->y<<", ";
      if ((*gtarIt)->z>=0) 
	cout<<" ";
      cout<<(*gtarIt)->z<<" )"<<endl;

      targetstruct target;
      target.wnr = (*gtarIt)->wnr;
      target.x   = (*gtarIt)->x;
      target.y   = (*gtarIt)->y;
      target.z   = (*gtarIt)->z;
      target.dx  = dx;
      target.dy  = dy;
      target.dz  = dz;

      if((*gtarIt)->dx > (*gtarIt)->dy){
	target.x = beam_x;
	target.flaghorver = 0; // hor
      }
      else{
	target.y = beam_y;
	target.flaghorver = 1; // ver
      }
      target.n_tr = 100;
      target.n_vx = 100;
      
      if (target.z<-2.5){
	nsta1++;
	zsta1 += target.z;
      }
      else{
	nsta2++;
	zsta2 += target.z;
      }

      targets.insert(pair<int,targetstruct>((*gtarIt)->wnr,target));
    }
    cout<<endl;

    if ((nsta1!=0)||(nsta2!=0)){

      if (nsta1>nsta2)
	zsta = zsta1/nsta1;
      else
	zsta = zsta2/nsta2;
    }

  }
  


  for(Target_mi tarit = targets.begin(); tarit != targets.end(); ++tarit){
    
    float xt     = (tarit->second).x;
    float yt     = (tarit->second).y;
    float dxt    = (tarit->second).dx;
    float dyt    = (tarit->second).dy;
    int   nt_tr  = (tarit->second).n_tr;
    int   nt_vx  = (tarit->second).n_vx;
    
    for(Tracks_mi it = tracks.begin();it != tracks.end();++it){
      trackpropagate(it->second,(tarit->second).z);
      
      float x = (it->second).x;
      float y = (it->second).y;
      
      
      if ((fabs(x-xt)<dx)&&(fabs(y-yt)<dy)){
	xt = xt*(nt_tr+nt_vx)+x;
	yt = yt*(nt_tr+nt_vx)+y;
	nt_tr++;
	xt /= (nt_tr+nt_vx);
	yt /= (nt_tr+nt_vx);
	
	dxt = (dxt*(nt_tr-1+nt_vx)+fabs(x-xt))/(nt_tr+nt_vx);
	dyt = (dyt*(nt_tr-1+nt_vx)+fabs(y-yt))/(nt_tr+nt_vx);
      }
    }

    if ((nt_tr+nt_vx) > 10000){
      nt_tr = 100;
      nt_vx = 100;
    }    

    (tarit->second).x     = xt;;
    (tarit->second).y     = yt;
    (tarit->second).dx    = dxt;
    (tarit->second).dy    = dyt;
    (tarit->second).n_tr  = nt_tr;
    (tarit->second).n_vx  = nt_vx;

    /*
    cout<<"ROVER : target wire "<<(tarit->second).wnr << " at (";
    if ((tarit->second).x>=0) 
      cout<<" ";
    cout<<(tarit->second).x<<", ";
    if ((tarit->second).y>=0) 
      cout<<" ";
    cout<<(tarit->second).y<<", ";
    if ((tarit->second).z>=0) 
      cout<<" ";
    cout<<(tarit->second).z<<" )"<<endl;
    */


  }
  
} 

void Lover::trackpropagate(Tracks_t& track,float z){
  
  if(track.z == z)
    return;
  
  float dz = z - track.z;
  
  if (fabs(dz)< 5.0){   
    track.x += dz * track.tx;
    track.y += dz * track.ty;
    track.z  = z;
    
    track.C[0]  += dz * (dz*track.C[5] + track.C[3] + track.C[3]);
    track.C[1]  += dz * (dz*track.C[8] + track.C[6] + track.C[4]);
    track.C[2]  += dz * (dz*track.C[9] + track.C[7] + track.C[7]);
    track.C[3]  += dz * track.C[5]; 
    track.C[4]  += dz * track.C[8]; 
    track.C[6]  += dz * track.C[8];
    track.C[7]  += dz * track.C[9];
    track.C[10] += dz * track.C[12];
    track.C[11] += dz * track.C[13];
  }
  else{
    CatsExtrapolate(track.rseg, z, track.x,track.y,track.tx,track.ty,track.C);
    track.z = z;
    track.C[10] += dz * track.C[12];
    track.C[11] += dz * track.C[13];
  }
  
}


int Lover::target_near(Tracks_t& track, int wire, float sigcut){
  float dist = 0.;
  float cut = 0.;
  
  if(targets[wire].flaghorver == 0){
    dist = track.y - (targets[wire].y);
    cut = sigcut*sigcut*track.C[2];
  }
  else{
    dist = track.x - (targets[wire].x);
    cut = sigcut*sigcut*track.C[0];
  }          

  if(dist*dist < cut )
    return 1;
  else
    return 0;  

} 


void Lover::vertex_refit(void){
  
  typedef struct{
    float cov[4][4];
    float Tcov[2][2];
    float Tpar[2];
    float dzeta[4];
    float zf;
    int flag;
  } TRACK_TYPE, *pTRACK_TYPE;
  
  TRACK_TYPE atr[200];
  int n_track;

  const float dchi2=10.0;
  const float Pfalse=0.05;

  float Vcov[3][3];
  float Rk[5];
  float Vpar[3];
  float dG[5][5],KH[5][5],Gk[5][5];
  float Hk[4][5];
  //  float M[4][5];
  //  float Ht[5][4];
  float Kk[5][4];
  float L[4][4];
  float Fk[5][4];
  float lambda,ax,ay,beta1,dz;
  float res[4],ext[4];
  int i,j,itr,k;

  Vpar[0]=vertex.xv; Vpar[1]=vertex.yv; Vpar[2]=vertex.zv;

  n_track=0;
  for(int itr=1; itr<=ntrincluster; itr++){
    Tracks_mi it = trackincluster[itr];
    trackpropagate(it->second,Vpar[2]);
    atr[n_track].cov[0][0]=(it->second).C[0];
    atr[n_track].cov[0][1]=(it->second).C[1];
    atr[n_track].cov[1][1]=(it->second).C[2];
    atr[n_track].cov[0][2]=(it->second).C[3];
    atr[n_track].cov[1][2]=(it->second).C[4];
    atr[n_track].cov[2][2]=(it->second).C[5];
    atr[n_track].cov[0][3]=(it->second).C[6];
    atr[n_track].cov[1][3]=(it->second).C[7];
    atr[n_track].cov[2][3]=(it->second).C[8];
    atr[n_track].cov[3][3]=(it->second).C[9];
    atr[n_track].cov[1][0]=atr[n_track].cov[0][1];
    atr[n_track].cov[2][0]=atr[n_track].cov[0][2];
    atr[n_track].cov[3][0]=atr[n_track].cov[0][3];
    atr[n_track].cov[2][1]=atr[n_track].cov[1][2];
    atr[n_track].cov[3][1]=atr[n_track].cov[1][3];
    atr[n_track].cov[3][2]=atr[n_track].cov[2][3];
    atr[n_track].dzeta[0]=(it->second).x;
    atr[n_track].dzeta[1]=(it->second).y;
    atr[n_track].dzeta[2]=(it->second).tx;
    atr[n_track].dzeta[3]=(it->second).ty;
    atr[n_track].zf=Vpar[2];
    atr[n_track].Tpar[0]= atr[n_track].dzeta[2];
    atr[n_track].Tpar[1]= atr[n_track].dzeta[3];
    
    for(i=0;i<2;i++) for(j=0;j<2;j++)  
      atr[n_track].Tcov[i][j]=atr[n_track].cov[i+2][j+2];
    
    atr[n_track].flag=1;
    n_track++;
  }
  
  
  for(int jjj=0;jjj<3;jjj++){
    // (re)initialization of the vertex covariance before iteration 
    
    for(i=0;i<3;i++) for(j=0;j<3;j++) Vcov[i][j]=0.0;
    Vcov[0][0]=vertex.cv[0];Vcov[1][1]=vertex.cv[2];Vcov[2][2]=vertex.cv[5];
    
    // Part one: vertex position estimation 
    
    for(itr=0;itr<n_track;itr++){
      // Construct covariance from vertex and track parts 
      
      for(i=0;i<5;i++) for(j=0;j<5;j++) Gk[i][j]=0.0;
      for(i=0;i<3;i++) for(j=0;j<3;j++) Gk[i][j]=Vcov[i][j];
      for(i=3;i<5;i++) for(j=3;j<5;j++) Gk[i][j]=atr[itr].Tcov[i-3][j-3];
      
      // Construct state vector from vertex and track parts
      
      for (i=0;i<3;i++) Rk[i]=Vpar[i];
      Rk[3]=atr[itr].Tpar[0];Rk[4]=atr[itr].Tpar[1];
      
      // Linearized measurement matrix 
      
      dz=atr[itr].zf-Rk[2];ax=Rk[3];ay=Rk[4];
      
      for(i=0;i<4;i++) for(j=0;j<5;j++) Hk[i][j]=0.0;
      Hk[0][0]=Hk[1][1]=Hk[2][3]=Hk[3][4]=1.0;
      Hk[0][2]=-ax;Hk[1][2]=-ay;Hk[0][3]=dz;Hk[1][4]=dz;
      
      // Check if track is in the validation region
      
      for(i=0;i<4;i++) for(j=0;j<4;j++) L[i][j]=0.0;
      
      L[0][0]=Gk[0][0]+ax*ax*Gk[2][2]+dz*dz*Gk[3][3]+
	2.0*(dz*Gk[0][3]-ax*Gk[0][2]-ax*dz*Gk[2][3]);
      L[1][1]=Gk[1][1]+ay*ay*Gk[2][2]+dz*dz*Gk[4][4]+
	2.0*(dz*Gk[1][4]-ay*Gk[1][2]-ay*dz*Gk[2][4]);
      L[0][1]=Gk[0][1]+ax*ay*Gk[2][2]+dz*dz*Gk[3][4]+dz*(Gk[0][4]+Gk[1][3])-
	ay*(Gk[0][2]+dz*Gk[2][3])-ax*(Gk[1][2]+dz*Gk[2][4]);
      L[1][0]=L[0][1];L[2][2]=Gk[3][3];
      L[0][2]=Gk[0][3]-ax*Gk[2][3]+dz*Gk[3][3];
      L[2][0]=L[0][2];L[0][3]=Gk[0][4]-ax*Gk[2][4]+dz*Gk[3][4];
      L[3][0]=L[0][3];L[1][2]=Gk[1][3]-ay*Gk[2][3]+dz*Gk[3][4];
      L[2][1]=L[1][2];L[1][3]=Gk[1][4]-ay*Gk[2][4]+dz*Gk[4][4];
      L[3][1]=L[1][3];L[3][3]=Gk[4][4];L[2][3]=L[3][2]=Gk[3][4];
      
      for(i=0;i<4;i++) for (j=0;j<4;j++) L[i][j]+=atr[itr].cov[i][j];
      
      four_dim_inv(L);
      
      ext[0]=Rk[0]+dz*Rk[3];ext[1]=Rk[1]+dz*Rk[4];ext[2]=Rk[3];ext[3]=Rk[4];
      for(i=0;i<4;i++) res[i]=atr[itr].dzeta[i]-ext[i];
      lambda=0.0;
      for(i=0;i<4;i++) for(j=0;j<4;j++) lambda+=res[i]*res[j]*L[i][j];
      if (lambda>dchi2) {
	//	printf("Track %d out of val. region\n",itr);
	atr[itr].flag=0;
	continue;
      }
      else if (atr[itr].flag){
	// Probability of correct classification
	beta1=1.0-Pfalse/(Pfalse+exp(-lambda));
	
	// Kalman gain
	
	Fk[0][0]=Gk[0][0]-ax*Gk[0][2]+dz*Gk[0][3];
	Fk[0][1]=Gk[0][1]-ay*Gk[0][2]+dz*Gk[0][4];
	Fk[0][2]=Gk[0][3];Fk[0][3]=Gk[0][4];
	Fk[1][0]=Gk[0][1]-ax*Gk[1][2]+dz*Gk[1][3];
	Fk[1][1]=Gk[1][1]-ay*Gk[1][2]+dz*Gk[1][4];
	Fk[1][2]=Gk[1][3];Fk[1][3]=Gk[1][4];
	Fk[2][0]=Gk[0][2]-ax*Gk[2][2]+dz*Gk[2][3];
	Fk[2][1]=Gk[1][2]-ay*Gk[2][2]+dz*Gk[2][4];
	Fk[2][2]=Gk[2][3];Fk[2][3]=Gk[2][4];
	Fk[3][0]=Gk[0][3]-ax*Gk[2][3]+dz*Gk[3][3];
	Fk[3][1]=Gk[1][3]-ay*Gk[2][3]+dz*Gk[3][4];
	Fk[3][2]=Gk[3][3];Fk[3][3]=Gk[3][4];
	Fk[4][0]=Gk[0][4]-ax*Gk[2][4]+dz*Gk[3][4];
	Fk[4][1]=Gk[1][4]-ay*Gk[2][4]+dz*Gk[4][4];
	Fk[4][2]=Gk[3][4];Fk[4][3]=Gk[4][4];
	for(i=0;i<5;i++) for(j=0;j<4;j++){
	  Kk[i][j]=0.0;
	  for(k=0;k<4;k++) Kk[i][j]+=Fk[i][k]*L[k][j];
	}
	
	// New estimation of the vertex position 
	
	for(i=0;i<5;i++) 
	  for(j=0;j<4;j++) 
	    Rk[i]+=beta1*Kk[i][j]*res[j];
	
	// New covariance matrix
	
	for(i=0;i<5;i++) for(j=0;j<5;j++){
	  KH[i][j]=0.0;
	  for(k=0;k<4;k++) KH[i][j]+=Kk[i][k]*Hk[k][j];
	}
	
	for(i=0;i<5;i++) for(j=0;j<5;j++){
	  dG[i][j]=0.0;
	  for(k=0;k<5;k++) dG[i][j]+=KH[i][k]*Gk[k][j];
	}
	
	for(i=0;i<5;i++) for(j=0;j<5;j++) Gk[i][j]-=beta1*dG[i][j];
	
	// Extract vertex part
	
	for(i=0;i<3;i++) Vpar[i]=Rk[i];
	for(i=0;i<3;i++) for (j=0;j<3;j++) Vcov[i][j]=Gk[i][j];
      }
    }
    
    // Part two: smoothing track parameters using estimated vertex position
    
    for(itr=0;itr<n_track;itr++) if(atr[itr].flag){
      // Construct covariance from vertex and track parts
      
      for(i=0;i<5;i++) for(j=0;j<5;j++) Gk[i][j]=0.0;
      for(i=0;i<3;i++) for(j=0;j<3;j++) Gk[i][j]=Vcov[i][j];
      for(i=3;i<5;i++) for(j=3;j<5;j++) Gk[i][j]=atr[itr].Tcov[i-3][j-3];
      
      // Construct state vector from vertex and track parts
      
      for (i=0;i<3;i++) Rk[i]=Vpar[i];
      Rk[3]=atr[itr].Tpar[0];Rk[4]=atr[itr].Tpar[1];
      
      // Linearized measurement matrix
      
      dz=atr[itr].zf-Rk[2];ax=Rk[3];ay=Rk[4];
      
      for(i=0;i<4;i++) for(j=0;j<5;j++) Hk[i][j]=0.0;
      Hk[0][0]=Hk[1][1]=Hk[2][3]=Hk[3][4]=1.0;
      Hk[0][2]=-ax;Hk[1][2]=-ay;Hk[0][3]=dz;Hk[1][4]=dz;
      
      // Check if track is still in the validation region
      
      for(i=0;i<4;i++) for(j=0;j<4;j++) L[i][j]=0.0;
      
      L[0][0]=Gk[0][0]+ax*ax*Gk[2][2]+dz*dz*Gk[3][3]+
	2.0*(dz*Gk[0][3]-ax*Gk[0][2]-ax*dz*Gk[2][3]);
      L[1][1]=Gk[1][1]+ay*ay*Gk[2][2]+dz*dz*Gk[4][4]+
	2.0*(dz*Gk[1][4]-ay*Gk[1][2]-ay*dz*Gk[2][4]);
      L[0][1]=Gk[0][1]+ax*ay*Gk[2][2]+dz*dz*Gk[3][4]+dz*(Gk[0][4]+Gk[1][3])-
	ay*(Gk[0][2]+dz*Gk[2][3])-ax*(Gk[1][2]+dz*Gk[2][4]);
      L[1][0]=L[0][1];L[2][2]=Gk[3][3];
      L[0][2]=Gk[0][3]-ax*Gk[2][3]+dz*Gk[3][3];
      L[2][0]=L[0][2];L[0][3]=Gk[0][4]-ax*Gk[2][4]+dz*Gk[3][4];
      L[3][0]=L[0][3];L[1][2]=Gk[1][3]-ay*Gk[2][3]+dz*Gk[3][4];
      L[2][1]=L[1][2];L[1][3]=Gk[1][4]-ay*Gk[2][4]+dz*Gk[4][4];
      L[3][1]=L[1][3];L[3][3]=Gk[4][4];L[2][3]=L[3][2]=Gk[3][4];
      
      for(i=0;i<4;i++) for (j=0;j<4;j++) L[i][j]+=atr[itr].cov[i][j];
      
      four_dim_inv(L);
      
      ext[0]=Rk[0]+dz*Rk[3];ext[1]=Rk[1]+dz*Rk[4];ext[2]=Rk[3];ext[3]=Rk[4];
      for(i=0;i<4;i++) res[i]=atr[itr].dzeta[i]-ext[i];
      lambda=0.0;
      for(i=0;i<4;i++) for(j=0;j<4;j++) lambda+=res[i]*res[j]*L[i][j];
      if(lambda>dchi2) {
	//	printf("Track %d out of val. region\n",itr);
	atr[itr].flag=0;
	continue;
      }
      else{
	// Probability of correct classification
	beta1=1.0-Pfalse/(Pfalse+exp(-lambda));
	
	// Kalman gain
	
	Fk[0][0]=Gk[0][0]-ax*Gk[0][2]+dz*Gk[0][3];
	Fk[0][1]=Gk[0][1]-ay*Gk[0][2]+dz*Gk[0][4];
	Fk[0][2]=Gk[0][3];Fk[0][3]=Gk[0][4];
	Fk[1][0]=Gk[0][1]-ax*Gk[1][2]+dz*Gk[1][3];
	Fk[1][1]=Gk[1][1]-ay*Gk[1][2]+dz*Gk[1][4];
	Fk[1][2]=Gk[1][3];Fk[1][3]=Gk[1][4];
	Fk[2][0]=Gk[0][2]-ax*Gk[2][2]+dz*Gk[2][3];
	Fk[2][1]=Gk[1][2]-ay*Gk[2][2]+dz*Gk[2][4];
	Fk[2][2]=Gk[2][3];Fk[2][3]=Gk[2][4];
	Fk[3][0]=Gk[0][3]-ax*Gk[2][3]+dz*Gk[3][3];
	Fk[3][1]=Gk[1][3]-ay*Gk[2][3]+dz*Gk[3][4];
	Fk[3][2]=Gk[3][3];Fk[3][3]=Gk[3][4];
	Fk[4][0]=Gk[0][4]-ax*Gk[2][4]+dz*Gk[3][4];
	Fk[4][1]=Gk[1][4]-ay*Gk[2][4]+dz*Gk[4][4];
	Fk[4][2]=Gk[3][4];Fk[4][3]=Gk[4][4];
	for(i=0;i<5;i++) for(j=0;j<4;j++){
	  Kk[i][j]=0.0;
	  for(k=0;k<4;k++) Kk[i][j]+=Fk[i][k]*L[k][j];
	}
	
	// New estimation
	
	for(i=0;i<5;i++) for(j=0;j<4;j++) Rk[i]+=beta1*Kk[i][j]*res[j];
	
	// New covariance matrix
	
	for(i=0;i<5;i++) for(j=0;j<5;j++){
	  KH[i][j]=0.0;
	  for(k=0;k<4;k++) KH[i][j]+=Kk[i][k]*Hk[k][j];
	}
	
	for(i=0;i<5;i++) for(j=0;j<5;j++){
	  dG[i][j]=0.0;
	  for(k=0;k<5;k++) dG[i][j]+=KH[i][k]*Gk[k][j];
	}
	
	for(i=0;i<5;i++) for(j=0;j<5;j++) Gk[i][j]-=beta1*dG[i][j];
	
	// Extract track and vertex parts
	
	atr[itr].Tpar[0]=Rk[3];atr[itr].Tpar[1]=Rk[4];
	Vpar[0]=Rk[0];Vpar[1]=Rk[1];Vpar[2]=Rk[2];
	//
	//atr[itr].Tcov[0][0]=Gk[3][3];atr[itr].Tcov[1][1]=Gk[4][4];
	//atr[itr].Tcov[0][1]=Gk[3][4];atr[itr].Tcov[1][0]=Gk[4][3];
	//
      }
    }
    else 
      ;
      //printf("Track was excluded from the fit\n");
  }
  
  vertex.xv = Vpar[0];
  vertex.yv = Vpar[1];
  vertex.zv = Vpar[2];

  vertex.cv[0] = Vcov[0][0];     
  vertex.cv[1] = Vcov[0][1];
  vertex.cv[2] = Vcov[1][1];
  vertex.cv[3] = Vcov[0][2];     
  vertex.cv[4] = Vcov[1][2];
  vertex.cv[5] = Vcov[2][2];
  
  vertex.nt = 0;
  for(int itr=1; itr<=ntrincluster; itr++){
    if (atr[itr-1].flag==0)
      continue;
    vertex.nt++;
    Tracks_mi it = trackincluster[itr];
    (it->second).used = 1;
  }

  if (vertex.nt>=mintracksinvertex){
    ArteTable<RVER>::push_back(vertex);
  }
  else{
    for(int itr=1; itr<=ntrincluster; itr++){
      Tracks_mi it = trackincluster[itr];
      (it->second).used = 1;// remove ALL tracks of the cluster
    }
  }

  return; 
}

//===================================================================

void Lover::four_dim_inv(float a[4][4]){
  /**** Gaussian algorithm for 4x4 matrix inversion ****/
  
  int i,j,k,l;
  float factor;
  float temp[4];
  float b[4][4];
  
  for(i=0;i<4;i++) for(j=0;j<4;j++) 
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;
  
  for(i=0;i<4;i++){
    for(j=i+1;j<4;j++) 
      if (fabs(a[i][i])<fabs(a[j][i])){
	for(l=0;l<4;l++) temp[l]=a[i][l];
	for(l=0;l<4;l++) a[i][l]=a[j][l];
	for(l=0;l<4;l++) a[j][l]=temp[l];
	for(l=0;l<4;l++) temp[l]=b[i][l];
	for(l=0;l<4;l++) b[i][l]=b[j][l];
	for(l=0;l<4;l++) b[j][l]=temp[l];
      }
    factor=a[i][i];
    for(j=4-1;j>-1;j--) {
      b[i][j]/=factor;a[i][j]/=factor;
    }
    for(j=i+1;j<4;j++) {
      factor=-a[j][i];
      for(k=0;k<4;k++){
	a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }
  for(i=3;i>0;i--){
    for(j=i-1;j>-1;j--){
      factor=-a[j][i];
      for(k=0;k<4;k++){
	a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  }
  for(i=0;i<4;i++) for(j=0;j<4;j++) a[i][j]=b[i][j];
}

//===================================================================

/***************** Graphics ******************/



/******************************************/
/************ graph_ini() ******************/
/******************************************/



void Lover::graph_ini(void){

  iginit(0);
  igsse(0, 7);


  iscr(1, Black       ,0.0000, 0.0000, 0.0000);
  iscr(1, DarkGray    ,0.5000, 0.5400, 0.5300);
  iscr(1, Blue        ,0.0000, 0.0000, 1.0000);
  iscr(1, Green       ,0.1333, 0.5451, 0.1333);
  iscr(1, Cyan        ,0.0000, 0.8078, 0.8196);
  iscr(1, Red         ,1.0000, 0.0000, 0.0000);
  iscr(1, Magenta     ,1.0000, 0.0000, 1.0000);
  iscr(1, Brown       ,0.5000, 0.1647, 0.1647);
  iscr(1, LightBlue   ,0.6784, 0.8471, 0.9020);
  iscr(1, LightGreen  ,0.5961, 0.9843, 0.5961);
  iscr(1, LightCyan   ,0.4980, 1.0000, 0.8314);
  iscr(1, LightRed    ,0.9804, 0.5020, 0.4471);
  iscr(1, LightMagenta,0.8549, 0.4392, 0.8392);
  iscr(1, Yellow      ,1.0000, 1.0000, 0.0000);
  iscr(1, LightGray   ,0.8275, 0.8275, 0.8275);
  iscr(1, White       ,1.0000, 1.0000, 1.0000);


  float beam_x = 0.0, beam_y = 0.0;  // beam center for simu data
  
  if (!ArteTable<MCIN>::booked()){
    beam_x =  0.30;                  // beam center for real data
    beam_y = -0.05;
  }


  iswn(2, beam_x-0.5, beam_x+0.5, beam_y-0.5, beam_y+0.5);
  isvp(2,0.0,1.0,0.0,1.0);
  iselnt(2);

  isfais(1);

  isfaci(LightGray);
  isplci(Red);

  igbox(beam_x-0.5, beam_x+0.5, beam_y-0.5, beam_y+0.5);

  iuwk(1,1);

}
/**************************************************/
/************** graph_close() *********************/
/**************************************************/

void Lover::graph_close(void){
  igend();
}

void Lover::draw_event(void){
  
  float xpl[100],ypl[100],xpm[10],ypm[10];

  isfaci(White);
  isplci(Red);

  float beam_x = 0.0, beam_y = 0.0;  // beam center for simu data
  
  if (!ArteTable<MCIN>::booked()){
    beam_x =  0.30;                  // beam center for real data
    beam_y = -0.05;
  }
  igbox(beam_x-0.5, beam_x+0.5, beam_y-0.5, beam_y+0.5);
       

  
  for(Target_mi tarit = targets.begin(); tarit != targets.end(); ++tarit){
    
    float xt   = (tarit->second).x;
    float yt   = (tarit->second).y;
    float zt   = (tarit->second).z;
    float dxt  = (tarit->second).dx;
    float dyt  = (tarit->second).dy;

    isplci(Yellow);
    if ((tarit->second).flaghorver==0){
      xpl[0] = xt-0.2;
      ypl[0] = yt;
      xpl[1] = xt+0.2;
      ypl[1] = yt;
    }
    else{
      xpl[0] = xt;
      ypl[0] = yt-0.2;
      xpl[1] = xt;
      ypl[1] = yt+0.2;
    }
    ipl(2,xpl,ypl);
    
    if (zt>-2.5)
      isplci(Black);
    else
      isplci(DarkGray);

    xpl[0] = xt-dxt;
    ypl[0] = yt-dyt;
    xpl[1] = xt+dxt;
    ypl[1] = yt-dyt;
    xpl[2] = xt+dxt;
    ypl[2] = yt+dyt;
    xpl[3] = xt-dxt;
    ypl[3] = yt+dyt;
    xpl[4] = xt-dxt;
    ypl[4] = yt-dyt;
    ipl(5,xpl,ypl);
  }


  ismk(3);
  ismksc(0.5);
  ispmci(Cyan);

  for (ArteTable<MTRA>::iterator mtraIt = ArteTable<MTRA>::begin();
       mtraIt != ArteTable<MTRA>::end(); ++mtraIt){
    if ((*mtraIt)->topp->chrg == 0 )
      continue;

    int nhitb = 0;
    for ( ArteRelationIterator<MIMP> mimpIt = (*mtraIt)->mimp.begin();
	  mimpIt != (*mtraIt)->mimp.end(); ++mimpIt){
      if ((*mimpIt)->gede->cmp != 1) continue;
      nhitb += distance((*mimpIt)->hitb.begin(),(*mimpIt)->hitb.end()); 
    }
    if (nhitb<5)
      continue;
    
    float px = (*mtraIt)->px / (*mtraIt)->pz;
    float py = (*mtraIt)->py / (*mtraIt)->pz;
    float xm = (*mtraIt)->bver->x;
    float ym = (*mtraIt)->bver->y;
    float zm = (*mtraIt)->bver->z;

    float zt = zsta;

    xpm[0]  = xm+(zt-zm)*px;
    ypm[0]  = ym+(zt-zm)*py;
    ipm(1,xpm,ypm);
  }

  /*
  ismk(3);
  ismksc(0.5);
  ispmci(LightBlue);
  isplci(LightBlue);

  for(Tracks_mi it = tracks.begin();it != tracks.end();++it){
    trackpropagate(it->second,zsta);
    xpm[0] = (it->second).x;
    ypm[0] = (it->second).y;
    ipm(1,xpm,ypm);
    
    float xm = (it->second).x;
    float ym = (it->second).y;
    float dx = sqrt((it->second).C[0]);
    float dy = sqrt((it->second).C[2]);

    igarc(xm, ym, 0.5*(dx+dy),0.5*(dx+dy), 0.0, 0.0);

  }
  */

  ismk(28);
  ismksc(5.0);

  for( ArteTable<MVER>::iterator mverIt = ArteTable<MVER>::begin();
       mverIt != ArteTable<MVER>::end();++mverIt){
    if( (*mverIt)->flag != 1) continue;

    int mtrafound = 0;
    for (ArteTable<MTRA>::iterator mtraIt = ArteTable<MTRA>::begin();
	 mtraIt != ArteTable<MTRA>::end(); ++mtraIt){
      if ((*mtraIt)->bver != (*mverIt)) continue;
      int nhitb = 0;
      for ( ArteRelationIterator<MIMP> mimpIt = (*mtraIt)->mimp.begin();
	    mimpIt != (*mtraIt)->mimp.end(); ++mimpIt){
	if ((*mimpIt)->gede->cmp != 1) continue;
	nhitb += distance((*mimpIt)->hitb.begin(),(*mimpIt)->hitb.end()); 
      }
      if (nhitb>=5){
	mtrafound++;
      }
    }
    if (mtrafound<mintracksinvertex) continue;
    
    if ((*mverIt)->z>-2.5)
      ispmci(Black);
    else
      ispmci(DarkGray);

    xpm[0]=(*mverIt)->x;
    ypm[0]=(*mverIt)->y;

    ipm(1,xpm,ypm);
  }

  iuwk(1,1);
}

