
#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


extern DbFileDesc *DbFileDescList;
extern int DbLedaCompatibility;

void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);
DbTableDesc *DbArteTableDesc(char *);
int DbMatchType(char *);


void DbWRelTab(int lun, int *buffer, int Nrows, DbTableDesc *TD, 
		int major, int minor, int *ierr) 
{

  /*************************************************************************
   *
   * Author: A. Semenov (JINR, Dubna)
   * Created 01.06.99: 
   *
   * Description: Write to a MIZZI file LEDA relation table.
   *
   * Called from: .
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  lun     in     LUN
   *  buffer  in     pointer to the table
   *  Nrows   in     number of rows in the table
   *  TD      in     table description (from DbLedaRelTabDesc)
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  ierr    out    operation status
   *              0  OK
   *              2  Bank name too long for MIZZI
   *              3  LUN is not connected to data file/server
   *              4  Can not book table
   *              5  Table already exists (at interactive mode, user 
    					refused overwriting)
   *              6  Can not allocate memory for buffer
   *              7  field type not implemented 
   *              8  lun is opened read-only
   *
   ************************************************************************/

  int n, j, Tab, FieldType, *databuffer;
  short *databuffer1;
  char mess[80]="DBWRTRELTAB: ", *warn, what;
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  Tables Id;
  int var;
  DbFileDesc  *FD;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  strcpy(Name,TD->Name);

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
    *ierr=2;
	return;
  }
  

  /* File id (given by OpenTables). */
  
  
  for(FD=DbFileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

  if( FD == NULL ) 
    {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr=3;
    return;
    }
    
  if( FD->rwflag == 0 ) 
    {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "attemt to write to read-only stream %s", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr=8;
    return;
    }  

  Id   =    FD->Id;
      
  /*
  ** Book Table T->Name
  */
    
  /* Check Format string */
  
  Tab=0;
  if(SearchTable(Id, 0, TD->Name, TD->Name))
	  Tab  =  BookTable( Id, TD->Name, 0, TD->Comm );
	  
  if (Tab != 0)
     {
     char *oldFmt;
     
     oldFmt = TableDescription( Id, Tab );
     if (strlen(oldFmt) != strlen(TD->Fmt) || strcmp(oldFmt,TD->Fmt) != 0)
        {
        static int errPtr;
        char       warnMess[80];
        
        /* Warning if new description conflicts with the old one */
        if (strlen(TD->Fmt) < strlen(oldFmt) ||
            strncmp(TD->Fmt,oldFmt,strlen(oldFmt)) != 0)
           {
           sprintf(warnMess,
              "DBWRTRELTAB: bank %s: new description conflicts with the old one",
				    TD->Name );
           mecWarningCount( &errPtr, warnMess );
		   printf("Old format: '%s'\n",oldFmt);
		   printf("New format: '%s'\n",TD->Fmt);
           }
        
        UnbookTable( Id, Tab );
        Tab = 0;
        }
     }

  /* Book Table again */
    
  if (Tab == 0)
     Tab  =    BookTable( Id, TD->Name, TD->Fmt, TD->Comm );
  if( Tab == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "cannot book table %s", TD->Name );
    mecErrorCount( &errPtr, mess );
    *ierr=4;
	return;
  }
  
    
  TableStamp( Id, major , minor );
    
    
  /*
  ** Check if Major and Minor revision numbers have already been used.
  */
    
  if( MatchTable( Id, Tab, 0 ) )
    {
    if (arteflag_.ibatch == 0) /* (for interactive application only) */
      {
      sprintf(mess+strlen(mess),
         "table %s with major and minor %d %d already exists",
          Name , major, minor );
      mecAllPrint( mess );
      warn = "Overwrite y/n ?";
      mecAllPrint( warn );
      do
        {
        scanf( "%c", &what );
        what = toupper ( what );
        } while ( what != 'Y' && what != 'N' );
    
      if( what == 'N' )
        {
        UnbookTable( Id, Tab );
		*ierr=5;
        return;
        }
      }

    sprintf(mess, "DBWRTRELTAB: overwriting table %s", Name );
    if (arteflag_.ibatch == 0)
      mecAllPrint( mess );
    else
      {
      static int errPtr;
      mecWarningCount( &errPtr, mess );
      }
    }

  SkipTableField( Id, 1 );
  
  
  /* Calculate data buffer size, allocate memory */
  
  var=(2+TD->Entries)*sizeof(int);
  for(n=0;n<TD->Entries;n++)
  	{
  	j=Nrows*sizeof(int);
  	if(j>var)
  		var=j;
  	}
  	
  
  databuffer=(void *)malloc(var);
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr=6;
      return;
      }
	  
   databuffer1=(short *)databuffer;

  
  if(Nrows==0)
  {
	StoreTable( Id, Tab, 0 );
  	UnbookTable( Id, Tab );
  	*ierr=0;
  	return;
  }
  
  /* Finally record the bank into the db file. */
  
  /* Cycle on the fields of the bank. */
  
  for( n = 0, j = 0 ; n < TD->Entries ; n++ ) {

    switch( TD->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B':
      FieldType = TABLEuint;
      break;
      
    case 'H':
      FieldType = TABLEbyte;
      break;
	  
	case 'U':
	  FieldType = TABLEushort;
	  break;
	  
	case 'S':
	  FieldType = TABLEshort;
	  break;
	        
    default:
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "wrong field type \"%c\" in table %s",
         TD->Type[n], TD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr=7;
      return;
      }

    }
    
    /* Cycle on the dimension of each field. */
	/* Procedure differs in the cases of Leda/nonLeda */
    
	{ /* Leda case */
	    	  
      int nCopy;
 
      nCopy = Nrows;
 
      if(TD->Type[n]!='H')
	  {
		  if(TD->Type[n]=='S' || TD->Type[n]=='U')
		  {
			  int k;
			 for(k=0;k<Nrows;k++)
      				databuffer1[k] = *(buffer + j + k*TD->Entries);
			  PutTableField(Id, FieldType, Nrows, databuffer);
		  }
		  else
    	  PutTableStruct( Id, FieldType, TD->Entries * 4, nCopy,
                    	  (void *)(buffer + j));
      }
      else
      {
              PutTableStruct( Id, FieldType, TD->Entries * 4, nCopy,
                  ((void *)(buffer + j ) + 0));
              PutTableStruct( Id, FieldType, TD->Entries * 4, nCopy,
                  ((void *)(buffer + j ) + 1));
              PutTableStruct( Id, FieldType, TD->Entries * 4, nCopy,
                  ((void *)(buffer + j ) + 2));
              PutTableStruct( Id, FieldType, TD->Entries * 4, nCopy,
                  ((void *)(buffer + j ) + 3));
      }
 
      
		
	}
    
    j += 1;
    
  }
  StoreTable( Id, Tab, 0 );
  UnbookTable( Id, Tab );
  *ierr=0;
  return;
  
}


extern int Db_dbr_last_major, Db_dbr_last_minor, Db_dbr_empty;

void DbRRelTab(int lun, int **Buffer, int *nRows, DbTableDesc *TD, 
		int major, int minor, char *opt, int *ierr) 
		
{

  /*************************************************************************
   *
   * Author: A. Semenov (JINR, Dubna)
   * Created 01.06.99: 
   *
   * Description: Read from a MIZZI file LEDA relation table.
   *
   * Called from: .
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  lun     in     LUN
   *  Buffer  out    pointer to the table
   *  nRows   out    number of rows in the table
   *  TD      in     table description (from DbLedaRelTabDesc)
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  opt     in     "eq","le",... - applied to major, minor
   *  ierr    out    operation status
   *              0  OK
   *              2  'opt' has incorrect value
   *              3  bank name too long for MIZZI
   *              4  LUN is not connected to data file/server
   *              5  bank does not exist in file/server
   *              6  Can not find requested major/minor table
   *              7  Can not allocate memory for buffer
   *              8  Field type is not implemented
   *              9  Database table format differs from Arte table
   *
   *
   ************************************************************************/
  
  int n, m, j, Tab, FieldType, Nrows, *databuffer, *buffer;
  short *databuffer1;
  char mess[80]="DBRRELTAB: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  DbFileDesc *FD;
  Tables Id;
  int var;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  
  /*
  ** Check 'opt' parameter has allowed value.
  */
  
  if( DbMatchType(opt)==-1 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
    "Unknown option '%s'; should use one of \"eq\", \"le\", \"ge\", \"lt\", \"gt\"",
       opt);
    mecErrorCount( &errPtr, mess );
	*ierr = 2;
    return;
  }

  strcpy(Name, TD->Name );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr = 3;
    return;
  }


  /* File id (given by OpenTables). */


  for(FD=DbFileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

  if( FD == NULL ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr = 4;
    return;
  }
  
  Id   =    FD->Id;

  /* Check that table T->Name exists */
  
  if(SearchTable(Id, 0, TD->Name, TD->Name)==0)
  {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "bank %s does not exist in file/server %s", Name, FD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr = 5;
      return;
  }

  /* Book Table T->Name for reading. */

  Tab  =    BookTable( Id, TD->Name, 0, 0 );

  if( Tab == 0 ) 
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s can not be booked in file/server %s", Name, FD->Name );
    mecErrorCount( &errPtr, mess );
	*ierr = 5;
    return;
  }


  TableStamp( Id, major, minor );

  if( FetchTable( Id, Tab, DbMatchType(opt) ) == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "table %s with major and minor %d %d doesn't exist",
       Name, major, minor );
    UnbookTable( Id, Tab );
    mecErrorCount( &errPtr, mess );
	*ierr = 6;
    return;
  }
  
  Db_dbr_last_major=TableMajor(Id);
  Db_dbr_last_minor=TableMinor(Id);
  
  SkipTableField( Id, 1 );

  Nrows=TableFieldSize(Id);
      
 
  if(Nrows==0)
  {
	  Db_dbr_empty=1;
	  *nRows=0;
	  *Buffer=NULL;
	  UnbookTable(Id,Tab);
	  *ierr=0;
	  return;
  }
  
   /* Calculate data buffer size, allocate memory */
  
  var=(2+TD->Entries)*sizeof(int);
  for(n=0;n<TD->Entries;n++)
  	{
  	m=Nrows*sizeof(int);
  	if(m>var)
  		var=m;
  	}
  	
  
  databuffer=malloc(var*sizeof(int));
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr = 7;
      return;
      }

  databuffer1=(short *)databuffer;

  buffer=malloc(var*sizeof(int)*TD->Entries);
  if(buffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr = 7;
      return;
      }
  

  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  

  /* Cycle on the fields of the bank. */

  for( n = 0, j = 0 ; n < TD->Entries ; n++ ) {

    switch( TD->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B':
      FieldType = TABLEuint;
      break;
      
    case 'H':
      FieldType = TABLEbyte;
      break;
	  
    case 'S':
      FieldType = TABLEshort;
      break; 
	  
    case 'U':
      FieldType = TABLEushort;
      break;
		  
    default:
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "wrong field type \"%c\" in table %s",
         TD->Type[n], TD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr = 8;
      return;
      }
    }

	
	{
		int nCopy = Nrows;
		
		if(TD->Type[n]=='H')
		{
			GetTableStruct( Id, FieldType, TD->Entries*4, nCopy,
                  ((void *)(buffer + j )) + 0);
            GetTableStruct( Id, FieldType, TD->Entries*4, nCopy,
                  ((void *)(buffer + j )) + 1);
            GetTableStruct( Id, FieldType, TD->Entries*4, nCopy,
                  ((void *)(buffer + j )) + 2);
            GetTableStruct( Id, FieldType, TD->Entries*4, nCopy,
                  ((void *)(buffer + j )) + 3);
		}
		else if(TD->Type[n]=='S' || TD->Type[n]=='U')
		  {
		  int k;
		  GetTableField(Id, FieldType, Nrows, databuffer1); 
    	  for(k=0;k<Nrows;k++)
      		*(buffer + j + k*TD->Entries ) =
      			databuffer1[k];
	  	  }
		else
			GetTableStruct( Id, FieldType, TD->Entries*4, nCopy,
                      (void *)(buffer + j ) );

		
	}
    j += 1;


  }
  UnbookTable( Id, Tab );

  free(databuffer);

  *nRows=Nrows;
  *Buffer=buffer;
  
  Db_dbr_empty=0;
  
  *ierr = 0;
  return;
}
