#include <stdio.h>
#include <strings.h>
#include <ctype.h>
#include <stdlib.h>
#include "arte/Arte.h"
#include "auto/MEMO.h"
#include "auto/MEMA.h"
#include "auto/MEMS.h"
#include "auto/MEIO.h"
#include "dbarte/DbArteBridge.h"
#include "arte/ArteFlag.h"
#include "arte/mecCount.h"


extern DbFileDesc *DbFileDescList;
extern int DbLedaCompatibility;

void mmbook_(int *, int *), mmrupd_(int *), mmdrop_(int *), mmfree_(int *);
void chhtot_(char *, char *, int *);
void iounit_(char *, int *, int);
int  mmrstat_(int *, int *);
int  mmwstat_(int *, int *);
DbTableDesc *DbArteTableDesc(char *);
int DbMatchType(char *);


int Db_dbr_last_major, Db_dbr_last_minor, Db_dbr_empty;

void DbReadTable(int lun, int *Mlink, int major, int minor, 
		char *opt, int *ierr) 
{

  /*************************************************************************
   *
   * Author: HERA-B Bologna.
   *
   * Description: Read from a MIZZI file a single bank.
   *
   * Called from: anywhere.
   *
   * External references: MIZZI functions.
   *
   * Argument in/out description
   *  Csym    in     symbolic name (GEOMIN, GEOMOUT, ...)
   *  Mlink   in     bank link
   *  major   in     major revision number
   *  minor   in     minor revision number
   *  opt     in     "eq","le",... - applied to major, minor
   *  ierr    out    operation status
   *              0  OK
   *              1  link not inside link area
   *              2  'opt' has incorrect value
   *              3  bank name too long for MIZZI
   *              4  LUN is not connected to data file/server
   *              5  bank does not exist in file/server
   *              6  Can not find requested major/minor table
   *              7  Can not allocate memory for buffer
   *              8  Field type is not implemented
   *              9  Database table format differs from Arte table
   *
   *
   ************************************************************************/

  int n, m, j, nlink, *Link, Tab, FieldType, Nrows, *databuffer;
  short *databuffer1;
  char mess[80]="DBRTAB: ";
  char Name[DB_MAX_NAME_SIZE+1];
  char Csym[DB_MAX_NAME_SIZE+1];
  DbTableDesc *TD;
  DbFileDesc  *FD;
  Tables Id;
  int var;

  strncpy(Csym,MEIO(lun).stream,DB_MAX_NAME_SIZE);
  for(var=0;var<DB_MAX_NAME_SIZE && Csym[var] && Csym[var]!=' ';var++);
  Csym[var]=0;

  nlink = (int)Mlink / 4 - artelink_.larln1;

  /* Check if Mlink is a correct Zebra link. */

  if( nlink <= 0 || nlink >= artelink_.narlnk ) {
    static int errPtr;
    strcat( mess, "link not inside link area" );
    mecErrorCount( &errPtr, mess );
    *ierr = 1;
	return;
  }
  
  /*
  ** Check 'opt' parameter has allowed value.
  */
  
  if( DbMatchType(opt)==-1 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
    "Unknown option '%s'; should use one of \"eq\", \"le\", \"ge\", \"lt\", \"gt\"",
       opt);
    mecErrorCount( &errPtr, mess );
	*ierr = 2;
    return;
  }


  var = DB_MAX_NAME_SIZE/4;
  chhtot_(MEMO(nlink).name, Name, &var );

  /*
  ** Check bank name length. Note that MIZZI allows only 16 characters names.
  */
  
  if( strlen(Name) > DB_MAX_TABLE_NAME_SIZE ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank name \"%s\" is too long for MIZZI (NameLen <= %d)",
       Name,DB_MAX_TABLE_NAME_SIZE);
    mecErrorCount( &errPtr, mess );
	*ierr = 3;
    return;
  }

  TD=DbArteTableDesc(Name);
 

  /* File id (given by OpenTables). */


  for(FD=DbFileDescList;FD;FD=FD->next)
	  if(FD->lun==lun)
		  break;

  if( FD == NULL ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "symbolic name %s is not connected to a file/server", Csym );
    mecErrorCount( &errPtr, mess );
	*ierr = 4;
    return;
  }
  
  Id   =    FD->Id;

  /* Check that table T->Name exists */
  
  if(SearchTable(Id, 0, TD->Name, TD->Name)==0)
  {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "bank %s does not exist in file/server %s", Name, FD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr = 5;
      return;
  }

  /* Book Table T->Name for reading. */

  Tab  =    BookTable( Id, TD->Name, 0, 0 );

  if( Tab == 0 ) 
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "bank %s can not be booked in file/server %s", Name, FD->Name );
    mecErrorCount( &errPtr, mess );
	*ierr = 5;
    return;
  }


  TableStamp( Id, major, minor );

  if( FetchTable( Id, Tab, DbMatchType(opt) ) == 0 ) {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "table %s with major and minor %d %d doesn't exist",
       Name, major, minor );
    UnbookTable( Id, Tab );
    mecErrorCount( &errPtr, mess );
	*ierr = 6;
    return;
  }
  
  Db_dbr_last_major=TableMajor(Id);
  Db_dbr_last_minor=TableMinor(Id);
  
  SkipTableField( Id, 1 );

  var=TableFieldSize(Id);
  
  if(!DbLedaCompatibility && var!=TD->Entries+1)
  {
    static int errPtr;
    sprintf(mess+strlen(mess),
       "different format for table %s ", Name );
    UnbookTable( Id, Tab );
    mecErrorCount( &errPtr, mess );
	*ierr = 9;
    return;
  }
  
  databuffer=malloc(var*sizeof(int));
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr = 7;
      return;
      }
  databuffer1 = (short *) databuffer;
      
  /* Get rows number and entries dimensions (first field). */

  if(DbLedaCompatibility)
	  databuffer[0]=var;
  else
  	GetTableField( Id, TABLEint, var, databuffer );
  
  
  if(!DbLedaCompatibility)
  for(n=0;n<TD->Entries;n++)
  	if(TD->Dim[n]!=databuffer[n+1])
	  {
	    static int errPtr;
	    sprintf(mess+strlen(mess),
	       "different format for table %s ", Name );
	    UnbookTable( Id, Tab );
	    mecErrorCount( &errPtr, mess );
	    *ierr = 9;
	    return;
	  }
  
  Nrows=databuffer[0];

  free(databuffer);

  /* Check if bank exists already. */

  if( *Mlink != 0 ) {
	mmdrop_( Mlink );
    sprintf(mess+strlen(mess),
       "existing bank %s will be overwritten", Name );
    if (arteflag_.ibatch == 0)
       mecAllPrint( mess );
    else
       {
       static int errPtr;
       mecWarningCount( &errPtr, mess );
       }
  }
 
  if(Nrows==0)
  {
	  Db_dbr_empty=1;
	  UnbookTable(Id,Tab);
	  *ierr=0;
	  return;
  }
  
   /* Calculate data buffer size, allocate memory */
  
  var=(2+TD->Entries)*sizeof(int);
  for(n=0;n<TD->Entries;n++)
  	{
  	m=Nrows*(TD->Dim[n]==0?1:TD->Dim[n])*sizeof(int);
  	if(m>var)
  		var=m;
  	}
  	
  
  databuffer=malloc(var*sizeof(int));
  if(databuffer==NULL)
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "can not allocate memory for buffer");
      mecErrorCount( &errPtr, mess );
      UnbookTable(Id,Tab);
	  *ierr = 7;
      return;
      }
	  
  databuffer1=(short *)databuffer;


  /* Book space in memory. */

  mmbook_( Mlink, &Nrows );


  /*
   * Memory address of the bank. Points directly to the columns number.
   */
  
  Link  =    gcbank_.iq + *Mlink;

  /* Cycle on the fields of the bank. */

  for( n = 0, j = 0 ; n < TD->Entries ; n++ ) {

    switch( TD->Type[n] ) {
      
    case 'I':
      FieldType = TABLEint;
      break;
      
    case 'F':
      FieldType = TABLEfloat;
      break;
      
    case 'B':
      FieldType = TABLEuint;
      break;
      
    case 'H':
      FieldType = (DbLedaCompatibility?TABLEbyte:TABLEascii);
      break;
	  
    case 'S':
      FieldType = TABLEshort;
      break; 
	  
    case 'U':
      FieldType = TABLEushort;
      break;
	
	 case 'R':
      FieldType = 0;
      break; 
	  
    default:
      {
      static int errPtr;
      sprintf(mess+strlen(mess),
         "wrong field type \"%c\" in table %s",
         TD->Type[n], TD->Name );
      mecErrorCount( &errPtr, mess );
	  *ierr = 8;
      return;
      }
    }

	
	if(!DbLedaCompatibility)
	{
    	GetTableField(Id, FieldType, 
    	  Nrows*(TD->Type[n]=='H'?4:1)*(TD->Dim[n]==0?1:TD->Dim[n]), 
    	  databuffer);    

    	/* Cycle on the dimension of each field (scalar 0, vector n). */

    	for( m = 0 ; m < ( ( TD->Dim[n] == 0 ) ? 1 : TD->Dim[n] ) ; m++ ) 
    	{
    	  int k;
    	  int nCopy;

    	  nCopy = Nrows;

		  if(TD->Type[n]=='S' || TD->Type[n]=='U')
		  {
    	  for(k=0;k<Nrows;k++)
      		*(Link + 2 + j + k*MEMO(nlink).col + m ) =
      			databuffer1[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m];
    	  }
		  else
		  {
		  for(k=0;k<Nrows;k++)
      		*(Link + 2 + j + k*MEMO(nlink).col + m ) =
      			databuffer[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m];  
		  }

    	}
    }
	else if(TD->Type[n]!='R')
		for( m = 0 ; m < ( ( TD->Dim[n] == 0 ) ? 1 : TD->Dim[n] ) ; m++ ) 
	{
		int nCopy = Nrows;
		
		if(TD->Type[n]=='H')
		{
			GetTableStruct( Id, FieldType, MEMO(nlink).col*4, nCopy,
                  ((void *)(Link + 2 + j + m)) + 0);
            GetTableStruct( Id, FieldType, MEMO(nlink).col*4, nCopy,
                  ((void *)(Link + 2 + j + m)) + 1);
            GetTableStruct( Id, FieldType, MEMO(nlink).col*4, nCopy,
                  ((void *)(Link + 2 + j + m)) + 2);
            GetTableStruct( Id, FieldType, MEMO(nlink).col*4, nCopy,
                  ((void *)(Link + 2 + j + m)) + 3);
		}
		else if(TD->Type[n]=='S' || TD->Type[n]=='U')
		  {
		  int k;
		  GetTableField(Id, FieldType, 
    	  	Nrows*(TD->Type[n]=='H'?4:1)*(TD->Dim[n]==0?1:TD->Dim[n]), 
    	  		databuffer); 
    	  for(k=0;k<Nrows;k++)
      		*(Link + 2 + j + k*MEMO(nlink).col + m ) =
      			databuffer1[k*(TD->Dim[n] == 0 ? 1 : TD->Dim[n] )+m];
	  	  }
		else
			GetTableStruct( Id, FieldType, MEMO(nlink).col*4, nCopy,
                      (void *)(Link + 2 + j + m) );

		
	}
    j += ( ( TD->Dim[n] == 0 ) ? 1 : TD->Dim[n] );


  }
  UnbookTable( Id, Tab );

  /* Free unused rows. */

  mmfree_( Mlink );

  free(databuffer);

  Db_dbr_empty=0;
  *ierr = 0;
  return;
}
