
// *****************************************************************************
//
// source:    DQ.C
//
// type:      source code
//
// created:   19 Oct 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// called by: DQ routines
//
// changes:
// 20 Oct 1999 (AG) make DQ_flag() usable for C         
// 01 Nov 1999 (AG) use LVL_M_ABC for nrclevel
// 04 Nov 1999 (AG) repair kuip
// 26 Nov 1999 (AG) introduce DQ_RECO
// 12 Jan 2000 (AG) introduce GLOB = CORR for AC
// 01 Feb 2000 (AG) AC works without levels now
// 03 Feb 2000 (AG) always use level LVL_REC for AC_ID
// 21 Feb 2000 (AG) fixed bug
// 25 Feb 2000 (AG) moved IDs to dq.h (ONLINE)
// 26 Feb 2000 (AG) extract AC stuff to alca/AC.C
//  3 Mar 2000 (AG) VDS=SVD and HPT=HIPT
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

// ONLINE includes
#include "component.h"

// Arte includes
#include "dq/DQ.h"

// DQ externals
extern int DQ_DB;
extern int DQ_EVAL;

extern int DQ_SVD;
extern int DQ_ITR;
extern int DQ_HIPT;
extern int DQ_OTR;
extern int DQ_RICH;
extern int DQ_TRD;
extern int DQ_ECAL;
extern int DQ_MUON;

extern int DQ_FLT;
extern int DQ_SLT;
extern int DQ_TLT;
extern int DQ_FARM;

extern int DQ_CORR;
extern int DQ_RECO;
extern int DQ_TARG;

// prototypes
extern "C" void kugetc_( char *string, int *len, int len1 );


//
// preliminary, will go to ONLINE
//

unsigned int 
DQ_ID( int COMP_XYZ, int LVL_ABC, int histID )
{
  return ( ( MOD_DQ<<28 ) | ( COMP_XYZ<<24 ) | ( LVL_ABC<<20 ) | histID );
}
  
unsigned int 
AC_ID( int COMP_XYZ, int LVL_ABC, int histID )
{
  return ( ( MOD_AC<<28 ) | ( COMP_XYZ<<24 ) | ( LVL_REC<<20 ) | histID );
}

//
//
//


//
// DQ_flag() function
//
extern "C" int
DQ_flag( int COMP_XYZ, int LVL_M_ABC )
{
  int rc = 0;

  //cout << "DQ_flag(): COMP_XYZ=" << COMP_XYZ << " LVL_M_ABC=" << LVL_M_ABC << endl;

  //cout << "DQ_flag(): DQ_SVD= " << DQ_SVD  << endl;
  //cout << "DQ_flag(): DQ_RICH=" << DQ_RICH << endl;

  if( ( COMP_XYZ == COMP_SVD  ) && ( DQ_SVD  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ITR  ) && ( DQ_ITR  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_OTR  ) && ( DQ_OTR  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_HIPT ) && ( DQ_HIPT & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RICH ) && ( DQ_RICH & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TRD  ) && ( DQ_TRD  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_ECAL ) && ( DQ_ECAL & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_MUON ) && ( DQ_MUON & LVL_M_ABC ) ) rc = 1;

  if( ( COMP_XYZ == COMP_FLT  ) && ( DQ_FLT  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_SLT  ) && ( DQ_SLT  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_TLT  ) && ( DQ_TLT  & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_FARM ) && ( DQ_FARM & LVL_M_ABC ) ) rc = 1;

  if( ( COMP_XYZ == COMP_CORR ) && ( DQ_CORR & LVL_M_ABC ) ) rc = 1;
  if( ( COMP_XYZ == COMP_RECO ) && ( DQ_RECO & LVL_M_ABC ) ) rc = 1;

  return rc; 
}

//
// dq_comp *************************************************************
//
extern "C"
void dq_comp_()
{  
  //--------------------------------------------------------------------
  //   read in subdetector
  //--------------------------------------------------------------------
  
  int len;        
  int len1 = 4;
  char input[7];
  kugetc_( input, &len, len1 );
  input[len] = 0;
  
  char *subdetector;
  subdetector  = (char *) malloc( len+1 );
  strncpy( subdetector, input, len );
  subdetector[len] = '\0';
  
  for( int i=0; i<len; i++ ) subdetector[i] = toupper( subdetector[i] );
  
  if( strncmp( subdetector,"SVD",   3 ) &&
      strncmp( subdetector,"VDS",   3 ) &&
      strncmp( subdetector,"ITR",   3 ) && 
      strncmp( subdetector,"HPT",   3 ) &&
      strncmp( subdetector,"HIPT",  4 ) &&
      strncmp( subdetector,"OTR",   3 ) && 
      strncmp( subdetector,"RICH",  4 ) &&
      strncmp( subdetector,"TRD",   3 ) && 
      strncmp( subdetector,"ECAL",  4 ) &&
      strncmp( subdetector,"MUON",  3 ) && 
      strncmp( subdetector,"SLT",   3 ) &&
      strncmp( subdetector,"TLT",   3 ) &&
      strncmp( subdetector,"FARM",  4 ) && 
      strncmp( subdetector,"TARG",  4 ) &&
      strncmp( subdetector,"CORR",  4 ) &&
      strncmp( subdetector,"RECO",  4 ) && 
      strncmp( subdetector,"ALL",   3 ) ){
    cout << "DQ_init(): ********* DQ/SUBD: unknown subdetector: "
	 << subdetector << endl;
  }
  else {
    //cout << "DQ/SUBD: " << subdetector << endl;
  }
  
  //--------------------------------------------------------------------
  //   read in subdetetcor status
  //--------------------------------------------------------------------

  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  char *status = (char *) malloc( len+1 );
  strncpy( status, input, len );
  status[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) status[i] = toupper( input[i] );
  
  if(strncmp(status,"ON",2) && strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/SUBD: unknown subdetector status: "
	 << status << endl;
  }
  else {
    //cout << "DQ/SUBD: " << status << endl;
  }
  
  //--------------------------------------------------------------------
  //   read in dq level (hit/raw/rec)
  //--------------------------------------------------------------------
  
  len  = 0;
  len1 = 3;
  kugetc_( input, &len, len1 );
  input[len] = 0;
  int  level = 0;  
  char *rclevel  = (char *) malloc( len+1 );
  strncpy( rclevel, input, len );
  rclevel[len] = '\0';
  
  if( len>0 ) for( int i=0; i<len; i++ ) rclevel[i] = toupper(input[i]);
  
  if(      !strncmp( rclevel, "RAW", 3 ) ){
    level = LVL_M_RAW;
  }
  else if( !strncmp( rclevel, "HIT", 3 ) ){
    level = LVL_M_HIT;
  } 
  else if( !strncmp( rclevel, "REC", 3 ) ){
    level = LVL_M_REC;
  }
  else {
    cout << "DQ_init(): ****** DQ/SUBD: unknown level: " << rclevel << " "
	 << "Taking default." << endl;
  }
  //cout << "DQ/SUBD: " << level << endl;

  //--------------------------------------------------------------------
  //   set dq status
  //--------------------------------------------------------------------
  
  if( !strncmp( status,"OFF", 3 ) ) level=0;
  
  if( !strncmp( subdetector, "SVD" ,   3 ) ) DQ_SVD  |= level;
  if( !strncmp( subdetector, "VDS" ,   3 ) ) DQ_SVD  |= level;
  if( !strncmp( subdetector, "ITR" ,   3 ) ) DQ_ITR  |= level;
  if( !strncmp( subdetector, "HPT",    3 ) ) DQ_HIPT |= level;
  if( !strncmp( subdetector, "HIPT",   4 ) ) DQ_HIPT |= level;
  if( !strncmp( subdetector, "OTR" ,   3 ) ) DQ_OTR  |= level;
  if( !strncmp( subdetector, "RICH",   4 ) ) DQ_RICH |= level;
  if( !strncmp( subdetector, "TRD" ,   3 ) ) DQ_TRD  |= level;
  if( !strncmp( subdetector, "ECAL",   4 ) ) DQ_ECAL |= level;
  if( !strncmp( subdetector, "MUON",   4 ) ) DQ_MUON |= level;
  if( !strncmp( subdetector, "FLT" ,   3 ) ) DQ_FLT  |= level;
  if( !strncmp( subdetector, "SLT" ,   3 ) ) DQ_SLT  |= level;
  if( !strncmp( subdetector, "TLT" ,   3 ) ) DQ_TLT  |= level;
  if( !strncmp( subdetector, "FARM",   4 ) ) DQ_FARM |= level;
  if( !strncmp( subdetector, "CORR",   4 ) ) DQ_CORR |= level;
  if( !strncmp( subdetector, "RECO",   4 ) ) DQ_RECO |= level;
  
  if (!strncmp( subdetector, "ALL" , 3 ) ){
    DQ_SVD  |= level;
    DQ_HIPT |= level;
    DQ_OTR  |= level;
    DQ_ITR  |= level;
    DQ_RICH |= level;
    DQ_TRD  |= level;
    DQ_ECAL |= level;
    DQ_MUON |= level;
    DQ_FLT  |= level;
    DQ_SLT  |= level;
    DQ_TLT  |= level;
    DQ_FARM |= level;
    DQ_CORR |= level;
    DQ_RECO |= level;
  }

  // print
  cout << "DQ/SUBD: " << subdetector << " " << status << " " << level << endl;
}

//
// dq_db ***************************************************************
//
extern "C"
void dq_db_()
{  
  //--------------------------------------------------------------------
  //   read in status for saving histograms to the DB
  //--------------------------------------------------------------------
  
  int len;        
  int len1 = 3;
  char input[7];
  kugetc_(input, &len, len1);
  input[len] = 0;
  
  char *status;
  status  = (char *) malloc(len+1);
  strncpy(status, input, len);
  status[len] = '\0';
  
  if(len>0) for(int i=0; i<len; i++) status[i] = toupper(status[i]);
  
  if(strncmp(status,"ON",2)&&strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/DB: unknown status for saving histograms to DB: "
	 << status << endl;
  }

  //--------------------------------------------------------------------
  //   set status
  //--------------------------------------------------------------------
  
  if (!strncmp(status,"OFF",3)) DQ_DB=0;
  if (!strncmp(status,"ON", 2)) DQ_DB=1;
}

//
// dq_eval *************************************************************
//
extern "C"
void dq_eval_()
{  
  //--------------------------------------------------------------------
  //   read in demand for evaluating histograms
  //--------------------------------------------------------------------
  int len;        
  int len1 = 3;
  char input[7];
  kugetc_(input, &len, len1);
  input[len] = 0;
  
  char *status;
  status  = (char *) malloc(len+1);
  strncpy(status, input, len);
  status[len] = '\0';
  
  if(len>0) for(int i=0; i<len; i++) status[i] = toupper(status[i]);
  
  if(strncmp(status,"ON",2)&&strncmp(status,"OFF",3)){
    cout << "DQ_init(): ********* DQ/EVAL: unknown demand for evaluating the histograms: "
	 << status << endl;
  }

  //--------------------------------------------------------------------
  //   set status
  //--------------------------------------------------------------------  
  if (!strncmp(status,"OFF",3)) DQ_EVAL=0;
  if (!strncmp(status,"ON", 2)) DQ_EVAL=1;
}

//
// print
//
void
DQ_print( const char *txt )
{
  cout << "DQ_print(): Called from " << txt << ":" << endl;
  cout << "            DQ_SVD= " << DQ_SVD  << endl;
  cout << "            DQ_ITR= " << DQ_ITR  << endl;
  cout << "            DQ_HIPT=" << DQ_HIPT << endl;
  cout << "            DQ_OTR= " << DQ_OTR  << endl;
  cout << "            DQ_RICH=" << DQ_RICH << endl;
  cout << "            DQ_TRD= " << DQ_TRD  << endl;
  cout << "            DQ_ECAL=" << DQ_ECAL << endl;
  cout << "            DQ_MUON=" << DQ_MUON << endl;
  cout << "            DQ_FLT= " << DQ_FLT  << endl;
  cout << "            DQ_SLT= " << DQ_SLT  << endl;
  cout << "            DQ_TLT= " << DQ_TLT  << endl;
  cout << "            DQ_FARM=" << DQ_FARM << endl;
  cout << "            DQ_CORR=" << DQ_CORR << endl;
  cout << "            DQ_RECO=" << DQ_RECO << endl;
}
