
// ******************************************************************
//
// D. Samtleben 10 Aug 1999
//
// Filling of histograms
//
// changes:
// 28 Oct 1999 (AG) use defines from DQ.h
// 01 Nov 1999 (AG) get rid of dq_hid.
// 05 Nov 1999 (AG) improve histogramming
//
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ECAL.h"

//
// function code
//
int
DQ_ECAL_rec_evnt()
{
  double Mass, CosTheta;
  double TotalClusterEnergy=0;
  
  double E1, X1, Y1; 
  int Wid1;

  double E2, X2, Y2;
  int Wid2;  

  int NrOfClusters;
  int NrOfTracks;
 
  double Z1, Z2;
  double ThisEt1,  ThisEt2;
 
  //int trigflag=100*faDDAQFCS_tcode();

  // tcode
  int tcode = faDDAQFCS_tcode();
  int trigflag = 0;
  if( tcode > 0 ) trigflag = 900; // fill histos for tcode =0 and #0

  NrOfClusters=ArtePointer<RCCL>::nrow();
  NrOfTracks=ArtePointer<RSEG>::nrow();
  double EtMax=0;
  double EtMax2=0;
  double EtMax3=0;
  double MassMax=0;
  double MassMaxSLT=0;

  double XEtMax=0.;
  double YEtMax=0.;

  int NrOfClustersPtGT03=0;
  //===========================================================================
  // count number of clusters with Pt >0.3 for pi0 cut
  //===========================================================================
  for( ArteTableIterator<RCCL> Cluster1; Cluster1.valid(); Cluster1.next() ){
      rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,2001+trigflag),
                Cluster1->x,Cluster1->y,1);
      /* fill occupancy weighted by energy */
      rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,2002+trigflag),
               Cluster1->x,Cluster1->y, Cluster1->esum);

      TotalClusterEnergy+=Cluster1->esum;
      ThisEt1=Cluster1->esum*
           sqrt(Cluster1->x*Cluster1->x +Cluster1->y*Cluster1->y)/Cluster1->z;

      if(ThisEt1 > 0.3) {
          
          NrOfClustersPtGT03++;
        }
    }


  for( ArteTableIterator<RCCL> Cluster1; Cluster1.valid(); Cluster1.next() ){
      E1=Cluster1->esum;
      X1=Cluster1->x;
      Y1=Cluster1->y;
      Z1=Cluster1->z;
      Wid1=(int)Cluster1->wid;
      ThisEt1=E1*sqrt(X1*X1 +Y1*Y1)/Z1;

      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1005+trigflag),E1,1);
      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1006+trigflag),ThisEt1,1);

  //===========================================================================
  /* trigger conditions */
  //===========================================================================

      if(ThisEt1>EtMax)
        {
          EtMax3=EtMax2;
          EtMax2=EtMax;
          EtMax=ThisEt1;
          XEtMax=X1;
          YEtMax=Y1;
        }
      else if(ThisEt1>EtMax2)
        {
          EtMax3=EtMax2;
          EtMax2=ThisEt1;
        }
      else if(ThisEt1>EtMax3)
        {
          EtMax3=ThisEt1;
        }
      
      for( ArteTableIterator<RCCL> Cluster2(Cluster1); Cluster2.valid(); Cluster2.next() ){
          if(Cluster2.index()>Cluster1.index())
            {
              E2=Cluster2->esum;
              X2=Cluster2->x;
              Y2=Cluster2->y;
              Z2=Cluster2->z;
              Wid2=(int)Cluster2->wid;
              ThisEt2=E2*sqrt(X2*X2 +Y2*Y2)/Z2;

  //===========================================================================
  /* now calculate the invariant mass of this cluster pair */
  //===========================================================================
              CosTheta=(X1*X2+Y1*Y2+Z1*Z2)/ 
                 ( sqrt(X1*X1+Y1*Y1+Z1*Z1) *sqrt(X2*X2+Y2*Y2+Z2*Z2) );
              Mass=sqrt(2*E1*E2*(1-CosTheta));
             
  //===========================================================================
  /*  check for SLT mass cut */
  //===========================================================================
              if(Mass>MassMax)
                {
                  MassMax=Mass;
                }
              if((Mass>MassMaxSLT) && (Wid1>2) && (Wid2>2) 
                   && (faDDAQFCS_tcode()<1))
                {
                  MassMaxSLT=Mass;
                }

  //===========================================================================
  /* Cuts for Pi 0 's */
  //===========================================================================
        
	  //                     && (X1<-220||X1>-120||Y1<-40||Y1>0)
	  //                    && (X2<-220||X2>-120||Y2<-40||Y2>0))
       if((Mass<1) && (ThisEt1>0.3) &&(ThisEt2>0.3) 
                     &&(Wid1<10) && (Wid1>=3) &&(Wid2<10) && (Wid2>=3) 
                     &&(NrOfClustersPtGT03<10))
                {
      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1002+trigflag),Mass,1);
                  
  //===========================================================================
  /* look for position of cell */
  //===========================================================================
                  ArtePointer<ECHS> echs1;
                  ArtePointer<ECHS> echs2;
                  for(ArteRelationIterator<HITC>::iterator rcclhitc=Cluster1->hitc.begin(); rcclhitc!=Cluster1->hitc.end();rcclhitc++)
                    {
                      echs1 = ArtePointer<ECHS>(((ArtePointer<HITC>)rcclhitc)->gcce.index());
                      break;
                    }
                  for(ArteRelationIterator<HITC>::iterator rcclhitc=Cluster2->hitc.begin(); rcclhitc!=Cluster2->hitc.end();rcclhitc++)
                    {
                     echs2= ArtePointer<ECHS>(((ArtePointer<HITC>)rcclhitc)->gcce.index());
                     break;
                    }

                  if((echs1->fdid<100) &&(echs2->fdid <100))
                    {
                      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1003+trigflag),Mass,1);
                    }
                  if((echs1->fdid>100) &&(echs2->fdid >100))
                    {
                      rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1004+trigflag),Mass,1);
                    }
	  
                }
	                    
	    }
      }
  }
              
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1007+trigflag),EtMax,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1008+trigflag),EtMax2,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1009+trigflag),MassMax,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1010+trigflag),MassMaxSLT,1);
  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,1011+trigflag),EtMax3,1);

  rhp_hf1(DQ_ID(COMP_ECAL,LVL_REC,2010+trigflag),TotalClusterEnergy,1);

  if(NrOfClusters>0) rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,3000+trigflag),XEtMax,YEtMax,1);
  if(TotalClusterEnergy>5) rhp_hfill(DQ_ID(COMP_ECAL,LVL_REC,3001+trigflag),XEtMax,YEtMax,1);

  return 0; // return error code (0 = okay)
}
