
// ******************************************************************
//
// D. Samtleben 10 Aug 1999
//
// Filling of histograms
//
// changes:
// 27 Oct 1999 (AG) use defines from DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
//  4 Nov 1999 (DD) beta=1 radius histogram 
//  5 Nov 1999 (AG) some polishing
// 16 Dec 1999 (RP) likelihood histograms DQ_RICH_rec_pid()
// ******************************************************************


// standard includes
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "dq/DQ.h"
#include "dq/DQ_RICH.h"
#include "farm/farm.hh"
#include "rich/rich.h"
#include "rise/RingSearchARTE.h"

// externals
extern int FAGRR;

extern float DQ_RICH_rec_xmin;
extern float DQ_RICH_rec_xmax;
extern int   DQ_RICH_rec_nbin;

extern float DQ_RICH_rec_thc_xmin;
extern float DQ_RICH_rec_thc_xmax;
extern int   DQ_RICH_rec_thc_nbin;

// prototypes
int DQ_RICH_rec_beta1( ArtePointer<RSEG> rich );
int DQ_RICH_rec_pid();
int DQ_RICH_rec_thc();

//////////////// Fill RICH DQ histograms /////////////////////////////
int DQ_RICH_rec_evnt() {
  DQ_RICH_rec_pid();
  DQ_RICH_rec_thc();
  int nrRICHrings=0; 
  for(ArteTable<RSEG>::iterator irseg=ArteTable<RSEG>::begin();
      irseg!=ArteTable<RSEG>::end(); ++irseg) {
    if ((*irseg)->cmp!=Rsegc::rich) continue;
    ArtePointer<RSEG> rich=(*irseg);
    nrRICHrings++;

    float tx=rich->txf;
    float ty=rich->tyf;
    rhp_hfill(DQ_ID(COMP_RICH, LVL_REC, 201),tx,ty,1);
    rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 202),rich->time,1);
    if (!DQ_RICH_rec_beta1(rich))
      rhp_hf1(DQ_ID(COMP_RICH, LVL_REC, 203),rich->time,1);
  }
  rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 200),nrRICHrings,1);

  return 0; // return error code (0 = okay)
}

//////////////  Select RICH Beta=1 w/ ECAL ////////////////////////
int DQ_RICH_rec_beta1(ArtePointer<RSEG> rich) {
  static float max_dty=3e-3;  // angle match
  static float min_esum=8;    // minimum cluster energy
  static float max_p=200;     // maximum p
  static float min_p=10;      // minimum p
  for (ArteTable<RCCL>::iterator irccl=ArteTable<RCCL>::begin();
       irccl!=ArteTable<RCCL>::end(); ++irccl) {
    if ((*irccl)->esum<min_esum) continue;
    if (fabs((*irccl)->ty - rich->tyf)>max_dty) continue;
    float p=288/fabs((*irccl)->x - rich->txf*(*irccl)->z);
    if (p<min_p) continue;
    if (p>max_p) continue;
    return 0; // cluster found
  }
  return 1; // cluster not found
}

////////////// calculate cherenkov angle for all ( track, hit ) pairs;

//extern float DQ_RICH_rec_thc_xarray0[100];
//extern float DQ_RICH_rec_thc_xarray1[100];
//extern float DQ_RICH_rec_thc_xarray2[100];
//extern float DQ_RICH_rec_thc_xarray3[100];

int DQ_RICH_rec_thc(){ 

  /*static int   nbin = DQ_RICH_rec_thc_nbin;
  static float xmin = DQ_RICH_rec_thc_xmin;
  static float xmax = DQ_RICH_rec_thc_xmax;
  //  static float *DQ_RICH_rec_thc_xarray;
  //  if (!DQ_RICH_rec_thc_xarray) DQ_RICH_rec_thc_xarray= new float[ nbin ];
  if (!rhp_hnoent(DQ_ID(COMP_RICH, LVL_REC, 204) ) ) 
    for (int i=0;i<nbin;i++) DQ_RICH_rec_thc_xarray0[i] =0; 
  else   
    rhp_hpak(DQ_ID(COMP_RICH, LVL_REC, 204),DQ_RICH_rec_thc_xarray0);   
  
  if (!rhp_hnoent(DQ_ID(COMP_RICH, LVL_REC, 205))  )
    for (int i=0;i<nbin;i++) DQ_RICH_rec_thc_xarray1[i] =0; 
  else   
    rhp_hpak(DQ_ID(COMP_RICH, LVL_REC, 205),DQ_RICH_rec_thc_xarray1);   

  
  if (!rhp_hnoent(DQ_ID(COMP_RICH, LVL_REC, 206)) )
    for (int i=0;i<nbin;i++) DQ_RICH_rec_thc_xarray2[i] =0; 
  else   
    rhp_hpak(DQ_ID(COMP_RICH, LVL_REC, 206),DQ_RICH_rec_thc_xarray2);   
  
  if (!rhp_hnoent(DQ_ID(COMP_RICH, LVL_REC, 207)) )
    for (int i=0;i<nbin;i++) DQ_RICH_rec_thc_xarray3[i] =0; 
  else   
    rhp_hpak(DQ_ID(COMP_RICH, LVL_REC, 207),DQ_RICH_rec_thc_xarray3);   
  

    */
  return 0;
}


int DQ_RICH_rec_pid(){ 
  /*  float logp=0;
  static int   nbin=DQ_RICH_rec_nbin;
  static float xmin=DQ_RICH_rec_xmin;
  static float xmax=DQ_RICH_rec_xmax; 
  static float dx=xmax-xmin;
  int flr[nbin], tra[nbin];  
  for (int i=0;i<nbin;i++) {
    flr[i]=0;
    tra[i]=0;
  }
  int nflr=0;
  int ntra=0;
  for(ArteTable<RTRA>::iterator itra=ArteTable<RTRA>::begin();
      itra!=ArteTable<RTRA>::end(); ++itra) {
    ArtePointer<RTRA> rtra(itra); 
    int ok=1;
    if ( fabs(rtra->pf) > 0.001)      logp=fabs(1/rtra->pf); else ok=0;
    if (logp>0) logp=log(logp); else ok=0;
    int bin=int((logp-xmin)/dx);
    if ((bin>=nbin)||(bin<0)) ok=0;
    if (rtra->flr) { 
      nflr++;
      float lrother=1-rtra->lre-rtra->lrmu-rtra->lrpi-rtra->lrk+rtra->lrp;
      int which=0;
      float max=lrother;
      if (rtra->lre>max)  { max= rtra->lre;  which=1;}
      if (rtra->lrmu>max) { max= rtra->lrmu; which=2;}
      if (rtra->lrpi>max) { max= rtra->lrpi; which=3;}
      if (rtra->lrk>max)  { max= rtra->lrk;  which=4;}
      if (rtra->lrp>max)  { max= rtra->lrp;  which=5;}
      
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 210),which,1);
      
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 211),rtra->lre,1);
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 212),rtra->lrmu,1);
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 213),rtra->lrpi,1);
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 214),rtra->lrk,1);
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 215),rtra->lrp,1);
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 216),lrother,1);
      rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 217),rtra->hri,1);
      if (ok) {
	flr[bin]++;
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 221),logp,rtra->lre,1);
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 222),logp,rtra->lrmu,1);
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 223),logp,rtra->lrpi,1);
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 224),logp,rtra->lrk,1);
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 225),logp,rtra->lrp,1);
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 226),logp,lrother,1);
	rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 227),logp,rtra->hri,1);
      }
    }
    ntra++;
    if (ok) tra[bin]++;
  }
  rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 218),     nflr,1);
  if (ntra) rhp_hf1(DQ_ID(COMP_RICH,LVL_REC, 219),     float(nflr)/ntra,1);
  for (int i=0;i<nbin;i++) {
    float x=xmin+dx/nbin*(i+0.5);
    rhp_hfill(  DQ_ID(COMP_RICH,LVL_REC, 228),x,flr[i],1);
    if (tra[i]) 
      rhp_hfill(DQ_ID(COMP_RICH,LVL_REC, 229),x,float(flr[i])/tra[i],1);
  }
  */
  return 0; 
} 





