
// *********************************************************************
//
// Routine:      DQ_init.C
// Author:       D.Samtleben
//
// Description:  calls the different DQ-routines
//
// Changes:
// 09 Sep 1999 (AG) replace cerr by cout
// 16 Sep 1999 (DS) introduced evaluation of component set-up
// 16 Sep 1999 (AG) more print-outs
// 22 Sep 1999 (AG) move faComp() to farm/faUtil.C
// 07 Oct 1999 (AG) code polishing
// 19 Oct 1999 (AG) introduce DQ.hh
// 20 Oct 1999 (AG) use DQ.h
// 22 Oct 1999 (AG) print RCOMPMASK
// 26 Oct 1999 (AG) one DQ-flag per COMP_N_XYZ (component.h)
// 27 Oct 1999 (AG) include TLT
// 09 Nov 1999 (AG) polishing
// 16 Nov 1999 (AG) DQ-flags always hold unsigned short int mask
// 16 Nov 1999 (AG) introdcue DB-checking
// 17 Nov 1999 (AG) invoke DQ_FARM_rec_init()
// 23 Nov 1999 (AG) remove print-outs
// 24 Nov 1999 (AG) add MUON rec
// 24 Nov 1999 (AG) improve print-out
// 26 Nov 1999 (AG) intorduce rcDQ99()
//
// *********************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "component.h"
#include "runcfg.h"
#include "grr.h"

// Arte includes
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"
//#include "dq/DQ_ITR.h"
#include "dq/DQ_HIPT.h"
#include "dq/DQ_OTR.h"
#include "dq/DQ_RICH.h"
//#include "dq/DQ_TRD.h"
#include "dq/DQ_ECAL.h"
#include "dq/DQ_MUON.h"
//#include "dq/DQ_FLT.h"
//#include "dq/DQ_SLT.h"
#include "dq/DQ_TLT.h"
#include "dq/DQ_FARM.h"
#include "dq/DQ_CORR.h"
#include "dq/DQ_RECO.h"
//#include "dq/DQ_USR1.h"
//#include "dq/DQ_USR2.h"

// globals
int DQ_SVD  = 0;
int DQ_ITR  = 0;
int DQ_HIPT = 0;
int DQ_OTR  = 0;
int DQ_RICH = 0;
int DQ_TRD  = 0;
int DQ_ECAL = 0;
int DQ_MUON = 0;
int DQ_FLT  = 0;
int DQ_SLT  = 0;
int DQ_TLT  = 0;
int DQ_FARM = 0;

int DQ_CORR = 0;
int DQ_RECO = 0;

int DQ_USR1 = 0;
int DQ_USR2 = 0;

// special
int DQ_DB   = 0;
int DQ_EVAL = 0;

// old
int DQ_TARG = 0;

// externals
extern int FADEB;

//
// program code
//
int
DQ_init()
{
  // show up
  cout << "DQ_init()" << endl;
  
  // component flags
  char *rcompmask = getenv( RC_Comp_Mask_ENV );
  int mask = 0;
  if( rcompmask ) mask = atoi( rcompmask );
  char txt[80];
  memset( txt, 0, 80 );
  sprintf( txt, "DQ_init(): DQ Mask=%d", mask );
  ARTE_error( GRR_SINFO, txt, 0 );

  // print
  //if( FADEB ){
    cout << "DQ_init(): Components in run (RC_Comp_Mask_ENV):" << endl;
    cout << "           Mask=" << mask                  << endl;
    cout << "           DAQ= " << faComp( COMP_M_DAQ  ) << endl;
    cout << "           SVD= " << faComp( COMP_M_SVD  ) << endl;
    cout << "           ITR= " << faComp( COMP_M_ITR  ) << endl;
    cout << "           HIPT=" << faComp( COMP_M_HIPT ) << endl;
    cout << "           OTR= " << faComp( COMP_M_OTR  ) << endl;
    cout << "           RICH=" << faComp( COMP_M_RICH ) << endl;
    cout << "           TRD= " << faComp( COMP_M_TRD  ) << endl;
    cout << "           ECAL=" << faComp( COMP_M_ECAL ) << endl;
    cout << "           MUON=" << faComp( COMP_M_MUON ) << endl;
    cout << "           FLT= " << faComp( COMP_M_FLT  ) << endl;
    cout << "           SLT= " << faComp( COMP_M_SLT  ) << endl;
    cout << "           TLT= " << faComp( COMP_M_TLT  ) << endl;
    cout << "           FARM=" << faComp( COMP_M_FARM ) << endl;
    //}

  // print
  //DQ_print( "DQ_init(): BEFORE faComp()" );

  //
  // enable DQ only for components which are in RCOMPMASK
  //
  DQ_SVD  = DQ_SVD  & faComp( COMP_M_SVD  );
  DQ_HIPT = DQ_HIPT & faComp( COMP_M_HIPT );
  DQ_OTR  = DQ_OTR  & faComp( COMP_M_OTR  );
  DQ_RICH = DQ_RICH & faComp( COMP_M_RICH );
  DQ_ECAL = DQ_ECAL & faComp( COMP_M_ECAL );
  DQ_MUON = DQ_MUON & faComp( COMP_M_MUON );
  DQ_FLT  = DQ_FLT  & faComp( COMP_M_FLT  );
  DQ_SLT  = DQ_SLT  & faComp( COMP_M_SLT  );
  DQ_TLT  = DQ_TLT  & faComp( COMP_M_TLT  );
  DQ_FARM = DQ_FARM & faComp( COMP_M_FARM );

  DQ_CORR = DQ_CORR & faComp( COMP_M_DAQ  );
  DQ_RECO = DQ_RECO & faComp( COMP_M_DAQ  );

  DQ_USR1 = DQ_USR1 & faComp( COMP_M_DAQ  );
  DQ_USR2 = DQ_USR2 & faComp( COMP_M_DAQ  );

  // print
  //DQ_print( "DQ_init(): AFTER faComp()" );

  //
  // call  init functions
  //
  int dq_flag=0;

  if( DQ_flag( COMP_SVD,  LVL_M_HIT ) ) dq_flag = DQ_SVD_hit_init();
  if( DQ_flag( COMP_SVD,  LVL_M_REC ) ) dq_flag = DQ_SVD_rec_init();

  if( DQ_flag( COMP_HIPT, LVL_M_RAW ) ) dq_flag = DQ_HIPT_raw_init();

  if( DQ_flag( COMP_OTR,  LVL_M_HIT ) ) dq_flag = DQ_OTR_hit_init();
  if( DQ_flag( COMP_OTR,  LVL_M_REC ) ) dq_flag = DQ_OTR_rec_init();

  if( DQ_flag( COMP_RICH, LVL_M_RAW ) ) dq_flag = DQ_RICH_raw_init();
  if( DQ_flag( COMP_RICH, LVL_M_HIT ) ) dq_flag = DQ_RICH_hit_init();
  if( DQ_flag( COMP_RICH, LVL_M_REC ) ) dq_flag = DQ_RICH_rec_init();

  if( DQ_flag( COMP_ECAL, LVL_M_HIT ) ) dq_flag = DQ_ECAL_hit_init();
  if( DQ_flag( COMP_ECAL, LVL_M_REC ) ) dq_flag = DQ_ECAL_rec_init();

  if( DQ_flag( COMP_MUON, LVL_M_HIT ) ) dq_flag = DQ_MUON_hit_init();
  if( DQ_flag( COMP_MUON, LVL_M_REC ) ) dq_flag = DQ_MUON_rec_init();

  if( DQ_flag( COMP_TLT,  LVL_M_REC ) ) dq_flag = DQ_TLT_rec_init();

  if( DQ_flag( COMP_FARM, LVL_M_HIT ) ) dq_flag = DQ_FARM_hit_init();
  if( DQ_flag( COMP_FARM, LVL_M_REC ) ) dq_flag = DQ_FARM_rec_init();

  if( DQ_flag( COMP_CORR, LVL_M_HIT ) ) dq_flag = DQ_CORR_hit_init();

  if( DQ_flag( COMP_RECO, LVL_M_REC ) ) rcDQinit();

  //if( DQ_flag( COMP_USR1, LVL_M_REC ) ) dq_flag = ;
  //if( DQ_flag( COMP_USR2, LVL_M_REC ) ) dq_flag = ;


  // after initialization has been done read database
  dq_flag = DQ_db();

  // print
  //DQ_print( "DQ_init(): AFTER DQ_db()" );

  //
  // take only dq which is asked for in the kumac & RCOMPMASK
  //
  if( FADEB ){

    cout << "DQ_init(): DQ settings (final):" << endl;
    
    cout << "           SVD:  RAW/HIT/REC "
	 << DQ_flag( COMP_SVD, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_SVD, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_SVD, LVL_M_REC ) << endl;
    
    cout << "           ITR:  RAW/HIT/REC "
	 << DQ_flag( COMP_ITR, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_ITR, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_ITR, LVL_M_REC ) << endl;
    
    cout << "           HIPT: RAW/HIT/REC "
	 << DQ_flag( COMP_HIPT, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_HIPT, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_HIPT, LVL_M_REC ) << endl;
    
    cout << "           OTR:  RAW/HIT/REC "
	 << DQ_flag( COMP_OTR, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_OTR, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_OTR, LVL_M_REC ) << endl;
    
    cout << "           RICH: RAW/HIT/REC "
	 << DQ_flag( COMP_RICH, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_RICH, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_RICH, LVL_M_REC ) << endl;
    
    cout << "           TRD:  RAW/HIT/REC "
	 << DQ_flag( COMP_TRD, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_TRD, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_TRD, LVL_M_REC ) << endl;
    
    cout << "           ECAL: RAW/HIT/REC "
	 << DQ_flag( COMP_ECAL, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_ECAL, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_ECAL, LVL_M_REC ) << endl;
    
    cout << "           MUON: RAW/HIT/REC "
	 << DQ_flag( COMP_MUON, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_MUON, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_MUON, LVL_M_REC ) << endl;
    
    cout << "           FLT:  RAW/HIT/REC "
	 << DQ_flag( COMP_FLT, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_FLT, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_FLT, LVL_M_REC ) << endl;
    
    cout << "           SLT:  RAW/HIT/REC "
	 << DQ_flag( COMP_SLT, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_SLT, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_SLT, LVL_M_REC ) << endl;
    
    cout << "           TLT:  RAW/HIT/REC "
	 << DQ_flag( COMP_TLT, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_TLT, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_TLT, LVL_M_REC ) << endl;
    
    cout << "           FARM: RAW/HIT/REC "
	 << DQ_flag( COMP_FARM, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_FARM, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_FARM, LVL_M_REC ) << endl;
    
    cout << "           CORR: RAW/HIT/REC "
	 << DQ_flag( COMP_CORR, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_CORR, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_CORR, LVL_M_REC ) << endl;

    cout << "           RECO: RAW/HIT/REC "
	 << DQ_flag( COMP_RECO, LVL_M_RAW ) << " "
	 << DQ_flag( COMP_RECO, LVL_M_HIT ) << " "
	 << DQ_flag( COMP_RECO, LVL_M_REC ) << endl;
   
    //cout << "           USR1: RAW/HIT/REC "
    // << DQ_flag( COMP_USR1, LVL_M_RAW ) << " "
    // << DQ_flag( COMP_USR1, LVL_M_HIT ) << " "
    // << DQ_flag( COMP_USR1, LVL_M_REC ) << endl;

    //cout << "           UIR2: RAW/HIT/REC "
    // << DQ_flag( COMP_USR2, LVL_M_RAW ) << " "
    // << DQ_flag( COMP_USR2, LVL_M_HIT ) << " "
    // << DQ_flag( COMP_USR2, LVL_M_REC ) << endl;
  }
  
  return 0;
}
