
// *****************************************************************************
//
// source:    faCatchSIG.C
//
// type:      source code
//
// created:   23 Sep 1997
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:         
// 23 Feb 1999 (AG) include prototypes here
// 11 Mar 1999 (AG) polishing
// 25 Mar 1999 (AG) use atstopc_()
// 01 Jun 1999 (AG) put includes
// 18 Aug 1999 (AG) provide only one signal catching routine
// 02 Sep 1999 (AG) fixed segmentation violation problem
// 07 Sep 1999 (AG) use atstopcc_( char * )
// 13 Sep 1999 (AG) signal catcher output
// 13 Oct 1999 (AG) write to log-file
// 14 Oct 1999 (AG) added unistd.h
// 18 Oct 1999 (AG) polishing
// 03 Nov 1999 (AG) add process id to logfile
// 03 Nov 1999 (AG) log sig to file
// 11 Nov 1999 (AG) improve log
// 24 Nov 1999 (AG) use ARTE_error()
// 26 Nov 1999 (AG) correct logfile/logsig handling
// 29 Nov 1999 (AG) polishing
// 29 Feb 2000 (AG) linux -> __linux
// 22 Mar 2000 (AG) improve print-out
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <signal.h>
#include <string.h>
#include <unistd.h>

// Arte includes
#include "farm/fa.hh"
#include "farm/farm.hh"

// externals
extern char *LOGFILE; // log-file
extern char *LOGSIG;  // log-file

// C prototypes
extern "C" void atstopcc_( char *txt );
#ifdef __linux
extern "C" long gethostid();
extern "C" int  gethostname( char *hostname, unsigned int length );
#endif

//
// program code
//
void
faCatchSIG( int sig )
{
#ifdef _OUT_
  switch( sig ){
  case SIGINT:
    cout << "faCatchSIG(): SIGINT  sig=" << sig << " Invoking atstopcc_()" << endl;
    break;
  case SIGQUIT:
    cout << "faCatchSIG(): SIGQUIT sig=" << sig << " Invoking atstopcc_()" << endl;
    break;
  case SIGILL:
    cout << "faCatchSIG(): SIGILL  sig=" << sig << " Invoking atstopcc_()" << endl;
    break;
  case SIGBUS:
    cout << "faCatchSIG(): SIGBUS  sig=" << sig << " Invoking atstopcc_()" << endl;
    break;
  case SIGSEGV:
    cout << "faCatchSIG(): SIGSEGV sig=" << sig << " Invoking atstopcc_()" << endl;
    break;
  case SIGTERM:
    cout << "faCatchSIG(): SIGTERM sig=" << sig << " Invoking atstopcc_()" << endl;
    break;
  default:
    cout << "faCatchSIG(): sig=" << sig << " Invoking atstopcc_()" << endl;
  }
#endif

  // error logger
  switch( sig ){
  case SIGINT:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught SIGINT", sig );
    break;
  case SIGQUIT:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught SIGQUIT", sig );
    break;
  case SIGILL:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught SIGILL", sig );
    break;
  case SIGBUS:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught SIGBUS", sig );
    break;
  case SIGSEGV:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught SIGSEGV", sig );
    break;
  case SIGTERM:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught SIGTERM", sig );
    break;
  default:
    ARTE_error( GRR_SINFO, "faCatchSIG(): Caught signal", sig );
  }
  
  //
  // host name
  //
  char *hostName = new char[30];
  gethostname( hostName, 29 );
  
  //
  // current date/time
  //
  time_t t = time( NULL ); 
  char date[25];
  strcpy( date, ctime( &t ) );
  date[24] = '\0';

  //
  // process id
  //
  int pid = getpid();

  //  
  // check signal
  //
  char txt[80];
  switch( sig ){
  case SIGINT:
    strcpy( txt, "Called from faCatchSIG() with SIGINT." );
    break;
  case SIGQUIT:
    strcpy( txt, "Called from faCatchSIG() with SIGQUIT." );
    break;
  case SIGILL:
    strcpy( txt, "Called from faCatchSIG() with SIGILL." );
    break;
  case SIGBUS:
    strcpy( txt, "Called from faCatchSIG() with SIGBUS." );
    break;
  case SIGSEGV:
    strcpy( txt, "Called from faCatchSIG() with SIGSEGV." );
    break;
  case SIGTERM:
    strcpy( txt, "Called from faCatchSIG() with SIGTERM." );
    break;
  default:
    strcpy( txt, "Called from faCatchSIG()." );
  }

  //
  // write to logfile
  //
  if( LOGFILE ){
#ifdef _OUT_
    cout << "SIG: Write to logfile=" << LOGFILE << endl;
#endif
    FILE *log1 = fopen( LOGFILE, "a+" );
    if( log1 != NULL ){
      fprintf( log1, "SIG   on %s node:%s pid:%d sig=%d\n",
	       date, hostName, pid, sig );
      fclose( log1 );
    }
  }

  //
  // write to logfile
  //
  if( LOGSIG && ( sig != 0 ) ){
#ifdef _OUT_
    cout << "SIG: Write to logsig=" << LOGSIG << endl;
#endif
    FILE *log2 = fopen( LOGSIG, "w" );
    if( log2 != NULL ){
      fprintf( log2, "signal=%d", sig );
      fclose( log2 );
    }
  }

  // call atstop
  atstopcc_( txt );
}
