
#ifndef _FARM_HH_
#define _FARM_HH_

// *****************************************************************************
//
// source:    farm.hh
//
// type:      header file
//
// created:   23 Feb 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// changes:   
//                  Fortran/C callable functions (AG: 8 Mar 99)
//                  polishing (AG: 11 Mar 99)
//                  change return values of utility functions (AG: 26 Mar 99)
//                  introduce DEVT (AG: 6 May 99)
//                  introduce ARTE_get_recid_DDAQ/DKEY (AG: 11 May 99)
//                  EVHD has more el;ements now (AG: 26 May 99)
//                  remove DKEY again (AG: 27 May 99)
// 09 Sep 1999 (AG) complete utilities to access tables
// 22 Sep 1999 (AG) include faComp()
// 07 Oct 1999 (AG) include faComp_evt()
// 19 Oct 1999 (AG) polishing
// 16 Nov 1999 (AG) introduce error logging function for online
// 22 Nov 1999 (AG) introduce faReco()
// 03 Dec 1999 (AG) adapt to new EVHD table (ipaddr and cputime in EVRC now)
// 16 Dec 1999 (AG) introduce EVRC table; remove ipaddr and cputime from EVHD
// 17 Dec 1999 (AG) add filling of EVRC
// 23 Feb 2000 (AG) adapt to new EVRC
//  1 Mar 2000 (CP) introduce faHit()
// 26 Mar 2000 (AG) introduce faCompStrg()
//
// *****************************************************************************

// ONLINE includes
#include "etb.h"
#include "tgts_raw.h"

// prototypes

// get pointer to event
extern "C" EVENT_HDR *ARTE_get_event();

// get recid
extern "C" int ARTE_get_recid_DEVT( int *recid );
extern "C" int ARTE_get_recid_DSVD( int *recid );
extern "C" int ARTE_get_recid_DITR( int *recid );
extern "C" int ARTE_get_recid_DPTC( int *recid );
extern "C" int ARTE_get_recid_DOTR( int *recid );
extern "C" int ARTE_get_recid_DRIC( int *recid );
extern "C" int ARTE_get_recid_DTRD( int *recid );
extern "C" int ARTE_get_recid_DCAL( int *recid );
extern "C" int ARTE_get_recid_DMUT( int *recid );
extern "C" int ARTE_get_recid_DMUX( int *recid );
extern "C" int ARTE_get_recid_DMUP( int *recid );

// get FEDs
extern "C" int ARTE_get_feds_DSVD( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DITR( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DPTC( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DOTR( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DRIC( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DTRD( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DCAL( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DMUT( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DMUX( int max, ETB_fed *feds, int *n );
extern "C" int ARTE_get_feds_DMUP( int max, ETB_fed *feds, int *n );

// evaluate presence of component COMP_M_XYZ in run from RCOMPMSK
int faComp( int COMP_M_XYZ );

// evaluate presence of component COMP_M_XYZ for HITB from RHITMSK
int faHit( int COMP_M_XYZ );

// evaluate presence of component COMP_M_XYZ for reconstruction from RRECOMSK
int faReco( int COMP_M_XYZ );

// return string of components in run
char *faCompStrg( int mask );

// evaluate presence of component  COMP_M_XYZ in event from DIHD->cmsk
int faComp_evt( int COMP_M_XYZ );

// error logging
int ARTE_error( int severity, const char *message, int error );

// general
int faPrintNrow();

// EVHD
int   faPrintEVHD(); // print table

int   faEVHDevt();
int   faEVHDrun();
int   faEVHDexp();
int   faEVHDdate();
int   faEVHDtime();
float faEVHDvfld();
int   faEVHDflagdst();
int   faEVHDdbmajor();
int   faEVHDdbminor();
int   faEVHDflag();

// EVRC
int          faPrintEVRC(); // print table

unsigned int faEVRCipaddr  ( int row );
unsigned int faEVRCerrmask ( int row );
int          faEVRCdate    ( int row );
float        faEVRCcputime ( int row );
float        faEVRCartevers( int row );
unsigned int faEVRCalca    ( int row );

// DIHD
int faPrintDIHD();

// DEVT
int faDEVTlength();

// DDAQ
int   faPrintDDAQ();        // print table

int   faDDAQrun_number();   // Run number
int   faDDAQevent_time();   // Time event was built
int   faDDAQevent_status(); // event status/error code
int   faDDAQflt_status();   // FLT trigger number
int   faDDAQslp_node();     // SLP processing node
int   faDDAQslt_status();   // SLT trigger number/status
int   faDDAQexp_number();   // Experiment Number
int   faDDAQfarm_node();    // Farm node
int   faDDAQBX_long();      // bits 0 to 31 of the BX tag (at time trigger issued)
short faDDAQBX_upper();     // bits 32 to 47 of the BX tag (at time trigger issued)
short faDDAQFCS_tcode();    // FCS trigger code for event
short faDDAQphysBX();       // physical BX == hera bucket ID of event
short faDDAQFLT_BX();       // pipeline cell triggered on by FLT for this event
short faDDAQfltevt();       // flt event number as delivered by FCS

// DTAR
int   faPrintDTAR();

float faDTARia();
float faDTARbg();
void  faDTARtgts( TGTS_RAW *tgts );

// raw data logging (DAQ format)
int faDaqLog();

// C prototypes
extern "C" float faDTARia_();
extern "C" float faDTARbg_();

#endif
