/************************************************************************
*
* CRKIBK
* written by Siegmund Nowak, 13/02/96 19.56.54
*
* This routine transforms the temporary GEANT vertex and kinematic banks
* into  ARTE banks MVER and MTRA
*
*       called from GUOUT
*
***********************************************************************/
#include <vector.h>
#include <stdio.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"


extern "C" {
  void crkibk_();
  int  tpgear_(int*);
  void mmfree_(int&);
}

void crkibk_() {
   extern vector<hgmtra> *hg_mtra;
   extern vector<int>    *hg_vert;

   vector<hgmtra>::iterator mt, mo;   
   vector<int>::iterator mv;     
   zgkine ZGKINE;

   if(!arteflag_.mimpsw || hg_mtra->size() == 0) return;

// mark tracks with mimps or where daughters have mimps
   for(int itr=hg_mtra->size(); itr >0; itr--){
      mt = hg_mtra->begin() + itr -1;
      if(!mt->nhit) continue;                  //    no hits
      if(mt->mtra)  continue;                  //    track already booked
      int jtr = *(gcbank_.lq + gclink_.jkine -1 - itr);
      int *pji = gcbank_.iq + jtr +5 ; 
      float *pjx = (float*)pji;
      unsigned int tr_vertex=int(*pjx); 
      if(!tr_vertex) continue;
      int jvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
      pji = gcbank_.iq + jvx +4;
      pjx = (float*)pji;
      int mo_track=int(*pjx); 
      if(!mo_track) continue;
      mo = hg_mtra->begin() + mo_track -1;        if(!mo->nhit) mo->nhit = -1;
   }

// install mtra and mver and relations

   for(unsigned int itr=1; itr <= hg_mtra->size(); itr++){
      mt = hg_mtra->begin() + itr -1;
      if(mt->mtra) {                              // track already booked
         ArtePointer<MTRA> map(mt->mtra);   
         map->ml   = mt->len;
         map->leng = mt->len;
         map->flag = mt->flag;
         continue;}

      if(!mt->nhit) continue;                    // no hits

//    define geant particle code, geant vertex origin and geant mother track
      int jtr = *(gcbank_.lq + gclink_.jkine -1 - itr);
      int *pji = gcbank_.iq + jtr +4; 
      float *pjx = (float*)pji;       
      int ipart =int(*pjx);
      pji = gcbank_.iq + jtr +5;
      pjx = (float*)pji; 
      unsigned int tr_vertex=int(*pjx); 
      int jvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
      pji = gcbank_.iq + jvx +4;
      pjx = (float*)pji;
      int mo_track=int(*pjx); 

//    book and fill track bank
      MTRA ma;
      memcpy(&ma.px,gcbank_.iq + jtr +1,12);
      ma.ml   = mt->len;
      ma.flag = mt->flag;
      int itopp = tpgear_(&ipart); //transform particle id from geant to arte
      if(!itopp) {           
         arteMsg << "crkibk: particle code = 0";        
         static int idcount = 8762135;  arteMsg.warn(idcount);
      }
      ArtePointer<TOPP> tp(itopp);
      if(tp->wid > 1.e-9)      ma.flag = 2;              // it is a resonance
      ArteTable<MTRA>::push_back(ma);

      // update temporary track bank and fill MTRA bank and rel topp - track
      int nmtra = ArteTable<MTRA>::size();              
      mt->mtra = nmtra;
      ArtePointer<MTRA> map(nmtra);
      map->leng = mt->len;
      if(tp->wid > 1.e-9)          map->mass = tp->mass;
      createRelation( tp, map, &TOPP::mtra);

//    book and fill vertex bank
//    first look for the vertex which track origin is not a resonance
      int iorig = 0;
      if(mo_track > 0) {
         mo = hg_mtra->begin() + mo_track -1;
         createRelation(ArtePointer<MTRA>(mo->mtra), map,&MTRA::etra);

         //define particle id of mother 
         int mjtr = *(gcbank_.lq + gclink_.jkine -1 - mo_track);
         int *pji = gcbank_.iq + mjtr +4;
         float *pjx = (float*)pji;
         int mpart =int(*pjx);
         int mtopp = tpgear_(&mpart);
         ArtePointer<TOPP> mtp(mtopp);                
         if(mtp->wid > 1.e-9) {            // mother is a resonance 
            iorig = -mo->mtra;
            // as long as mother = resonance go to the previous mother track
            while (mo_track &&  mtp->wid > 1.e-9) {  
               mjtr = *(gcbank_.lq + gclink_.jkine -1 - mo_track);
               pji = gcbank_.iq + mjtr +5;   
               pjx = (float*)pji;
               unsigned int tr_vertex=int(*pjx); 
               int mjvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
               pji = gcbank_.iq + mjvx +4;    
               pjx = (float*)pji;
               mo_track=int(*pjx); 
               if(mo_track) {
                  mjtr = *(gcbank_.lq + gclink_.jkine -1 - mo_track);
                  pji = gcbank_.iq + mjtr +5;
                  pjx = (float*)pji;
                  mpart=int(*pjx);
                  mtopp = tpgear_(&mpart);
                  mtp = ArtePointer<TOPP>(mtopp);
               }
            } 
         }
      }

//    no more mother or mother is not a resonance - look for the vertex origin
      mv = hg_vert->begin() + tr_vertex - 1;
      if(!*mv || tr_vertex > hg_vert->size()) { // vertex not yet  booked
         MVER mver;                         // book and fill hg_vert and mver
         int jvx = *(gcbank_.lq + gclink_.jvertx -1 - tr_vertex);
         memcpy(&mver.x,gcbank_.iq + jvx,12);
         mver.z=mver.z - zgmoth_.ZZERO ; 
         ArteTable<MVER>::push_back(mver);
         int nmver = ArteTable<MVER>::size();                 
         ZGKINE.mvpush(tr_vertex,nmver);
         ArtePointer<MVER> mv(nmver);
         if(!mo_track) {}                       
//        hier muss gecheckt werden
         else {
            mv->flag = 4;
            mo = hg_mtra->begin() + mo_track -1;         
            int nmoth = mo->mtra;
            ArtePointer<MTRA> moth(nmoth);
            if(iorig < 0 || moth->flag == 3) mv->flag = 2;
            createRelation( moth, mv, &MTRA::ever);  
	 }
      }
//    fill mtra_orig and relation mtmv
      mv = hg_vert->begin() + tr_vertex -1;
      int nmver = *mv;
      createRelation(ArtePointer<MVER>(nmver),map,&MVER::etra);

      if(iorig < 0) map->orig = iorig; 
      else map->orig = nmver;
   }                                                     // end track loop

   mmfree_(ArtePointer<MTRA>::getLink().getRef());
   mmfree_(ArtePointer<MVER>::getLink().getRef());  
}


