#ifndef _NOARTE_

//----------------------------------------------------------------------------
//
//  Subroutines to fill HITB Table from ITR_marvin's structures
//  ===========================================================
// 
//----------------------------------------------------------------------------

// include files
#include  <stdio.h>
#include  <stdlib.h>
#include  <string.h>
#include  <math.h>

// Arte banks,  C++ version
#include  "arte/arte.hh"

#include "aframelib.h"
#include "itrleda.h"
#include "ITR_marvin.h"
#include "ITR_arte.h"
#include "grr.h"

int ITR_STEERDB_MAJ=-999;
int ITR_STEERDB_MIN=-999; 
int ITR_CLUSTER_ALG=-1;
int ITR_DYNTHRESH=0;
int ITR_NEVT_MON=0;
float ITR_THRESHOLD_A=0.;
float ITR_THRESHOLD_B=0.;
float ITR_HOTCUT=0.;
float ITR_MAXSIGNAL=0.;

extern "C" {

/* ------------------------------------------------------------------------
 *
 *  Initialize aframe.
 *
 *  To be called before everything else.
 *  Done is (among others):
 *    - Some initialization
 *
 * ------------------------------------------------------------------------ */
int ITR_arte_param_init()
{
  char *routine = "ITR_arte_param_init";

  aframe_mess(routine,GRR_SINFO,"ITR_arte started");

  return GRR_OK;

}

}  /* end of extern C */  

static int ITR_gedeindex[10][10][8];

// ---------------------------------------------------------------------------
//  Fill HITB table from ITR_Marvin_Sevt_t structure 
//  last change 15/11/1999  TZ
// 
//  Returns:   GRR_OK    in case of no problems
//             GRR_FISHY otherwise
// ---------------------------------------------------------------------------

int ITR_fillHITBfromSPARSEVT()
{
  char *routine = "ITR_fillHITBfromSPARSEVT";
  ArtePointer<GEDE> oldgede = 0;

  // First of all get event from ITR_marvin

  ITR_Marvin_Sevt_t *sevt = ITR_marvin_fetch_sevt();

  if ( sevt == NULL ) {
    aframe_mess(routine,GRR_SERROR,"NULL pointer to sparsified information!");
    return GRR_FISHY;
  }

  // check reasonable number of clusters in format 3

  ITR_Marvin_Header_t *header = ITR_marvin_sevtHeader( (char*) sevt->data );

  if ( header->format == 3 && header->ncl > 0x7FFF / 8 ) {
    aframe_mess(routine,GRR_SERROR,"Too many clusters in event --> skipping it!");
    return GRR_FISHY;
  }
 
  // loop over clusters to fill HITB bank
  
  ITR_Marvin_Cluster_t *cluster = ITR_marvin_sevtCluster( (char*) sevt->data, -1 );
   
  while ( cluster ) {

    // get GEDE id from database

    LedaPointer<class ITR_Chamber> chamber = ITR_Chamber->get->key( cluster->chamber );
    LedaPointer<class ITR_nede> nede = chamber->tonede;

    if ( !chamber ) {
      aframe_mess(routine,GRR_SERROR,"ITR chamber %d not in database");
      cluster = ITR_marvin_sevtCluster(NULL, -1);
      continue;
    }


    ArtePointer<GEDE> gede, gede1;

    int gederow = ITR_arte_returnGEDE( (int) nede->superl_db, (int) nede->lay_db, (int) nede->sec_db[0] );

    if (nede->sec_db[0] == 5 || nede->sec_db[0] == 7) {
      gede = ArtePointer<GEDE>( gederow );
      gede1 = ArtePointer<GEDE> (gederow+1);
    }
    else if (nede->sec_db[0] == 2 || nede->sec_db[0] == 4) {
      gede1 = ArtePointer<GEDE>( gederow );
      gede = ArtePointer<GEDE> (gederow+1);
    }                    
    
    int found = 1;

    if (gede == 0 || gede1 == 0 ) {
      aframe_mess(routine,GRR_SERROR,"GEDE row is non existing !");
      cluster = ITR_marvin_sevtCluster(NULL,-1);
      found = 0;
      continue;
    }

    if ( gede->cmp != 3 || gede1->cmp != 3 ) {
      aframe_mess(routine,GRR_SERROR,"GEDE row is not an ITR GEDE table");
      cluster = ITR_marvin_sevtCluster(NULL, -1);
      found = 0;
      continue;
    }

    // cluster coordinates -> HITB
    //
    // strip counting in ITR_marvin starts always at the beam pipe
    // (0..767) in Arte first wire is always at the right corner of
    // the corresponding gede plane 
    // for type I+II 8 (III 5)  wires at both sides are not connected 
    // therefor corresponding Arte gede planes have only 752 (758) wires 

    HITB hit;
    int ITR_hitoutofsens = 0;
    float relpos=0;
    float gasgap = gede->zmax-gede->zmin;

    hit.z    = gede->zfw[0]+gasgap/2;

    if ( (int) chamber->type == 3 && (int) nede->sec_db[0] >= 5 ) {
      // -x side Typ III 
      if (cluster->cog>4 && cluster->cog<763) {
        relpos = gede->nwi[0]+gede1->nwi[0]+4-cluster->cog;
        hit.x    = relpos*gede->pit + gede->xfw[0];
      }
      else ITR_hitoutofsens = 1;
    }  

    if ( (int) chamber->type == 3 && (int) nede->sec_db[0] <= 3 ) {
      // +x side Typ III
      if (cluster->cog>4 && cluster->cog<763) {
        relpos = cluster->cog-5;
        hit.x    = relpos*gede->pit + gede->xfw[0];
      }
      else ITR_hitoutofsens = 1;
    }      

    if ( (int) chamber->type < 3 && (int) nede->sec_db[0] >= 5 ) {
      // -x side Typ I+II
      if (cluster->cog>7 && cluster->cog<760) {
        relpos = gede->nwi[0]+gede1->nwi[0]+7-cluster->cog;
        hit.x    = relpos*gede->pit + gede->xfw[0];
      }
      else ITR_hitoutofsens = 1;
    }

    if ( (int) chamber->type < 3 && (int) nede->sec_db[0] <= 3 ) {
      // +x side Typ I+II
      if (cluster->cog>7 && cluster->cog<760) {
        relpos = cluster->cog-8;
        hit.x    = relpos*gede->pit + gede->xfw[0];
      }
      else ITR_hitoutofsens = 1;
    }
    if (!ITR_hitoutofsens) {

      if (relpos > gede->nwi[0]) 
        hit.gede = gede1;
      else 
        hit.gede = gede;

      hit.y    = 0;
      hit.q    = cluster->charge;

      // hit resolution set to 100um

      hit.r    = 0.01;
    
      ArtePointer<HITB> hitb = ArteTable<HITB>::push_back( hit );

      // update GEDE bank: hit1 only , if GEDEnr has changed,
      //                   hitl always (every additional hit overwrites it)
      // be carefull : 1 ITR-Chamber corresponds to 2 Gede planes

      if ( hitb->gede != oldgede ) {
        if (relpos > gede->nwi[0]) gede->hit1 = hitb;
        else gede1->hit1 = hitb;
        oldgede = hitb->gede;
      }
    
      if (relpos > gede->nwi[0]) gede->hitl = hitb;
      else gede1->hitl = hitb;
       
      if ( found == 0 ) {
        aframe_mess(routine,GRR_SERROR,"GEDE row is non-existent");
        cluster = ITR_marvin_sevtCluster(NULL, -1);
        continue;
      }
    } // of (!ITR_hitoutofsens)

      // get next cluster
      cluster = ITR_marvin_sevtCluster(NULL, -1);
  } 
 
  return GRR_OK;
}



// ---------------------------------------------------------------------------
//
//  update GEDE from NEDE + ALIGNMENT
//
// ---------------------------------------------------------------------------


int ITR_arte_updateGEDE( ) {

  // routine for update of GEDE using Alignment information
  // has to be called by acReadITR

  return -1;

}


// ---------------------------------------------------------------------------
//
//  fill gedeindex
//
// --------------------------------------------------------------------------- 

int ITR_fillgedeindex() {

  // routine to fill ITR_gedeindex 
  // allows faster access to gede table 
  // gesl->nsl, gede->lay, gede->sec  -> gede.index

  int error = 0;
  char* ITR_SLNAME[] = {"M01","M03","M05","M06","M10","M11","M12","M13","M14","M15"}; 
  char ITR_hwc[13] = "none";
  char *routine = "ITR_fillITRgedeindex";

  // loop over CHAMBER entries in LEDA

  for ( LedaTableIterator<class ITR_Chamber> chamber; chamber.valid(); chamber.next() ) {
    LedaPointer<class ITR_nede> nede = chamber->tonede;
    LedaPointer<class ITR_nesl> nesl = nede->tonesl;

    // loop over ARTE GEDE table to get corresponding GEDE row
    int found = 0;

    for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
      ArtePointer<GEDE> gede(*igede);
      ArtePointer<GEDE> gede1;
      ArtePointer<GESL> gesl(gede->gesl);
      
      if (   (gede->cmp == 3)
          && (gesl->nsl == (int) nede->superl_db)
          && (gede->lay == (int) nede->lay_db)
          && (gede->sec == (int) nede->sec_db[0]) ) {

        if (nede->sec_db[0] == 5 || nede->sec_db[0] == 7)
          gede1 = ArtePointer<GEDE> (gede.index()+1);
        else if (nede->sec_db[0] == 2 || nede->sec_db[0] == 4)
          gede1 = ArtePointer<GEDE>( gede.index()+1 );                
        // fill gedeindex array

        if (    (int) nede->superl_db  >= 1 && (int) nede->superl_db  <= 10
                && (int) nede->lay_db >= 1 && (int) nede->lay_db <= 10
		&& (int) nede->sec_db[0] >= 1 && (int) nede->sec_db[0] <= 8 ) {
          char ITR_ori[3] = "++";
          ITR_gedeindex[ (int) nede->superl_db-1 ][ (int) nede->lay_db-1 ][ (int) nede->sec_db[0]-1 ] = gede.index();

          // gede->hwc is filled 
          if (nede->sec_db[0]== 3 || nede->sec_db[0]== 4) ITR_ori[1] = '-';
          if (nede->sec_db[0]== 6 || nede->sec_db[0]== 5) { ITR_ori[0] = '-'; ITR_ori[1] = '-'; }
          if (nede->sec_db[0]== 8 || nede->sec_db[0]== 7) ITR_ori[0] = '-';
          char ITR_ang[3] = "+0";
          if (gede->rot[3] > 0.05) ITR_ang[1] = '5';
          if (gede->rot[3] < -0.05) { ITR_ang[0] = '-'; ITR_ang[1] = '5';}
          sprintf(ITR_hwc,"%s%s%d%.2s%.4s",ITR_SLNAME[nede->superl_db-1],ITR_ori,nede->lay_db,ITR_ang,chamber->name);
          strncpy(gede->hwc,ITR_hwc,12);
          strncpy(gede1->hwc,ITR_hwc,12);
        }
        else {
          aframe_mess(routine,GRR_SWARNING,"GEDE");
        }
        found = 1;
     
      }
    }   // end loop over ARTE GEDE rows

    // complain in case no corresponding ARTE GEDE row was found

    if ( !found ) {
      aframe_mess(routine,GRR_SWARNING,"ITR chamber not found in GEDE table!");
      error++;
    }

  }
  if ( error )
    return GRR_FISHY;

  return GRR_OK;

}


// ---------------------------------------------------------------------------
//
//  return GEDE row from SL name, layer, and sector numbers
//
// ---------------------------------------------------------------------------

int ITR_arte_returnGEDE( int sl, int lay, int sec )
{
  
  char *routine = "ITR_arte_returnGEDE";

  if ( sl >= 1 && sl <= 10 && lay >= 1 && lay <= 10 && sec >= 1 && sec <= 8 )
    return ITR_gedeindex[ sl-1 ][ lay-1 ][ sec-1 ];
  else
    return 0;

}


#endif   /* of #ifndef _NOARTE_ */


/************************************************************************
* ITRSTEER
*
* called from  KUIP
***********************************************************************/

#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <ctype.h>

extern "C" void kugeti_( int* );
extern "C" void kugetr_( float* );

extern "C" void itrsteer_() {

  int number = -999;
  kugeti_( &number);
  ITR_STEERDB_MAJ=number;
  kugeti_( &number);
  ITR_STEERDB_MIN=number;
} 

extern "C" void itrcluster_() {

  int number = 1;
  float thresh = 3.;
  kugeti_( &number);
  if (number==1 || number==2 || number==3)
    ITR_CLUSTER_ALG=number;
  kugetr_( &thresh);
  if (ITR_CLUSTER_ALG==1 || ITR_CLUSTER_ALG==2 || ITR_CLUSTER_ALG==3)
    ITR_THRESHOLD_A=thresh;
  thresh = 5.;
  kugetr_( &thresh);
  if (ITR_CLUSTER_ALG==1 || ITR_CLUSTER_ALG==2 || ITR_CLUSTER_ALG==3)
    ITR_THRESHOLD_B=thresh;    
} 

extern "C" void itrdyn_() {

  int number = 0;
  kugeti_( &number);
  ITR_DYNTHRESH=number;
} 

extern "C" void itrnevtmon_() {

  int number = 200;
  kugeti_( &number);
  ITR_NEVT_MON=number;
} 

extern "C" void itrnoise_() {

  float number = 9.;
  kugetr_( &number);
  ITR_HOTCUT=number;
  number = 25.;
  kugetr_( &number);
  ITR_MAXSIGNAL=number;
}
