*CMZ :  1.00/00 26/11/94  15.25.19  by  Dominik Ressing
*CMZ :  0.01/02 22/08/94  14.17.13  by  Dominik Ressing
*-- Author :    Dominik Ressing   27/07/94
cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
c
c     calculate x,y koordinates at a given z-koordinate for a track, which is given
c     by its 3-momentum and its vertex.
c     z or Vz may not be inside the magnet - Vz may before and z after or vice versa
c     or both on one side.
c
c     INPUTS:
c
c     Px,Py,Pz   3-momentum of the track
c     Vx,Vy,Vz   Vertex of track
c     z          z-position to calculate x,y
c
c     Otputs:
c
c     x,y        point in the x,y plane at z
c
cxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxxx
c
      SUBROUTINE l1kinpo(Px,Py,Pz,Vx,Vy,Vz,z,x,y)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      REAL    Px,Py,Pz,Vx,Vy,Vz,z,x,y,rLeMag,
     &        dz,zM,xM,yM,zMfi,zMla,Pxz,xxM,zzM,rrM,
     &        a,alpha,beta,Pzz,Pxx,Pyy
c
c     --- check input:
      IF ( (Pz.gt.0 .and. z.lt.Vz) .or.
     &     (Pz.lt.0 .and. z.gt.Vz) .or.
     &      Pz.eq.0. ) THEN
         x = -99999.
         y = -99999.
         RETURN
      END IF
      Pxz = sqrt(Px*Px + Pz*Pz )
c
c     --- no pass through the magnet ==> straight line :
      zMla = T1DETmagz + T1DETmagd/2.   ! end of magnet
      zMfi = T1DETmagz - T1DETmagd/2.   ! begin of magnet
      IF (   ((Vz.gt.zMla).and.(Pz.gt.0))
     &  .or. ((Vz.gt.zMla).and.(z.gt.zMla))
     &  .or. ((Vz.lt.zMfi).and.(Pz.lt.0))
     &  .or. ((Vz.lt.zMfi).and.(z.lt.zMfi))) THEN
c
         dz = z - Vz
         x  = Vx + Px/Pz*dz
         y  = Vy + Py/Pz*dz
         RETURN
c
      END IF
c
c     --- extrapolation to center of Magnet
      zM = T1DETmagz-Vz
      xM = Px/Pz*zM+Vx
      yM = Py/Pz*zM+Vy
c
c     --- now get the length of the magnetic field ( in track direction )
      xxM = xM**2.
      zzM = zM**2.
      IF ( abs(xM) .gt. T1DETmagd/2. ) THEN
         rLeMag = 0.
      ELSE
         rrM = (T1DETmagd/2.)**2.
c        --- get rLeMag in meter !!!
         rLeMag = 2.*sqrt(rrM-xxM/(1.+xxM/zzM))/100.
      END IF
c
c     --- alpha is the bending angle in the Magnet / beta the new track slope
c         a = rLeMag/R; R is the bending radius of the track
      a = 0.3*T1DETmagB*rLeMag/Pxz
c     alpha = atan(2./a*(1.-sqrt(1.-a**2))) ! this is the ``correct'' formular
      alpha = atan(a*(1. - (a**2)/4.))      ! this is an approximation of above
      beta  = atan(Px/Pz) + alpha
c
c     --- track momenta behind magnet
      Pzz = sign(Pxz/sqrt(1+tan(beta)**2),Pz) ! maintain sign of original Pz
      Pxx = Pzz*tan(beta)
      Pyy = Py
c     --- distance of point from bending
      dz = z - zM
c     --- extrapolate track as straight line:
      x = xM + dz*Pxx/Pzz
      y = yM + dz*Pyy/Pzz
C
      RETURN
      END
