//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcMatchBuilder class
//
//**********************************************************************/
#include <math.h>
#include <algo.h>


#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

#include "marple/RcPoint.hh"
#include "marple/RcMatchPoint.hh"
#include "marple/RcMatchConst.h"
#include "marple/RcMcFunction.hh"

#include "marple/RcHbook.hh"
#include "marple/RcTuples.h"

//#include "marple/RcMatchBuilder.hh"

template <class PointLeft, class PointRight>
RcMatchBuilder<PointLeft,PointRight >::RcMatchBuilder(  
                                       std::vector<PointLeft *> vecLeft,
				       std::vector<PointRight *> vecRight,
				       RcMatchConst & rcConst)
  :  fvecLeft(), fvecRight(), fmatchList(), fptRcConst(0),
     fNrealPair(0), fNfastReal(0), fNbad(0), fNlost(0)

{
  addLeft(  vecLeft );
  addRight( vecRight);
  setConst( rcConst );
  doMatch();
}

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::doMatch()
{
  if( vectorLeft().size() == 0 || vectorRight().size() == 0 )
    return;  // nothing to match

  //**********************************************************************/
  // create a list of matching points with chi2 
  //**********************************************************************/
  fNrealPair = 0;
  fNfastReal = 0;

  int compareChi2 = 1;
  if( ptToMatchConst()->chi2 > RcMatchConst::maxConsideredChi2 
      || ptToMatchConst()->chi2<=0 )
    compareChi2  = 0;


 if( ptToMatchConst()->use_pair_cut )
   {
     for( unsigned int iLeft=0; iLeft < vectorLeft().size(); ++iLeft)
       vectorLeft()[iLeft]->setmark(ptToMatchConst()->n_pair_left);  
     for( unsigned int iRight=0; iRight < vectorRight().size(); ++iRight)
       vectorRight()[iRight]->setmark(ptToMatchConst()->n_pair_right);
   }

  for( unsigned int iLeft=0; iLeft < vectorLeft().size(); ++iLeft)
    {
      for( unsigned int iRight=0; iRight < vectorRight().size(); ++iRight)
        {
	  if( ArteTable<MTRA>::size() )
	    {
	      ArtePointer<MTRA> imtra = 
		RcMtraPointer( (*vectorLeft()[ iLeft ]).mtraList(), 
			       (*vectorRight()[iRight]).mtraList() );
	      if( imtra != 0 )
		{  
		  fNrealPair++; 	   
		  if( RcMtraReference( imtra ) )
		    fNfastReal++;
		}
	    }

          //*************************************************/
          // check combination 
          int takeCombination = 1;
          // in case of delta_cut is used, save pairs passed delta_cut
          if( (*ptToMatchConst()).use_delta_cut &&
              !(*ptToMatchConst()).delta_cut_ok( *vectorLeft()[  iLeft],
						 *vectorRight()[ iRight]))
                { takeCombination = 0 ; }

          // save combination passed cuts
          if( takeCombination )
            {
	      RcMatchPoint * matchPoint = 
		new RcMatchPoint( *vectorLeft()[iLeft],
				  *vectorRight()[iRight],
				  ptToMatchConst()->flag);
	      if(  (compareChi2 &&   // chi2 is ok
		    matchPoint->chi2() > (*ptToMatchConst()).chi2  ) ||
		   matchPoint->mark() )  // match was ok
		{
                  delete matchPoint;		  
		}else{
		  fmatchList.push_back( matchPoint );
		}
	      matchPoint = 0;
            }
	}
    }	
  
  //**********************************************************************/
  // sort vector of RcMatchPoints by chi2() 
  //**********************************************************************/
  if( compareChi2 )
    sort( fmatchList.begin(), fmatchList.end(), comp_chi2 );

  //**********************************************************************/
  // check the number of combinations with given RcPoints
  //**********************************************************************/

  int goodCombCut = 0;
  int goodFastCut = 0;

  for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
    {
      int checkOk = 1;
      PointLeft* ptLeft   =  leftOrigin(iMatch);
      PointRight* ptRight =  rightOrigin(iMatch);

      if( compareChi2 && (*ptToMatchConst()).use_pair_cut ) 
	{
	  if( !(ptToMatchConst()->pair_cut_ok( *ptLeft  )) ) checkOk = 0 ;
	  if( !(ptToMatchConst()->pair_cut_ok( *ptRight )) ) checkOk = 0 ;
          if( !checkOk )
            { 
  	      delete( fmatchList[iMatch]);
              fmatchList.erase( &fmatchList[iMatch]) ; 
	      iMatch--;
            }else{
	      // mark origin
	      int mark = ptLeft->mark();
	      mark--;
	      ptLeft->setmark(mark );
	      mark = ptRight->mark();
	      mark--;
	      ptRight->setmark(mark );
	    }
	}
      
      // calculate MTRA statistics
      if( checkOk && ArteTable<MTRA>::size() )
	{
	  ArtePointer<MTRA> imtra = RcMtraPointer( ptLeft->mtraList(), 
						   ptRight->mtraList() );
	  if( imtra != 0 )
	    { 
	      goodCombCut++ ; 
	      if ( RcMtraReference(imtra ) )
		goodFastCut++;
	    }     
	}
    }
  fNbad = matchList().size() - goodCombCut;
  fNlost = fNfastReal - goodFastCut ;
  
}

//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::refitLeftFound(int & iErr)
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); ++iMatch)
    {
      RcPoint * ptLeft = leftOrigin(iMatch);
      *ptLeft = *matchList()[iMatch];
      ptLeft->refit( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::refitRightFound(int & iErr)
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      RcPoint * ptRight = rightOrigin(iMatch);
      *ptRight = *matchList()[iMatch];
      ptRight->refit( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}

//--------------------------------------------------------------
template <class PointL, class PointR>
void RcMatchBuilder<PointL,PointR >::updateARTELeftFound(int & iErr) const
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      leftOrigin(iMatch)->updateARTE( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}

template <class PointL, class PointR>
void RcMatchBuilder<PointL,PointR >::updateARTERightFound(int & iErr) const
{
  iErr = 0;
  int iErr1 = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      rightOrigin(iMatch)->updateARTE( iErr1);
      if( iErr1 ) iErr = iErr1;
    }
}
//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::stat() const
{
  if( MARPLE_DEBUG > 0 && MARPLE_DEBUG != 10 && MARPLE_DEBUG != 20 )
    {
      static int first = 1;
      if( first )
	{
	  cout << ptToMatchConst();
	  first = 0;
	}
      stat(MARPLE_DEBUG);
    }
}

//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::stat( int iLong ) const
{
  if( iLong < 0 ) return; 
  unsigned int i;
  iLong = iLong%10;
  switch (iLong)
    {
    case 2 :
      arteMsg << "Left Points :" << vectorLeft().size(); 
      arteMsg.allp();
      for( i=0; i< vectorLeft().size(); i++)
        {  cout << *(vectorLeft()[i]) ; }
      
      arteMsg << "Right Points :" << vectorRight().size(); 
      arteMsg.allp();
      for( i=0; i< vectorRight().size(); i++)
        {  cout << *(vectorRight()[i]) ;  }
      
      arteMsg << "Match Points : "<< matchList().size(); 
      arteMsg.allp();
      for( i=0; i< matchList().size(); i++)
        {  cout << *(matchList()[i]) ;  }

    case 1 :
      cout << "MARPLE: *** constants used for matching ****" << endl 
	   << *ptToMatchConst(); 
      //      arteMsg.allp();

    case 0 :
    default :     
      arteMsg << "*********** result of matching *******";
      arteMsg.allp();
      arteMsg << "Left Points      : " << vectorLeft().size(); 
      arteMsg.allp();
      arteMsg << "Right Points     : " << vectorRight().size(); 
      arteMsg.allp();
      arteMsg << "All pairs with the same MTRA: " << NrealPair(); 
      arteMsg.allp();
      arteMsg << "Reference pairs  : " << NrefPair(); 
      arteMsg.allp();
      arteMsg << "Match Points     : " << matchList().size();
      arteMsg.allp();
      arteMsg << "Bad  pairs       : " << Nbad() ; 
      arteMsg.allp();
      arteMsg << "Lost pairs       : " << Nlost(); 
      arteMsg.allp();
    }
}

extern "C"
{
  void hfntb_(int*, char*, int);
}

//--------------------------------------------------------------
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::fillDeltas(int iNtup,
						       int pulsNtup) const
{
  if( ! FORTRAN_ROUTINE( hexist )(&iNtup) ) 
    {
      int type = 3;  
      FORTRAN_ROUTINE( mamakent )(&iNtup, &type);  
    }

  if( pulsNtup && ! FORTRAN_ROUTINE( hexist )(&pulsNtup) ) 
    {
      int type = 4;
      FORTRAN_ROUTINE( mamakent )(&pulsNtup, &type);  
    }

  for( unsigned int iMatch=0; iMatch < matchList().size(); iMatch++ )
    {
      RcMatchPoint * ptMatchPoint = matchList()[iMatch];
      RcPoint * ptRp1 = leftOrigin(iMatch);
      RcPoint * ptRp2 = rightOrigin(iMatch);

      FORTRAN_COMMON(deltas).chi2    = (*ptMatchPoint).chi2();
      FORTRAN_COMMON(deltas).quality = (*ptMatchPoint).quality();

      // MC information
      if( ArteTable<MTRA>::size() != 0 )
	{	
	  FORTRAN_COMMON(deltas).flag    =
	    RcMtraOk( (*ptRp1).mtraList(), (*ptRp2).mtraList() );
	  int ipart;
	  ArtePointer<MTRA> mtra;
	  if( (ptRp1->mtraList()).size() !=0)
	    {
	      mtra  = (*ptRp1).mtraList()[0];
	      float momentum = sqrt( mtra->px*mtra->px + mtra->py*mtra->py +
				     mtra->pz*mtra->pz );
	      FORTRAN_COMMON(deltas).leftp = momentum;
	      
	      ipart = ArtePointer<TOPP>(mtra->topp)->lund;
	      FORTRAN_COMMON(deltas).ipart1  = ipart;
	      ipart = ArtePointer<TOPP>(mtra->btra->topp)->lund;
	      FORTRAN_COMMON(deltas).imoth1  = ipart;

	    }else{
	      FORTRAN_COMMON(deltas).ipart1  = 0;
	      FORTRAN_COMMON(deltas).imoth1  = 0;
	      FORTRAN_COMMON(deltas).leftp   = 0;
	    }
      
	  if( ((*ptRp2).mtraList()).size() != 0 )
	    {
	      mtra  = (*ptRp2).mtraList()[0];
	      float momentum = sqrt( mtra->px*mtra->px + mtra->py*mtra->py +
				     mtra->pz*mtra->pz );
	      FORTRAN_COMMON(deltas).rightp = momentum;
	      
	      ipart = ArtePointer<TOPP>(mtra->topp)->lund;
	      FORTRAN_COMMON(deltas).ipart2  = ipart;
	      ipart = ArtePointer<TOPP>(mtra->btra->topp)->lund;
	      FORTRAN_COMMON(deltas).imoth2  = ipart;
	      
	    }else{
	      FORTRAN_COMMON(deltas).ipart2  = 0;
	      FORTRAN_COMMON(deltas).imoth2  = 0;
	      FORTRAN_COMMON(deltas).rightp  = 0;
	    }
	}else{
	  FORTRAN_COMMON(deltas).flag    = ptMatchPoint->mark();
	  if( ptRp1->pe() != 0 ) FORTRAN_COMMON(deltas).leftp = 1/ptRp1->pe(); 
	  else                   FORTRAN_COMMON(deltas).leftp = 0;
	  if( ptRp2->pe() != 0 ) FORTRAN_COMMON(deltas).rightp= 1/ptRp2->pe(); 
	  else                   FORTRAN_COMMON(deltas).rightp = 0;
	}

      // deltas
      
      FORTRAN_COMMON(deltas).deltaX  = ptMatchPoint->deltaX() ;
      FORTRAN_COMMON(deltas).deltaY  = ptMatchPoint->deltaY() ;
      FORTRAN_COMMON(deltas).deltaTX = ptMatchPoint->deltaTX() ;
      FORTRAN_COMMON(deltas).deltaTY = ptMatchPoint->deltaTY() ;
      FORTRAN_COMMON(deltas).deltaPE = ptMatchPoint->deltaPE() ;

      
      FORTRAN_COMMON(deltas).matchZ  = ptMatchPoint->z();
      FORTRAN_COMMON(deltas).matchX  = ptMatchPoint->x();
      FORTRAN_COMMON(deltas).matchY  = ptMatchPoint->y();
      FORTRAN_COMMON(deltas).matchTX = ptMatchPoint->tx();
      FORTRAN_COMMON(deltas).matchTY = ptMatchPoint->ty();
      FORTRAN_COMMON(deltas).matchPE = ptMatchPoint->pe();
      
      if( iNtup )
	  FORTRAN_ROUTINE( hfnt)(&iNtup);

      //--------------------------------------------------
      // fill Pulses if specified
      //--------------------------------------------------
      if( pulsNtup == 0 )
	{
	  continue;
	}

      int createTmp1 = 0;
      // here we do the same way as in RcMatchPoint
      if(  fabs(ptRp1->z() - ptRp2->z()) > 0.1e-3 )
        {
          ptRp1 = new RcPoint( *ptRp1);
          ptRp1->prolongate( ptRp2->z() );
          createTmp1 = 1;
        }

      if( ptToMatchConst()->flag.ifmarked_X() )
	{
	  FORTRAN_COMMON(deltas).lpulsX = ( ptMatchPoint->x() - ptRp1->x() )
	    / sqrt((*ptRp1).cov()(1,1));
	  FORTRAN_COMMON(deltas).rpulsX = ( ptMatchPoint->x() - ptRp2->x() )
	    / sqrt((*ptRp2).cov()(1,1));
	}else{
	  FORTRAN_COMMON(deltas).lpulsX = 0;
	  FORTRAN_COMMON(deltas).rpulsX = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_Y() )
	{
	  FORTRAN_COMMON(deltas).lpulsY = ( ptMatchPoint->y() - ptRp1->y() )
	    / sqrt((*ptRp1).cov()(2,2));
	  FORTRAN_COMMON(deltas).rpulsY = ( ptMatchPoint->y() - ptRp2->y() )
	    / sqrt((*ptRp2).cov()(2,2));
	}else{
	  FORTRAN_COMMON(deltas).lpulsY = 0;
	  FORTRAN_COMMON(deltas).rpulsY = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_TX() )
	{
	  FORTRAN_COMMON(deltas).lpulsTX = ( ptMatchPoint->tx() - ptRp1->tx())
	    / sqrt((*ptRp1).cov()(3,3));
	  FORTRAN_COMMON(deltas).rpulsTX = ( ptMatchPoint->tx() - ptRp2->tx())
	    / sqrt((*ptRp2).cov()(3,3));
	}else{
	  FORTRAN_COMMON(deltas).lpulsTX = 0;
	  FORTRAN_COMMON(deltas).rpulsTX = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_TY() )
	{
	  FORTRAN_COMMON(deltas).lpulsTY = ( ptMatchPoint->ty() - ptRp1->ty())
	    / sqrt((*ptRp1).cov()(4,4));
	  FORTRAN_COMMON(deltas).rpulsTY = ( ptMatchPoint->ty() - ptRp2->ty())
	    / sqrt((*ptRp2).cov()(4,4));
	}else{
	  FORTRAN_COMMON(deltas).lpulsTY = 0;
	  FORTRAN_COMMON(deltas).rpulsTY = 0;
	}

      if( ptToMatchConst()->flag.ifmarked_PE() )
	{
	  FORTRAN_COMMON(deltas).lpulsPE = ( ptMatchPoint->pe() - ptRp1->pe())
	    / sqrt((*ptRp1).cov()(5,5));
	  FORTRAN_COMMON(deltas).rpulsPE = ( ptMatchPoint->pe() - ptRp2->pe())
	    / sqrt((*ptRp2).cov()(5,5));
	}else{
	  FORTRAN_COMMON(deltas).lpulsPE = 0;
	  FORTRAN_COMMON(deltas).rpulsPE = 0;
	}
      
      FORTRAN_ROUTINE( hfnt)(&pulsNtup);      
      if( createTmp1){ delete ptRp1; ptRp1= 0; }
      //if( createTmp2){ delete ptRp2; ptRp2= 0; }

    } 
  return;
}

//--------------------------------------------------------------

template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::fillMatch( int iNtup) const
{
  if( ! FORTRAN_ROUTINE( hexist )(&iNtup) ) 
    {
      int type = 1;  
      FORTRAN_ROUTINE( mamakent )(&iNtup, &type);  
    }
  FORTRAN_COMMON(matchnt).Nevt   = (*ArteTable<EVHD>::begin())->evt;
  FORTRAN_COMMON(matchnt).Nleft  = vectorLeft().size();
  FORTRAN_COMMON(matchnt).Nright = vectorRight().size();
  FORTRAN_COMMON(matchnt).Nmatch = matchList().size();
  FORTRAN_COMMON(matchnt).Nreal  = NrealPair();
  FORTRAN_COMMON(matchnt).Nbad   = Nbad();
  FORTRAN_COMMON(matchnt).Nlost  = Nlost() ;
  if( matchList().size() != 0 )
    {
      FORTRAN_COMMON(matchnt).Rbad = float(Nbad()) / float(matchList().size());
    }else{
      FORTRAN_COMMON(matchnt).Rbad = 0;
    }
  if( NrefPair() != 0)
    {
      FORTRAN_COMMON(matchnt).Rlost = float( Nlost()) / float( NrefPair());
    }else{
      FORTRAN_COMMON(matchnt).Rlost = 0;
    }
  if( iNtup )  FORTRAN_ROUTINE( hfnt)(&iNtup); 
}

//********************************************************/
// save information into ARTE banks
//********************************************************/

//template <class PointLeft, class PointRight>
//void RcMatchBuilder<PointLeft,PointRight >::saveRTRA(int & iErr, 
//						     int withRefit = 1)
//const
//{
/*
  int iErr = 0;
  for( unsigned int iMatch=0; iMatch< matchList().size(); iMatch++)
    {
      RcPoint * ptLeft  = leftOrigin(iMatch);
      RcPoint * ptRight  = rightOrigin(iMatch);
      if( withRefit )
	{
	  *ptLeft = *matchList()[iMatch];
	  ptLeft->refit( iErr1);
	  if( iErr1 ) iErr = iErr1;

	  *ptRight = *matchList()[iMatch];
	  ptRight->refit( iErr1);
	  if( iErr1 ) iErr = iErr1;
	}
      RcPoint * ptSmaller;
      RcPoint * ptLarger;
      if( ptLeft->z() < ptRight->z() )
	{
	  ptSmaller = ptLeft;
	  ptLarger  = ptRight;
	}
      else
	{
	  ptSmaller = ptRight;
	  ptLarger  = ptLeft;
	}
      ArtePointer<RTRA> rtra;
      rtra->xf  = ptSmaller->x();
      rtra->yf  = ptSmaller->y() ;
      rtra->zf  = ptSmaller->z() ;
      rtra->txf = ptSmaller->tx() ;
      rtra->tyf = ptSmaller->ty();
      rtra->pf  = ptSmaller->pe();
      for( int i=0; i<5; i++){
	for(int j=i; j<5; j++){
	  rtra->cvf[c_matrix_pack[i][j]] = ptSmaller->cov()[i][j]; 
	}
      }
      std::vector<ArtePointer<HITB> > hitbList = 
                                      matchList()[iMatch]->hitbList();
      int hsi = 0, hin = 0, hou = 0, hri = 0, hca = 0, htr = 0, hmu = 0;
      for( unsigned ihit = 0; ihit < hitbList.size(); ihit++ )
	{
	  switch (ArtePointer<GEDE>(hitbList[ihit]->gede)->cmp)
	    {
	    case 1 :
	      hsi++;
	    case 5 :
	      hri++;
	    case 25 :
	      hmu++;
	    }
	}
      

    }
*/  
//}
                  
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRSEG(int & iErr ) const
{
  iErr = 0;
  for( unsigned int iMatch=0; iMatch < fmatchList.size(); iMatch++ )
    {
      RSEG rseg;
      RcMatchPoint * matchPoint = fmatchList[iMatch];
      rseg.zf  = matchPoint->z();
      rseg.xf  = matchPoint->x();
      rseg.yf  = matchPoint->y();
      rseg.txf = matchPoint->tx();
      rseg.tyf = matchPoint->ty();
      rseg.pf  = matchPoint->pe();

      for( unsigned int i = 0; i < 5 ; ++i )
	for( unsigned int j = i; j < 5 ; ++j )
	  rseg.cvf[c_matrix_pack[i][j]] = matchPoint->cov()(i+1,j+1);
      
      rseg.chi   = matchPoint->chi2();
      rseg.fit   = 7;
      rseg.time  = 888888;
      rseg.dtim  = matchPoint->mark();

      PointLeft* ptLeft   =  leftOrigin(iMatch);
      PointRight* ptRight =  rightOrigin(iMatch);
      rseg.cmp   = ptLeft->link()->cmp | ptRight->link()->cmp;

      rseg.xe  = matchPoint->deltaX();
      rseg.ye  = matchPoint->deltaY();
      rseg.txe = matchPoint->deltaTX();
      rseg.tye = matchPoint->deltaTY();
      rseg.pe  = matchPoint->deltaPE();

      for( unsigned int i = 0; i < 15 ; ++i )
	rseg.cve[i] = 0;
      

      ArtePointer<RSEG> Rseg=ArteTable<RSEG>::push_back(rseg);
      int iErr1 = 0;
      saveRelationWithMatch( ptLeft, ptRight, Rseg, iErr1 );
      if( iErr1 ) iErr = iErr1;

      ptLeft = 0;
      ptRight = 0;
      matchPoint = 0;
    } 
}

/*
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRCCL() const
{}
*/
template <class PointLeft, class PointRight>
void RcMatchBuilder<PointLeft,PointRight >::saveRelation( int & iErr) const
{
  if( !matchList().size() ){ iErr = 0; return; }
  iErr = 1;
  for(unsigned int iMatch=0; iMatch < matchList().size(); iMatch++)
    {      
      int iErr1=1;
      saveRelation(leftOrigin(iMatch),
		   rightOrigin(iMatch), 
		   iErr1); 
      if( !iErr1 ){ iErr = 0; }
    }
}

