//---------------------------------------------------------------------//
//         matching of OTR segments with RICH pseudo-segments          //
//         This routine  stores relations                              //
//         RSEG_RBEG between the two segments found                    //
//---------------------------------------------------------------------//

    
#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"
#include "arte/RSEG_cmp.hh"
#include "arte/arte.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchOtrRich = 0;
#endif


RcMatchConst rcOTRRichConst = 
  {
    110 /* = flag: use only tx, ty info */, 
    1000. /*= Z value of plane */ , 0. /*= zmin*/ , 1500. /*zmax */ ,
    0 /*use pair cut */,2 /* =n pair left */ ,1	/*= n pair right */,
    1 /*= use delta cut */, 
    1E+10 /* = delta x */ ,     1E+10 /* = delta y */ ,
    0.0075 /*=delta tx*/,       0.05, // 0.01 /*=delta ty */ ,
    1E+10 /* = delta_pe */,
    1500 /* = chi2 */
  };

// soft cuts to search correlation  
RcMatchConst rcSoftORConst = 
  {
    110 /* = flag: use only tx, ty info */, 
    1000. /*= Z value of plane */ , 0. /*= zmin*/ , 1500. /*zmax */ ,
    0 /*use pair cut */,2 /* =n pair left */ ,1	/*= n pair right */,
    1 /*= use delta cut */, 
    1E+10 /* = delta x */ ,   1E+10 /* = delta y */ ,
    0.05 /*=delta tx*/,      0.1, // 0.01 /*=delta ty */ ,
    1E+10 /* = delta_pe */,
    100000 /* = chi2 */
  };
  

extern "C"
void FORTRAN_ROUTINE(matchotrrich) ()
{
# ifdef MARPLEPERFORMANCES
  if(chMatchOtrRich == 0){
    ChronoManager CM;
    chMatchOtrRich = CM.createChrono("match Otr Rich");
  }
  chMatchOtrRich->start();
# endif

  RcMatchBuilder<RcSegPoint, RcSegPoint>  OTRRichMatch;

  //------------------------------------------------------------------
  // put the defined constants in this builder:
  //------------------------------------------------------------------
  
  if( MARPLE_SOFT )
    OTRRichMatch.setConst(rcSoftORConst);
  else
    OTRRichMatch.setConst(rcOTRRichConst);


  int iErr; 

  //------------------------------------------------------------------
  // create two vectors: one for the OTR (RSEG) segments and
  //  one for the (quasi-)segments in the RICH
  // first the OTR segment - vector:
  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
       irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if(  (*irseg)->cmp & Rsegc::patt )
	{
	  RcSegPoint * MyptToPoint = new RcSegPoint( *irseg , 1); // endPoint
	  MyptToPoint->prolongate( rcOTRRichConst.z_plane);
	  OTRRichMatch.addPointLeft (*MyptToPoint);
	  MyptToPoint = NULL;
	}
    }
  if( OTRRichMatch.vectorLeft().size() == 0 )
    {
# ifdef MARPLEPERFORMANCES
    chMatchOtrRich->stop();
# endif
      return;
    }
  
  //------------------------------------------------------------------
  // secondly the quasi-segments from the rich
  //------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator richsegment = ArteTable<RSEG>::begin();
       richsegment != ArteTable<RSEG>::end(); ++richsegment )
    {
      if( ((*richsegment)->cmp & Rsegc::bitsign) != 0 ) continue;
      if( (*richsegment)->cmp == Rsegc::rich  )
	 // (*richsegment)->ze/sqrt((*richsegment)->ye) > 5 )
	{
	  RcSegPoint *MyptToPoint2 = new RcSegPoint (*richsegment ,0 );
	  MyptToPoint2->setZ( rcOTRRichConst.z_plane);
	  OTRRichMatch.addPointRight (*MyptToPoint2);
	  MyptToPoint2 = NULL;
	}
    }

  //------------------------------------------------------------------
  // go for it:
  //------------------------------------------------------------------
  OTRRichMatch.doMatch();
  
  
  // and estimate momentum just for case
  for( unsigned int iMatch = 0; 
       iMatch < OTRRichMatch.matchList().size(); iMatch++ )
    {
      RcSegPoint * otr  = OTRRichMatch.leftOrigin(iMatch);
      RcPoint * matchPt = OTRRichMatch.matchList()[iMatch];
      if( fabs(otr->link()->pf )>0.e-5 )
	{
	  matchPt->setpe( otr->link()->pf );
	  matchPt->setcov(5,5, otr->link()->cvf[c_matrix_pack[4][4]] );
	  continue;
	}
      RcPoint tmp = *otr ;

      static float bp = 288;
      float overMomentum = (otr->z()*otr->tx() - otr->x())/bp;
      // in principle we should correct it by factor 
      // sqrt((1+tx**2)/(1+tx**2+ty**2)) 
      // but it does not make big difference since the angles are small

      if( fabs( overMomentum ) > 2 ) // Pt < 0.5 GeV
	{
	  continue;
	}

      tmp.setpe( overMomentum );

      if ( fabs(tmp.pe()) < 1 )
	{
	  tmp.estimatePEfromTAR(6, iErr);
	}else{
	  if( tmp.pe() < 0 ) tmp.setpe(-1);
	  else               tmp.setpe(1);
	}
      matchPt->setpe( tmp.pe() );
      matchPt->setcov(5,5, tmp.cov()(5,5) );
    }


  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------
  OTRRichMatch.stat();
    
      
  if( MARPLE_DEBUG >= 20)
    {
      OTRRichMatch.fillMatch(8134);
      OTRRichMatch.fillDeltas(8234, 8334);
    }else if ( MARPLE_DEBUG >= 10 ){
      OTRRichMatch.fillDeltas(8234);
    }



  //------------------------------------------------------------------
  // save the relations to the RSEG table:
  //------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) OTRRichMatch.saveRSEG(iErr);

  OTRRichMatch.saveRelation(iErr);
  if(iErr)
    {
      cout << "MATCHOTRICH: not all relations were saved. " << endl;
    }
 
  OTRRichMatch.deleteAll();
# ifdef MARPLEPERFORMANCES
    chMatchOtrRich->stop();
# endif
}

