//--------------------------------------------------------------------
// Created: 02.09.97 by   Olya Igonkina

//   MARPLE matching of SVD-ECAL segments and segments reconstructed 
//   in the Tracker area behind magnet(OT)
//   with estimation of momentum information 
//   by requirement of segment point to target

//--------------------------------------------------------------------
  

//--------------------------------------------------------------------

#include "marple/marple.hh"
#include "marple/RcMatchBuilder.hh"

#include "arte/ArteMessage.hh"
#include "arte/RSEG_cmp.hh"

#ifdef MARPLEPERFORMANCES
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
Chrono * chMatchSeOtr = 0;
#endif

//definition of constants for SE-OT matching ( see \Ref{RcMatchConst})
// flag, z_plane, z_min, z_max, use_pair_cut, n_pair_left, n_pair_right, 
// use_delta_cut, delta_x, delta_y, delta_tx, delta_ty, delta_pe, chi2, 
RcMatchConst rcSeOtConst = { 
11110,     700.,   -30.,  800.,      0,          1,            1,
    1,      1.,      15.,      0.005,      .025,   1.e+10,   1000.,  
};

// MC RcMatchConst rcSeOtConst = { 
//11110,     160.,   30.,  800.,      1,          1,            1,
//    1,      3.,      5.,      0.01,      .01,   1.e+10,   100.,  
//};

RcMatchConst rcSoftSeOtConst = { 
11110,     700.,   30.,  800.,      0,          1,            1,
    1,      35.,      35.,      0.5,      0.5,   1.e+10,   1000000.,  
};


extern "C"  
void FORTRAN_ROUTINE(matchseotr)() 
{

# ifdef MARPLEPERFORMANCES
  if(chMatchSeOtr == 0){
    ChronoManager CM;
    chMatchSeOtr = CM.createChrono("Se-Otr match");
  }
  chMatchSeOtr->start();
# endif
  

  RcMatchBuilder< RcSegPoint, RcSegPoint> SeOtMatch;

  //--------------------------------------------------------------------
  // put constants to the MatchBuilder
  //--------------------------------------------------------------------
 if( MARPLE_SOFT )
   SeOtMatch.setConst(rcSoftSeOtConst);
  else
    SeOtMatch.setConst(rcSeOtConst);

  //--------------------------------------------------------------------
  // create vector of SE segments
  //--------------------------------------------------------------------
  
  int iErr;
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if ( (*irseg)->fit != 7 ) continue; // marple segment
      if ( ( (*irseg)->cmp & Rsegc::vxd ) == 0 || 
           ( (*irseg)->cmp & Rsegc::ecal) == 0 ) continue;      

      ArtePointer<RSEG> svd = *((*irseg)->rend.begin());
      if( !(svd->cmp & Rsegc::vxd ) ) continue;
      ArtePointer<RCCL> ecal = *((*irseg)->rccl.begin());
      if ( ecal == 0 ) continue;
      //float pt = ecal->esum*sqrt(ecal->x*ecal->x+ecal->y*ecal->y)/1350;
      //      if ( pt < .8 ) continue;  // pt cut

      //RcSegPoint * ptToPoint = new RcSegPoint( svd , 1 ); // end point
      RcSegPoint * ptToPoint = new RcSegPoint( svd , 0 ); // first point
      ptToPoint->setpe((*irseg)->pf);
      ptToPoint->prolongate( SeOtMatch.ptToMatchConst()->z_plane,
			     iErr);
      if( iErr )
	{
	  delete ptToPoint;
	}else{
	  SeOtMatch.addPointLeft( *ptToPoint);
	}
      ptToPoint = 0;
    }
  

  // no candidate found --- no reason go through OTR segments
  if( SeOtMatch.vectorLeft().size() == 0 ) 
    {
# ifdef MARPLEPERFORMANCES
      chMatchSeOtr->stop();
# endif
      return;
    }
  //--------------------------------------------------------------------
  // create vector of Outer Tracker segments 
  //--------------------------------------------------------------------
  
  for( ArteTable<RSEG>::iterator irseg = ArteTable<RSEG>::begin();
        irseg != ArteTable<RSEG>::end(); ++irseg )
    {
      if ( (*irseg)->fit != 0 ) continue;
      if( ((*irseg)->cmp & Rsegc::bitsign) != 0 ) continue;
      if(  (*irseg)->cmp & Rsegc::patt && 
	   SeOtMatch.ptToMatchConst()->geomAccept(HepDouble((*irseg)->zf)) )
	{
	  iErr = 0;
	  RcSegPoint * ptToPoint = new RcSegPoint( (*irseg), 0 ); //first point
	  /*
	  if( fabs(ptToPoint->pe() )> 1 || fabs(ptToPoint->pe() )<0.0001 )
	    {
	      // initial value from kick :
	      float overMomentum = 
		( ptToPoint->tx()*ptToPoint->z() - ptToPoint->x() ) / 288;
	      if( fabs(overMomentum) >  2 ) // Pt < 0.5 GeV
		{
		  iErr = 1;
		}else{
		  ptToPoint->setpe( overMomentum);
		  ptToPoint->estimatePEfromTAR( 5, iErr);
		}
	    }else if( fabs(ptToPoint->pe() )> 2 ){
	      iErr = 1;
	    }

	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }

	  ptToPoint->prolongate( SeOtMatch.ptToMatchConst()->z_plane, iErr);
	  if( iErr ){ delete ptToPoint; ptToPoint = 0; continue; }
	  */

	  SeOtMatch.addPointRight( *ptToPoint);
	  ptToPoint = 0;	      
	}
    }  
  
  //--------------------------------------------------------------------
  // create a list of matching points with chi2 
  //--------------------------------------------------------------------

  SeOtMatch.doMatch();

  //--------------------------------------------------------------------
  // fill OTR estimated momentum to matchPoint
  //--------------------------------------------------------------------
  unsigned int nCombinations = SeOtMatch.matchList().size();
  for( unsigned int iMatch = 0;  iMatch<nCombinations; ++iMatch)
    {
      SeOtMatch.matchList()[iMatch]->
	setpe( SeOtMatch.rightOrigin(iMatch)->pe() );
    }
  
  //--------------------------------------------------------------------
  // print result of matching
  //--------------------------------------------------------------------

  SeOtMatch.stat();
    
  
  if( MARPLE_DEBUG >= 20)
    {  
      SeOtMatch.fillDeltas(8213,8313);
      SeOtMatch.fillMatch( 8113);
    }else if( MARPLE_DEBUG >= 10){
      SeOtMatch.fillDeltas(8213);
    }
  //--------------------------------------------------------------------
  // save found relation to the table
  //--------------------------------------------------------------------
  if( MARPLE_MAKERSEG ) SeOtMatch.saveRSEG(iErr);
  
  SeOtMatch.saveRelation(iErr);
  if( iErr )
    { arteMsg << "MARPLE: some relations were not saved "; arteMsg.allp();}
  
  //    refit information
  //SeOtMatch.refitLeftFound(       iErr);
  //SeOtMatch.refitRightFound(      iErr);
  //    and update RSEG tables
  //SeOtMatch.updateARTELeftFound(  iErr);
  //SeOtMatch.updateARTERightFound( iErr);

  //--------------------------------------------------------------------
  // clean everything
  //--------------------------------------------------------------------
  SeOtMatch.deleteAll();  

# ifdef MARPLEPERFORMANCES
  chMatchSeOtr->stop();
# endif
}

