
// Boris Fominykh  10.02.2000
// initialization of muon detector reconstruction
// initialization = alignment correction + initialization of digi2hitb
// program has to be called after read of first event
// (keytables and run number are needed)
//
// input parameter KTstatus is not used
//
// return code      2  -  data updated from DB using the run number
//                  1  -  data updated from DB using keytable
//                  0  -  no actions because data are uptodate 
//                 -1  -  data updated from ASCII file
//                 -2  -  error
//
// program returns  10*retal + retdh
//           where  retal  - error code for alignment correction
//           where  retdh  - error code for digi2hitb initialization

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "database/keytable.h"

static int first1 =0;
static int first2 =0;

extern "C" {
  void getrun_();
  int MuAlignment(int,int,char*);
  int MuD2Hini(int,int,char*);

int acReadMuon(int KTstatus) {
//  ========================

  char* DBname="/MUON_DB";

// check that it is not MC
  if(ArteTable<MTRA>::size() <= 0) {

// make alignment corrections
    int retal=-2;
// ASCII file has the highest priority 
    if(getenv("MUON_ALIGN") != NULL){

// make alignment corrections only one time
      if (first1 == 0){
        first1 = 1;
        int ret=MuAlignment(-3,0,DBname);
        if(ret == 2) retal=-1;
        else         retal=-2;
      }
// correction is not needed
      else retal=0;
    }

// if there is no ASCII file or the error took place let's read DB
    if(retal == -2) {

// keytable has larger priority
      int MUserver = KeyFindServer(DBname);
      if(MUserver > 0) {
        int MUtable = KeyFindTable(MUserver,"MuonAlignment");
        if(MUtable > 0) {
// table was updated
          int MUnew = KeyTableNew(MUtable);
          if(MUnew > 0) {
            int ret=MuAlignment(2,0,DBname);
            if(ret == 1) retal=1;
            else         retal=-2;
          }
// table was not updated - no action
          else retal=0;
        }
      }          
// use run number if there is a problem with keytable 
      if(retal == -2) {
        if(first1 == 1) retal=0;
// get the run number
        int run=0;
        getrun_(&run);
        if(first1 == 0 && run > 0) {
            first1=1;
            int ret=MuAlignment(-2,run,DBname);
            if(ret == 2) retal=2;
            else         retal=-2;
        }
      }
    }


// initialization of digi2hitb
    int retdh=-2;
// two ASCII files have the highest priority 
    if((getenv("MUON_BADFILE") != NULL) &&
       (getenv("MUON_CONFIG")  != NULL)) {

// initialize only one time
      if (first2 == 0){
        first2 = 1;
        int ret=MuD2Hini(-3,0,DBname);
        if(ret == 2) retdh=-1;
        else         retdh=-2;
      }
// correction is not needed
      else retdh=0;
    }

// if there are no ASCII files (at least one of them)
// or the error took place let's read DB
    if(retdh == -2) {

// keytable has larger priority
      int MUserver = KeyFindServer(DBname);
      if(MUserver > 0) {
// check that both table are here
        int MUtab1 = KeyFindTable(MUserver,"MuonConfig");
        int MUtab2 = KeyFindTable(MUserver,"MuonHots");
        if(MUtab1 > 0 && MUtab2 > 0) {
// at least one of two tables has to be updated
          int MUnew1=KeyTableNew(MUtab1);
          int MUnew2=KeyTableNew(MUtab2);
          if(MUnew1 > 0 || MUnew2 > 0) {
            int ret=MuD2Hini(2,0,DBname);
            if(ret == 1) retdh=1;
            else         retdh=-2;
          }
// both tables were not updated - no actions
          else retdh=0;
        }
      }          
// use run number if there is problem with keytable 
      if(retdh == -2) {
        if(first2 == 1) retdh=0;
// get the run number
        int run=0;
        getrun_(&run);
        if(first2 == 0 && run > 0) {
            first2=1;
            int ret=MuD2Hini(-2,run,DBname);
            if(ret == 2) retdh=2;
            else         retdh=-2;
        }
      }
    }
    return 10*retal+retdh;
  }
// the case of MC
  return 0;
}
}
