/* Usage:
   
   bind<i>(f, (Arg1*)0, ..., arg<i>, (Arg<i+1>*)0, ..., (Arg<n>*)0);

   where 1 <= <n> <= 5, <i> <= <n>. (E.g. bind1(...), bind2(...), ...).
   F f - some function object which takes <n> arguments and
   expression f(a1, .., aN) 
   (a1, .., aN are of types Arg1, .., ArgN respectively) 
   is valid.
   bind<i> returns the function object which has <n>-1 arguments of types
   Arg1, .., Arg<i-1>, Arg<i+1>,... Arg<n>;
   the <i>'th argument is fixed to arg<i>.

   The version bind1(...) works with up to 10 arguments.
*/

// Author V.Balagura (balagura@mail.desy.de)   (09.04.98).

#ifndef _binder_hh
#define _binder_hh 1

/* --------------------  bind 1st, 1 arg -------------------- */
template <class F, class A1>
class Binder_1st_1arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_1arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()() { function(arg1); }
};
template <class F, class A1>
inline Binder_1st_1arg<F, A1> bind1(F f, A1 a1) { 
  return Binder_1st_1arg<F, A1>(f, a1); 
}
/* --------------------  bind 1st, 2 arg -------------------- */
template <class F, class A1, class A2>
class Binder_1st_2arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_2arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2) { function(arg1, a2); }
};
template <class F, class A1, class A2>
inline Binder_1st_2arg<F, A1, A2> bind1(F f, A1 a1, A2*) { 
  return Binder_1st_2arg<F, A1, A2>(f, a1); 
}
/* --------------------  bind 1st, 3 arg -------------------- */
template <class F, class A1, class A2, class A3>
class Binder_1st_3arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_3arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3) { function(arg1, a2, a3); }
};
template <class F, class A1, class A2, class A3>
inline Binder_1st_3arg<F, A1, A2, A3> bind1(F f, A1 a1, A2*, A3*) { 
  return Binder_1st_3arg<F, A1, A2, A3>(f, a1); 
}
/* --------------------  bind 1st, 4 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4>
class Binder_1st_4arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_4arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4) { function(arg1, a2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline Binder_1st_4arg<F, A1, A2, A3, A4> bind1(F f, A1 a1, A2*, A3*, A4*) { 
  return Binder_1st_4arg<F, A1, A2, A3, A4>(f, a1); 
}
/* --------------------  bind 1st, 5 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class Binder_1st_5arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_5arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4, A5 a5) { 
    function(arg1, a2, a3, a4, a5); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline Binder_1st_5arg<F, A1, A2, A3, A4, A5> 
bind1(F f, A1 a1, A2*, A3*, A4*, A5*) { 
  return Binder_1st_5arg<F, A1, A2, A3, A4, A5>(f, a1); 
}
/* --------------------  bind 1st, 6 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5, class A6>
class Binder_1st_6arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_6arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4, A5 a5, A6 a6) { 
    function(arg1, a2, a3, a4, a5, a6); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5, class A6>
inline Binder_1st_6arg<F, A1, A2, A3, A4, A5, A6> 
bind1(F f, A1 a1, A2*, A3*, A4*, A5*, A6*) { 
  return Binder_1st_6arg<F, A1, A2, A3, A4, A5, A6>(f, a1); 
}
/* --------------------  bind 1st, 7 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7>
class Binder_1st_7arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_7arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7) { 
    function(arg1, a2, a3, a4, a5, a6, a7); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7>
inline Binder_1st_7arg<F, A1, A2, A3, A4, A5, A6, A7> 
bind1(F f, A1 a1, A2*, A3*, A4*, A5*, A6*, A7*) { 
  return Binder_1st_7arg<F, A1, A2, A3, A4, A5, A6, A7>(f, a1); 
}
/* --------------------  bind 1st, 8 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7, class A8>
class Binder_1st_8arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_8arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8) { 
    function(arg1, a2, a3, a4, a5, a6, a7, a8); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7, class A8>
inline Binder_1st_8arg<F, A1, A2, A3, A4, A5, A6, A7, A8> 
bind1(F f, A1 a1, A2*, A3*, A4*, A5*, A6*, A7*, A8*) { 
  return Binder_1st_8arg<F, A1, A2, A3, A4, A5, A6, A7, A8>(f, a1); 
}
/* --------------------  bind 1st, 9 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7, class A8, class A9>
class Binder_1st_9arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_9arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8, A9 a9) { 
    function(arg1, a2, a3, a4, a5, a6, a7, a8, a9); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7, class A8, class A9>
inline Binder_1st_9arg<F, A1, A2, A3, A4, A5, A6, A7, A8, A9> 
bind1(F f, A1 a1, A2*, A3*, A4*, A5*, A6*, A7*, A8*, A9*) { 
  return Binder_1st_9arg<F, A1, A2, A3, A4, A5, A6, A7, A8, A9>(f, a1); 
}
/* --------------------  bind 1st, 10 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7, class A8, class A9, class A10>
class Binder_1st_10arg {
protected:
  A1 arg1;
public:
  F function;

  Binder_1st_10arg(F f, A1 a1) : arg1(a1), function(f) {}
  void operator()(A2 a2, A3 a3, A4 a4, A5 a5, A6 a6, A7 a7, A8 a8, A9 a9, 
		  A10 a10) { 
    function(arg1, a2, a3, a4, a5, a6, a7, a8, a9, a10); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5, class A6,
  class A7, class A8, class A9, class A10>
inline Binder_1st_10arg<F, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10> 
bind1(F f, A1 a1, A2*, A3*, A4*, A5*, A6*, A7*, A8*, A9*, A10*) { 
  return Binder_1st_10arg<F, A1, A2, A3, A4, A5, A6, A7, A8, A9, A10>(f, a1); 
}

/* --------------------  bind 2nd -------------------- */
/* --------------------  bind 2nd, 2 arg -------------------- */
template <class F, class A1, class A2>
class Binder_2nd_2arg {
protected:
  A2 arg2;
public:
  F function;

  Binder_2nd_2arg(F f, A2 a2) : arg2(a2), function(f) {}
  void operator()(A1 a1) { function(a1, arg2); }
};
template <class F, class A1, class A2>
inline Binder_2nd_2arg<F, A1, A2> bind2(F f, A1*, A2 a2) { 
  return Binder_2nd_2arg<F, A1, A2>(f, a2); 
}
/* --------------------  bind 2nd, 3 arg -------------------- */
template <class F, class A1, class A2, class A3>
class Binder_2nd_3arg {
protected:
  A2 arg2;
public:
  F function;

  Binder_2nd_3arg(F f, A2 a2) : arg2(a2), function(f) {}
  void operator()(A1 a1, A3 a3) { function(a1, arg2, a3); }
};
template <class F, class A1, class A2, class A3>
inline Binder_2nd_3arg<F, A1, A2, A3> bind2(F f, A1*, A2 a2, A3*) { 
  return Binder_2nd_3arg<F, A1, A2, A3>(f, a2); 
}
/* --------------------  bind 2nd, 4 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4>
class Binder_2nd_4arg {
protected:
  A2 arg2;
public:
  F function;

  Binder_2nd_4arg(F f, A2 a2) : arg2(a2), function(f) {}
  void operator()(A1 a1, A3 a3, A4 a4) { function(a1, arg2, a3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline Binder_2nd_4arg<F, A1, A2, A3, A4> bind2(F f, A1*, A2 a2, A3*, A4*) { 
  return Binder_2nd_4arg<F, A1, A2, A3, A4>(f, a2); 
}
/* --------------------  bind 2nd, 5 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class Binder_2nd_5arg {
protected:
  A2 arg2;
public:
  F function;

  Binder_2nd_5arg(F f, A2 a2) : arg2(a2), function(f) {}
  void operator()(A1 a1, A3 a3, A4 a4, A5 a5) { 
    function(a1, arg2, a3, a4, a5); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline Binder_2nd_5arg<F, A1, A2, A3, A4, A5> 
bind2(F f, A1*, A2 a2, A3*, A4*, A5*) { 
  return Binder_2nd_5arg<F, A1, A2, A3, A4, A5>(f, a2); 
}

/* --------------------  bind 3d -------------------- */
/* --------------------  bind 3d, 3 arg -------------------- */
template <class F, class A1, class A2, class A3>
class Binder_3d_3arg {
protected:
  A3 arg3;
public:
  F function;

  Binder_3d_3arg(F f, A3 a3) : arg3(a3), function(f) {}
  void operator()(A1 a1, A2 a2) { function(a1, a2, arg3); }
};
template <class F, class A1, class A2, class A3>
inline Binder_3d_3arg<F, A1, A2, A3> bind3(F f, A1*, A2*, A3 a3) { 
  return Binder_3d_3arg<F, A1, A2, A3>(f, a3); 
}
/* --------------------  bind 3d, 4 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4>
class Binder_3d_4arg {
protected:
  A3 arg3;
public:
  F function;

  Binder_3d_4arg(F f, A3 a3) : arg3(a3), function(f) {}
  void operator()(A1 a1, A2 a2, A4 a4) { function(a1, a2, arg3, a4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline Binder_3d_4arg<F, A1, A2, A3, A4> bind3(F f, A1*, A2*, A3 a3, A4*) { 
  return Binder_3d_4arg<F, A1, A2, A3, A4>(f, a3); 
}
/* --------------------  bind 3d, 5 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class Binder_3d_5arg {
protected:
  A3 arg3;
public:
  F function;

  Binder_3d_5arg(F f, A3 a3) : arg3(a3), function(f) {}
  void operator()(A1 a1, A2 a2, A4 a4, A5 a5) { 
    function(a1, a2, arg3, a4, a5); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline Binder_3d_5arg<F, A1, A2, A3, A4, A5> 
bind3(F f, A1*, A2*, A3 a3, A4*, A5*) { 
  return Binder_3d_5arg<F, A1, A2, A3, A4, A5>(f, a3); 
}

/* --------------------  bind 4th -------------------- */
/* --------------------  bind 4th, 4 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4>
class Binder_4th_4arg {
protected:
  A4 arg4;
public:
  F function;

  Binder_4th_4arg(F f, A4 a4) : arg4(a4), function(f) {}
  void operator()(A1 a1, A2 a2, A3 a3) { function(a1, a2, a3, arg4); }
};
template <class F, class A1, class A2, class A3, class A4>
inline Binder_4th_4arg<F, A1, A2, A3, A4> bind4(F f, A1*, A2*, A3*, A4 a4) { 
  return Binder_4th_4arg<F, A1, A2, A3, A4>(f, a4); 
}
/* --------------------  bind 4th, 5 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class Binder_4th_5arg {
protected:
  A4 arg4;
public:
  F function;

  Binder_4th_5arg(F f, A4 a4) : arg4(a4), function(f) {}
  void operator()(A1 a1, A2 a2, A3 a3, A5 a5) { 
    function(a1, a2, a3, arg4, a5); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline Binder_4th_5arg<F, A1, A2, A3, A4, A5> 
bind4(F f, A1*, A2*, A3*, A4 a4, A5*) { 
  return Binder_4th_5arg<F, A1, A2, A3, A4, A5>(f, a4); 
}

/* --------------------  bind 5th -------------------- */
/* --------------------  bind 5th, 5 arg -------------------- */
template <class F, class A1, class A2, class A3, class A4, class A5>
class Binder_5th_5arg {
protected:
  A5 arg5;
public:
  F function;

  Binder_5th_5arg(F f, A5 a5) : arg5(a5), function(f) {}
  void operator()(A1 a1, A2 a2, A3 a3, A4 a4) { 
    function(a1, a2, a3, a4, arg5); 
  }
};
template <class F, class A1, class A2, class A3, class A4, class A5>
inline Binder_5th_5arg<F, A1, A2, A3, A4, A5> 
bind5(F f, A1*, A2*, A3*, A4*, A5 a5) { 
  return Binder_5th_5arg<F, A1, A2, A3, A4, A5>(f, a5); 
}

#endif
