// Classes defined here allows
// 1) to create "menu" which marks cells as BAD/NOISY.
// It allows to read "hit-map" high-pt files.
// Typical usage can be found in HPT_DAQ_Abstract.C
//
// 2) To write the "hit-map" file in the same format one can use
// write_cell_states(Detector& det, const string& setup_name, ostream& os);
// 

// Author: Vladislav Balagura (balagura@hera-b.desy.de, 03.11.1999)

#ifndef _io_cell_states_hh
#define _io_cell_states_hh 1

#include "ptset/detector.hh"
#include "ptset/high_pt.hh"
#include "ptset/delimit.hh"
#include <iostream>


template <class AnyISection>
class BAD_NOISY_Cells_Marker {
  Fired_Cell::State state;
public:
  BAD_NOISY_Cells_Marker(Fired_Cell::State fState) : state(fState) {
    if ( state != Fired_Cell::BAD && state != Fired_Cell::NOISY ) {
      string tmp;
      tmp << "Wrong state is given: " << int(state) 
	  << " (neither BAD nor NOISY)";
      global_error()(tmp);
      global_error() << "BAD_NOISY_Cells_Marker: program error\n";
    }
  }
  void operator()(istream* is, AnyISection isection) {
    Index ind;
    typedef typename iterator_traits<AnyISection>::value_type AnySection;
    AnySection& section = (*isection);
    typedef typename AnySection::ICell AnyICell;
    AnyICell icell;
    while (1) {
      *is >> ind;
      if ( is->eof() ) return;
      else if ( !(*is) ) { global_error()("can not read Index"); return; }
      if ( (icell = section.find(ind)) == section.end() ) {
	string tmp;
	tmp << "there is no cell with Index " << ind;
	global_error()(tmp);
	return;
      }
      (*icell).second.state = state;
    }	
  }
};

class Write_Cell_States {
  ostream& os;
public:
  Write_Cell_States(ostream& fOs) : os(fOs) {}

  /*
  template <class CellT, SectionT, ChamberTSetupT>
  void operator()(Setup<...>) {...}
  */
  void operator()(HPT_Setup& setup);
};

void write_cell_states(Detector& det, const string& setup, ostream& os);

#endif
