// See io_cell_states.hh for details.

// Author: Vladislav Balagura (balagura@hera-b.desy.de, 03.11.1999)

#include "ptset/io_cell_states.hh"
#include <iomanip.h>

 
void Write_Cell_States::operator()(HPT_Setup& setup) {
  for (HPT_Setup::IChamber ich=setup.begin(); ich!=setup.end();
       ++ich) {
    HPT_Chamber& chamber = (*ich).second;
    os << "  /chamber{ " << (*ich).first << '\n';
    for (HPT_Setup::ISection is=chamber.begin(); is!=chamber.end();
	 ++is) {
      HPT_Section& section = (*is);
      os << "    /section{ " << is - chamber.begin() << '\n';
      {
	os << "      /BAD{   ";
	Delimiter delim("  ", 5, "\n              ");
	for (HPT_Setup::ICell ic=section.begin(); ic!=section.end();
	     ++ic) {
	  HPT_Cell& cell = (*ic).second;
	  if (cell.state == Fired_Cell::BAD) {
	    Index ind = (*ic).first;
	    os << '[' << setw(3) << ind.ix << ',' << setw(3) << ind.iy
	       << ']' << delim;
	  }
	}
	os << "}\n\n";
      }
      {
	os << "      /NOISY{ ";
	Delimiter delim("  ", 5, "\n              ");
	for (HPT_Setup::ICell ic=section.begin(); ic!=section.end();
	     ++ic) {
	  HPT_Cell& cell = (*ic).second;
	  if (cell.state == Fired_Cell::NOISY) {
	    Index ind = (*ic).first;
	    os << '[' << setw(3) << ind.ix << ',' << setw(3) << ind.iy
	       << ']' << delim;
	  }
	}
	os << "}\n";
      }
      os << "    }\n";
    }
    os << "  }\n";
  }
}

void write_cell_states(Detector& det, const string& setup, ostream& os) {
  os << "/sub-detector{ " << setup << '\n';
  apply_virtually(Write_Cell_States(os))(det, setup);
  os << "}\n";
}
