// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include "arte/arte.hh"
#include "rang/Node.hh"
#include "rang/Hit.hh"
#include "rang/HitStrip.hh"
#include "rang/Geom.hh"
#include "rang/StateP.hh"
#include "rang/StatePI.hh"
#include "rang/StateL.hh"
#include "rang/StateLI.hh"
#include <math.h>

//-- Constructor
rangerHitStrip::rangerHitStrip() {};
rangerHitStrip::rangerHitStrip(ArtePointer<RHIT> rhit,rangerFitMode fm) 
: rangerHit(rhit,fm) {
  m = new HepVector(1);
  (*m)(1) = rhit->hitb->x;
  V = new HepSymMatrix(1);
  (*V)(1,1) = (rhit->hitb->r) * (rhit->hitb->r);
//  cout << "This is the rangerHitStrip constructor" << endl;
};

rangerHitStrip::rangerHitStrip(ArtePointer<HITB> hb,rangerFitMode fm) 
: rangerHit(hb,fm) {
  m = new HepVector(1);
  (*m)(1) = hb->x;
  V = new HepSymMatrix(1);
  (*V)(1,1) = (hb->r) * (hb->r);
//  cout << "This is the rangerHitStrip constructor" << endl;
};

rangerHitStrip::~rangerHitStrip() {
  delete m;
  delete V;
//  cout << "The HitStrip destructor deletes m and V" << endl;
}

void rangerHitStrip::projectP(HepVector& resid,
			      HepMatrix& HH,rangerStateP* s) {

#ifdef FULL
  cout << "This is rangerHitStrip::projectP" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  double Hx = s->x() * cosTot - s->y() * sinTot;
  resid(1) = (*m)(1) - Hx;

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

}
  
void rangerHitStrip::projectPI(HepVector& resid,
			       HepMatrix& HH,rangerStatePI* s) {

#ifdef FULL
  cout << "This is rangerHitStrip::projectPI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  double Hx = s->x() * cosTot - s->y() * sinTot;
  resid(1) = (*m)(1) - Hx;

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;
  HH(1,6) = 0;

}
  
void rangerHitStrip::projectL(HepVector& resid,
			      HepMatrix& HH,rangerStateL* s) {

#ifdef FULL
  cout << "This is rangerHitStrip::projectL" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  double Hx = s->x() * cosTot - s->y() * sinTot;
  resid(1) = (*m)(1) - Hx;

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;

}
  
void rangerHitStrip::projectLI(HepVector& resid,
			       HepMatrix& HH,rangerStateLI* s) {

#ifdef FULL
  cout << "This is rangerHitStrip::projectLI" << endl;
#endif
  ArtePointer<GEDE> gede = getHitb()->gede;
#ifdef FULL
  cout << "                       hitb=" << getHitb() << endl;
  cout << "                      xhitb=" << getHitb()->x << endl;
  cout << "                       gede=" << gede << endl;
#endif
  const float& cosTot = rangerGeom::vol( gede )->Cos();
  const float& sinTot = rangerGeom::vol( gede )->Sin();

  double Hx = s->x() * cosTot - s->y() * sinTot;
  resid(1) = (*m)(1) - Hx;

  HH(1,1) =   cosTot;
  HH(1,2) = - sinTot;
  HH(1,3) = 0;
  HH(1,4) = 0;
  HH(1,5) = 0;

}
  
