// 
//  Author:  Rainer Mankel                     Created:  08-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerLayPl

#include <math.h>
#include "rang/LayPl.hh"
#include "rang/DomLay.hh"
#include "rang/DomWin.hh"
#include "rang/Geom.hh"
#include "rang/ExclusionRange.hh"

extern "C" {
  void tfuwind_(const int& ge,const float& xmin,const float& xmax,
		const float& ymin,const float& ymax,
		float& umin,float& umax);
}

rangerLayPl::rangerLayPl(rangerDomLay* dl,ArtePointer<GEDE> ge,
			 const int subGLay1,const int subGLay2){
  dLay = dl;
  gede = ge;
  subGL1 = subGLay1;
  subGL2 = subGLay2;
  float umin = 0;
  float umax = 0;
  tfuwind_(ge.index(),dl->win()->xmin(),dl->win()->xmax(),
	   dl->win()->ymin(),dl->win()->ymax(),umin,umax);
  if (umin > umax) {
    cout << "rangerLayPl::rangerLayPl: umin=" << umin
	 << " umax=" << umax << endl;
  }
  uMin = umin - 2;   // 2cm tolerance
  uMax = umax + 2;
}


int rangerLayPl::excludeRange(double uExclMin,double uExclMax,
			      int subgl1,int subgl2) {
  // install exclusion range (return 1 if modification)
  // convention: uExclMin < uExclMax

  if (uExclMin > uMax || uExclMax < uMin) return 0;  // no intersection

  double uemin = uExclMin;
  double uemax = uExclMax;

  if (uemin < uMin) uemin = uMin;  // truncate to valid region
  if (uemax > uMax) uemax = uMax;

  // check if included in any of the existing exclusion ranges
  for (list<rangerExclusionRange*>::iterator exi=exrs.begin();
       exi != exrs.end(); ++exi) {
    if ((*exi)->umin() <= uemax && (*exi)->umax() >= uemin) {
      (*exi)->extend(uemin,uemax);
      return 1;
    }
  }

  // new exclusion range
  rangerExclusionRange* er = new rangerExclusionRange(this,uemin,uemax);
  if (er) exrs.push_back(er);
  return 1;
}


int rangerLayPl::clearExclusionRanges() {
  // remove all exclusion ranges (return number of deletes)

  int nDel = 0;
  for (list<rangerExclusionRange*>::iterator exi=exrs.begin();
       exi != exrs.end(); ++exi) {
    delete *exi;
    ++nDel;
  }
  exrs.erase(exrs.begin(),exrs.end());
  return nDel;
}



void rangerLayPl::dump(const int level){
  if (level > 0) {
    cout << "    LayPl  ge=" << gede << " cmp " << gede->cmp
	 << " sec " << gede->sec << " lay " << gede->lay
	 << " alpha " << (int) 57*asin(rangerGeom::vol( gede )->Sin())
	 << " subGl " << subGL1 << "-" << subGL2
	 << " uMin " << uMin
	 << " uMax " << uMax 
	 << " N(excl) " << exrs.size()
	 << endl;
  }
}
