// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomain
// basic ranger domain

#ifndef RANGER_DOMAIN_HH
#define RANGER_DOMAIN_HH

#include <vector.h>
#include <string>
#include "arte/arte.hh"

class rangerDomWin;
class rangerDomSeg;
class rangerXWin;

class rangerDomain {

public:
  rangerDomain(){};
  rangerDomain(const string dName,const int dScope,const int dActive,
	       const string sl1, const string sl2="", const string sl3="");
  ~rangerDomain(){};

  vector <rangerDomWin*>& win() { return windows; };
  rangerDomain* cloneSl() const;  // produce copy (containing sl names only)
  const vector<ArtePointer<GESL> >& domSl() const { return sls; }; 
  const string& nam()  { return name; };
  int sco() { return scope; };
  float zref() { return zRef;};
  int nb() { return number; };
  rangerXWin* xwin() const { return xWi; };
  void updSls();  // update sls from slNames
  int addWin(int cmp,int firstSector,int lastSector,
	     float sAlphaMin,float sAlphaMax,int maxMod);
  void setXWin( rangerXWin* xwi ) { xWi = xwi; };
  void setZRef();
  rangerDomSeg* domSeg() { return dseg; };
  void setDomSeg(rangerDomSeg* ds) { dseg = ds; };
  void setNum(int n ) { number = n; };
  int addProxy(const int cmp,const int sec,const int position,
	       const float width,const int cmpP,const int secP);
  int setProxyLayers();
  friend ostream& operator<<(ostream &os,const rangerDomain &d);
  void dump(const int level=1) const;

private:
  vector<string> slNames;
  vector<ArtePointer<GESL> > sls;
  vector <rangerDomWin*> windows;
  int scope;
  int active;
  string name;
  float zRef;
  rangerDomSeg* dseg;
  rangerXWin* xWi;
  int number;
};

#endif
