//-- Author : A.Spiridonov  24.03.98

#include <iostream.h>
#include <math.h>
#include <stdio.h>

#ifndef SIZE_IT

#define QP_step_control 1.1 
#define QP_curls 20.
#define SIZE_IT 20.
#define RK_step_min 20.
#define RK_step_max 60.

float RK_scale = 1.;
float RK5_step_min = 2.;

#endif

extern "C" {
  void rk4order_(double& z_in ,double* p_in
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rk4fast_(double& z_in ,double* p_in
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rk5order_(double& z_in ,double* p_in,float* error
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rk5fast_(double& z_in ,double* p_in,float* error
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rk5numde_(double& z_in ,double* p_in,float* error
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rktrans_(double& z_in ,double* p_in
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rktransc_(double& z_in, double* p_in, double* c_in, double& z_out
                ,double* p_out, double* c_out, int& ierror);

  void rktranscd_(double& z_in, double* p_in, double* c_in, double& z_out
                ,double* p_out, double* c_out,double* rkd , int& ierror);

  void rk1fast_(double& z_in ,double* p_in
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rk5clip_(double& z_in ,double* p_in
		,double& z_out,double* p_out,double* rkd,int& ierror);

  void rkzfield_(float& z_field_min, float& z_field_max); 

  void rkscale_(float& rk_step_scale); 

  void rksmin_(float& step_min); 

  void utfeld_(float* x, float* b);

#define F_C_1(f13,f15,x,y,z) ( x + f13 * y + f15 * z)
#define F_C_2(f24,x,y) ( x + f24 * y )
#define F_C_3(f35,x,y) ( x + f35 * y )

}
//              Without derivatives rkd and ierror flag 
  void rk5fast_(double& , double* , float * , double& ,double* );


//              Without derivatives rkd and ierror flag
  void rk4fast_(double& , double* , double& , double* );

//
// HISTORY
//
// In rktranscd :
//  errror for OT and IT are 0.00001
//  rk5order was used only
//

//-- Author : A.Spiridonov  26.03.98

  void rk5order_(
		 
		 double& z_in , // z value for input parameters
		 double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
		 float* error,  // acceptable errors in cm
                                // error(1) for Inner tracker
                                // error(2) for Outer tracker
		 double& z_out, // z value for output parameters
		 double* p_out, // output track parameters
		 double* rkd,   // derivatives d p_out[0-4] / d p_in[0-4]
		 int&    ierror)
                                // = 0 ok
                                // = 1 track curls
//
// Fifth-order Runge-Kutta method with adaptive stepsize control
// for solution of the equation of motion of a particle with 
//       parameter qp = Q /P in the magnetic field B()  
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{
  static float C  = 0.000299792;          // assume field in kG
  static float a[6]  = {0., 0.2, 0.3, 0.6, 1., 7./8.};
  static float c[6]  = {37./378., 0., 250./621., 125./594., 0., 512./1771.};
  static float c1[6] = {2825./27648., 0., 18575./48384., 13525./55296.,
			  277./14336.,1./4.};
  static float b[16] = {0., 1./5., 3./40., 9./40., 3./10., -9./10., 6./5.,
  -11./54., 5./2., -70./27., 35./27.,
  1631./55296., 175./512., 575./13824., 44275./110592., 253./4096.};

  float qp,hC,h;
  float k[24],x0[4],x[4],k1[24];
  float tx,ty,tx2,ty2,txty,tx2ty2,tx2ty2qp,tx2ty21,I_tx2ty2,I_tx2ty21;
  float Ax[6],Ay[6],Ax_tx[6],Ay_tx[6],Ax_ty[6],Ay_ty[6];
  float x_field[3],B[3];

  int step_j,step4;
  double p1_out[5],z2_out,p2_out[5],rkd1[25],rkd2[25];

//----------------------------------------------------------------

//  cout << "\n Z_in  " << z_in << " Z_out " << z_out << endl;

  qp    = (float) p_in[4];
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  h = z_out - z_in;
  hC   = h * C;
  
  x0[0] = (float) p_in[0];
  x0[1] = (float) p_in[1];
  x0[2] = (float) p_in[2];
  x0[3] = (float) p_in[3];

//
//   Runge-Kutta step
//

  for (int step = 0; step < 6; ++step)
    {
      for(int i=0; i < 4; ++i)
	{
	  x[i] = x0[i]; step_j = ((step-1)*step)/2 + 1;
	  for(int j=0; j < step; ++j)
	    {
	      x[i] += b[step_j + j] * k[j*4+i];
	    }
	}

      x_field[0] = x[0]; x_field[1] = x[1]; 
      x_field[2] = (float) z_in  + a[step] * h;

      utfeld_(x_field,B);

      tx = x[2]; ty = x[3]; tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty;
      tx2ty21= 1. + tx2 + ty2; I_tx2ty21 = 1. / tx2ty21 * qp;
      tx2ty2 = sqrt( tx2ty21 ) ; I_tx2ty2 = qp * hC / tx2ty2 ; 
      tx2ty2 *= hC; tx2ty2qp = tx2ty2 * qp; 

      Ax[step] = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
      Ay[step] = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;

      Ax_tx[step] = Ax[step]*tx*I_tx2ty21 + (ty*B[0]-2.*tx*B[1])*tx2ty2qp;
      Ax_ty[step] = Ax[step]*ty*I_tx2ty21 + (tx*B[0]+B[2])*tx2ty2qp;
      Ay_tx[step] = Ay[step]*tx*I_tx2ty21 + (-ty*B[1]-B[2])*tx2ty2qp;
      Ay_ty[step] = Ay[step]*ty*I_tx2ty21 + (-tx*B[1]+2.*ty*B[0])*tx2ty2qp;
       
      step4 = step * 4;
      k[step4  ] = tx * h;
      k[step4+1] = ty * h;
      k[step4+2] = Ax[step] * qp;
      k[step4+3] = Ay[step] * qp;

    }  // end of Runge-Kutta steps  

  for(int i=0; i < 4; ++i)
    {
      p_out[i]=x0[i]+c[0]*k[i]+c[2]*k[8+i]+c[3]*k[12+i]+c[5]*k[20+i];
    }
  p_out[4]=p_in[4];

//      printf("p_out %8f3 %8f3 %8f3 %8f3  \n"
//      ,p_out[0],p_out[1],p_out[2],p_out[3]);  
  
//
//     The embedded fourth-order formula for x and y
//
  for (int i = 0; i < 2; ++i)
    {
      p1_out[i]=x0[i]+c1[0]*k[i]+c1[2]*k[8+i]+c1[3]*k[12+i]+c1[4]*k[16+i]
	             +c1[5]*k[20+i];
    }

//
//     Derivatives    dx/dqp
//

  x0[0] = 0.; x0[1] = 0.; x0[2] = 0.; x0[3] = 0.;

//
//   Runge-Kutta step for derivatives dx/dqp

  for (int step = 0; step < 6; ++step)
    
    {
      for(int i=0; i < 4; ++i)
	{
	  x[i] = x0[i]; step_j = ((step-1)*step)/2 + 1;
	  for(int j=0; j < step; ++j)
	    {
	      x[i] += b[step_j + j] * k1[j*4+i];
	    }
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+1] = x[3] * h;
      k1[step4+2] = Ax[step] + Ax_tx[step] * x[2] + Ax_ty[step] * x[3];
      k1[step4+3] = Ay[step] + Ay_tx[step] * x[2] + Ay_ty[step] * x[3];

    }  // end of Runge-Kutta steps for derivatives dx/dqp

  for (int i = 0; i < 4; ++i ) 
    { 
      rkd[20+i]=x0[i]+c[0]*k1[i]+c[2]*k1[8+i]+c[3]*k1[12+i]+c[5]*k1[20+i];
    }  
  rkd[24] = 1.;

//      end of derivatives dx/dqp
//
//     Derivatives    dx/tx
//

  x0[0] = 0; x0[1] = 0.; x0[2] = 1.; x0[3] = 0.;

//
//   Runge-Kutta step for derivatives dx/dtx
//

  for (int step = 0; step < 6; ++step)    
    {
      for(int i=0; i < 4; ++i)
	{
	  x[i] = x0[i]; step_j = ((step-1)*step)/2 + 1;
	  for(int j=0; j < step; ++j)
	    {                                                 
	      if(i != 2) {x[i] += b[step_j + j] * k1[j*4+i];} // tx fixed
	    }
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+1] = x[3] * h;
//    k1[step4+2] = Ax_tx[step] * x[2] + Ax_ty[step] * x[3];
      k1[step4+3] = Ay_tx[step] * x[2] + Ay_ty[step] * x[3];
	  
    }  // end of Runge-Kutta steps for derivatives dx/dtx 

  for (int i = 0; i < 4; ++i ) 
    { 
      if(i != 2) 
	{
	  rkd[10+i]=x0[i]+c[0]*k1[i]+c[2]*k1[8+i]+c[3]*k1[12+i]+c[5]*k1[20+i];
	}
    }  
//      end of derivatives dx/dtx
  rkd[12] = 1.;
  rkd[14] = 0.;

//     Derivatives    dx/ty
//

  x0[0] = 0; x0[1] = 0.; x0[2] = 0.; x0[3] = 1.;

//
//   Runge-Kutta step for derivatives dx/dty
//

  for (int step = 0; step < 6; ++step)    
   
    {
      for(int i=0; i < 4; ++i)                              
	{
	  x[i] = x0[i]; step_j = ((step-1)*step)/2 + 1;
	  for(int j=0; j < step; ++j)
	    {
	      if( i != 3){ x[i] += b[step_j + j] * k1[j*4+i];} //    ty fixed
	    }
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+1] = x[3] * h;
      k1[step4+2] = Ax_tx[step] * x[2] + Ax_ty[step] * x[3];
//    k1[step4+3] = Ay_tx[step] * x[2] + Ay_ty[step] * x[3];
	  
    }  // end of Runge-Kutta steps for derivatives dx/dty 

  for (int i = 0; i < 3; ++i ) 
    { 
      rkd[15+i]=x0[i]+c[0]*k1[i]+c[2]*k1[8+i]+c[3]*k1[12+i]+c[5]*k1[20+i];
    }  
//      end of derivatives dx/dty
  rkd[18] = 1.;
  rkd[19] = 0.;

//
//    derivatives dx/dx and dx/dy

    for(int i = 0; i < 10; ++i){ rkd[i] = 0.;}
  rkd[0] = 1.; rkd[6] = 1.;

//
//      stepsize control
//
//  if( (fabs(p1_out[0]-p_out[0])  > error[0] && 
//       (fabs(p_out[0]) < SIZE_IT && fabs(p_out[1]) < SIZE_IT ) ) ||
//      (fabs(p1_out[0]-p_out[0])  > error[1] ) )
 
  if( 
      ((fabs(p1_out[0]-p_out[0])  > error[0] && 
       (fabs(p_out[0]) < SIZE_IT && fabs(p_out[1]) < SIZE_IT ) ) ||
      (fabs(p1_out[0]-p_out[0])  > error[1] ))
      ||
      ((fabs(p1_out[1]-p_out[1])  > error[0] && 
       (fabs(p_out[0]) < SIZE_IT && fabs(p_out[1]) < SIZE_IT ) ) ||
      (fabs(p1_out[1]-p_out[1])  > error[1] ))
      )
    {
      if(fabs(h) > RK5_step_min ) 
	{
	  z2_out = z_in + 0.5 * h;
	  rk5order_(z_in  , p_in,   error, z2_out, p2_out, rkd1, ierror);
	  rk5order_(z2_out, p2_out, error, z_out,  p_out , rkd2, ierror);

	  rkd[0]  = 1.; rkd[1] = 0.; rkd[2] = 0.; rkd[3] = 0.; rkd[4] = 0;
	  rkd[5]  = 0.; rkd[6] = 1.; rkd[7] = 0.; rkd[8] = 0.; rkd[4] = 0;
	  rkd[10] = rkd1[10] + rkd2[10]            + rkd2[15] * rkd1[13];
	  rkd[11] = rkd1[11] + rkd2[11]            + rkd2[16] * rkd1[13];
	  rkd[12] = 1.;
	  rkd[13] =            rkd2[13]            +            rkd1[13];
          rkd[14] = 0.;
	  rkd[15] = rkd1[15] + rkd2[10] * rkd1[17] + rkd2[15];
	  rkd[16] = rkd1[16] + rkd2[11] * rkd1[17] + rkd2[16];
	  rkd[17] =                       rkd1[17] + rkd2[17];
	  rkd[18] = 1.;
	  rkd[19] = 0.;
	  rkd[20] = rkd1[20] + rkd2[10] * rkd1[22] + rkd2[15] * rkd1[23]
	           +rkd2[20];
	  rkd[21] = rkd1[21] + rkd2[11] * rkd1[22] + rkd2[16] * rkd1[23]
	           +rkd2[21];
	  rkd[22] =                       rkd1[22] + rkd2[17] * rkd1[23]
	           +rkd2[22];
	  rkd[23] =            rkd2[13] * rkd1[22] +            rkd1[23]
	           +rkd2[23];
	  rkd[24] = 1.;

	}

    }       // end of stepsize control

}    // end of rk5order


//-- Author : A.Spiridonov  26.03.98

  void rk5fast_(

		double& z_in , // z value for input parameters
		double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
		float* error,  // acceptable errors in cm
                               // error(1) for Inner tracker
                               // error(2) for Outer tracker
		double& z_out, // z value for output parameters
		double* p_out, // output track parameters
		double* rkd,   // derivatives d p_out[0-4] / d p_in[0-4]
		int&    ierror)
                               // = 0 ok
                               // = 1 track curls
//
// Fifth-order Runge-Kutta method with adaptive stepsize control
// for solution of the equation of motion of a particle with 
//       parameter qp = Q /P in the magnetic field B()  
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{

  static float C  = 0.000299792;          // assume field in kG

  static float a[6]  = {0., 0.2, 0.3, 0.6, 1., 7./8.};
  static float c[6]  = {37./378., 0., 250./621., 125./594., 0., 512./1771.};
  static float c1[6] = {2825./27648., 0., 18575./48384., 13525./55296.,
			  277./14336.,1./4.};
  static float b[16] = {0., 1./5., 3./40., 9./40., 3./10., -9./10., 6./5.,
  -11./54., 5./2., -70./27., 35./27.,
  1631./55296., 175./512., 575./13824., 44275./110592., 253./4096.};

  int step_j,step4;
  float qp,hC,h;
  float k[24],x0[4],x[4],k1[24];
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax[6],Ay[6];

  float x_field[3],B[3];

  double p1_out[5],z2_out,p2_out[5];

//----------------------------------------------------------------

  qp    = (float) p_in[4];
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  h = z_out - z_in;
  hC   = h * C;

  x0[0] = (float) p_in[0];  
  x0[1] = (float) p_in[1];
  x0[2] = (float) p_in[2];  
  x0[3] = (float) p_in[3];

//
//   Runge-Kutta step
//

  for (int step = 0; step < 6; ++step)
    {
      for(int i=0; i < 4; ++i)
	{
	  x[i] = x0[i]; step_j = ((step-1)*step)/2 + 1;
	  for(int j=0; j < step; ++j)
	    {
	      x[i] += b[step_j + j] * k[j*4+i];
	    }
	}

      x_field[0] = x[0]; x_field[1] = x[1]; 
      x_field[2] = (float) z_in  + a[step] * h;

      utfeld_(x_field,B);

      tx = x[2]; ty = x[3]; tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
      tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;

      Ax[step] = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
      Ay[step] = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;

      step4 = step * 4;
      k[step4  ] = tx * h;
      k[step4+1] = ty * h;
      k[step4+2] = Ax[step] * qp;
      k[step4+3] = Ay[step] * qp;

    }  // end of Runge-Kutta steps  

  for(int i=0; i < 4; ++i)
    {
      p_out[i]=x0[i]+c[0]*k[i]+c[2]*k[8+i]+c[3]*k[12+i]+c[5]*k[20+i];
    }
  p_out[4]=p_in[4];
  
  
//
//     The embedded fourth-order formula for x 
//
  p1_out[0]=x0[0]+c1[0]*k[0]+c1[2]*k[8]+c1[3]*k[12]+c1[4]*k[16]+c1[5]*k[20];

//     Derivatives    dx/dqp
//
//
//   Runge-Kutta step for derivatives dx/dqp

  for (int step = 0; step < 6; ++step)
    
    {
      x[0] = 0.; step_j = ((step-1)*step)/2 + 1;
      for(int j=0; j < step; ++j) { x[0] += b[step_j + j] * k1[j*4  ];}
      x[2] = 0.; step_j = ((step-1)*step)/2 + 1;
      for(int j=0; j < step; ++j) { x[2] += b[step_j + j] * k1[j*4+2];}

      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+2] = Ax[step];

}  // end of Runge-Kutta steps for derivatives dx/dqp

  rkd[20]=c[0]*k1[0]+c[2]*k1[8 ]+c[3]*k1[12]+c[5]*k1[20];
  rkd[21]=0.;
  rkd[22]=c[0]*k1[2]+c[2]*k1[10]+c[3]*k1[14]+c[5]*k1[22];
  rkd[23]=0.;
  rkd[24] = 1.;

//      end of derivatives dx/dqp
//
//
//      other derivatives

  for(int i = 0; i <= 19; ++i){ rkd[i] = 0.;}
  rkd[0]  = 1.; rkd[6]  = 1.; rkd[10] = h; 
  rkd[12] = 1.; rkd[16] = h;  rkd[18] = 1.;

//
//      stepsize control
//
  if( (fabs(p1_out[0]-p_out[0])  > error[0] && 
       (fabs(p_out[0]) < SIZE_IT && fabs(p_out[1]) < SIZE_IT ) ) ||
      (fabs(p1_out[0]-p_out[0])  > error[1] ) )
    {
      if(fabs(h) > RK_step_min *3.) 
	{
	  z2_out = z_in + 0.5 * h;
	  rk5fast_(z_in  , p_in,   error, z2_out, p2_out);
	  rk5fast_(z2_out, p2_out, error, z_out,  p_out );
	}
      
      else if(fabs(h) > RK_step_min ) 
	{
	  z2_out = z_in + 0.5 * h;
	  rk4fast_(z_in  , p_in,   z2_out, p2_out);
	  rk4fast_(z2_out, p2_out, z_out,  p_out );
	}
    }       // end of stepsize control

//      printf("p_out %8f3 %8f3 %8f3 %8f3  \n"
//      ,p_out[0],p_out[1],p_out[2],p_out[3]);  

}    // end of rk5fast

//-- Author : A.Spiridonov  26.03.98

void rk5fast_(                 // Without derivatives

		double& z_in , // z value for input parameters
		double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
		float* error,  // acceptable errors in cm
                               // error(1) for Inner tracker
                               // error(2) for Outer tracker
		double& z_out, // z value for output parameters
		double* p_out )// output track parameters
//
// Fifth-order Runge-Kutta method with adaptive stepsize control
// for solution of the equation of motion of a particle with 
//       parameter qp = Q /P in the magnetic field B()  
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{

  static float C  = 0.000299792;          // assume field in kG

  static float a[6]  = {0., 0.2, 0.3, 0.6, 1., 7./8.};
  static float c[6]  = {37./378., 0., 250./621., 125./594., 0., 512./1771.};
  static float c1[6] = {2825./27648., 0., 18575./48384., 13525./55296.,
			  277./14336.,1./4.};
  static float b[16] = {0., 1./5., 3./40., 9./40., 3./10., -9./10., 6./5.,
  -11./54., 5./2., -70./27., 35./27.,
  1631./55296., 175./512., 575./13824., 44275./110592., 253./4096.};

  int step_j,step4;
  float qp,hC,h;
  float k[24],x0[4],x[4];
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax[6],Ay[6];

  float x_field[3];
  float B[3];

  double p1_out[5],z2_out,p2_out[5];

//----------------------------------------------------------------

  qp    = (float) p_in[4];
  h = z_out - z_in;
  hC   = h * C;

  x0[0] = (float) p_in[0];  
  x0[1] = (float) p_in[1];
  x0[2] = (float) p_in[2];  
  x0[3] = (float) p_in[3];

//
//   Runge-Kutta step
//

  for (int step = 0; step < 6; ++step)
    {
      for(int i=0; i < 4; ++i)
	{
	  x[i] = x0[i]; step_j = ((step-1)*step)/2 + 1;
	  for(int j=0; j < step; ++j)
	    {
	      x[i] += b[step_j + j] * k[j*4+i];
	    }
	}

      x_field[0] = x[0]; x_field[1] = x[1]; 
      x_field[2] = (float) z_in  + a[step] * h;

      utfeld_(x_field,B);

      tx = x[2]; ty = x[3]; tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
      tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
      Ax[step] = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
      Ay[step] = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;

      step4 = step * 4;
      k[step4  ] = tx * h;
      k[step4+1] = ty * h;
      k[step4+2] = Ax[step] * qp;
      k[step4+3] = Ay[step] * qp;

    }  // end of Runge-Kutta steps  

  for(int i=0; i < 4; ++i)
    {
      p_out[i]=x0[i]+c[0]*k[i]+c[2]*k[8+i]+c[3]*k[12+i]+c[5]*k[20+i];
    }
  
  p_out[4]=p_in[4];

//
//     The embedded fourth-order formula for x 
//
  p1_out[0]=x0[0]+c1[0]*k[0]+c1[2]*k[8]+c1[3]*k[12]+c1[4]*k[16]+c1[5]*k[20];

//
//      stepsize control
//
  if( (fabs(p1_out[0]-p_out[0])  > error[0] && 
       (fabs(p_out[0]) < SIZE_IT && fabs(p_out[1]) < SIZE_IT ) ) ||
      (fabs(p1_out[0]-p_out[0])  > error[1] ) )
    {
      if(fabs(h) > RK_step_min *3.) 
	{
	  z2_out = z_in + 0.5 * h;
	  rk5fast_(z_in  , p_in,   error, z2_out, p2_out);
	  rk5fast_(z2_out, p2_out, error, z_out,  p_out );
	}
      
      else if(fabs(h) > RK_step_min ) 
	{
	  z2_out = z_in + 0.5 * h;
	  rk4fast_(z_in  , p_in,   z2_out, p2_out);
	  rk4fast_(z2_out, p2_out, z_out,  p_out );
	}
    }       // end of stepsize control
  
}         // end of rk5fast without derivatives


//-- Author : A.Spiridonov  09.04.98

  void rk5numde_(               // Numerical Derivatives ( quite slow )
		 
		 double& z_in , // z value for input parameters
		 double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
		 float* error,  // acceptable errors in cm
                                // error[0] for Inner tracker
                                // error[1] for Outer tracker
		 double& z_out, // z value for output parameters
		 double* p_out, // output track parameters
		 double* rkd,   // derivatives d p_out[0-4] / d p_in[0-4]
		 int&    ierror)
                                // = 0 ok
                                // = 1 track curls
//
// Fifth-order Runge-Kutta method with adaptive stepsize control
// for solution of the equation of motion of a particle with 
//       parameter qp = Q /P in the magnetic field B()  
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*B(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+B(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  Derivatives are calculated by "numerical differentiation" 
//         of p_out as function of p_in
//

{
  float qp;
  static double delta[5] = {2.5, 2.5, 0.01, 0.01, 0.05};
  double p1_out[5],p1_in[5],d_p[5];

//----------------------------------------------------------------

  qp    = (float) p_in[4];
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  rk5fast_(z_in  , p_in,   error, z_out, p_out);
  for(int i=0; i < 4; ++i){d_p[i] = delta[i];} 
  d_p[4] = qp * delta[4];
  for(int j=0; j < 5 ; ++j)
    {
      for( int i=0; i < 5; ++i){ p1_in[i] = p_in[i];}
      p1_in[j] += d_p[j]; rk5fast_(z_in  , p1_in,   error, z_out, p1_out);
      for( int i=0; i < 5; ++i){rkd[5*j+i] = (p1_out[i]-p_out[i])/d_p[j];}
    }
}    // end of rk5nuder


//-- Author : A.Spiridonov  23.03.98

  void rk4order_(

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
	       double& z_out, // z value for output parameters
               double* p_out, // output track parameters
               double* rkd,  // derivatives d p_out(1-5) / d p_in(1-5)
               int&    ierror)
                             // = 0 ok
                             // = 1 track curls
//
// Forth-order Runge-Kutta method for solution of the equation 
// of motion of a particle with parameter qp = Q /P 
//              in the magnetic field B()
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{

  static float C  = 0.000299792;          // assume field in kG

  static float a[4] = {0., 0.5, 0.5, 1.};
  static float c[4] = {1./6., 1./3., 1./3., 1./6.};
  static float b[4] = {0., 0.5, 0.5, 1.};

  int step4;
  float qp,hC,h;
  float k[16],x0[4],x[4],k1[16];
  float tx,ty,tx2,ty2,txty,tx2ty2,tx2ty2qp,tx2ty21,I_tx2ty2,I_tx2ty21;
  float Ax[4],Ay[4],Ax_tx[4],Ay_tx[4],Ax_ty[4],Ay_ty[4];

  float x_field[3],B[3];

//----------------------------------------------------------------

  qp    = (float) p_in[4]; 
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  h = z_out - z_in; hC   = h * C;
  x0[0] = (float) p_in[0]; x0[1] = (float) p_in[1];
  x0[2] = (float) p_in[2]; x0[3] = (float) p_in[3];
//
//   Runge-Kutta step
//

  for (int step = 0; step < 4; ++step)
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[i] = x0[i];}
	  else {x[i] = x0[i] + b[step] * k[step*4-4+i];}
	}

      x_field[0] = x[0]; x_field[1] = x[1]; 
      x_field[2] = (float) z_in  + a[step] * h;
      utfeld_(x_field,B);
      tx = x[2]; ty = x[3]; tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty;
      tx2ty21= 1. + tx2 + ty2; I_tx2ty21 = 1. / tx2ty21 * qp;
      tx2ty2 = sqrt( tx2ty21 ) ; I_tx2ty2 = qp * hC / tx2ty2 ; 
      tx2ty2 *= hC; tx2ty2qp = tx2ty2 * qp; 
      Ax[step] = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
      Ay[step] = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;

      Ax_tx[step] = Ax[step]*tx*I_tx2ty21 + (ty*B[0]-2.*tx*B[1])*tx2ty2qp;
      Ax_ty[step] = Ax[step]*ty*I_tx2ty21 + (tx*B[0]+B[2])*tx2ty2qp;
      Ay_tx[step] = Ay[step]*tx*I_tx2ty21 + (-ty*B[1]-B[2])*tx2ty2qp;
      Ay_ty[step] = Ay[step]*ty*I_tx2ty21 + (-tx*B[1]+2.*ty*B[0])*tx2ty2qp;
       
      step4 = step * 4;
      k[step4  ] = tx * h;
      k[step4+1] = ty * h;
      k[step4+2] = Ax[step] * qp;
      k[step4+3] = Ay[step] * qp;

    }  // end of Runge-Kutta steps  

  for(int i=0; i < 4; ++i)
    {
      p_out[i]=x0[i]+c[0]*k[i]+c[1]*k[4+i]+c[2]*k[8+i]+c[3]*k[12+i];
    }
  p_out[4]=p_in[4];
//
//     Derivatives    dx/dqp
//

  x0[0] = 0.; x0[1] = 0.; x0[2] = 0.; x0[3] = 0.;

//
//   Runge-Kutta step for derivatives dx/dqp

  for (int step = 0; step < 4; ++step)
    
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[i] = x0[i];}
	  else {x[i] = x0[i] + b[step] * k1[step*4-4+i];}
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+1] = x[3] * h;
      k1[step4+2] = Ax[step] + Ax_tx[step] * x[2] + Ax_ty[step] * x[3];
      k1[step4+3] = Ay[step] + Ay_tx[step] * x[2] + Ay_ty[step] * x[3];

    }  // end of Runge-Kutta steps for derivatives dx/dqp

  for (int i = 0; i < 4; ++i ) 
    { 
      rkd[20+i]=x0[i]+c[0]*k1[i]+c[1]*k1[4+i]+c[2]*k1[8+i]+c[3]*k1[12+i];
    }  
  rkd[24] = 1.;
//
//      end of derivatives dx/dqp
//

//     Derivatives    dx/tx
//

  x0[0] = 0; x0[1] = 0.; x0[2] = 1.; x0[3] = 0.;

//
//   Runge-Kutta step for derivatives dx/dtx
//

  for (int step = 0; step < 4; ++step)    
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[i] = x0[i];}           // tx fixed
	  else {if(i!=2){x[i] = x0[i] + b[step] * k1[step*4-4+i];}}
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+1] = x[3] * h;
//    k1[step4+2] = Ax_tx[step] * x[2] + Ax_ty[step] * x[3];
      k1[step4+3] = Ay_tx[step] * x[2] + Ay_ty[step] * x[3];
	  
    }  // end of Runge-Kutta steps for derivatives dx/dtx 

  for (int i = 0; i < 4; ++i ) 
    { 
      if(i != 2)
	{
	  rkd[10+i]=x0[i]+c[0]*k1[i]+c[1]*k1[4+i]+c[2]*k1[8+i]+c[3]*k1[12+i];
	}
    }  
//      end of derivatives dx/dtx
  rkd[12] = 1.;
  rkd[14] = 0.;

//     Derivatives    dx/ty
//

  x0[0] = 0; x0[1] = 0.; x0[2] = 0.; x0[3] = 1.;

//
//   Runge-Kutta step for derivatives dx/dty
//

  for (int step = 0; step < 4; ++step)    
   
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[i] = x0[i];}           // ty fixed
	  else {if(i!=3){x[i] = x0[i] + b[step] * k1[step*4-4+i];}}
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+1] = x[3] * h;
      k1[step4+2] = Ax_tx[step] * x[2] + Ax_ty[step] * x[3];
//    k1[step4+3] = Ay_tx[step] * x[2] + Ay_ty[step] * x[3];
	  
    }  // end of Runge-Kutta steps for derivatives dx/dty 

  for (int i = 0; i < 3; ++i ) 
    { 
      rkd[15+i]=x0[i]+c[0]*k1[i]+c[1]*k1[4+i]+c[2]*k1[8+i]+c[3]*k1[12+i];
    }  
//      end of derivatives dx/dty
  rkd[18] = 1.;
  rkd[19] = 0.;

//
//    derivatives dx/dx and dx/dy

    for(int i = 0; i < 10; ++i){ rkd[i] = 0.;}
  rkd[0] = 1.; rkd[6] = 1.;

}    // end of rk4order


//-- Author : A.Spiridonov  27.03.98

  void rk4fast_(

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
	       double& z_out, // z value for output parameters
               double* p_out, // output track parameters
               double* rkd,  // derivatives d p_out[0-4] / d p_in[0-4]
               int&    ierror)
                             // = 0 ok
                             // = 1 track curls
//
// Forth-order Runge-Kutta method for solution of the equation 
// of motion of a particle with parameter qp = Q /P 
//              in the magnetic field B()
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{
  static float C  = 0.000299792;          // assume field in kG
  static float a[4] = {0., 0.5, 0.5, 1.};
  static float c[4] = {1./6., 1./3., 1./3., 1./6.};
  static float b[4] = {0., 0.5, 0.5, 1.};
  int step4;
  float qp,hC,h;
  float k[16],x0[4],x[4],k1[16];
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax[4],Ay[4];
  float x_field[3],B[3];
//----------------------------------------------------------------
  qp    = (float) p_in[4]; 
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  h = z_out - z_in; hC   = h * C;
  x0[0] = (float) p_in[0]; x0[1] = (float) p_in[1];
  x0[2] = (float) p_in[2]; x0[3] = (float) p_in[3];
//
//   Runge-Kutta step
//
  for (int step = 0; step < 4; ++step)
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[i] = x0[i];}
	  else {x[i] = x0[i] + b[step] * k[step*4-4+i];}
	}
      x_field[0] = x[0]; x_field[1] = x[1]; 
      x_field[2] = (float) z_in  + a[step] * h;
      utfeld_(x_field,B);
      tx = x[2]; ty = x[3]; tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
      tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
      Ax[step] = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
      Ay[step] = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;

      step4 = step * 4;
      k[step4  ] = tx * h;
      k[step4+1] = ty * h;
      k[step4+2] = Ax[step] * qp;
      k[step4+3] = Ay[step] * qp;

    }  // end of Runge-Kutta steps  

  for(int i=0; i < 4; ++i)
    {
      p_out[i]=x0[i]+c[0]*k[i]+c[1]*k[4+i]+c[2]*k[8+i]+c[3]*k[12+i];
    }
  p_out[4]=p_in[4];

//
//     Derivatives    dx/dqp and dtx/dqp
//
//   Runge-Kutta step for derivatives dx/dqp

  for (int step = 0; step < 4; ++step)    
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[0] = 0.; x[2] = 0.;}
	  else {x[0] = b[step] * k1[step*4-4];
		x[2] = b[step] * k1[step*4-2]; }
	}
      step4 = step * 4;
      k1[step4  ] = x[2] * h;
      k1[step4+2] = Ax[step] ;

    }  // end of Runge-Kutta steps for derivatives dx/dqp

  rkd[20] = c[0]*k1[0]+c[1]*k1[4]+c[2]*k1[8]+c[3]*k1[12];
  rkd[21] = 0.;
  rkd[22] = c[0]*k1[2]+c[1]*k1[6]+c[2]*k1[10]+c[3]*k1[14];
  rkd[23] = 0.;
  rkd[24] = 1.;

//      end of derivatives dx/dqp
//
//      other derivatives

  for(int i = 0; i <= 19; ++i){ rkd[i] = 0.;}
  rkd[0]  = 1.; rkd[6]  = 1.; rkd[10] = h; 
  rkd[12] = 1.; rkd[16] = h;  rkd[18] = 1.;

}    // end of rk4fast


//-- Author : A.Spiridonov  27.03.98

void rk4fast_(                // Without derivatives              

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
	       double& z_out, // z value for output parameters
               double* p_out) // output track parameters
//
// Forth-order Runge-Kutta method for solution of the equation 
// of motion of a particle with parameter qp = Q /P 
//              in the magnetic field B()
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{
  static float C  = 0.000299792;          // assume field in kG
  static float a[4] = {0., 0.5, 0.5, 1.};
  static float c[4] = {1./6., 1./3., 1./3., 1./6.};
  static float b[4] = {0., 0.5, 0.5, 1.};
  int step4;
  float qp,hC,h;
  float k[16],x0[4],x[4];
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax[4],Ay[4];
  float x_field[3],B[3];
//----------------------------------------------------------------

  qp    = (float) p_in[4]; 
  h = z_out - z_in;
  hC   = h * C;

  x0[0] = (float) p_in[0];  x0[1] = (float) p_in[1];
  x0[2] = (float) p_in[2];  x0[3] = (float) p_in[3];
//
//   Runge-Kutta step
//
  for (int step = 0; step < 4; ++step)
    {
      for(int i=0; i < 4; ++i)
	{
	  if(step == 0) {x[i] = x0[i];}
	  else {x[i] = x0[i] + b[step] * k[step*4-4+i];}
	}

      x_field[0] = x[0]; x_field[1] = x[1]; 
      x_field[2] = (float) z_in  + a[step] * h;
      utfeld_(x_field,B);
      tx = x[2]; ty = x[3]; tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
      tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
      Ax[step] = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
      Ay[step] = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;

      step4 = step * 4;
      k[step4  ] = tx * h;
      k[step4+1] = ty * h;
      k[step4+2] = Ax[step] * qp;
      k[step4+3] = Ay[step] * qp;

    }  // end of Runge-Kutta steps  
  for(int i=0; i < 4; ++i)
    {
      p_out[i]=x0[i]+c[0]*k[i]+c[1]*k[4+i]+c[2]*k[8+i]+c[3]*k[12+i];
    }
  p_out[4]=p_in[4];

}    // end of rk4fast without derivatives


//-- Author : A.Spiridonov  30.05.98

  void rktrans_(

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
	       double& z_out, // z value for output parameters
               double* p_out, // output track parameters
               double* rkd,  // derivatives d p_out(1-5) / d p_in(1-5)
               int&    ierror)
                             // = 0 ok
                             // = 1 track curls
//
// Transport the vector of track parameters p_in defined at the  
// reference plane z_in to the vector p_out defined at z_out 
//      using Runge-Kutta methods of different orders
//

{
  static float C  = 0.000299792;          // assume field in kG
  static float error[2] = { 0.0025, 0.0050 }; // IT , OT - errors
  float qp,hC,h;
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax,Ay,Axqp,Ayqp;
  float x_field[3],B[3];
//----------------------------------------------------------------

  h = z_out - z_in; 

  if ( fabs(h) < RK_step_min)                         // short distance
    {
  qp    = (float) p_in[4]; 
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  hC   = h * C;

  x_field[0] = (float) p_in[0];
  x_field[1] = (float) p_in[1];
  x_field[2] = (float) z_in;

  utfeld_(x_field,B);

  tx = (float) p_in[2]; ty = (float) p_in[3]; 
  tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
  tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
  Ax = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
  Ay = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;
  Axqp = Ax * qp; Ayqp = Ay * qp;

  p_out[0] = p_in[0] + tx * h + 0.5 * Axqp * h;
  p_out[1] = p_in[1] + ty * h + 0.5 * Ayqp * h;
  p_out[2] = p_in[2] + Axqp;
  p_out[3] = p_in[3] + Ayqp;
  p_out[4] = p_in[4];

  for(int i = 0; i <= 19; ++i){ rkd[i] = 0.;}
  rkd[0]  = 1.; rkd[6]  = 1.; rkd[10] = h; 
  rkd[12] = 1.; rkd[16] = h;  rkd[18] = 1.;
  rkd[20] = 0.5 * Ax * h;  rkd[21] = 0.;  rkd[22] = Ax;
  rkd[23] = 0.; rkd[24] = 1.; 

    }
  else if ( fabs(h) < RK_step_min * 3.)     // medium distance 
    {
      rk4fast_(z_in, p_in, z_out, p_out, rkd, ierror);
    }
  else                                      // long distance
    {
      rk5fast_(z_in, p_in, error, z_out, p_out, rkd, ierror);
    }

}    // end of rktrans


//-- Author : A.Spiridonov  31.05.98

  void rktransc_(

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
               double* c_in,  // input cov matrix of track parameters 
	       double& z_out, // z value for output parameters
               double* p_out, // output track parameters
               double* c_out, // output cov matrix of track parameters
               int&    ierror)
                             // = 0 ok
                             // = 1 track curls
//
// Transport  vector of track parameters p_in and covariance
// matrix c_in defined at the reference plane z_in to the 
// vector p_out (cov marix c_out) defined at z_out 
//  using Runge-Kutta methods of different orders
//

{
  static float C  = 0.000299792;          // assume field in kG
  static float error[2] = { 0.0025, 0.0050 }; // IT , OT - errors
  float qp,hC,h;
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax,Ay,Axqp,Ayqp;
  float x_field[3],B[3];
  double rkd[25],g[25];
//----------------------------------------------------------------

  h = z_out - z_in; 

  if ( fabs(h) < RK_step_min)                         // short distance
    {
  qp    = (float) p_in[4]; 
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  hC   = h * C;

  x_field[0] = (float) p_in[0];
  x_field[1] = (float) p_in[1];
  x_field[2] = (float) z_in;

  utfeld_(x_field,B);

  tx = (float) p_in[2]; ty = (float) p_in[3]; 
  tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
  tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
  Ax = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
  Ay = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;
  Axqp = Ax * qp; Ayqp = Ay * qp;

  p_out[0] = p_in[0] + tx * h + 0.5 * Axqp * h;
  p_out[1] = p_in[1] + ty * h + 0.5 * Ayqp * h;
  p_out[2] = p_in[2] + Axqp;
  p_out[3] = p_in[3] + Ayqp;
  p_out[4] = p_in[4];

  for(int i = 0; i <= 19; ++i){ rkd[i] = 0.;}
  rkd[0]  = 1.; rkd[6]  = 1.; rkd[10] = h; 
  rkd[12] = 1.; rkd[16] = h;  rkd[18] = 1.;
  rkd[20] = 0.5 * Ax * h;  rkd[21] = 0.;  rkd[22] = Ax;
  rkd[23] = 0.; rkd[24] = 1.; 

    }
  else if ( fabs(h) < RK_step_min * 3.)     // medium distance 
    {
      rk4fast_(z_in, p_in, z_out, p_out, rkd, ierror);
    }
  else                                      // long distance
    {
      rk5fast_(z_in, p_in, error, z_out, p_out, rkd, ierror);
    }
//          covariance matrix transport 

  for( int i = 0; i < 25; i += 5) {
    g[i]  = F_C_1(rkd[10],rkd[20],c_in[i], c_in[i+2], c_in[i+4]);}
  for( int i = 1; i < 25; i += 5) {g[i] = F_C_2(rkd[16],c_in[i], c_in[i+2]);}
  for( int i = 2; i < 25; i += 5) {g[i] = F_C_3(rkd[22],c_in[i], c_in[i+2]);}
  for (int i = 3; i < 25; i += 5) {g[i] = c_in[i]; g[i+1] = c_in[i+1];}

  for( int i = 0; i < 5; ++i ) {
    c_out[i]  = F_C_1(rkd[10],rkd[20],g[i], g[i+10], g[i+20]);}
  for( int i = 5;  i < 10; ++i) {c_out[i] = F_C_2(rkd[16],g[i], g[i+10]);}
  for( int i = 10; i < 15; ++i) {c_out[i] = F_C_3(rkd[22],g[i], g[i+10]);}
  for( int i = 15; i < 25; ++i) {c_out[i] = g[i];}

}    // end of rktransc


//-- Author : A.Spiridonov  20.08.98

  void rktranscd_(

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
               double* c_in,  // input cov matrix of track parameters 
	       double& z_out, // z value for output parameters
               double* p_out, // output track parameters
               double* c_out, // output cov matrix of track parameters
               double* rkd,  // derivatives d p_out(1-5) / d p_in(1-5)
               int&    ierror)
                             // = 0 ok
                             // = 1 track curls
//
// Transport  vector of track parameters p_in and covariance
// matrix c_in defined at the reference plane z_in to the 
// vector p_out (cov marix c_out) defined at z_out 
//  using Runge-Kutta methods of different orders
//

{
  static float C  = 0.000299792;          // assume field in kG
//  static float error[2] = { 0.0025, 0.0050 }; // IT , OT - errors
  static float error[2] = { 0.00001, 0.00001 }; // IT , OT - errors
  float qp,hC,h;
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax,Ay,Axqp,Ayqp;
  float x_field[3],B[3];
  double g[25];
//----------------------------------------------------------------

  h = z_out - z_in; 

  if ( fabs(h) < - RK_step_min)                         // short distance
    {
  qp    = (float) p_in[4]; 
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  hC   = h * C;

  x_field[0] = (float) p_in[0];
  x_field[1] = (float) p_in[1];
  x_field[2] = (float) z_in;

  utfeld_(x_field,B);

  tx = (float) p_in[2]; ty = (float) p_in[3]; 
  tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
  tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
  Ax = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
  Ay = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;
  Axqp = Ax * qp; Ayqp = Ay * qp;

  p_out[0] = p_in[0] + tx * h + 0.5 * Axqp * h;
  p_out[1] = p_in[1] + ty * h + 0.5 * Ayqp * h;
  p_out[2] = p_in[2] + Axqp;
  p_out[3] = p_in[3] + Ayqp;
  p_out[4] = p_in[4];

  for(int i = 0; i < 25; ++i){ rkd[i] = 0.;}
  rkd[0]  = 1.; rkd[6]  = 1.; rkd[10] = h; 
  rkd[12] = 1.; rkd[16] = h;  rkd[18] = 1.;
  rkd[20] = 0.5 * Ax * h;  rkd[21] = 0.;  rkd[22] = Ax;
  rkd[23] = 0.; rkd[24] = 1.; 

    }
  else if ( fabs(h) < - RK_step_min * 3.)     // medium distance 
    {
      rk4fast_(z_in, p_in, z_out, p_out, rkd, ierror);
    }
  else                                      // long distance
    {

//      rk5fast_(z_in, p_in, error, z_out, p_out, rkd, ierror);

      rk5order_(z_in, p_in, error, z_out, p_out, rkd, ierror);


    }
//          covariance matrix transport 

  for( int i = 0; i < 25; i += 5) {
    g[i]  = F_C_1(rkd[10],rkd[20],c_in[i], c_in[i+2], c_in[i+4]);}
  for( int i = 1; i < 25; i += 5) {g[i] = F_C_2(rkd[16],c_in[i], c_in[i+2]);}
  for( int i = 2; i < 25; i += 5) {g[i] = F_C_3(rkd[22],c_in[i], c_in[i+2]);}
  for( int i = 3; i < 25; i += 5) {g[i] = c_in[i]; g[i+1] = c_in[i+1];}

  for( int i = 0; i < 5; ++i ) {
    c_out[i]  = F_C_1(rkd[10],rkd[20],g[i], g[i+10], g[i+20]);}
  for( int i = 5;  i < 10; ++i) {c_out[i] = F_C_2(rkd[16],g[i], g[i+10]);}
  for( int i = 10; i < 15; ++i) {c_out[i] = F_C_3(rkd[22],g[i], g[i+10]);}
  for( int i = 15; i < 25; ++i) {c_out[i] = g[i];}

}    // end of rktranscd


//
//-- Author : A.Spiridonov  15.04.98

  void rk5clip_(
		 
		 double& z_in , // z value for input parameters
		 double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
		 double& z_out, // z value for output parameters
		 double* p_out, // output track parameters
		 double* rkd,   // derivatives d p_out[0-4] / d p_in[0-4]
		 int&    ierror)
                                // = 0 ok
                                // = 1 track curls
//
// Particle trajectory is clipped on small pieces and for each
// of them fifth-order Runge-Kutta method with adaptive stepsize 
// control is used for solution of the equation of motion of a 
// particle with parameter qp = Q /P in the magnetic field B()  
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//
//  In the following for RK step  :
//     x=x[0], y=x[1], tx=x[3], ty=x[4].
//

{
  double z1,z2,p1[5],p2[5],rkd1[25],rkd2[25];
  float error1[2],error[2]={0.00005,0.00005};

  double h,step_1;
  float RK_step_clip;
  int N_step;

//----------------------------------------------------------------

//  cout << "\n Z_in  " << z_in << " Z_out " << z_out << endl;

  h = z_out - z_in;

  float p_0 = 1./fabs(p_in[4]);
  if(p_0 < 3. ) {
    RK_step_clip = 6.;
  }
  else if(p_0 < 10.) {
    RK_step_clip = p_0 * 2.;
  }
  else if(p_0 < 30.) {
    RK_step_clip =   20. + ( p_0 - 10.) * 0.5  ;
  }
  else {
    RK_step_clip = p_0 * 1.;
  }
//  cout << "\n RK_step_clip " << RK_step_clip << endl;
  
  if(fabs(h) > (double) RK_step_clip) 
    {
      N_step = (int) fabs(h/(RK_step_clip / RK_scale)) + 1; 
      step_1 = h / ((float) N_step);
    }
  else    
    {
      N_step = 1;
      step_1 = h; 
    }

  for ( int i=0; i < 2; ++i) 
    {error1[i] = error[i]/sqrt(float( N_step));}

  z1 = z_in; for (int j = 0; j < 5; ++j){p1[j] = p_in[j];}
  for ( int i=0; i < N_step  ; ++i)
    {
      z2 = z_out - ((double)( N_step - i - 1 )) * step_1;

//  cout << "\n z1  " << z1 << " z2 " << z2 << endl;

      rk5order_(z1,p1,error1,z2,p2,rkd2,ierror);

      if(i == 0) 
	{
	  for(int i1 = 0; i1 < 25; ++i1){rkd[i1] = rkd2[i1];}
	}
      else
	{
	  rkd[0]  = 1.; rkd[1] = 0.; rkd[2] = 0.; rkd[3] = 0.; rkd[4] = 0;
	  rkd[5]  = 0.; rkd[6] = 1.; rkd[7] = 0.; rkd[8] = 0.; rkd[4] = 0;

	  rkd[10] = rkd1[10] + rkd2[10]            + rkd2[15] * rkd1[13];
	  rkd[11] = rkd1[11] + rkd2[11]            + rkd2[16] * rkd1[13];
	  rkd[12] = 1.;
	  rkd[13] =            rkd2[13]            +            rkd1[13];
          rkd[14] = 0.;
	  rkd[15] = rkd1[15] + rkd2[10] * rkd1[17] + rkd2[15];
	  rkd[16] = rkd1[16] + rkd2[11] * rkd1[17] + rkd2[16];
	  rkd[17] =                       rkd1[17] + rkd2[17];
	  rkd[18] = 1.;
	  rkd[19] = 0.;
	  rkd[20] = rkd1[20] + rkd2[10] * rkd1[22] + rkd2[15] * rkd1[23]
	           +rkd2[20];
	  rkd[21] = rkd1[21] + rkd2[11] * rkd1[22] + rkd2[16] * rkd1[23]
	           +rkd2[21];
	  rkd[22] =                       rkd1[22] + rkd2[17] * rkd1[23]
	           +rkd2[22];
	  rkd[23] =            rkd2[13] * rkd1[22] +            rkd1[23]
	           +rkd2[23];
	  rkd[24] = 1.;
	}
      z1 = z2;
      for(int i1 = 0; i1 <  5; ++i1){ p1  [i1] = p2 [i1];}
      for(int i1 = 0; i1 < 25; ++i1){ rkd1[i1] = rkd[i1];}
    }
  for(int i1 = 0; i1 <  5; ++i1){ p_out  [i1] = p2 [i1];}

}    // end of rk5clip


//-- Author : A.Spiridonov  02.07.98

  void rk1fast_(

               double& z_in , // z value for input parameters
               double* p_in,  // input track parameters (x,y,tx,ty,Q/p)
	       double& z_out, // z value for output parameters
               double* p_out, // output track parameters
               double* rkd,   // derivatives d p_out[0-4] / d p_in[0-4]
               int&    ierror)
                             // = 0 ok
                             // = 1 track curls
//
//    Parabolic expansion of the particle trajectory
// for solution of the equation of motion of a particle with 
//       parameter qp = Q /P in the magnetic field B()  
//
//        | x |          tx
//        | y |          ty 
// d/dz = | tx| = ft * ( ty * ( B(3)+tx*b(1) ) - (1+tx**2)*B(2) ) 
//        | ty|   ft * (-tx * ( B(3)+ty+b(2)   - (1+ty**2)*B(1) )  ,
//
//   where  ft = C*qp*sqrt ( 1 + tx**2 + ty**2 ) .
//

{
  static float C  = 0.000299792;          // assume field in kG
  float qp,hC,h;
  float tx,ty,tx2,ty2,txty,tx2ty2;
  float Ax,Ay,Axqp,Ayqp;
  float x_field[3],B[3];
//----------------------------------------------------------------

  h = z_out - z_in; 
  qp    = (float) p_in[4]; 
  ierror = (fabs(qp) > QP_curls) ? 1 : 0;
  hC   = h * C;

  x_field[0] = (float) p_in[0];
  x_field[1] = (float) p_in[1];
  x_field[2] = (float) z_in;

  utfeld_(x_field,B);

  tx = (float) p_in[2]; ty = (float) p_in[3]; 
  tx2 = tx * tx; ty2 = ty * ty; txty = tx * ty; 
  tx2ty2 =  sqrt( 1. + tx2 + ty2 ) *  hC;
  Ax = ( txty * B[0] + ty * B[2] - ( 1. + tx2 ) * B[1] ) * tx2ty2; 
  Ay = (-txty * B[1] - tx * B[2] + ( 1. + ty2 ) * B[0] ) * tx2ty2;
  Axqp = Ax * qp; Ayqp = Ay * qp;

  p_out[0] = p_in[0] + tx * h + 0.5 * Axqp * h;
  p_out[1] = p_in[1] + ty * h + 0.5 * Ayqp * h;
  p_out[2] = p_in[2] + Axqp;
  p_out[3] = p_in[3] + Ayqp;
  p_out[4] = p_in[4];

  for(int i = 0; i <= 19; ++i){ rkd[i] = 0.;}
  rkd[0]  = 1.; rkd[6]  = 1.; rkd[10] = h; 
  rkd[12] = 1.; rkd[16] = h;  rkd[18] = 1.;
  rkd[20] = 0.5 * Ax * h;  rkd[21] = 0.;  rkd[22] = Ax;
  rkd[23] = 0.; rkd[24] = 1.; 

}    // end of rk1fast


//-- Author : A.Spiridonov  24.06.98

  void rkscale_(float& rk_step_scale) 

//    Set RK_scale factor for reduction of RK_step_max

{
  RK_scale = rk_step_scale;
}


//-- Author : A.Spiridonov  24.06.98

  void rksmin_(float& step_min) 

//    Give maximal step for RK 

{
  RK5_step_min = step_min;
}

//-- Author : A.Spiridonov  02.07.98

  void rkzfield_(float& z_field_min, float& z_field_max) 

//    Give lower and upper bounds of the region 
//  where magnetig field if defined in gufld,utfeld

{
  z_field_min = -470.;     //  in centimeter
  z_field_max =  490.;
}

