// ********************************************************************
//
//  Routine:      rc99AStandAlone
//  Author:       T.Oest/D.Samtleben
//
//  Description:  stand alone reconstruction
//
//                reco level 0: no reconstruction
//                           1: hitb only
//                           2: reconstruction only
//                           3: hitb + reconstruction
//
//  Modification:  
//      - drop RICH hits
//      - recoTARGET,... variables no longer static
//                                                        27/05/99 Oest
//      - VDS==SVD for kumac-steering
//      - error-message if given subdetector is not known
//                                                        10/06/99 Samtleben
//      - if at least one reconstruction package of a subdetector
//        is called the Rxxx-tables are cleared
//                                                        01/07/99 Samtleben
//      - bug fix: proper termination of input[]
//      - include CATS: recoVDSmodule
//      - correct calling sequenz for rc99ATARGET
//                                                        26/07/99 Oest
//     -- global alignment transformation added 
//                                                        06/09/99 T.Oest
//     -- RICH hits are no longer removed since they are needed for
//        the particle ID                                 14/09/99 T.Oest 
//     -- move VDS global alignment transformation to rc99AVDS
//                                                        20/09/99 T.Oest
//     -- filling of GTAR bank moved to acReadTARGET      24/09/99 T.Oest
//     -- include global VDS alignment transformation
//                                                        29/10/99 T.Oest
//     -- ITR added                                       07/02/00 T.Oest
//
//
// ********************************************************************

#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include "arte/arte.hh"

#include "reco/rc99Areco.hh"
#include "reco/rcflags.hh"

void acRSEGinGlobalCoordinates(const int CMPnumber);

//********************************************************************
// rc99AStandAlone
//********************************************************************
  int rc99AStandAlone() {

//===================================================================
// remove the Rxxx tables (for real data)
//===================================================================

//------  remove Rxxx-tables
    if(recoVDS||recoECAL||recoRICH||recoHPT||recoTRD||recoMUON){
        ArteTable<RREL>::clear();
        ArteTable<RPNT>::clear();
        ArteTable<RVER>::clear();
        ArteTable<RHIT>::clear();
        ArteTable<RSEG>::clear();
        ArteTable<RTRA>::clear();
        ArteTable<RCCL>::clear();
}
//===================================================================
//  stand alone tracking
//===================================================================

    if (recoVDS)    int SVDflag     = rc99AVDS(recoVDS);
    if (recoITR)    int ITRflag     = rc99AITR(recoITR);
    if (recoOTR)    int OTRflag     = rc99AOTR(recoOTR);
    if (recoHPT)    int HPTflag     = rc99AHPT(recoHPT);
    if (recoMUON)   int MUONflag    = rc99AMUON(recoMUON);
    if (recoTRD)    int TRDflag     = rc99ATRD(recoTRD);
    if (recoRICH)   int RICHflag    = rc99ARICH(recoRICH);

//===================================================================
//  transformation of segments/RHIT to global coordinates
//===================================================================

   if (recoVDSmodule & 1) acRSEGinGlobalCoordinates(1);
   if (recoVDSmodule & 2) acRSEGinGlobalCoordinates(65537);
   if (recoRICH)          acRSEGinGlobalCoordinates(67108864);

//===================================================================
//  ECAL needs tracks for the 2. reconstruction step
//===================================================================

    if (recoECAL)   int ECALflag    = rc99AECAL(recoECAL);


//===================================================================
//  flag for stand alone reconstruction
//===================================================================

    return 0;

  }

