////////////////////////////////////////////////////////////////
//
// Author: D.Dujmic
//
// Particle ID using RICH stand-alone reconstruction 
// and matching with momentum measurement.
// Returns an array filled with deviations from expected radius
// in sigmas^2 or fills RTRA fileds with the same quantities
// 
// For a good id the deviation in sigmas^2 should be less than 5 
// i.e. for good pion id:  rtra->lrpi<5 or
//                         dd[2]<5
//      depending which function you call
//
////////////////////////////////////////////////////////////////

#include "arte/arte.hh"
#include "rich/rich.h"
#include "arte/RSEG_cmp.hh"

// calculate deviations
int rc99idRISE(float ip, float rad, float *dd) {
  float ipp=ip*ip;
  float pp=1/ipp;
  static float aa=.03*.03;
  static float bb=.0025*.0025;
  static float sigma2_rr  = 1.5e-5 * 1.5e-5;
  static int   n_mass=5;
  static float mm[]={2.6e-7, 1.12e-2, 1.95e-2, .24372, .88035};
  float sigma2_ipp = 4 * ipp*ipp * (aa+bb*pp);
  float chi2=100;

  float rr=rad*rad;

  float r0=getRadius(ArtePointer<EVHD>(1)->run,1);
  float r0r0=r0*r0;

  int bestM=-1;
  for (int i=0; i<n_mass; i++) {

    float isigma2 = 1/(sigma2_rr + mm[i]*mm[i] * sigma2_ipp);
    float d       = (rr - r0r0 + mm[i] * ipp);
    dd[i]         = d*d*isigma2;
    if (dd[i]<chi2) { chi2=dd[i]; bestM=i; }
  }

  return bestM;
}

// RTRA is filled with  deviations of radius in sigma^2
int rc99idRISE(ArtePointer<RTRA> t) {

  // find rich ring
  ArteRelationIterator<RSEG> irseg=t->rseg.begin();
  for (;irseg!=t->rseg.end(); ++irseg) {
    if ((*irseg)->cmp==Rsegc::rich) break; 
  };
  if (irseg==t->rseg.end()) return 1;

  // calculate deviation from r2p2 line
  float rad=(*irseg)->time;
  float ip =t->pf;
  float dd[5];
  t->lre  = t->lrmu = t->lrpi = t->lrk  = t->lrp  = 1e5; 
  if (rc99idRISE(ip,rad,dd)<0) return -1;

  // fill RTRA fields
  t->lre  = dd[0];
  t->lrmu = dd[1]; 
  t->lrpi = dd[2]; 
  t->lrk  = dd[3]; 
  t->lrp  = dd[4]; 

  return 0;
}

// returns array dd[0..4] with deviations 
// in sigma^2 for ecah particle
int rc99idRISE(ArtePointer<RSEG> r, float *dd) {
  float rad = r->time;
  float ip  = r->pf;
  rc99idRISE(ip,rad,dd);
  return 0;
}
