 
// ********************************************************************
//
//  Routine:      rcSetTrackPoints
//
//  Description:  set track points
//
//  Modification:
//  -- only called for 99 track selection          17/02/00 T.Oest
//  -- called for all track selections, use RHITs  10/03/00 T.Oest
//
// ********************************************************************
 
 
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include "arte/arte.hh"
#include "reco/rcflags.hh"

int rcSetTrackPointfromRTRA(ArtePointer<RTRA> , const float);
int rcSetTrackPointfromRHIT(ArtePointer<RTRA> , const float);


//********************************************************************
// function
//********************************************************************

int rcSetTrackPoints() {

//-------------------------------------------------------------------
//   new points only for /RECON/RTRA ON
//-------------------------------------------------------------------

  if (rcSelectTracksStatus == 0) return 0; 

//-------------------------------------------------------------------
//   loop over tracks
//-------------------------------------------------------------------


  for(ArteTable<RTRA>::iterator rtra=ArteTable<RTRA>::begin();
      rtra!=ArteTable<RTRA>::end();rtra++){

  if ((*rtra)->fit == 0 && 
      (*rtra)->rpnt.begin() == (*rtra)->rpnt.end()){

//-------------------------------------------------------------------
//   check for RHITs
//-------------------------------------------------------------------

  int test = 0;

  ArteTable<RHIT>::iterator rhit = ArteTable<RHIT>::begin();

  while ( rhit != ArteTable<RHIT>::end() && test == 0){
      rhit++;
      if (ArtePointer<RTRA>((*rhit)->rtra) == *rtra) test = 1;
  };



//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

   if ((*rtra)->hou + (*rtra)->hin == 0 || test == 0){

     rcSetTrackPointfromRTRA(*rtra, 1000.);
     rcSetTrackPointfromRTRA(*rtra, 1340.);

    } else {

     rcSetTrackPointfromRHIT(*rtra, 1000.);
     rcSetTrackPointfromRHIT(*rtra, 1340.);

    };
  
  };
  };      
       
  return 0;      
};
//********************************************************************
// function
//********************************************************************
extern "C" void rkmain_(const int    & icovar, 
			const double & zout, 
			const double & fstep, 
			const double * parameters, 
			const double * prec, 
			const double * p_track, 
			const double * derivatives, 
			const int    & ierror);


int rcSetTrackPointfromRTRA(ArtePointer<RTRA> rtra, const float z) {

  const int index[5][5] = { {0, 1,  3, 6,10},
                            {1, 2,  4, 7,11},
                            {3, 4,  5, 8,12},
                            {6, 7,  8, 9,13},
                            {10,11,12,13,14}};

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
      double zout = double(z);
      
      const int    icovar=2;    
      double       prec[4];
      double       derivatives[5][5]={};
      double       p_track[6], parameters[6];
      double       fstep=50.;
      
      parameters[0] = double(rtra->zf);
      parameters[1] = double(rtra->xf);
      parameters[2] = double(rtra->yf);
      parameters[3] = double(rtra->txf);
      parameters[4] = double(rtra->tyf);
      parameters[5] = double(rtra->pf);


      if (zout != parameters[0]) {
	
	int    ierror;
	prec[0] = 0.001/fabs(zout-parameters[0]);
	prec[1] = prec[0];
	prec[2] = 0.0001/fabs(zout-parameters[0]);
	prec[3] = prec[2];
      
	rkmain_(icovar, zout, fstep, parameters, prec, p_track, 
		& derivatives[0][0], ierror);

        if (ierror == 0) {
         RPNT rpnt;

         rpnt.zf  = float(p_track[0]);
         rpnt.xf  = float(p_track[1]);
         rpnt.yf  = float(p_track[2]);
         rpnt.txf = float(p_track[3]);
         rpnt.tyf = float(p_track[4]);
         rpnt.pf  = float(p_track[5]);

         rpnt.cvf[index[4][0]] = 0.0;
         rpnt.cvf[index[4][1]] = 0.0;
         rpnt.cvf[index[4][2]] = 0.0;
         rpnt.cvf[index[4][3]] = 0.0;
         rpnt.cvf[index[4][4]] = rtra->cvf[index[4][4]];

         for (int i=0; i<=3 ; i++){
         for (int j=i; j<=3 ; j++){

          float sigma = 0.0;
	  for (int k=0; k<=3; k++) { for (int l=0; l<=3; l++){
           sigma = sigma + float(derivatives[k][i])
                         * rtra->cvf[index[k][l]]
                         * float(derivatives[l][j]);
          };};

          rpnt.cvf[index[i][j]] = sigma;
         };};

         ArtePointer<RPNT> Prpnt = ArteTable<RPNT>::push_back(rpnt);
         createRelation( rtra, Prpnt, &RPNT::rtra );

        };
      };

       return 0;      
     };
//********************************************************************
// function
//********************************************************************

int rcSetTrackPointfromRHIT(ArtePointer<RTRA> rtra, const float z) {

  const int index[5][5] = { {0, 1,  3, 6,10},
                            {1, 2,  4, 7,11},
                            {3, 4,  5, 8,12},
                            {6, 7,  8, 9,13},
                            {10,11,12,13,14}};

//-------------------------------------------------------------------
// find the closest RHIT
//-------------------------------------------------------------------

  ArtePointer<RHIT> rhit = 0;

  for(ArteTable<RHIT>::iterator rhit1 = ArteTable<RHIT>::begin();
      rhit1 != ArteTable<RHIT>::end(); rhit1++){

     if ( ArtePointer<RTRA>((*rhit1)->rtra) == rtra &&
         (*rhit1)->hitb->z < z && 
	 (rhit == 0 || ( (*rhit1)->hitb->z > rhit->hitb->z))) 
         rhit = *rhit1;

    };


//-------------------------------------------------------------------
//
//-------------------------------------------------------------------

    if (rhit != 0){


      double zout = double(z);
      
      const int    icovar=2;    
      double       prec[4];
      double       derivatives[5][5]={};
      double       p_track[6], parameters[6];
      double       fstep=50.;
      
      parameters[0] = double(rhit->hitb->z);
      parameters[1] = double(rhit->x);
      parameters[2] = double(rhit->y);
      parameters[3] = double(rhit->tx);
      parameters[4] = double(rhit->ty);

      if (rhit->p == 0.0) {
       parameters[5] = double(rtra->pf);
      } else {
       parameters[5] = double(rhit->p);
      };


      if (zout != parameters[0]) {
	
	int    ierror;
	prec[0] = 0.001/fabs(zout-parameters[0]);
	prec[1] = prec[0];
	prec[2] = 0.0001/fabs(zout-parameters[0]);
	prec[3] = prec[2];
      
	rkmain_(icovar, zout, fstep, parameters, prec, p_track, 
		& derivatives[0][0], ierror);

        if (ierror == 0) {
         RPNT rpnt;

         rpnt.zf  = float(p_track[0]);
         rpnt.xf  = float(p_track[1]);
         rpnt.yf  = float(p_track[2]);
         rpnt.txf = float(p_track[3]);
         rpnt.tyf = float(p_track[4]);
         rpnt.pf  = float(p_track[5]);


//-------------------------------------------------------------------
// now estimate the error
//-------------------------------------------------------------------


         parameters[0] = double(rtra->zf);
         parameters[1] = double(rtra->xf);
         parameters[2] = double(rtra->yf);
         parameters[3] = double(rtra->txf);
         parameters[4] = double(rtra->tyf);
         parameters[5] = double(rtra->pf);

 	 rkmain_(icovar, zout, fstep, parameters, prec, p_track, 
	     	 & derivatives[0][0], ierror);

        if (ierror == 0) {

         rpnt.cvf[index[4][0]] = 0.0;
         rpnt.cvf[index[4][1]] = 0.0;
         rpnt.cvf[index[4][2]] = 0.0;
         rpnt.cvf[index[4][3]] = 0.0;
         rpnt.cvf[index[4][4]] = rtra->cvf[index[4][4]];

         for (int i=0; i<=3 ; i++){
         for (int j=i; j<=3 ; j++){

          float sigma = 0.0;
	  for (int k=0; k<=3; k++) { for (int l=0; l<=3; l++){
           sigma = sigma + float(derivatives[k][i])
                         * rtra->cvf[index[k][l]]
                         * float(derivatives[l][j]);
          };};

          rpnt.cvf[index[i][j]] = sigma;
         };};

	};


//-------------------------------------------------------------------
// store point
//-------------------------------------------------------------------

         ArtePointer<RPNT> Prpnt = ArteTable<RPNT>::push_back(rpnt);
         createRelation( rtra, Prpnt, &RPNT::rtra );

        };
      };

     };
       return 0;      
};
