
// *****************************************************************************
// 
//  Author:  Rainer Mankel                     Created:  17-Apr-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
// rcevnt.C
// reconstruction main program
// dispatch selecting the appropriate main program according to
// time or geometry version
//
// changes:
// 17 Dec 1999 (AG) introduce timing
//  8 Feb 2000      reconstruction module is selected by flags   T.Oest
//
// *****************************************************************************

// standard includes
#include <math.h>

// ONLINE includes
#include "rhp.h"

// ARTE includes
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "reco/rcflags.hh"
#include "reco/reco.hh"
#include "reco/recoOptions.hh"

// prototypes
extern "C" void rcZevnt_();

unsigned int acFlag();

extern recoOptions RecoOpt;

//
// program code
//
extern "C" void
rcevnt_()
{
  //-------------------------------------------------------------------
  //    initialize the reco error flag
  //-------------------------------------------------------------------

  if (recoFlag != 0) rcerror("init");

  //-------------------------------------------------------------------
  //    create new EVRC bank if reco is switched on 
  //-------------------------------------------------------------------

  ArtePointer<EVRC> evrc;

  if (recoFlag != 0) {
    int size = ArteTable<EVRC>::size();
    mmbook_( ArtePointer<EVRC>::getLink().getRef(), 1 );
    evrc = ArtePointer<EVRC>( size + 1 );
    evrc->ipaddr   = arteflag_.ipaddr;
    evrc->errmask  = 0x0;
    evrc->date     = time( NULL );
    evrc->cputime  = 0.0;
    evrc->alca     = acFlag();
    evrc->artevers = arteflag_.artevers;
  };

  //-------------------------------------------------------------------
  //     start timing
  //-------------------------------------------------------------------
  static Chrono *cron_rcevnt = 0;
  if( !cron_rcevnt ){
    ChronoManager CM;
    cron_rcevnt = CM.createChrono( "cron_rcevnt" );
  }
  cron_rcevnt->start();

  //-------------------------------------------------------------------
  //     call event reconstruction
  //-------------------------------------------------------------------

  if (recoFlag != 0) {
      rc99Aevnt();
  } else {
      if (RecoOpt.CheckActiveAlg("RITER")  || 
          RecoOpt.CheckActiveAlg("CATS")   ||
          RecoOpt.CheckActiveAlg("HOLMES") ||
          RecoOpt.CheckActiveAlg("MARPLE") ||
          RecoOpt.CheckActiveAlg("RIRE")   ||
          RecoOpt.CheckActiveAlg("RITER")  ||
          RecoOpt.CheckActiveAlg("VERTEX"))  rcZevnt_();
  };

  //-------------------------------------------------------------------
  //     stop timing
  //-------------------------------------------------------------------
  cron_rcevnt->stop();
  float cputime = cron_rcevnt->lastTime();
  //cout << "rcevnt_(): cputime=" << cputime << " sec" << endl;
  if( DQ_flag( COMP_FARM, LVL_M_HIT ) )
    rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 5 ), cputime, 1 );
  
  //-------------------------------------------------------------------
  //     fill in EVRC bank 
  //-------------------------------------------------------------------
  //faFillEVRC( cputime ); // > ARTE-03-05-r1

  if (recoFlag != 0) evrc->cputime = cputime;
  if (recoFlag != 0) evrc->errmask = recoErrorFlag();

}
