// ********************************************************************
//
//  Routine:      rcinventRTRA
//  Author:       T.Oest
//
//  Description:  select and store RTRA tracks based on VDS/main tracker
//
//  Modification:
//  -- rcSetRtraHitRelations: set RTRA-HITC relations only 
//     for charged tracks                               T.Oest 28/03/00
//
// ********************************************************************

#include <iostream>
#include <set>
#include <math.h>

#include <arte/arte.hh>
#include <arte/RSEG_cmp.hh>

int rcinventRtraVdsOtr();
int rcinventRtraSingleSegment();
int rcinventRtraOtr();
int rcinventRtraNeutral();
int rcCountRtraHits();
int rcSetRtraHitRelations();
int rcRTRAmatchMTRA();

//*****************************************************************
// function
//*****************************************************************

int rcinventRTRA(){

 rcinventRtraVdsOtr();
 rcinventRtraSingleSegment();
 rcinventRtraNeutral();
 rcSetRtraHitRelations();
 rcCountRtraHits();

 rcRTRAmatchMTRA();

return 0;

};

//*****************************************************************
// function
//*****************************************************************

int rcinventRtraVdsOtr(){

  RTRA rtra;
  
  rtra.chi = 0.0;
  
  rtra.flr = 0;
  rtra.flt = 0;
  rtra.fle = 0;
  rtra.flm = 0;
  rtra.fid = 0;
  
  rtra.lmmu = 0;
  rtra.lmh  = 0;

  rtra.lte = 0;
  rtra.lth = 0;

  rtra.lee = 0;
  rtra.leh = 0;
  rtra.lemi= 0;
  
  rtra.lre = 0.;
  rtra.lrmu= 0.;
  rtra.lrpi= 0.;
  rtra.lrk = 0.;
  rtra.lrp = 0.;
  
  rtra.le  = 0.;
  rtra.lmu = 0.;
  rtra.lpi = 0.;
  rtra.lk  = 0.;
  rtra.lp  = 0.;

  rtra.hca = 0;
  rtra.hsi = 0;
  rtra.hin = 0;
  rtra.hou = 0;
  rtra.hri = 0;
  rtra.htr = 0;
  rtra.hmu = 0;


//==================================================================
// 1. loop over RSEG, select VDS/main tracker matches
//==================================================================

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

    if ((*rseg)->fit == 7 && 
        ((*rseg)->cmp & Rsegc::vxd)  != 0 &&
        ((*rseg)->cmp & Rsegc::magt) != 0 ){


    ArtePointer<RSEG> vds     = 0;
    ArtePointer<RSEG> otr     = 0;

    for( ArteRelationIterator<RSEG> rseg2 = rseg->rend.begin();
	 rseg2 != rseg->rend.end(); ++rseg2 ) {
      if( ((*rseg2)->cmp & Rsegc::vxd) != 0 )  vds = *rseg2;
      if( ((*rseg2)->cmp & Rsegc::magt) != 0 ) otr = *rseg2;
    }


//==================================================================
// 2. store track parameter
//==================================================================
  
  rtra.fit = 0;

  rtra.xf  = vds->xf;
  rtra.yf  = vds->yf;
  rtra.zf  = vds->zf;
  rtra.txf = vds->txf;
  rtra.tyf = vds->tyf;
  rtra.pf  = otr->pf;

  const int index[5][5] = { {0, 1,  3, 6,10},
			    {1, 2,  4, 7,11},
			    {3, 4,  5, 8,12},
			    {6, 7,  8, 9,13},
			    {10,11,12,13,14}};
  
  for (int i=0; i<=3; i++){for (int j=0; j<=3; j++)
    rtra.cvf[index[i][j]] = vds->cvf[index[i][j]];};

  for (int i=0; i<=4; i++){
    rtra.cvf[index[i][4]] = 0.;
  };
  
  
  rtra.cvf[index[4][4]] = otr->cvf[index[4][4]];;

  ArtePointer<RTRA> Prtra = ArteTable<RTRA>::push_back( rtra );
  
//==================================================================
// 3. set links
//==================================================================

  createRelation( Prtra, *rseg, &RTRA::rseg );
  createRelation( Prtra,   vds, &RTRA::rseg );
  createRelation( Prtra,   otr, &RTRA::rseg );

  for( ArteRelationIterator<RCCL> rccl = otr->rccl.begin();
	 rccl != otr->rccl.end(); ++rccl ) {

     createRelation( Prtra, *rccl, &RTRA::rccl );

  };

 };};


//==================================================================

return 0;
};


//*****************************************************************
// function
//*****************************************************************

int rcinventRtraSingleSegment(){


  RTRA rtra;
  
  rtra.chi = 0.0;
  
  rtra.flr = 0;
  rtra.flt = 0;
  rtra.fle = 0;
  rtra.flm = 0;
  rtra.fid = 0;
  
  rtra.lmmu = 0;
  rtra.lmh = 0;

  rtra.lte = 0;
  rtra.lth = 0;

  rtra.lee = 0;
  rtra.leh = 0;
  rtra.lemi= 0;
  
  rtra.lre = 0.;
  rtra.lrmu= 0.;
  rtra.lrpi= 0.;
  rtra.lrk = 0.;
  rtra.lrp = 0.;
  
  rtra.le  = 0.;
  rtra.lmu = 0.;
  rtra.lpi = 0.;
  rtra.lk  = 0.;
  rtra.lp  = 0.;

  rtra.hca = 0;
  rtra.hsi = 0;
  rtra.hin = 0;
  rtra.hou = 0;
  rtra.hri = 0;
  rtra.htr = 0;
  rtra.hmu = 0;


//==================================================================
// now select remaining VDS,OTR segments
//==================================================================

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

  if ((*rseg)->fit == 0 &&
      (*rseg)->rtra.begin() == (*rseg)->rtra.end() && 
       (((*rseg)->cmp & Rsegc::vxd)  != 0 ||
        ((*rseg)->cmp & Rsegc::magt) != 0 )){

    if (((*rseg)->cmp & Rsegc::vxd)  != 0) {
      rtra.fit = 11;
    } else {
      rtra.fit = 0;
    };

    rtra.xf  = rseg->xf;
    rtra.yf  = rseg->yf;
    rtra.zf  = rseg->zf;
    rtra.txf = rseg->txf;
    rtra.tyf = rseg->tyf;
    rtra.pf  = rseg->pf;

  
    for (int i=0; i<=14; i++)   rtra.cvf[i] = (*rseg)->cvf[i];

    ArtePointer<RTRA> Prtra = ArteTable<RTRA>::push_back( rtra );
  
    createRelation( Prtra, *rseg, &RTRA::rseg );

    for( ArteRelationIterator<RCCL> rccl = (*rseg)->rccl.begin();
	 rccl != (*rseg)->rccl.end(); ++rccl ) {
     createRelation( Prtra, *rccl, &RTRA::rccl );
    };


};};

//==================================================================

return 0;
};
//*****************************************************************
// function
//*****************************************************************

#include "care/ECALDetector.hh"

int rcinventRtraNeutral(){


  ECALDetector* ecal = ECALDetector::instance();
  ecal->storeNeutrals();  // store neutral particles

//==================================================================

return 0;
};


//*****************************************************************
// function
//*****************************************************************

int rcCountRtraHits(){

//==================================================================
// 1. loop over RTRA 
//==================================================================

  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {

    if (rtra->fit == 0 || rtra->fit == 11 || rtra->fit == 6){

//---------------------------------------------------------------
// set link to ECAL hitc, fill hca
//---------------------------------------------------------------

  rtra->hca = 0;

  for( ArteRelationIterator<RCCL> rccl = rtra->rccl.begin();
       rccl != rtra->rccl.end(); ++rccl ) {

    rtra->hca = rtra->hca + int((*rccl)->wid);

  };


//==================================================================
// 4. hits from tracking chambers
//==================================================================

  int HitsSvd   = 0;
  int BitMapSvd = 0;
  int HitsOtr   = 0;
  int HitsItr   = 0;


    for( ArteRelationIterator<HITB> hitb = rtra->hitb.begin();
	 hitb != rtra->hitb.end(); ++hitb ) {

            if ((*hitb)->gede->cmp == 1) HitsSvd++;
            if ((*hitb)->gede->cmp == 2) HitsOtr++;
            if ((*hitb)->gede->cmp == 3) HitsItr++;

            if ((*hitb)->gede->cmp == 1) {
               int layer = 4 * (*hitb)->gede->gesl->nsl - 4 +
                               (*hitb)->gede->lay;
               BitMapSvd |= 0x00000001 << layer;
            };


     };

    (*rtra)->hsi = HitsSvd;
    (*rtra)->hou = HitsOtr;
    (*rtra)->hin = HitsItr;

    (*rtra)->bsi = BitMapSvd;

 };};

//==================================================================

return 0;
};
//*****************************************************************
// function
//*****************************************************************

int rcSetRtraHitRelations(){

//==================================================================
// 1. loop over RTRA 
//==================================================================

  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {

    if (rtra->fit == 0 || rtra->fit == 11){

//---------------------------------------------------------------
// 1. set link to ECAL hitc
//---------------------------------------------------------------

  for( ArteRelationIterator<RCCL> rccl = rtra->rccl.begin();
       rccl != rtra->rccl.end(); ++rccl ) {

    for( ArteRelationIterator<HITC> hitc = (*rccl)->hitc.begin();
         hitc != (*rccl)->hitc.end(); ++hitc ) {
     createRelation( *rtra, *hitc, &RTRA::hitc );
    };

  };

  /*
//==================================================================
// 2. hits from VDS
//==================================================================

    for( ArteRelationIterator<RSEG> rseg = rtra->rseg.begin();
	 rseg != rtra->rseg.end(); ++rseg ) {
     if ( (*rseg)->fit != 7 && 
          ((*rseg)->cmp & Rsegc::vxd) != 0) {
  
       ArteRelationIterator<HITB> hitb = (*rseg)->hitb.end();

       while ( hitb != (*rseg)->hitb.begin()) {

        ArteRelationIterator<HITB> hit2 = (*rseg)->hitb.begin();
        ArteRelationIterator<HITB> hit3 = (*rseg)->hitb.begin();

        while (++hit2 != hitb) {++hit3;};
        hitb = hit3;

        createRelation( *rtra, *hitb, &RTRA::hitb );
       };

      };
     };


//==================================================================
// 2. hits from wire chambers
//==================================================================

    for( ArteRelationIterator<RSEG> rseg = rtra->rseg.begin();
	 rseg != rtra->rseg.end(); ++rseg ) {
     if ( (*rseg)->fit != 7 && 
          ((*rseg)->cmp & Rsegc::wch) != 0) {
  
       ArteRelationIterator<HITB> hitb = (*rseg)->hitb.end();

       while ( hitb != (*rseg)->hitb.begin()) {

        ArteRelationIterator<HITB> hit2 = (*rseg)->hitb.begin();
        ArteRelationIterator<HITB> hit3 = (*rseg)->hitb.begin();

        while (++hit2 != hitb) {++hit3;};
        hitb = hit3;

        createRelation( *rtra, *hitb, &RTRA::hitb );
       };

      };
     };

     */
//==================================================================
// 2. hits from wire chambers
//==================================================================

    for( ArteRelationIterator<RSEG> rseg = rtra->rseg.begin();
	 rseg != rtra->rseg.end(); ++rseg ) {
     if ( (*rseg)->fit != 7 && 
          ((*rseg)->cmp & Rsegc::wch) != 0) {

    int test = 0;

    for( ArteRelationIterator<HITB> hitb = (*rseg)->hitb.begin();
	 hitb != (*rseg)->hitb.end(); ++hitb ) {

     if ( (*hitb)->z < (*((*rseg)->hitb.begin()))->z ) test = 1;

    };


    if (test == 1){

       for( ArteRelationIterator<HITB> hitb = (*rseg)->hitb.begin();
	    hitb != (*rseg)->hitb.end(); ++hitb )
             createRelation( *rtra, *hitb, &RTRA::hitb );

    } else {

       ArteRelationIterator<HITB> hitb = (*rseg)->hitb.end();

       while ( hitb != (*rseg)->hitb.begin()) {

        ArteRelationIterator<HITB> hit2 = (*rseg)->hitb.begin();
        ArteRelationIterator<HITB> hit3 = (*rseg)->hitb.begin();

        while (++hit2 != hitb) {++hit3;};
        hitb = hit3;

        createRelation( *rtra, *hitb, &RTRA::hitb );
       };
    };

    };
    };
//==================================================================
// 3. hits from VDS
//==================================================================

    for( ArteRelationIterator<RSEG> rseg = rtra->rseg.begin();
	 rseg != rtra->rseg.end(); ++rseg ) {
     if ( (*rseg)->fit != 7 && 
          ((*rseg)->cmp & Rsegc::vxd) != 0) {
  
    int test = 0;

    for( ArteRelationIterator<HITB> hitb = (*rseg)->hitb.begin();
	 hitb != (*rseg)->hitb.end(); ++hitb ) {

     if ( (*hitb)->z < (*((*rseg)->hitb.begin()))->z ) test = 1;

    };


    if (test == 1){

       for( ArteRelationIterator<HITB> hitb = (*rseg)->hitb.begin();
	    hitb != (*rseg)->hitb.end(); ++hitb )
             createRelation( *rtra, *hitb, &RTRA::hitb );

    } else {

       ArteRelationIterator<HITB> hitb = (*rseg)->hitb.end();

       while ( hitb != (*rseg)->hitb.begin()) {

        ArteRelationIterator<HITB> hit2 = (*rseg)->hitb.begin();
        ArteRelationIterator<HITB> hit3 = (*rseg)->hitb.begin();

        while (++hit2 != hitb) {++hit3;};
        hitb = hit3;

        createRelation( *rtra, *hitb, &RTRA::hitb );
       };
    };

 };};

//==================================================================
};};


return 0;
};
