#include <stdio.h>
#include <string.h>
#include "rich/ArteTablesInit.h"
#include "rich/RICHDAQ.h"
#include "rich/vessel.h"
#include "rich/PhotonDetection.h"

ARTE_INIT_FROM_DB* ArteInitFromDB=0;

// ------------------------------------------------------------------

ARTE_INIT_FROM_DB::ARTE_INIT_FROM_DB(){

  char dbsours[256];
  char* RICHdbsours=getenv("RICH_INSTALLDB");
  if (RICHdbsours) strcpy(dbsours,RICHdbsours);
  else {
    fprintf( stdout," $$$ RICH: Set environment variable RICH_INSTALLDB before starting the program ... \n");
    Exit(1);
  }
  tid = OpenTables(RICHdbsours, 0);  
  if (!tid) {  
    fprintf( stdout," $$$ RICH: Cannot open configuration database (RICH_INSTALLDB=%s defined correctly?) ... \n",RICHdbsours);
    Exit(1);
  }
}

// ------------------------------------------------------------------

ARTE_INIT_FROM_DB::~ARTE_INIT_FROM_DB(){

  // Close the database. 
  CloseTables(tid);
  //printf ("Database is closed\n");
  }

// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CPMDInitialization() {

int entr_num=BookTable(tid, SearchTable(tid, 0, "CPMD", ""), 0, 0); // Book the Table CPMD
//Put the TableStamp on the entry which has a maximal major and minor numbers:
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
// Match the entry with the maximal entry number e.g. for looping through the
// whole table:
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid); // Minor number is not essential in the installation DB
ArtePointer<CPMD> a;
CPMD cpmd;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CPMD=%d\n", n);
  char sen[6][n], typ[12][n];
  int csup[n], row[n], col[n], clen[n], efv[n];
   for (j=0;j<6;j++) GetTableField(tid,TABLEascii  ,n, sen[j]);
   for (j=0;j<12;j++) GetTableField(tid,TABLEascii  ,n, typ[j]);        
   GetTableField(tid,TABLEint  ,n,csup);
   GetTableField(tid,TABLEint  ,n,row);
   GetTableField(tid,TABLEint  ,n,col);
   GetTableField(tid,TABLEint  ,n,clen);
   GetTableField(tid,TABLEint  ,n,efv);
   for (i=0; i<n; i++){
	for (j=0; j<6; j++) cpmd.seno[j]=sen[j][i];
	for (j=6; j<16; j++) cpmd.seno[j]='\40';
	for (j=0; j<12; j++) cpmd.type[j]=typ[j][i];
	for (j=12; j<16; j++) cpmd.type[j]='\40';
     cpmd.isup=csup[i];
     cpmd.irow=row[i];
     cpmd.icol=col[i];
     cpmd.lens=clen[i];
     cpmd.efv0=efv[i];
   a=ArteTable<CPMD>::push_back(cpmd);
//printf("%s,%s,%d,%d,%d,%d,%d\n",cpmd.seno,cpmd.type,cpmd.isup,cpmd.irow,cpmd.icol,cpmd.lens,cpmd.efv0);
 }
return 1;
}
// ------------------------------------------------------------------
int ARTE_INIT_FROM_DB::CPMTInitialization() {
 int entr_num=BookTable(tid, SearchTable(tid, 0, "CPMT", ""), 0, 0);
 TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
 MatchTable(tid, entr_num, 2);
 //int major=TableMajor(tid);
 //int minor=TableMinor(tid);
 ArtePointer<CPMT> a;
 CPMT cpmt;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CPMT=%d\n", n);
  char typ[12][n];
  int cnx[n], cny[n], cneff[n], cieff[n];
  float cxdiv[n], cydiv[n];
   for (j=0;j<12;j++) GetTableField(tid,TABLEascii  ,n, typ[j]);        
   GetTableField(tid,TABLEint  ,n,cnx);
   GetTableField(tid,TABLEfloat  ,n,cxdiv);
   GetTableField(tid,TABLEint  ,n,cny);
   GetTableField(tid,TABLEfloat  ,n,cydiv);
   GetTableField(tid,TABLEint  ,n,cneff);
   GetTableField(tid,TABLEint  ,n,cieff);
   for (i=0; i<n; i++){
	for (j=0; j<12; j++) cpmt.type[j]=typ[j][i];
	for (j=12; j<16; j++) cpmt.type[j]='\40';
     cpmt.nx=cnx[i];
     cpmt.xdiv=cxdiv[i];
     cpmt.ny=cny[i];
     cpmt.ydiv=cydiv[i];
     cpmt.neff=cneff[i];
     cpmt.ieff=cieff[i];
     a=ArteTable<CPMT>::push_back(cpmt);
//printf("%s,%d,%f,%d,%f,%d,%d\n",cpmt.type,cpmt.nx,cpmt.xdiv,cpmt.ny,cpmt.ydiv,cpmt.neff,cpmt.ieff);
 }
return 1;
}
// ------------------------------------------------------------------
int ARTE_INIT_FROM_DB::CSUPInitialization() {
 
int entr_num=BookTable(tid, SearchTable(tid, 0, "CSUP", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid); 
ArtePointer<CSUP> a;
CSUP csup;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CSUP=%d\n", n);
  float xx0[3][n], xxvec[3][n], yyvec[3][n], xxdiv[n], yydiv[n];
  int iid[n],nnx[n],nny[n];
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, xx0[j]);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, xxvec[j]);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, yyvec[j]);
   GetTableField(tid,TABLEint  ,n,nnx);
   GetTableField(tid,TABLEfloat  ,n,xxdiv);
   GetTableField(tid,TABLEint  ,n,nny);
   GetTableField(tid,TABLEfloat  ,n,yydiv);
   GetTableField(tid,TABLEint  ,n,iid);
   for (i=0; i<n; i++){
     for (j=0;j<3;j++) csup.x0[j]=xx0[j][i];
     for (j=0;j<3;j++) csup.xvec[j]=xxvec[j][i];
     for (j=0;j<3;j++) csup.yvec[j]=yyvec[j][i];
     csup.nx=nnx[i];
     csup.xdiv=xxdiv[i];
     csup.ny=nny[i];
     csup.ydiv=yydiv[i];
     csup.id=iid[i];
     a=ArteTable<CSUP>::push_back(csup);
//printf("%f,%f,%f,%f,%f,%f,%f,%f,%f,%d,%f,%d,%f,%d\n",csup.x0[0],csup.x0[1],csup.x0[2],csup.xvec[0],csup.xvec[1],csup.xvec[2],csup.yvec[0],csup.yvec[1],csup.yvec[2],csup.nx,csup.xdiv,csup.ny,csup.ydiv,csup.id);
 }
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CLENInitialization() {
int entr_num=BookTable(tid, SearchTable(tid, 0, "CLEN", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid);
ArtePointer<CLEN> a;
CLEN clen;
  int n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CLEN=%d\n", n);
  int lid[n], lnwav[n], liwav[n], lnang[n], liang[n];
   GetTableField(tid,TABLEint  ,n,lid);
   GetTableField(tid,TABLEint  ,n,lnwav);
   GetTableField(tid,TABLEint  ,n,liwav);
   GetTableField(tid,TABLEint  ,n,lnang);
   GetTableField(tid,TABLEint  ,n,liang);
  for (i=0; i<n; i++){
   clen.id=lid[i];
   clen.nwav=lnwav[i];
   clen.iwav=liwav[i];
   clen.nang=lnang[i];
   clen.iang=liang[i];
   a=ArteTable<CLEN>::push_back(clen);
  //printf("%d,%d,%d,%d,%d,\n",clen.id,clen.nwav,clen.iwav,clen.nang,clen.iang);
 }
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CEFFInitialization() {

//tablename=SearchTable(tid, 0, "CEFF", ""); 
int entr_num=BookTable(tid, SearchTable(tid, 0, "CEFF", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid); 
ArtePointer<CEFF> a;
CEFF ceff;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  int n=TableFieldSize(tid);
  //printf("Fieldsize CEFF=%d\n", n);
  float chaneff[n];

   GetTableField(tid,TABLEfloat  ,n,chaneff);
     for (int i=0;i<n;i++){
        ceff.eff=chaneff[i];
        a=ArteTable<CEFF>::push_back(ceff);
        //printf("chaneff[%d]=%f\n",i, chaneff[i]);
        }
 return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CPOIInitialization() {

//tablename=SearchTable(tid, 0, "CPOI", ""); 
int entr_num=BookTable(tid, SearchTable(tid, 0, "CPOI", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid); 
ArtePointer<CPOI> a;
CPOI cpoi;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  int n=TableFieldSize(tid);
  float arg[n],func[n];
  //printf("Fieldsize CPOI=%d\n", n);
   GetTableField(tid,TABLEfloat  ,n,&arg);
   GetTableField(tid,TABLEfloat  ,n,&func);
   for (int i=0; i<n; i++){
     cpoi.x[0]=arg[i];
     cpoi.x[1]=func[i];
     a=ArteTable<CPOI>::push_back(cpoi);
     //printf("%d arg=%e,func=%f\n",i,arg[i],func[i]);
     }
return 1;
}

// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CVESInitialization() {
int entr_num=BookTable(tid, SearchTable(tid, 0, "CVES", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid); 
ArtePointer<CVES> a;
CVES cves;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CVES=%d\n", n);
float vpori[3][n],vpez[3][n],vprad[n],vplen[n],veori[3][n],veez[3][n],verad[n],velen[n];
int virad[n];
    for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, vpori[j]);
    for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, vpez[j]);
    GetTableField(tid,TABLEfloat  ,n,vprad);
    GetTableField(tid,TABLEfloat  ,n,vplen);
    for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, veori[j]);
    for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, veez[j]);
    GetTableField(tid,TABLEfloat  ,n,verad);
    GetTableField(tid,TABLEfloat  ,n,velen);
    GetTableField(tid,TABLEint  ,n,virad);
   for (i=0; i<n; i++){
    for (j=0;j<3;j++) cves.pori[j]=vpori[j][i];
    for (j=0;j<3;j++) cves.pez[j]=vpez[j][i];
    cves.prad=vprad[i];
    cves.plen=vplen[i];
    for (j=0;j<3;j++) cves.eori[j]=veori[j][i];
    for (j=0;j<3;j++) cves.eez[j]=veez[j][i];
    cves.erad=verad[i];
    cves.elen=velen[i];
    cves.irad=virad[i];
   a=ArteTable<CVES>::push_back(cves);
 //printf("%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d\n",cves.pori[0],cves.pori[1],cves.pori[2],cves.pez[0],cves.pez[1],cves.pez[2],cves.prad,cves.plen,cves.eori[0],cves.eori[1],cves.eori[2],cves.eez[0],cves.eez[1],cves.eez[2],cves.erad,cves.elen,cves.irad);
}
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CSMIInitialization() {
int entr_num=BookTable(tid, SearchTable(tid, 0, "CSMI", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid);
ArtePointer<CSMI> a;
CSMI csmi;
  int n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CSMI=%d\n", n);
float smah[n], smah0[n], smav[n], smav0[n], smr[n], smrpc[n], smrec[n];
int smnref[n], smiref[n];
   GetTableField(tid,TABLEfloat  ,n,smah);
   GetTableField(tid,TABLEfloat  ,n,smah0);
   GetTableField(tid,TABLEfloat  ,n,smav);
   GetTableField(tid,TABLEfloat  ,n,smav0);
   GetTableField(tid,TABLEfloat  ,n,smr);
   GetTableField(tid,TABLEfloat  ,n,smrpc);
   GetTableField(tid,TABLEfloat  ,n,smrec);
   GetTableField(tid,TABLEint  ,n,smnref);
   GetTableField(tid,TABLEint  ,n,smiref);
  for (i=0; i<n; i++){
   csmi.ah=smah[i];
   csmi.ah0=smah0[i];
   csmi.av=smav[i];
   csmi.av0=smav0[i];
   csmi.r=smr[i];
   csmi.rpc=smrpc[i];
   csmi.rec=smrec[i];
   csmi.nref=smnref[i];
   csmi.iref=smiref[i];
   a=ArteTable<CSMI>::push_back(csmi);
 //printf("%f,%f,%f,%f,%f,%f,%f,%d,%d\n",csmi.ah,csmi.ah0,csmi.av,csmi.av0,csmi.r,csmi.rpc,csmi.rec,csmi.nref,csmi.iref);
}
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CSSMInitialization() {
int entr_num=BookTable(tid, SearchTable(tid, 0, "CSSM", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid);
ArtePointer<CSSM> a;
CSSM cssm;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CSSM=%d\n", n);
char ssmtype[8][n],ssmname[10][n];
float ssmorig[3][n],ssmnorm[3][n],ssmxvec[3][n],ssmhdia[n],ssmcorn[18][n];
int ssmrow[n],ssmcol[n],ssmnref[n],ssmiref[n],ssmncor[n];
   GetTableField(tid,TABLEint  ,n,ssmrow);
   GetTableField(tid,TABLEint  ,n,ssmcol);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, ssmorig[j]);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, ssmnorm[j]);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, ssmxvec[j]);
   for (j=0;j<8;j++) GetTableField(tid,TABLEascii  ,n, ssmtype[j]);
   for (j=0;j<10;j++) GetTableField(tid,TABLEascii  ,n, ssmname[j]);        
   GetTableField(tid,TABLEfloat  ,n, ssmhdia);
   GetTableField(tid,TABLEint  ,n,ssmnref);
   GetTableField(tid,TABLEint  ,n,ssmiref);
   GetTableField(tid,TABLEint  ,n,ssmncor);
   for (j=0;j<18;j++) GetTableField(tid,TABLEfloat  ,n, ssmcorn[j]);
  for (i=0; i<n; i++){
   cssm.row=ssmrow[i];
   cssm.col=ssmcol[i];
   for (j=0;j<3;j++) cssm.orig[j]=ssmorig[j][i];
   for (j=0;j<3;j++) cssm.norm[j]=ssmnorm[j][i];
   for (j=0;j<3;j++) cssm.xvec[j]=ssmxvec[j][i];
   for (j=0;j<8;j++) cssm.type[j]=ssmtype[j][i];
   for (j=0;j<10;j++) cssm.name[j]=ssmname[j][i];        
   for (j=10;j<20;j++) cssm.name[j]='\40';        
   cssm.hdia=ssmhdia[i];
   cssm.nref=ssmnref[i];
   cssm.iref=ssmiref[i];
   cssm.ncor=ssmncor[i];
   for (j=0;j<18;j++) cssm.corn[j]=ssmcorn[j][i];
   a=ArteTable<CSSM>::push_back(cssm);
  //printf("%d,%d,%s,%s\n,%f\t,%f\t,%f\n,%f\t,%f\t,%f\n,%f\t,%f\t,%f\n,%f\t,%f\t,%f\n,%f\t,%f\t,%f\n,%f\t,%f\t,%f\n\n",cssm.row,cssm.col,cssm.type,cssm.name,cssm.corn[0],cssm.corn[1],cssm.corn[2],cssm.corn[3],cssm.corn[4],cssm.corn[5],cssm.corn[6],cssm.corn[7],cssm.corn[8],cssm.corn[9],cssm.corn[10],cssm.corn[11],cssm.corn[12],cssm.corn[13],cssm.corn[14],cssm.corn[15],cssm.corn[16],cssm.corn[17]);
   }
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CPMIInitialization() {
int entr_num=BookTable(tid, SearchTable(tid, 0, "CPMI", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid); 
ArtePointer<CPMI> a;
CPMI cpmi;
  int n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CPMI=%d\n", n);
float pmrec[n];
int pmnref[n], pmiref[n];
   GetTableField(tid,TABLEfloat  ,n,pmrec);
   GetTableField(tid,TABLEint  ,n,pmnref);
   GetTableField(tid,TABLEint  ,n,pmiref);
  for (i=0; i<n; i++){
     cpmi.rec=pmrec[i];
	cpmi.nref=pmnref[i];
	cpmi.iref=pmiref[i];
   a=ArteTable<CPMI>::push_back(cpmi);
//printf("%f,%d,%d\n", cpmi.rec,cpmi.nref,cpmi.iref);
}
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CSPMInitialization() {
  int entr_num=BookTable(tid, SearchTable(tid, 0, "CSPM", ""), 0, 0);
  TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
  MatchTable(tid, entr_num, 2);
  //int major=TableMajor(tid);
  //int minor=TableMinor(tid); 
  ArtePointer<CSPM> a;
  CSPM cspm;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CSPM=%d\n", n);
  char sptype[8][n],spname[10][n];
  float sporig[3][n],spxvec[3][n],spyvec[3][n];
  int spnref[n], spiref[n], sprow[n], spcol[n];
  for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, sporig[j]);
  for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, spxvec[j]);
  for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, spyvec[j]);
  GetTableField(tid,TABLEint  ,n,spnref);
  GetTableField(tid,TABLEint  ,n,spiref);
  GetTableField(tid,TABLEint  ,n,sprow);
  GetTableField(tid,TABLEint  ,n,spcol);
  for (j=0;j<8;j++) GetTableField(tid,TABLEascii  ,n, sptype[j]);
  for (j=0;j<10;j++) GetTableField(tid,TABLEascii  ,n, spname[j]);        
  for (i=0; i<n; i++){
    for (j=0;j<3;j++) cspm.orig[j]=sporig[j][i];
    for (j=0;j<3;j++) cspm.xvec[j]=spxvec[j][i];
    for (j=0;j<3;j++) cspm.yvec[j]=spyvec[j][i];
    cspm.nref=spnref[i];
    cspm.iref=spiref[i];
    cspm.row=sprow[i];
    cspm.col=spcol[i];
    for (j=0;j<8;j++) cspm.type[j]=sptype[j][i];
    for (j=0;j<10;j++) cspm.name[j]=spname[j][i];        
    for (j=10; j<20; j++) cspm.name[j]='\0';
    a=ArteTable<CSPM>::push_back(cspm);

    /*cout<<cspm.row<<" "<<cspm.col<<" "
	<<cspm.name<<" | "<<cspm.type<<" | "
	<<cspm.orig[0]<<" "<<cspm.orig[1]<<" "<<cspm.orig[2]<<" | "
	<<cspm.xvec[0]<<" "<<cspm.xvec[1]<<" "<<cspm.xvec[2]<<" | "
	<<cspm.yvec[0]<<" "<<cspm.yvec[1]<<" "<<cspm.yvec[2]<<endl;*/
  }

  /*for (ArteTable<CSPM>::iterator icspm=ArteTable<CSPM>::begin();
       icspm!=ArteTable<CSPM>::end(); ++icspm) {
    ArtePointer<CSPM> mp=(*icspm);
    cout<<mp<<" "<<mp->row<<" "<<mp->col<<" "
	<<mp->name<<" "<<mp->type<<" "
	<<mp->orig[0]<<" "<<mp->orig[1]<<" "<<mp->orig[2]<<" | "
	<<mp->xvec[0]<<" "<<mp->xvec[1]<<" "<<mp->xvec[2]<<" | "
	<<mp->yvec[0]<<" "<<mp->yvec[1]<<" "<<mp->yvec[2]<<endl;
	}*/

  return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::CPDWInitialization() {
int entr_num=BookTable(tid, SearchTable(tid, 0, "CPDW", ""), 0, 0);
TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
MatchTable(tid, entr_num, 2);
//int major=TableMajor(tid);
//int minor=TableMinor(tid);
ArtePointer<CPDW> a;
CPDW cpdw;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CPDW=%d\n", n);
float worig[3][n],wxvec[3][n],wyvec[3][n],wt[n];
int wimat[n];
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, worig[j]);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, wxvec[j]);
   for (j=0;j<3;j++) GetTableField(tid,TABLEfloat  ,n, wyvec[j]);
   GetTableField(tid,TABLEfloat  ,n,wt);
   GetTableField(tid,TABLEint  ,n,wimat);
  for (i=0; i<n; i++){
   for (j=0;j<3;j++) cpdw.orig[j]=worig[j][i];
   for (j=0;j<3;j++) cpdw.xvec[j]=wxvec[j][i];
   for (j=0;j<3;j++) cpdw.yvec[j]=wyvec[j][i];
   cpdw.t=wt[i];
   cpdw.imat=wimat[i];
   a=ArteTable<CPDW>::push_back(cpdw);
 //printf("%f,%f,%f,%f,%f,%f,%f,%f,%f,%f,%d\n",cpdw.orig[0],cpdw.orig[1],cpdw.orig[2],cpdw.xvec[0],cpdw.xvec[1],cpdw.xvec[2],cpdw.yvec[0],cpdw.yvec[1],cpdw.yvec[2],cpdw.t,cpdw.imat);
 }
return 1;
}
// ------------------------------------------------------------------
int ARTE_INIT_FROM_DB::CDIMInitialization() {
 int entr_num=BookTable(tid, SearchTable(tid, 0, "CDIM", ""), 0, 0);
 TableStamp(tid, TABLEmaxrev, TABLEmaxrev);
 MatchTable(tid, entr_num, 2);
 //int major=TableMajor(tid);
 //int minor=TableMinor(tid); 
 ArtePointer<CDIM> a;
 CDIM cdim;
  int j, n, i;
  TableStamp    (tid, 1, 0);
  FetchTable(tid,entr_num,0);
  SkipTableField(tid,1);
  n=TableFieldSize(tid);
  //printf("Fieldsize CDIM=%d\n", n);
  char nname[10][n];
  float rrlos[n], ddens[n];
  int nnabs[n], iiabs[n], nnref[n], iiref[n];
    for (j=0;j<10;j++) GetTableField(tid,TABLEascii  ,n, nname[j]);
    GetTableField(tid,TABLEfloat  ,n, rrlos);
    GetTableField(tid,TABLEfloat  ,n, ddens);
    GetTableField(tid,TABLEint  ,n,nnabs);
    GetTableField(tid,TABLEint  ,n,iiabs);
    GetTableField(tid,TABLEint  ,n,nnref);
    GetTableField(tid,TABLEint  ,n,iiref);
   for (i=0; i<n; i++){
	for (j=0; j<10; j++) cdim.name[j]=nname[j][i];
	for (j=6; j<80; j++) cdim.name[j]='\40';
     cdim.rlos=rrlos[i];
     cdim.dens=ddens[i];
     cdim.nabs=nnabs[i];
     cdim.iabs=iiabs[i];
     cdim.nref=nnref[i];
     cdim.iref=iiref[i];
    a=ArteTable<CDIM>::push_back(cdim);
//printf("%f,%f,%d,%d,%d,%d\n",cdim.rlos,cdim.dens,cdim.nabs,cdim.iabs,cdim.nref,cdim.iref);
 }
 return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::ARTETablesInitialization() {
  CPMTInitialization();
  CPMDInitialization();
  CSUPInitialization();
  CLENInitialization();
  CEFFInitialization();
  CPOIInitialization();
  CVESInitialization();
  CSMIInitialization();
  CSSMInitialization();
  CPMIInitialization();
  CSPMInitialization();
  CPDWInitialization();
  CDIMInitialization();
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::RICHDetectorInitialization() {
  CPMTInitialization();
  CPMDInitialization();
  CSUPInitialization();
  CLENInitialization();
  CEFFInitialization();
  CPOIInitialization();
return 1;
}
// ------------------------------------------------------------------

int ARTE_INIT_FROM_DB::RICHVesselInitialization() {
  CPOIInitialization();
  CVESInitialization();
  CSMIInitialization();
  CSSMInitialization();
  CPMIInitialization();
  CSPMInitialization();
  CPDWInitialization();
  CDIMInitialization();
return 1;
}

//***********************************************************************
