// -*-C++-*-
#ifndef _RUNUTILITY_H_
#define _RUNUTILITY_H_

#include "rich/vessel.h"
#include "rich/PhotonDetection.h"
#include "rich/PhiLambda.h"

struct check {
  enum trace { ok,
	       noChannel,
	       noSM,
	       noPM };
};

class MirrorCheck {
private:
  SingleSphericalMirror* ssm;
  SinglePlaneMirror*     spm;
  Vec3                   xsm;
  Vec3                   xpm;
  int calc(StraightLine p);
public:
  /// Empty constructor
  MirrorCheck();
  /// Empty destructor
  ~MirrorCheck();
  /** Calculates reflexions on mirrors. Hit mirrors and
      coordinates are accessed via inline functions.
      Calculation assumes ray from origin.*/
  int calcFromTrack(float n[3]);
  int calcFromID(int id);
  /// Get sperical mirror.
  inline SingleSphericalMirror* sm()  { return ssm; }
  /// Get spherical mirror coordinate.
  inline Vec3                   smX() { return xsm; }
  /// Get plane mirror.
  inline SinglePlaneMirror*     pm()  { return spm; };
  /// Get plane mirror coordinate.
  inline Vec3                   pmX() { return xpm; }
};


#endif
