//--*-C++-*--
#ifndef __Ring_H__
#define __Ring_H__

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <time.h>
#include <vector.h>
#include <iostream.h>
#include "rich/ricutil.h"
#include "rich/PhiLambda.h"

#ifdef NO_ARTE
#include "RichHit.h"
#endif


#define MAXHITS        10000
#define MAXHITSPERRING 1000
#define MAXRINGS       500


extern float scaling;
extern float milliradian;
extern float setdualfac;


/////////////////////  fit description  ////////////////////////////////

struct RingSearchType {
  enum desc {
    standalone   =   0x1,   // stand alone ring search
    seed         =   0x2,   // track based
    nobg         =   0x4,   // old fitter (no background)
    lkbg         =   0x8,   // new fitter (background treated)
    seedrefit    =   0x10,  // use seed par in refit rather than ring search par
    bookrseg     =   0x20,  // book rseg
    bookhitb     =   0x40,  // book hitb
    fillrtra     =   0x80   // fill likelihoods and hits assigned to rich into rtra
  };
};


class RingSearchTrack {
public:
  static bool    exist;              // stand alone/track based
  static float   bx, by;             // impact parameters in meters !
  static float   phi, lambda;        // track direction in RICH!
  static float   ip;                 // inverse momentum*charge
  static ArtePointer<RTRA> rtra;     // RTRA row
  static float   max_phi_residual;   // seed residual
  static float   max_lambda_residual;// seed residual
  static float   aa;
  static float   bb;
  static float   sigma2_rr;
};

class RingSearchAttrib {
public:
  static int    type;         // fit type 
  static bool   reuse;        // reuse already found photons
  static bool   bookrseg;     // book rseg
  static bool   bookhitb;     // book rseg-hitb
  static bool   fillrtra;     // fill likelihoods and hits assigned to rich into rtra
  static float  SNcut;        // minimum S/Sqrt(N)
  static float  Scut;         // minimum significance (lkbg only)
};

/////////////////////  single hit ////////////////////////////////

typedef struct {
  float x,y;
  int   assigned,mctrack,plane,size;
#ifndef NO_ARTE
  ArtePointer<HITB> photon_hitb;
#else
  vector<RichHit>::iterator photon_hitb;
#endif
} Hit;



typedef struct corrHit {
  float x;
  float y;
  Hit*  h;   
};


/////////////////////  hit container /////////////////////////////

class Hits {
public:
  int numhit;
  Hit hit[MAXHITS];
  
  inline void reset() { numhit=0; }
  inline Hits()       { reset();  }
  inline void append(float xx,float yy,int plane,int size,int mctrack,
#ifdef USE_ARTE
ArtePointer<HITB> phitb
#else
vector<RichHit>::iterator phitb
#endif
) {
    if (numhit==MAXHITS) {
      printf("Hits::append(): Too many hits: numhit=%d>dimension=%d\n",
	     numhit,MAXHITS);
      return;
    }
    Hit& h=hit[numhit++];
    h.x=xx;
    h.y=yy;
    h.assigned=0;
    h.mctrack=mctrack;
    h.plane=plane;
    h.photon_hitb=phitb;
    h.size=size;
  }



  inline void randomize() {
    int i;
    Hit h;
    for (i=0;i<10*numhit;i++) {
      int n=int(rnd_normal()*numhit);
      int m=int(rnd_normal()*numhit);
      if (n!=m) {
	h.x            =hit[n].x;
	h.y            =hit[n].y;
	h.plane        =hit[n].plane;
	h.assigned     =hit[n].assigned;
	h.mctrack      =hit[n].mctrack;
	h.photon_hitb  =hit[n].photon_hitb;
	h.size         =hit[n].size;

	hit[n].x          =hit[m].x;
	hit[n].y          =hit[m].y;
	hit[n].plane      =hit[m].plane;
	hit[n].assigned   =hit[m].assigned;
	hit[n].mctrack    =hit[m].mctrack;
	hit[n].photon_hitb=hit[m].photon_hitb;
	hit[n].size       =hit[m].size;

	hit[m].x          =h.x;
	hit[m].y          =h.y;
	hit[m].plane      =h.plane;
	hit[m].assigned   =h.assigned;
	hit[m].mctrack    =h.mctrack;
	hit[m].photon_hitb=h.photon_hitb;
	hit[m].size       =h.size;
      }
    }
  }
  
  
  inline void print(FILE *fp=stdout) {
    int i;
    for (i=0;i<numhit;i++) {
      Hit& h=hit[i];
      fprintf(fp,"hit %4d %6.4f %6.4f %2d %2d %4d\n",
	      i,h.x,h.y,h.plane,h.assigned,h.mctrack);
    }
  }
};


/////////////////////// single ring ///////////////////////////////


class Ring  {
  double cov[3][3];
  int checkSN(Hits &h,float x0,float y0,float r0);


public:
  inline Ring()  { ; }
  float ix0;
  float iy0;
  float x0,y0;
  int numhit;
  Hit*    hit[MAXHITSPERRING];
  corrHit hit_list[MAXHITSPERRING];

  int   niter, nerro;
  float radius,numhit_refit,numhit_3sigma;
  float chi2,N,sigma,mu,f,numhit_bgrd,sigma_signal;
  float Rb, numhit_peak, refit_rms, refit_sn, av_accept;
  float search_area;
  float dphi_seed, dlam_seed;
  float refit_S, refit_Lbg, refit_L;
  int   fittype;
  bool  accept;

  // track
  inline float phi()    const { return x0*scaling; }
  inline float lambda() const { return y0*scaling; }
  inline float thetaC() const { return radius*scaling; }
  inline float covariance(int i,int j) const { return cov[i][j]*scaling*scaling; }
  inline float tx() const     { return tan(phi()); }
  inline float ty() const     { return tan(lambda()/cos(phi())); }

  // hits @@@
  inline float  hitPhi(int j)    const { return hit[j]->x*scaling; }
  inline float  hitLambda(int j) const { return hit[j]->y*scaling; }
  inline int    numHits()        const { return numhit; } 

  int  mctrack() const;
  int  calcring(Hits& hh);



  ///////////////// new fitter ///////////////////

  // -- prepare/finalize fit
  int            get_hits(Hits &hh);
  int            prepare();
  int            finalize();
  bool           IsCloseToSeed();

  // -- fit
  void           fitfun(int &npar, double *gin, double &f, double *par, int iflag);
  int            fitring(Hits &hh);
  int            check_limits();

};



///////////////////////  ring center ///////////////////////////////

class Center {
public:
  float x0;
  float y0;
  int   numhit;
  Hit*  hit[MAXHITSPERRING];
  inline Center() {
    x0=y0=0;
    numhit=0;
  }
};

/////////////////////// dual space /////////////////////////////////


class DualSpace {

private:
  Center center[300];
  Center *assignedcenter[300],*abovethresh[300];
  Ring   rings[MAXRINGS];

  int   threshold;
  float   r;
  float   r2;
  float   ri;

  int   nsqrtf;
  float sqrtbini;
  float sqrtf[10001];

  int     nphi;
  float   phibin;
  int     phitab[3001];

  int   numassigned,numabovethresh,numrings,ringfl;
  float dualfac,dualfaci;
  

  int findmax();
  void findring(Hit* hi,Hits* h);

public:

// ------------------------------------------------------------------------

  inline DualSpace() {
    memset(center,0,sizeof(center));
    numassigned=0;
    numrings=0;

    // @@@
    dualfac=setdualfac;                // 1/hit pos precision
    dualfaci=1/dualfac;
  }

// ------------------------------------------------------------------------
  inline void nextevent() {
    numassigned=0;
    numrings=0;
  }

  void define(float rad,int t);

// ------------------------------------------------------------------------


  inline float dualx(float x) const { return (dualfac*x); }
  inline float dualy(float y) const { return (dualfac*y); }
  inline float x(float dualx) const { return dualx*dualfaci; }
  inline float y(float dualy) const { return dualy*dualfaci; }

  inline int   numring()      const { return numrings; }
  inline Ring &getRing(int i)   {return rings[i];}

  inline int    mctrack(int i) const { return rings[i].mctrack(); }

// ------------------------------------------------------------------------

  inline void append(float dualx, float dualy, Hit* hit) {
    if (fabs(dualx)>.06/scaling) return;
    register int i=phitab[int(ri*(dualx+r))];
    if (dualy<0) i=nphi-i-1;
    if (i<0||i>3000) {
      printf("dualx %f %f %f\n",dualx,r,ri);
      //getchar();
      return;
    }
    register Center& c=center[i];
    c.x0+=dualx;
    c.y0+=dualy;
    c.hit[c.numhit]=hit;
    if (c.numhit++==0) assignedcenter[numassigned++]=&c;
    else if (c.numhit==threshold) ringfl=1;
  }
// ------------------------------------------------------------------------
  void ringsearch(Hits& h);
};


// ------------------------------------------------------------------------

#endif
