// *****************************************************************************
//
// source:    L3_SLP.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include "L3_SLP.hh"
//#ifdef L3_SLP_FRAME
//#define _BOOL
//#include "l2event.h"
//#undef _BOOL
//#endif

int L3_SLP::l2eventnr;
vector<L3_Track> L3_SLP::l2tracks;

void L3_SLP::init() {
  l2eventnr=0;
}

void L3_SLP::fill() {
  l2eventnr++;
}

void L3_SLP::clear() {
  l2tracks.clear();
}

void L3_SLP::setL2Tracks(vector<L3_Track> &v)
{
  l2tracks = v;
}

vector<L3_Track> L3_SLP::getL2Tracks() {
  return l2tracks;
}

int L3_SLP::getL2EventNr(){
  return l2eventnr;
}

void L3_SLP::setL2EventNr(int newnr){
  l2eventnr = newnr;
}

// ********************************************************************
//
// source:    PrintSltInfo.C
//
// type:      Example which shows SLT information
//
// author:    Brian Petersen
//
// needed:    link with $HBROOT/SLT/slt/Rev0500/$(BINTYPE)/libsltinfo.a
//            sltinfo.h is in $HBROOT/SLT/slt/Rev0500/include
//
// ********************************************************************
#if defined(L3_ARTE_FRAME) && defined(L3_SLT_INFO)
extern "C" {
#include "sltinfo.h"
}
void L3_SLP::PrintSltInfo()
{
   cout<<"*** Slt Info:***"<<endl;
   if (SLTReadDSLT()!=SLT_OK) {
     cout <<"No SLT information found"<<endl;
   }
   else {
     SLT_STATUS stat;
     SLT_TRIGGER trig;
     SLT_TRACK track;
     SLT_SEEDTRACK seed;
     SLT_SILITRACK sili;
     if (SLTStatus(&stat)!=SLT_OK) {
       cout <<"No status information"<<endl;
       return;
     }
     if (stat.status==SLTEVENT_REJECT) {
       cout <<"Event rejected"<<endl;
       return;
     }
     if (stat.status==SLTEVENT_BYPASS) {
       cout <<"Event was a FLTE BYPASS event!"<<endl;
       return;
     }
     cout<<"Event accepted with "<<stat.ntracks<<" tracks in ";
     cout<<stat.ntriggers<<" trigger pairs"<<endl;
     cout<<"-----Triggers:-------"<<endl;
     for(int i=0;i<stat.ntriggers;i++) {
       if (SLTTrigInfo(i,&trig)!=SLT_OK) {
	 cout<<"Error getting info on trigger number "<<i<<endl;
	 continue;
       }
       if (trig.status==SLTTRIG_REJECT) {
	 cout<<"Trigger "<<i<<" rejected"<<endl;
	 continue;
       }
       cout<<"Trigger "<<i<<" accepted"<<endl;
       for(int j=0;j<trig.ntracks;j++) {
	 if (SLTTrackInfo(trig.trackindex[j],&track)!=SLT_OK) {
	   cout<<"Error get info on track "<<trig.trackindex[j]<<endl;
	   continue;
	 }
	 switch(track.status) {
	 case SLTTRACK_ERROR:
	   cout<<"Error in data of track "<<j<<endl;
	   break;
	 case SLTTRACK_ACCEPT:
	   cout<<"Track "<<j<<" accepted:"<<endl;
	   break;
	 case SLTTRACK_REJECT_SILI:
	   cout<<"Track "<<j<<" not accepted L2Sili:"<<endl;
	   break;
	 case SLTTRACK_REJECT_OTHER:
	   cout<<"Track "<<j<<" rejected for unknown reason:"<<endl;
	   break;
	 }
	 if(track.status!=SLTTRACK_ERROR) {
	   if (SLTSeedTrackInfo(trig.trackindex[j],&seed)!=SLT_OK) {
	     cout<<"Error getting seed track for track"<<trig.trackindex[j];
	     cout<<endl;
	     continue;
	   }
	   cout<<"Seed energy:"<<seed.energy<<endl;
	   cout<<"Seed x,y,z:"<<seed.x2<<" "<<seed.y2<<" "<<seed.z2<<endl;
	   
	   if(track.status==SLTTRACK_ACCEPT) {
	     if (SLTSiliTrackInfo(trig.trackindex[j],&sili)!=SLT_OK) {
	       cout<<"Error getting L2sili track for track";
	       cout<<trig.trackindex[j]<<endl;
	       continue;
	     }
	     cout<<"slope,intersect,# hits and chisq in x: ";
	     cout<<sili.tx<<" "<<sili.bx<<" "<<sili.nhitx<<" "<<sili.chix<<endl;
	     cout<<"slope,intersect,# hits and chisq in y: ";
	     cout<<sili.ty<<" "<<sili.by<<" "<<sili.nhity<<" "<<sili.chiy<<endl;
	   }
	 }
       }
     }

   }
}
#else
void L3_SLP::PrintSltInfo(){
  cout << "L3_SLP::PrintSltInfo: no information available" << endl;
}
#endif
















