// *****************************************************************************
//
// source:    L3_Arte.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************
#ifndef _L3_ARTE_HH_
#define _L3_ARTE_HH_
#include "L3_Defines.hh"

class L3_Event;
class L3_Track;
#include "arte/arte.hh"

typedef struct ETLT {
  int wnr; //Nummer des Drahtes zu dem der Track den minimalen 
           //Impactparamter hat

  float imp1d;   //eindimensionaler Impactparameter
  float simp1d; //Fehler des eindim. Impactparameters

  float imp2d;   //zweidim. Impactparamter (<0.0 wenn kein Primaervertex auf
                 //dem Draht)
  float simp2d; //Fehler des zweidim. Impactparamters (<0.0 wenn kein
                 //Primaervertex auf dem Draht

  float chi2ndof; //Chi^2/ndof fuer den Track
  float fom;      //quality flag fuer den Track (figure of merit)
                  //Anzahl der Hits, die auch noch in anderen Tracks drin
                  //sind, um Geister zu erkennen usw.
  float theta;    //Winkel mit der z-Achse (man koennte daran denken,
                  //Tracks unter kleinen Winkeln, die ja hoeheren Impuls
                  //haben, staerker zu wichten.)
  int flag;       //noch ein Flag fuer besondere Zwecke
  inline void show(){ cout << wnr << ' ' << imp1d << "+-" << simp1d << ' ' << imp2d << "+-" << simp2d << ' ' << chi2ndof  
			   << ' ' << theta << endl; }
} ETLT;

class L3_Arte {
public:
  L3_Arte();
  L3_Arte(const L3_Arte&);
  L3_Arte& operator = (const L3_Arte&);
  void flush();
  void fill(const L3_Event&);

  static char* particleName(char *s);
  static ArtePointer<MTRA> particleOrigin(ArtePointer<MTRA> m);
  static char* Cstring(char *s,int size);
  static void hitb2svd( const ArtePointer<HITB> i,int *sl,int *sec=NULL,int *lay=NULL);
  static int svdHitsPresent(); //check whether this event has any svd HITB
  static int Btag(ArtePointer<MTRA> m,int depth=1);
  static int comesFrom(ArtePointer<MTRA>,char *name);
  static int fillDTLT(int *p,int anz);
  static int showDTLT();
private:
  ArtePointer<RSEG> track2rseg(L3_Track&);
  ArtePointer<RTRA> track2rtra(L3_Track&);
  void MakeRPNT(L3_Track&, ArtePointer<RSEG>, ArtePointer<RTRA> );

};

#endif
