#ifndef _L3_SILICON_HH_
#define _L3_SILICON_HH_
// *****************************************************************************
//
// source:    L3_Silicon.hh
//
// type:      include file
//
// created:   06 Dec 1999
//
// authors:   Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// changes:
//
// *****************************************************************************

#include "L3_Defines.hh"
#include <vector>
#include "L3_Hitb.hh"
#include "L3_SpacePoint.hh"
#include "L3_Track.hh"
#include "L3_Timer.hh"

// Utility structure for L3_Silicon::findValidTracks
struct TRACKinfo {
  TRACKinfo();
  TRACKinfo(double, int);
  double the_probability;
  int the_numHits;
};

class L3_Silicon {
public:
  inline L3_Silicon();
  L3_Silicon(const L3_Silicon&);
  L3_Silicon& operator = (const L3_Silicon&);
  int nHitbs() const;  //return number of HITBs 
  int nHitbs(int sl0,int sec0,int pl0) const;
  int nPoints() const; //return number of spacepoints
  inline int nTracks() const; //return number of reconstructed tracks
  vector<L3_Hitb> HList() const;  //return a new vector with all HITBs
  inline vector<L3_Hitb> Hitbs(const int& sl, const int& quad, const int& pl) const;
  vector<L3_SpacePoint> Points() const;  //return a new vector with all spacepoints 
  inline vector<L3_SpacePoint> Points(const int& sl, const int& quad) const;
  inline vector<L3_Track> Tracks() const; //return the vector of reconstructed tracks
  void reconstruct(L3_Timer&);
  void release();      //reset all lists....
  void FillHits();  
  void print() const;
  void printRecoSummary() const;
  vector<L3_Hitb> getUnusedHitbs() const;
  static const float AlphaS = 0.0436609429;
  void getSpacepointInfo(int*,int*,int*) const;
  void getTrackInfo(int*,int*,int*) const;
private:  
  vector<L3_Hitb> HitbList[7][4][4];
  vector<L3_SpacePoint> SpacePoint2List[7][4];
  vector<L3_SpacePoint> SpacePoint2XList[7][4];
  vector<L3_Track> TrackList;
  vector<L3_SpacePoint> SpacePointList[7][4];
  void SpacePointReco();
  void SpacePointRecoChi2();
  void SpacePairReco();
  void SpacePointAfterburner();
  void PurgeSpacePoints();  //discard spacepoints which are identical
  void PurgeSpacePointsOld();  //discard spacepoints which are identical
  void SortSpacePoints();
  void SortSpacePairs();
  void TrackReconstruction();
  void TrackReconstructionPrime();
  void TrackReconstruction98();
  int SearchNextSL(L3_Track&, const int&,int);
  int PropagateTrack(L3_Track&,int sl0);
  int PropagateTrackOld(L3_Track&,int sl0);
  static void findHits(int &from,int &to,vector<L3_SpacePoint> &l,L3_Window &SwinMean);
  static int findHits(vector<L3_SpacePoint> &l,L3_Window &SwinMean,vector<int> &IDs);
  int SearchNextSL98(L3_Track&, const int&, const int&, int, vector<L3_Track>&);
  void findValidTracks();
  void findValidTracksPrime();
  void print(const vector<L3_Hitb>&) const;
  void print(const vector<L3_SpacePoint>&) const;
  void print(const vector<L3_Track>&) const;
  void unused();
  void checkCompatibility(vector<L3_Track> &t);
#ifdef L3_ARTE_FRAME
  void fillTracksFromRseg();
#endif
  //some numbers to follow data flow
  int spc0,spc1,spc2; //number of spacepoints after 1st step, purge, afterburner
  int trk0,trk1,trk2; //number of tracks after reco, findvalidtracks, compatibility
};

#include "L3_Silicon.icc"

#endif
