#ifndef __WAX_DECODE_MANAGER_HH__
#define __WAX_DECODE_MANAGER_HH__

#ifndef __OTR_DECODE_H__
#include "OTrDecode.h"
#endif
#ifndef __WAX_BASICTYPES_HH__
#include "BasicTypes.hh"
#endif
#ifndef __STRING__
#include <string>
#endif
#ifndef __SGI_STL_VECTOR
#include <vector>
#endif

class waxDecodeManager ;
class waxDecodeAsd ;

class waxDecodeChannel
{
public:
  waxDecodeChannel(const OTrWire& chan) : fChannel(chan) {} 
  waxDecodeChannel() {}
  ~waxDecodeChannel() {}

  // decoding the wire structure //
  TdcCount_t     GetTdcCount()      const { return fChannel.dtime; }
  TdcCount_t     GetTZeroCount()    const { return fChannel.tzero; }
  TdcCount_t     GetTZeroErrCount() const { return fChannel.tzerr; }
  Status_t       GetStatus()        const { return fChannel.status; }
  GedeIndex_t    GetGedeIndex()     const { return fChannel.GEDE; }
  WireIndex_t    GetWireIndex()     const { return fChannel.cell; }
  AsdIndex_t     GetAsdIndex()      const { return fChannel.ASD; }
  ChannelIndex_t GetChannelIndex()  const { return fChannel.wire; }
  bool IsConnected() const { return fChannel.GEDE > 0 ; }

  const waxDecodeAsd* GetAsd() const ;
  FedIndex_t GetFedIndex() const ;
  ChannelIndex_t GetFedChannelIndex() const ;
  
  // decoding the tdccount
  DriftTime_t GetTZero()      const { return GetTZero(GetTZeroCount()); }
  DriftTime_t GetTZeroErr()   const { return GetTZeroErr(GetTZeroErrCount()) ; }
  DriftTime_t GetDriftTime()  const { return DriftTime_t(GetTdcCount()) - GetTZero() ; }
  void SetTdcCount(TdcCount_t t) { fChannel.dtime=t; }
  void MCSetTdcCount(TdcCount_t t) { if(t>fChannel.dtime) fChannel.dtime=t; }
  void SetStatus(Status_t st) { fChannel.status=st ; }
  void SetTZero(DriftTime_t tz, DriftTime_t tzerr) ;
  void SetTZeroCount(TdcCount_t tzc, TdcCount_t tzec) ;
  bool IsValidHit() const ;

  static DriftTime_t GetTZero(TdcCount_t tzc) { return float(tzc-128)/2 ; }
  static DriftTime_t GetTZeroErr(TdcCount_t tzec) { return float(tzec)/16 ; }

  static const waxDecodeChannel* GetChannel(AsdIndex_t asdid, ChannelIndex_t chid) ;
  static void SetDecodeManager(const waxDecodeManager* c) { fgDecodeManager=c ; }
private:
  OTrWire fChannel ;
  static const waxDecodeManager* fgDecodeManager ;
  friend class waxDecodeManager ;
} ;

class waxDecodeAsd
{
public:
  //typedefs
  typedef waxDecodeChannel* ChannelIterator ;
  typedef const waxDecodeChannel* ConstChannelIterator ;
 
  unsigned char GetNChannels() const { return 16 ; }
  ConstChannelIterator GetChannels() const { 
    return (ConstChannelIterator)(fAsd.wire) ; }
  ConstChannelIterator GetChannelsEnd() const { 
    return (ConstChannelIterator)(fAsd.wire +GetNChannels() ) ; }
  ConstChannelIterator GetChannel(ChannelIndex_t chId) const { 
    return GetChannels() + chId ; }
  ChannelIterator GetChannels()    { 
    return (ChannelIterator)(fAsd.wire) ; }
  ChannelIterator GetChannelsEnd() { 
    return (ChannelIterator)(fAsd.wire +GetNChannels() ) ; }

  std::string GetName() const ;
  int GetLayerIndex() const { return fAsd.Layer ; }
  int GetSuperLayerIndex() const { return fAsd.SuperLayer ; }
  int GetSectorIndex() const { return fAsd.Sector ; }
  int GetSubAsdIndex() const { return fAsd.ftb_ASD ; }
  FedIndex_t GetFedIndex() const { return fAsd.TDC_FED ; }
  int GetTdcConnectorIndex() const { return fAsd.TDC_Connector ; }
private:
  OTrASD8 fAsd ; 
  friend class waxDecodeManager ;
} ;

class waxDecodeTdc
{
public:
  CrateIndex_t GetCrateIndex() const { return fTdc.crate; }
  SlotIndex_t GetSlotIndex() const  { return fTdc.slot; }
  AsdIndex_t GetAsdIndex(int subasdid) const { return fTdc.ASD[subasdid] ; }
private:
  OTrTDC fTdc ; 
  friend class waxDecodeManager ;
} ;

class waxDecodeWire
{
public:
  waxDecodeWire() { fAsdId=MAXASD; fChannelId=0; }
  AsdIndex_t     GetAsdIndex()     const { return fAsdId ; }
  ChannelIndex_t GetChannelIndex() const { return fChannelId ; }
  const waxDecodeChannel* GetChannel() const ;
  bool IsConnected() const { return fAsdId != MAXASD ; }
private:
  AsdIndex_t fAsdId ;
  ChannelIndex_t fChannelId ;
  friend class waxDecodeManager ;
} ;

class waxDecodeGede
{
public:
  waxDecodeGede() ;
  typedef const waxDecodeWire* WireIterator ; 
  unsigned char GetNWires() const { return fWires[4]-fWires[0] ; }
  unsigned char GetNWires(int m)  const { return fWires[m+1]-fWires[m] ; }
  WireIterator GetWires()    const { return fWires[0] ; }
  WireIterator GetWiresEnd() const { return fWires[4] ; }
  WireIterator GetWire(WireIndex_t wid) const {
    WireIterator wire=fWires[wid/32] + wid%32;
    return ((wid/32)<4 && wire<fWires[(wid/32)+1]) ? wire : 0 ; }
private:
  waxDecodeWire* fWires[5] ;
  unsigned char fHVGroupIndex[8] ;
  friend class waxDecodeManager ;
} ;

class waxDecodeManager
{
public:
  // typedefs, enumerates
public:
  typedef vector<waxDecodeGede> GedeContainer ;
  typedef vector<waxDecodeWire> WireContainer ;
  typedef GedeContainer::const_iterator ConstGedeIterator ;
  typedef GedeContainer::iterator GedeIterator ;
  typedef vector<GedeIndex_t> GedeMapContainer ;
  typedef GedeMapContainer::const_iterator GedeMapIterator ;
  typedef const waxDecodeAsd*const* ConstAsdIterator ;
  typedef waxDecodeAsd*const* AsdIterator ;

  // constructors, destructor
  static waxDecodeManager* GetMe()  ;
  static waxDecodeManager* GetUncheckedMe() { return fgMe ; }
  static void DeleteMe() ;
  ~waxDecodeManager() ;
  
  // inlines
  AsdIterator GetAsds() ;
  ConstAsdIterator GetAsds() const ;
  const WireContainer& GetWires() const { return fWires ; }
  const GedeContainer& GetGedes() const { return fGedes ; }
  const GedeMapContainer& GetGedeMap() const { return fGedeMap ; }
  //int GetNGedes() const { return fGedes.size() ; }
  const waxDecodeGede* GetGede(GedeIndex_t gedeId) const ;
  const waxDecodeAsd* GetAsd(AsdIndex_t asdId) const ;
  waxDecodeAsd* GetAsd(AsdIndex_t asdId) ;
  GedeIndex_t GetMinGedeIndex() const { return fMinGedeIndex ; }
  const waxDecodeTdc* GetTdc(FedIndex_t fedId) const;
  OTR_Decode* GetDecodeTable() { return fDecodeTable ; }
  const OTR_Decode* GetDecodeTable() const { return fDecodeTable ; }

  // static
  static void SetListOfTablesFileName(std::string f) { fgListOfTablesFileName=f ; }
  static void SetTdcFedFileName(std::string f)       { fgTdcFedFileName=f ; }
  static void SetTZeroFileName(std::string f)        { fgTZeroFileName=f ; }
  static void SetMaskFileName(std::string f)         { fgMaskFileName=f ; }
  static void SetSwapCableFileName(std::string f)    { fgSwapCableFileName=f ; }
  
  // other methods
  void ReadTZeros() ;
  void WriteTZeros(const std::string&) ;
  void ReadStatus() ;
  int FillNewEvent() ;
  int GetNAsds() const ;
  waxDecodeAsd* GetAsd(std::string asdname) ;
  void SwapCables() ;
  void CreateDecodeGedes() ;
  void ResetTdcCounts() ;
  vector<FedIndex_t> GetActiveFedIds() const ;

private:
  waxDecodeManager() ;
  // GedeContainer& GetGedes() { return fGedes ; }
  waxDecodeTdc* GetTdc(FedIndex_t fedId) ;
private:
  
  OTR_Decode* fDecodeTable ;
  GedeIndex_t fMinGedeIndex ;
  WireContainer fWires ;
  GedeContainer fGedes ;
  vector<GedeIndex_t> fGedeMap ;
  
  static waxDecodeManager* fgMe ;
  static std::string fgListOfTablesFileName ;
  static std::string fgTdcFedFileName ;
  static std::string fgSwapCableFileName ;
  static std::string fgTZeroFileName ;
  static std::string fgMaskFileName ;
  friend class waxDecodeWire ;
  //map<decodeAsd::Id_t, decodeAsd*> fAsdMap ;
} ;

/*** inlines for waxDecodeChannel ****/
inline void waxDecodeChannel::SetTZeroCount(TdcCount_t tzc, TdcCount_t tzec) {
  fChannel.tzero=tzc; fChannel.tzerr=tzec; }

inline const waxDecodeChannel* waxDecodeWire::GetChannel() const { 
  return waxDecodeManager::fgMe->GetAsd(GetAsdIndex())->
    GetChannel(GetChannelIndex()) ; }

/********* inlines for waxDecodeManager *********/

inline waxDecodeManager::AsdIterator 
waxDecodeManager::GetAsds() {
  return (AsdIterator)(fDecodeTable->ASD8s) ; }

inline waxDecodeManager::ConstAsdIterator 
waxDecodeManager::GetAsds() const {
  return (ConstAsdIterator)(fDecodeTable->ASD8s) ; }

inline waxDecodeAsd*
waxDecodeManager::GetAsd(AsdIndex_t asdId) { 
  return GetAsds()[asdId] ; }
//(waxDecodeAsd*)(fDecodeTable->ASD8s + asdId) ; }

inline const waxDecodeAsd*
waxDecodeManager::GetAsd(AsdIndex_t asdId) const { 
  return GetAsds()[asdId] ; }
//  return (const waxDecodeAsd*)(fDecodeTable->ASD8s + asdId) ; }

inline const waxDecodeGede*
waxDecodeManager::GetGede(GedeIndex_t gedeId) const {
  const waxDecodeGede* gede = fGedes.begin() + gedeId - fMinGedeIndex ; 
  return (gede>=fGedes.begin() && gede<fGedes.end()) ? gede : 0 ;
}

inline const waxDecodeChannel* 
waxDecodeChannel::GetChannel(AsdIndex_t asdid, ChannelIndex_t chid) {
  return fgDecodeManager->GetAsd(asdid)->GetChannel(chid) ; }

inline const waxDecodeAsd* 
waxDecodeChannel::GetAsd() const {
  return fgDecodeManager->GetAsd(GetAsdIndex()) ; }

inline FedIndex_t
waxDecodeChannel::GetFedIndex() const {
  return GetAsd()->GetFedIndex() ; }

inline ChannelIndex_t
waxDecodeChannel::GetFedChannelIndex() const {
  return GetChannelIndex() + 16*(GetAsd()->GetTdcConnectorIndex()-1) ; }

inline const waxDecodeTdc* waxDecodeManager::GetTdc(FedIndex_t fedId) const {
  int tdcId=fDecodeTable->FEDs[fedId];
  if(tdcId==MAXFED) return 0;
  return (const waxDecodeTdc*)fDecodeTable->TDCs[tdcId] ; 
}

inline waxDecodeTdc* waxDecodeManager::GetTdc(FedIndex_t fedId) {
  int tdcId=fDecodeTable->FEDs[fedId];
  if(tdcId==MAXFED) return 0;
  return (waxDecodeTdc*)fDecodeTable->TDCs[tdcId] ; 
}

#endif
