#ifndef __WAX_HITB_HH__
#define __WAX_HITB_HH__

#ifndef __WAX_HIT_HH__
#include "Hit.hh"
#endif

class waxHITB
{
public:
  // constrcutors, destructor
  waxHITB(ArtePointer<HITB> hitb) : fHITB(hitb) {}
  static waxHITB* Create(ArtePointer<HITB> hitb) ;
  waxHITB() {}

  // data from hitb bank
  ArtePointer<HITB> GetHITB() const { return fHITB ; }
  GedeIndex_t GetGedeIndex() const { return fHITB->gede.index() ; }
  WireIndex_t GetWireIndex() const { 
    return waxHit::GetWireIndex(fHITB->chid) ; }
  AsdIndex_t GetAsdIndex() const { 
    return GetWire()->GetAsdIndex() ; }
  // return waxHit::GetAsdIndex(fHITB->chid) ; }
  ChannelIndex_t GetChannelIndex() const {
    return GetWire()->GetChannelIndex() ; }
  // return waxHit::GetChannelIndex(fHITB->chid) ; }
  
  const waxGede* GetGede() const {
    return waxHit::GetGedeManager()->GetGede(GetGedeIndex()) ; }
  float GetRawX() const { return fHITB->x ; }
  float GetRawZ() const { return fHITB->z ; }
  float GetRawY1() const { return GetGede()->GetRawY1(GetWireIndex()) ; }
  float GetRawY2() const { return GetGede()->GetRawY2(GetWireIndex()) ; }
  int GetMonoLayer() const { 
    return GetWireIndex()%(GetGede()->GetNMonoLayers()) ; }
  float GetDriftRadius() const { return fHITB->q ; }
  float GetDriftRadiusError() const { return fHITB->r ; }
  TdcCount_t GetTdcCount() const { return GetChannel()->GetTdcCount() ; } 
  bool IsOuterTracker() const { return fHITB->gede->cmp==2 ; }

  // data from wire structure (if availlable)
  const waxDecodeWire* GetWire() const {
    return waxDecodeManager::GetUncheckedMe()->GetGede(GetGedeIndex())->
      GetWire(GetWireIndex()) ; }
  const waxDecodeChannel* GetChannel() const {
    //return waxDecodeChannel::GetChannel(GetAsdIndex(),GetChannelIndex()) ; }
    return GetWire()->GetChannel() ; }
  DriftTime_t GetDriftTime() const { 
    return GetChannel()->GetDriftTime() ; }
  const waxRTRelation* GetRTRelation() const { 
    return GetGede()->GetRTRelation() ; }
  //   DriftTime_t GetTZero() const { 
  //     return GetChannel()->GetTZero() ; }
  
  ArtePointer<TAHB> AppendTAHB() const ;

private:
  ArtePointer<HITB> fHITB ;
} ;

#endif
