#ifndef __OTR_HIT_HH__
#define __OTR_HIT_HH__

/* virtual base class, for hits */

#ifdef USE_ARTE
 #ifndef __ARTE_HH__
 #include "arte/arte.hh"
 #endif
 typedef ArtePointer<HITB> ArtePointerHITB ;
#endif

class OTrHit
{
public:

  // setters and getter
  ArtePointerHITB GetHITB() const { return fHITB ; }
  
  // these must be implemented by child class
  virtual DriftTime_t GetTZeroCount() const = 0;
  virtual DriftTime_t GetTdcCount()   const = 0 ;
  virtual GedeIndex_t GetGedeIndex()  const = 0 ;
  virtual WireIndex_t GetWireNumber() const = 0 ;

  // decoding the tdccount
  DriftTime_t GetTZero()     const { return 0.25*(GetTZeroCount()-128); }
  DriftTime_t GetDriftTime() const { return GetTdcCount() + GetTZero() ; }
  
  // decoding the geometry
  const OTrGede* GetGede() const {
    return fgGedeContainer->GetGede(GetGedeIndex()) ; }
  float GetRawX() const { return GetGede()->GetRawX(GetWireNumber()) ; }
  float GetRawY1() const { return GetGede()->GetRawY1(GetWireNumber()) ; }
  float GetRawY2() const { return GetGede()->GetRawY2(GetWireNumber()) ; }
  float GetRawZ() const { return GetGede()->GetRawZ(GetWireNumber()) ; }
  PRWireIndex_t GetPRWireNumber() const { 
    return GetGede()->GetPRWireNumber(GetWireNumber()) ; }
  int GetMonoLayer() const { 
    return GetWireNumber()%(GetGede()->GetNMonoLayers()) ; }

protected:
#ifdef USE_ARTE
  OTrHit(ArtePointerHITB hitb) : fHITB(hitb) {}
  void SetHitB(ArtePointerHITB hitb) { fHITB = hitb ; }
#endif
private:
  static const OTrGedeContainer *fgGedeContainer ;
#ifdef USE_ARTE
  ArtePointerHITB fHITB ;
#endif
}

class OTrWireHit : public OTrHit
{
public:
  OTrWireHit(OTrWire* wire) : fWire(wire) {}
#ifdef USE_ARTE
  OTrWireHit(ArtePointer<OTHB> othb) ; 
#endif
  virtual TdcCount_t GetTZeroCount()  const { return fWire->GetTZeroCount() ; }
  virtual TdcCOunt_t GetTdcCount()    const { return fWire->GetTdcCount() ; }
  virtual GedeIndex_t GetGedeIndex()  const { return fWire->GetGedeIndex() ; }
  virtual WireIndex_t GetWireNumber() const { return fWire->GetWireNumber() ; }
  ArtePointerHITB AppendHITB() ;
private:
  OTrWire* fWire ;
}

#ifdef USE_ARTE
class OTrHITBHit : public OTrHit
{
public:
  OTrHITBHit(ArtePointerHITB hitb) : OTrHit(hitb) {}
  // declared by virtual base class
  virtual DriftTime_t GetTZeroCount() const { 
    return (int(GetHITB()->y))%256 ; }
  virtual DriftTime_t GetTdcCount()   const { 
    return ((int(GetHITB()->y))/256)%256 ; }
  virtual GedeIndex_t GetGedeIndex()  const { return GetHITB()->gede.index(); }
  virtual WireIndex_t GetWireNumber() const { return 0 ; }
}

class OTrDIGBHit : public OTrWireHit
{
public:
  OTrDIGBHit(ArtePointer DIGB
