#include <fstream>
#include <iomanip>
#include <algorithm>
#include "wax/DecodeManager.hh"
#include "wax/GedeManager.hh"
#include "wax/Gede.hh"
#include "wax/GedeManager.hh"
#include "wax/OTrAsdId.h"
#include "wax/OTrRead.h"
#ifdef USE_ARTE
#include "arte/arte.hh"
#endif

const waxDecodeManager*
waxDecodeChannel::fgDecodeManager = 0;

waxDecodeManager*
waxDecodeManager::fgMe = 0;

#ifdef __OTR_OLD_TABLES__
std::string waxDecodeManager::fgListOfTablesFileName =
"/afs/desy.de/group/hera-b/OTR/db/tables/list_of_tables_hallw" ;
std::string waxDecodeManager::fgTdcFedFileName =
"/afs/desy.de/group/hera-b/OTR/db/tables/tdc_fed_hallw" ;
#else
std::string waxDecodeManager::fgListOfTablesFileName =
"/afs/desy.de/group/hera-b/OTR/db/tables-new/list_of_tables_hallw" ;
std::string waxDecodeManager::fgTdcFedFileName =
"/afs/desy.de/group/hera-b/OTR/db/tables-new/tdc_fed_hallw" ;
#endif

std::string waxDecodeManager::fgSwapCableFileName = 
"/afs/desy.de/group/hera-b/OTR/db/swapcable/default.swap" ;

std::string waxDecodeManager::fgTZeroFileName = "" ; 
std::string waxDecodeManager::fgMaskFileName = "" ;

void waxDecodeChannel::SetTZero(DriftTime_t tz, DriftTime_t tzerr) {
  // check that tz and tzerr are within range
  int tzc = int(128 + 2*tz    + 0.5) ; 
  int tzec = int(16*tzerr + 0.5) ; 
  if(tzc>255) {
    tzc=255 ;
    tzec=255 ;
  } else if(tzc<0) {
    tzc=0 ;
    tzec=255 ;
  } else if(tzec>255) {
    tzec=255 ;
  } else if(tzec<0) 
    tzec=0 ;
  SetTZeroCount(tzc,tzec) ;
}

waxDecodeGede::waxDecodeGede()
{
  fWires[0]=fWires[1]=fWires[2]=fWires[3]=fWires[4]=0;
}

waxDecodeManager* waxDecodeManager::GetMe() 
{
  if(!fgMe) fgMe = new waxDecodeManager() ;
  return fgMe ;
}

void
waxDecodeManager::DeleteMe()
{
  delete fgMe ;
  fgMe=0 ;
}
 
waxDecodeManager::~waxDecodeManager() 
{
  //delete fDecodeTable ;
  if(fgMe==this) fgMe = 0;
}

waxDecodeManager::waxDecodeManager()
{
#ifdef __WAX_DEBUG__
  cout << "waxDecodeManager: initializing decoding." << endl ;
#endif
  if(!fgMe) fgMe = this ;
  
  int mode=0;
  fMinGedeIndex=117;
  fDecodeTable = OTr_Decode_init(mode,fgListOfTablesFileName.c_str(),
				 fgTdcFedFileName.c_str()) ;
  if(fDecodeTable!=NULL) {
    waxDecodeChannel::SetDecodeManager(this) ;
    for(AsdIterator asdIt = GetAsds() ; *asdIt != 0; asdIt++)
      for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels(); 
	  chanIt !=  (*asdIt)->GetChannelsEnd(); chanIt++) {
#ifdef USE_ARTE
	if(ArteTable<MCIN>::booked()) chanIt->SetTZero(0,0) ;
	else 
#endif
	  chanIt->SetTZero(0,100) ;
	chanIt->SetStatus(0) ;
      }
    ReadStatus() ;
    ReadTZeros() ;
    SwapCables() ;
 
#ifdef USE_ARTE
    CreateDecodeGedes() ;
#endif   

#ifdef __WAX_DEBUG__
    cout << "Wax DEBUGGING mode: " << endl ;
 
    AsdIterator firstAsd =  GetAsds() ;
    int asdId=0;
    for(AsdIterator asdIt = firstAsd; *asdIt != 0 ; 
	asdIt++,asdId++) { 
      int chanId=0;
      for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels() ;
	  chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++, chanId++) {
     
	int gedeId = chanIt->GetGedeIndex() ;
	if(gedeId != 0) {
	  int wireId = chanIt->GetWireIndex() ;
	  const waxDecodeGede* gedeIt = GetGede(gedeId) ;
	  if(gedeIt==0) cout << "No such gede " << gedeId << endl ;

	  const waxDecodeWire* wireIt = gedeIt->GetWire(wireId) ;
	  if(asdId  != wireIt->GetAsdIndex()  || chanId != wireIt->GetChannelIndex() ) {
	    waxDecodeChannel* altChanIt = 
	      const_cast<waxDecodeChannel*>(GetAsd(wireIt->GetAsdIndex())->
					    GetChannel( wireIt->GetChannelIndex())) ;
	    altChanIt->SetStatus(2) ;
	    chanIt->SetStatus(2) ;
	    ArtePointer<GEDE> artegede(gedeId) ;
	    std::cout << "Problem !!!! "  
		      << artegede->hwc << " "
		      << setw(6) << gedeId 
		      << setw(4) << wireId 
	      //<< setw(6) << asdId
		      << GetAsd(asdId)->GetName() << " "
		      << setw(4) << chanId 
	      //<< setw(6) << wireIt->GetAsdIndex() 
		      << GetAsd(wireIt->GetAsdIndex())->GetName() << " "
		      << setw(4) << int(wireIt->GetChannelIndex())
		      << setw(6) << altChanIt->GetGedeIndex() 
		      << setw(4) << int(altChanIt->GetWireIndex()) 
		      << std::endl ;
	    cout << "Wires per ml: " ;
	    for(int m=0; m<artegede->nsl; m++) 
	      cout << artegede->nwi[m] << " ";
	    cout << endl ;
	  }
	}
      }
    }
    
    int gedeid=fMinGedeIndex ;
    const waxDecodeWire* wireIt;
    for(GedeContainer::const_iterator gedeIt = fGedes.begin() ;
	gedeIt != fGedes.end() ; ++gedeIt, ++gedeid) {
      ArtePointer<GEDE> artegede(gedeid) ;
      const waxDecodeWire* firstwire = gedeIt->GetWire(0);
      const waxDecodeWire* lastwire  = gedeIt->GetWire(artegede->nwi[0]-1);
      if(firstwire && lastwire && firstwire->IsConnected() && 
	 lastwire->IsConnected() && 
	 artegede->hwc[1]=='p' &&
	 artegede->sec==4) {
	cout << artegede->hwc << " " 
	     << int(firstwire->GetChannelIndex()+1) << " "
	     << int(lastwire->GetChannelIndex()+1) <<  endl ;
      }
   
      for(int wireid=0; wireid<128; wireid++) {
	if( (wireIt = gedeIt->GetWire(wireid)) && 
	    wireIt->IsConnected()) {
	  const waxDecodeChannel* chanIt = wireIt->GetChannel() ;
	  if(chanIt->GetGedeIndex() != gedeid ||
	     chanIt->GetWireIndex() != wireid) {
	    cout << "ProblemTwo: "
		 << (ArtePointer<GEDE>(gedeid))->hwc << " "
		 << wireid << " "
		 << (ArtePointer<GEDE>(chanIt->GetGedeIndex()))->hwc << " "
		 << chanIt->GetWireIndex() << " "
		 << wireIt->GetAsdIndex() << " "
		 << wireIt->GetChannelIndex() << endl ;
	  }
	}
      }
    }
 
    /*
    cout << "These wires are not connected: " << endl ;
    const waxDecodeGede* decgedeIt ;
    for(ArteTable<GEDE>::iterator gedeIt = ArteTable<GEDE>::begin() ;
	gedeIt != ArteTable<GEDE>::end() ; ++gedeIt) 
      if(gedeIt->cmp==2 && gedeIt->nsl>0) {
	gedeIt->hwc[12]='\0' ;
	decgedeIt = GetGede(gedeIt.index()) ;
	if(decgedeIt) {
	  int ntot(0) ;
	  for(int m=0; m<gedeIt->nsl; m++) ntot += gedeIt->nwi[m]-1 ;
	  for(int m=0; m<gedeIt->nsl; m++)
	    for(int c=0; c<gedeIt->nwi[m]; ++c) {
	      int wireid = 32*m+c;
	      wireIt = decgedeIt->GetWire(wireid) ;
	      if(!wireIt) 
		cout << "wire not found: " 
		     << gedeIt->hwc << " " << m << " " << c << endl ;
	      else if(!wireIt->IsConnected())
		cout << "wire not connected: " 
		     << gedeIt->hwc << " " 
		     << m << " " << c << " " 
		     << gedeIt->nwi[m]-1 << " " << ntot << endl ;
	    }
	} else cout << "gede not found: " << gedeIt->hwc << endl;
      }
    */
    // 	int chanId = wireIt->GetChannelIndex() ;
// 	const waxDecodeAsd* asdIt = GetAsd(asdId) ;
// 	const waxDecodeChannel* chanIt = asdIt->GetChannel(chanId) ;
// 	// 	std::cout << gedeId << " " <<  wireId << " ==> "
// 	// 	     << asdId << " " << chanId << " "
// 	// 	     << chanIt->GetStatus() << std::endl ;
// 	if(gedeId != chanIt->GetGedeIndex() ||
// 	   wireId != chanIt->GetWireIndex()) {
// 	  std::cout << "Help !" << gedeId << " " << wireId << " "
// 		    << chanIt->GetGedeIndex()<< " " << chanIt->GetWireIndex()<< " "
// 		    << asdId << " " << chanIt - asdIt->GetChannels() << std::endl ;
       
// 	}
//       }
//       }
//     }

    // There seems to be a problem with the fed pointers in the asds. Let's check.
    asdId=0;
    for(AsdIterator asdIt = GetAsds(); *asdIt != 0 ; 
	asdIt++,asdId++) { 
      (*asdIt)->GetName() ;
      int fedid = (*asdIt)->GetFedIndex() ;

      if(fedid >0 && fedid <MAXFED) {
	int tdcid = fDecodeTable->FEDs[fedid] ;
	if(tdcid< MAXTDC) {
	  int tdcconnector = (*asdIt)->GetTdcConnectorIndex() ;
	  const OTrTDC* tdcIt = fDecodeTable->TDCs[tdcid] ;
	  if(tdcIt->ASD[tdcconnector-1] != asdId) {
	    cout << "ASD-TDC pointers not good: " 
		 << (*asdIt)->GetName() << " "
		 << asdId << " " << fedid << " " << tdcid << endl;
	  }
	} else {
	  cout << "Also strange:  Asd ->" << (*asdIt)->GetName() 
	       << "  fedid: " << fedid << " " << tdcid << endl ;
	}
      } else {
	cout << "Strange: Asd ->" << (*asdIt)->GetName() 
	     << "  fedid: " << fedid << endl ;
      }
    }
    cout << "last asd: " << asdId-1 << endl ;
    cout << "Pausing ... press a key. " << endl ;
    getchar() ;

    for(OTrTDC** tdcIt = fDecodeTable->TDCs ; *tdcIt !=0; tdcIt++) {
      for(int tdcconnid=0; tdcconnid<8; tdcconnid++) {
	int asdId = (*tdcIt)->ASD[tdcconnid] ;
	if(asdId>=0 && asdId<MAXASD && fDecodeTable->ASD8s[asdId]) {
	  const waxDecodeAsd* asdIt = GetAsd(asdId) ;
	  if((asdIt->GetFedIndex() != (*tdcIt)->FED) ||
	     (asdIt->GetTdcConnectorIndex() != tdcconnid+1)) {
	    cout << "ASD-TDC pointers not good: " 
		 << asdIt->GetName() << " "
		 << asdId << " " << (*tdcIt)->FED << " " << asdIt->GetFedIndex()
		 << " " <<asdIt->GetTdcConnectorIndex() << " " << tdcconnid+1  << endl;
	  }
	}
      }
    }
    cout << "Pausing ... press a key. " << endl ;
    getchar() ;

#endif
  } else {
    std::cout << "waxDecodeManager::waxDecodeManager(): "
	 << "Could not create OTR_Decode object." << std::endl ;
  }
}

bool waxCompareGede(const unsigned short lhsgedeId, const unsigned short rhsgedeId)
{
  int lhsindex(0),rhsindex(0) ;
#ifdef USE_ARTE
  static const waxGedeManager* gedemanager = waxGedeManager::GetMe() ;
  static const int mingedeid = waxDecodeManager::GetMe()->GetMinGedeIndex() ;
  const waxGede* lhsGede = gedemanager->GetGede(lhsgedeId+mingedeid) ;
  const waxGede* rhsGede = gedemanager->GetGede(rhsgedeId+mingedeid) ;
  if(lhsGede)
    lhsindex = 
      lhsGede->GetModuleId() + 100*
      (lhsGede->GetSectorId()+ 100*
       (lhsGede->GetLayerId() + 10*lhsGede->GetSuperLayerId())) ;
  if(rhsGede)
    rhsindex = 
      rhsGede->GetModuleId() + 100*
      (rhsGede->GetSectorId()+ 100*
       (rhsGede->GetLayerId() + 10*rhsGede->GetSuperLayerId())) ;
#endif
  return lhsindex<rhsindex ;
}

void
waxDecodeManager::CreateDecodeGedes()
{
#ifdef USE_ARTE
#ifdef __WAX_DEBUG__
  cout << "waxDecodeManager::CreateDecodeGedes()" << endl ;
#endif
  // count gedes and wires
  unsigned int mingedeid(100000),maxgedeid(0),totnwires(0) ;
  for(ArteTable<GEDE>::iterator gedeIt = ArteTable<GEDE>::begin() ;
      gedeIt != ArteTable<GEDE>::end() ; gedeIt++) 
    if(gedeIt->cmp==2 && gedeIt->hwc[0]!='n' && gedeIt->nsl>0) {
      if     (mingedeid>gedeIt.index()) mingedeid=gedeIt.index() ;
      else if(maxgedeid<gedeIt.index()) maxgedeid=gedeIt.index() ;
      for(int m=0; m<gedeIt->nsl; m++) totnwires+=gedeIt->nwi[m] ;
    }
#ifdef __WAX_DEBUG__
  cout <<"min/maxgede: " << mingedeid << " " << maxgedeid << endl ;
#endif
  // allocate gedes
  fMinGedeIndex = mingedeid;
  fGedes.resize(maxgedeid-mingedeid+1,waxDecodeGede()) ;

  // create map which contains gedes in correct order
  fGedeMap.resize(fGedes.size()) ;
  for(unsigned int i=0; i<fGedeMap.size(); i++) fGedeMap[i]=i;
  sort(fGedeMap.begin(),fGedeMap.end(),waxCompareGede) ;
  
  // allocate wires and set pointers in gedes. the wires are put in
  // the order in which they are needed for filling the HITB bank
  fWires.resize(totnwires) ;
  WireContainer::iterator wireIt = fWires.begin() ;
  
  for(waxDecodeManager::GedeMapIterator 
	gedeidIt = fGedeMap.begin() ;
      gedeidIt != fGedeMap.end(); ++gedeidIt) {
    GedeIndex_t gedeId = fMinGedeIndex + *gedeidIt ;
    ArtePointer<GEDE> gedeIt = ArtePointer<GEDE>(gedeId) ;
    if(gedeIt->cmp==2 && gedeIt->hwc[0]!='n' && gedeIt->nsl>0) {
      waxDecodeGede* decgedeIt = fGedes.begin() + *gedeidIt ;
      for(int m=0; m<5; m++) {
	decgedeIt->fWires[m] = wireIt;
	if(m<gedeIt->nsl) wireIt += gedeIt->nwi[m] ;
      }
    }
  }
#ifdef __WAX_DEBUG__
  if(wireIt != fWires.end() ) cout << "Strange, miscounted wires ? " << endl ;
#endif
  
  // connect
  for(AsdIterator asdIt = GetAsds() ; *asdIt != 0; asdIt++)
    for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels(); 
	chanIt !=  (*asdIt)->GetChannelsEnd(); chanIt++) 
      if(chanIt->GetGedeIndex()>0) {
	waxDecodeWire* wire = const_cast<waxDecodeWire*>
	  (fGedes[chanIt->GetGedeIndex()-fMinGedeIndex].
	   GetWire(chanIt->GetWireIndex())) ;
	if(wire) {
	  wire->fAsdId     = asdIt - GetAsds() ;
	  wire->fChannelId = chanIt - (*asdIt)->GetChannels();
	} else {
	  cout <<"Trying to connect non-existent wire !"
	       << chanIt->GetGedeIndex() << " "
	       << int(chanIt->GetWireIndex()) << " " 
	       << GetAsd(chanIt->GetAsdIndex())->GetSectorIndex() << " "
	       << endl ;
	}
      }
#endif
}

void
waxDecodeManager::ResetTdcCounts()
{
  for(AsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++)
    for(waxDecodeAsd::ChannelIterator chanIt = (*asdIt)->GetChannels() ;
	chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++) 
      chanIt->SetTdcCount(0) ;
}

int
waxDecodeManager::FillNewEvent()
{
  return OTr_fill_event(fDecodeTable) ;
}

int waxDecodeManager::GetNAsds() const {
  int n=0;
  for(ConstAsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++) n++ ; 
  return n;
}

waxDecodeAsd* waxDecodeManager::GetAsd(string asdname) 
{
  waxDecodeAsd* asdIt=0;
  unsigned char slid,layid,secid,ftbid ;
  if(asdid2num(asdname.c_str(),&slid,&layid,&secid,&ftbid)==0 && slid<32) {
    OTrSuperLayer* slIt =  fDecodeTable->Slayers + slid ;
    unsigned char firstasd = slIt->FirstASD[layid][secid] ;
    if(firstasd<255) {
      unsigned short asdId = 
	slIt->Line[slIt->pos[layid][secid] - firstasd + ftbid] ;
      asdIt = GetAsd(asdId) ;
    }
  }
  return asdIt ;
}

string waxDecodeAsd::GetName() const
{
  static char name[12] ;
  num2asdid(GetSuperLayerIndex(),GetLayerIndex(),
	    GetSectorIndex(),GetSubAsdIndex(),name) ;
  return string(name) ;
}

void waxDecodeManager::ReadTZeros() 
{
#ifdef __WAX_DEBUG__
  std::cout << "waxDecodeManager::ReadTZeros(): " ;
#endif
  const string filenameUser = fgTZeroFileName ;
  //if(!strncasecmp(fgTZeroFileName.c_str(),"none",4)) {
  if(!strncmp(fgTZeroFileName.c_str(),"none",4)) {
    std::cout << "TZeroFile = none -->Skip reading tzeros." << std::endl ;
    return ;
  }
  char runname[9] ;
  //sprintf(runname,"run%5.5d",arteflag_.numrun) ;
#ifdef USE_ARTE
  sprintf(runname,"run%5.5d",ArteTable<EVHD>::begin()->run) ;
#else
  sprintf(runname,"run_noa") ;
#endif
  const string filenameRun = "/afs/desy.de/group/hera-b/OTR/db/tzero/" +
    string(runname) + ".tzero" ;
  const string filenameGen = "/afs/desy.de/group/hera-b/OTR/db/tzero/otr.tzero"  ;
  
  std::ifstream ifs(filenameUser.c_str()) ;
  std::cout << "* Reading tzeros from:                           *" 
	    << std::endl << "* " ;
  if(ifs) std::cout << filenameUser << std::endl;
  else {
    ifs.open(filenameRun.c_str()) ;
    if(ifs) std::cout << filenameRun << std::endl ;
    else {
      ifs.open(filenameGen.c_str()) ;
      if(ifs) std::cout << filenameGen << std::endl ;
      else
	std::cout << "Could not open file under any of these names: " << std::endl
	     << filenameUser << std::endl << filenameRun << std::endl << filenameGen << std::endl ;
    }
  }
  while(ifs) {
    string asdname ;
    waxDecodeAsd* asdIt ;
    ifs >> asdname ;
    asdIt = GetAsd(asdname) ;
    int tzero,tzeroerr ;
    if(asdIt) {
      for(waxDecodeAsd::ChannelIterator chanIt = asdIt->GetChannels() ;
	  chanIt != asdIt->GetChannelsEnd() ; chanIt++) {
	ifs >> tzero >> tzeroerr ;
	chanIt->SetTZeroCount(tzero,tzeroerr) ;
      }
    } else for(int i=0; i<16; i++) ifs >> tzero >> tzeroerr ;
  }
  ifs.close() ;
}

void waxDecodeManager::WriteTZeros(const string& filename) 
{
  std::ofstream ofs(filename.c_str()) ;
  int asdId=0;
  for(ConstAsdIterator asdIt=GetAsds() ; *asdIt != 0 ; asdIt++, asdId++) {
    ofs << (*asdIt)->GetName() << " " ;
    for(waxDecodeAsd::ConstChannelIterator chanIt = (*asdIt)->GetChannels() ;
	chanIt != (*asdIt)->GetChannelsEnd() ; chanIt++) 
      ofs << setw(4) << int(chanIt->GetTZeroCount())
	  << setw(4) << int(chanIt->GetTZeroErrCount()) ;
    ofs << std::endl ;
  }
  ofs.close() ;
}

void waxDecodeManager::ReadStatus() 
{
#ifdef __WAX_DEBUG__
  std::cout << "waxDecodeManager::ReadStatus(): " ; 
#endif
  const string filenameUser = fgMaskFileName ;
  //if(!strncasecmp(fgMaskFileName.c_str(),"none",4)) {
  if(!strncmp(fgMaskFileName.c_str(),"none",4)) {
    std::cout << "MaskFile = none -->Skip reading masks." << std::endl ;
    return ;
  }
  char runname[9] ;
  //sprintf(runname,"run%5.5d",arteflag_.numrun) ;
#ifdef USE_ARTE
  sprintf(runname,"run%5.5d",ArteTable<EVHD>::begin()->run) ;
#else
  sprintf(runname,"run_noa") ;
#endif
  const string filenameRun = "/afs/desy.de/group/hera-b/OTR/db/masks/beam-" +
    string(runname) + "/otr.mask" ;
  const string filenameGen = "/afs/desy.de/group/hera-b/OTR/db/masks/beam/otr.mask"  ;
  
  std::ifstream ifs(filenameUser.c_str()) ;
  std::cout << "* Reading channel status from:                   *" 
	    << std::endl << "* " ;
  if(ifs) std::cout << filenameUser << std::endl;
  else {
    ifs.open(filenameRun.c_str()) ;
    if(ifs) std::cout << filenameRun << std::endl ;
    else {
      ifs.open(filenameGen.c_str()) ;
      if(ifs) std::cout << filenameGen << std::endl ;
      else
	std::cout << "Could not open file under any of these names: " << std::endl
		  << filenameUser << std::endl 
		  << filenameRun << std::endl 
		  << filenameGen << std::endl ;
    }
  }
  while(ifs) {
    char line[400];
    ifs.getline(line,400) ;
    if(line[0] != '#' && strlen(line)>12) {
      char asdname[12] ;
      char statusstr[17] ;
      sscanf(line,"%s %s",asdname,statusstr) ;
      waxDecodeAsd* asdIt = GetAsd(asdname) ;
      if(asdIt) {
	char* statusIt = statusstr ;
	for(waxDecodeAsd::ChannelIterator chanIt = asdIt->GetChannels() ;
	    chanIt != asdIt->GetChannelsEnd() ; chanIt++, statusIt++) {
	  switch(*statusIt) {
	  case 'D':
	  case 'W':
	  case 'H':
	  case 'X':
	  case 'I':
	  case 'V':
	  case 'N':
	  case 'F':
	    chanIt->SetStatus(1) ;
	    break;
	  case '.':
	  case 'C':
	  default:
	    chanIt->SetStatus(0) ;
	  }
	}
      } else std::cout << "Could not find asd: " << asdname << endl ;
    }
  }
  ifs.close() ;
}


void waxDecodeManager::SwapCables()
{
  
#ifdef __WAX_DEBUG__
  std::cout << "waxDecodeManager::ReadStatus(): " ; 
#endif
  const string filenameUser = fgMaskFileName ;
  //if(!strncasecmp(fgSwapCableFileName.c_str(),"none",4)) {
  if(!strncmp(fgSwapCableFileName.c_str(),"none",4)) {
    std::cout << "* SwapCableFileName = " << fgSwapCableFileName
	      << " --> No cables swapped." << std::endl ;
    return ;
  }

  std::ifstream ifs(fgSwapCableFileName.c_str()) ;
  if(!ifs) {
    std::cout << "* SwapCableFileName = " << fgSwapCableFileName
	      << " : Cannot find --> No cables swapped." 
	      << std::endl ;
    return ;
  } 

  std::cout << "* Correcting cabling mistakes from file:         *" 
	    << std::endl << "* " 
	    << fgSwapCableFileName << std::endl;
  
  int format ;
  ifs >> format ;
  if(format==1) {
    int nlines(0) ;
    int nasds ;
    ifs >> nasds ;
    while(ifs) {
      std::vector<waxDecodeAsd*> asdvec(nasds) ;
      string name;
      bool all_ok = true ;
      for(int i=0; i<nasds; i++) {
	ifs >> name ;
	asdvec[i] = GetAsd(name) ;
	if(asdvec[i]==0) {
	  all_ok = false ;
	  cout << "Strange name: " << name << endl ;
	}
      }
      
      if(all_ok) {
#ifdef __WAX_DEBUG__
	// let's check first
	for(int i=0; i<nasds; ++i) {
	  waxDecodeAsd* asdIt = asdvec[i];
	  waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ;
	  if(GetAsd(tdcIt->GetAsdIndex(asdIt->GetTdcConnectorIndex()-1)) !=
	     asdIt) cout << "Shit: already connection error." << endl ;
	}
#endif
	
	for(int i=0; i<nasds-1; ++i) {
	  waxDecodeAsd* asdIt = asdvec[i];
	  waxDecodeAsd* nextasdIt = asdvec[i+1];
	  waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ; 
	  waxDecodeTdc* nexttdcIt = GetTdc(nextasdIt->GetFedIndex()) ;
	  // swap info in tdc
	  swap(tdcIt->fTdc.ASD[asdIt->GetTdcConnectorIndex()-1],
	       nexttdcIt->fTdc.ASD[nextasdIt->GetTdcConnectorIndex()-1]) ;  
	  // swap info in asd
	  swap(asdIt->fAsd.TDC_FED,nextasdIt->fAsd.TDC_FED) ;
	  swap(asdIt->fAsd.TDC_Connector,nextasdIt->fAsd.TDC_Connector) ;
	}
	
#ifdef __WAX_DEBUG__
	// now check:
	for(int i=0; i<nasds; ++i) {
	  waxDecodeAsd* asdIt = asdvec[i];
	  waxDecodeTdc* tdcIt = GetTdc(asdIt->GetFedIndex()) ;
	  if(GetAsd(tdcIt->GetAsdIndex(asdIt->GetTdcConnectorIndex()-1)) !=
	     asdIt) cout << "Shit: connection error introduced." << endl ;
	}
#endif
      }
      nlines++;
      ifs >> nasds ;
    }
#ifdef __WAX_DEBUG__
    std::cout << "Read " << nlines << " lines." << endl ;
#endif
  } else std::cout << "SwapCables error: wrong format." << endl ;
  ifs.close() ;
}

std::vector<FedIndex_t> 
waxDecodeManager::GetActiveFedIds() const 
{
  // returns a vector with the ids of all outer tracker feds
  std::vector<FedIndex_t> fedids;
  for(FedIndex_t fedid = 0; fedid<MAXFED; fedid++) 
    if(fDecodeTable->FEDs[fedid]!=MAXFED)
      fedids.push_back(fedid) ;
  return fedids;
}
