#include "wax/Hit.hh"
#include "wax/RTRelation.hh"

//__________________________________________________________________
// waxHit
//
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

const waxGedeManager* waxHit::fgGedeManager=0 ;
float waxHit::fgDriftRadiusOffset=0 ;

#ifdef USE_ARTE
void
waxHit::AppendHITB(ArtePointerHITB hitb) {
  static waxRTRelation::RTResult rtresult ;
  if(fHITB.index()) {
    std::cout << "waxHit::AppendHITB(): Hit (" << fHITB.index() 
	 << ") already added !!" << std::endl ;
    return ;
  }
  fHITB=hitb ;
  HITB& hit = *hitb; 
  hit.z    = GetRawZ() ;
  hit.x    = GetRawX() ; 
  hit.y    = GetTdcCount() ;
  rtresult = GetGede()->GetRTRelation()->GetRTResultFromT(GetDriftTime()) ;
  hit.q    = GetDriftRadiusOffset() + rtresult.fR ;
  float r_rt = rtresult.fSigmaR ; // error estimated by rtcalibration
  float r_t0 = -fChannel->GetTZeroErr()/rtresult.fdTdR ; // error in t0
  hit.r    = sqrt(r_rt*r_rt + r_t0*r_t0) ;
  hit.gede = GetGedeIndex() ;
  hit.chid = GetCHID(GetWireIndex(),GetAsdIndex(),GetChannelIndex()) ;
}

ArtePointerHITB
waxHit::AppendHITB() {
  if(fHITB.index()) {
    std::cout << "waxHit::AppendHITB(): Hit (" << fHITB.index() 
	 << ") already added !!" << std::endl ;
    return fHITB ;
  }
  HITB hitb ;
  hitb.z    = GetRawZ() ;
  hitb.x    = GetRawX() ; 
  hitb.y    = GetTdcCount() ;
  waxRTRelation::RTResult rtresult = 
    GetGede()->GetRTRelation()->GetRTResultFromT(GetDriftTime()) ;
  hitb.q    = rtresult.fR ;
  hitb.r    = rtresult.fSigmaR ;
  hitb.gede = GetGedeIndex() ;
  hitb.chid = GetCHID(GetWireIndex(),GetAsdIndex(),GetChannelIndex()) ;
  fHITB = ArteTable<HITB>::push_back(hitb);
  return fHITB ;
}

ArtePointer<DIGB> waxHit::GetDIGB() const
{
  ArtePointer<DIGB> digb(0) ;
  ArtePointer<HITB> hitb = GetHITB() ;
  if(hitb.index()>0)
    for(ArteRelationIterator<DIGB> digbIt=hitb->digb.begin() ;
	digbIt!=hitb->digb.end() ; digbIt++) 
      if(fabs((*digbIt)->q - hitb->q) < 0.001) {
	digb = digbIt ;
	break ;
      }
  return digb ;
}

#endif
