#include <fstream>
#include <iomanip>
#include "wax/RTRelation.hh"
#include "wax/Parization.hh"

//__________________________________________________________________
// waxRTRelation
//
// Interface class for the rt-relations
//
// Author: Wouter Hulsbergen
//__________________________________________________________________

const waxRTRelation::Radius waxRTRelation::fgRMin[2] = {0.00125,0.00125 } ;//cm

const waxRTRelation::Radius waxRTRelation::fgRMax[2] = {0.29,0.58} ;//cm

const waxRTRelation::Time waxRTRelation::fgTMin = 0 ;

const waxRTRelation::Time waxRTRelation::fgTMax = 256 ;

int waxRTRelation::fgNTableBins = 500 ;

waxRTRelation::waxRTRelation() 
  : fTParization(0),fSParization(0),
    ftRTTable(0),ftMapTable(0) {}


waxRTRelation::waxRTRelation(int key, std::istream& ifs) 
  : fLedaKey(key),fEParType(kTofR),fTParization(0),fSParization(0),
    ftRTTable(0),ftMapTable(0)
{
  fTParization = waxParization::Create() ;
  fSParization = waxParization::Create() ;
  Read(ifs) ;
  fLedaKey = key ;
}

// waxRTRelation::waxRTRelation(LedaTableIterator<class OTR_r_t>& ledaIt)
//   : fLedaKey(ledaIt->irt),fEParType(kTofR),fTParization(0),fSParization(0),
//     ftRTTable(0),ftMapTable(0)
// {
//   fTParization = waxParization::Create() ;
//   fSParization = waxParization::Create() ;
//   ReadDB() ; 
// }

waxRTRelation::waxRTRelation(int ledakey, EPitch epitch, EParType partype,
			     int n_rt, int n_res) :
  fLedaKey(ledakey), fEPitch(epitch), fEParType(partype)
{
  fTParization = waxParization::Create(fgTMin,fgTMax,3,n_rt,0.2) ;
  fSParization = waxParization::Create(fgTMin,fgTMax,0,n_res,0.2) ;

  fTMin = fTParization->GetXMin() ;
  fTMax = fTParization->GetXMax() ;
  FillRTTable() ;
}

waxRTRelation::~waxRTRelation() 
{
  delete fTParization ;
  delete fSParization ;
} 

void waxRTRelation::ChangeXBounds(Time xmin, Time xmax) {
  fTMin = xmin ; fTMax = xmax ;
  fTParization->SetXBounds(xmin,xmax) ;
  fSParization->SetXBounds(xmin,xmax) ;
}

// void
// waxRTRelation::ReadDB()
// {
//   LedaPointer<class OTR_r_t> ledartIt(OTR_r_t->get->key(fLedaKey)) ;
//   fEPitch = ledartIt->pitch ;
//   HepVector tpar(ledartIt->n_rt) ;
//   for(int i=0; i<tpar.num_row(); i++) tpar[i] = ledartIt->par_rt[i] ;
//   HepVector spar(ledartIt->n_res) ;
//   for(int i=0; i<spar.num_row(); i++) spar[i] = ledartIt->par_res[i] ;
// //   fTParization = waxParization::Create(GetXMin(),GetXMax(),int(ledartIt->par_rt[19]),tpar) ;
// //   fSParization = waxParization::Create(GetXMin(),GetXMax(),int(ledartIt->par_res[19]),spar) ;
//   FillRTTable() ;
//   std::cout << "waxRTRelation::ReadDB() " << GetTMax() << " " 
//        << fLedaKey << std::endl ;
// }

// void
// waxRTRelation::WriteDB() const
// {
//   std::cout << "waxRTRelation::WriteDB()" << std::endl ;
//   LedaPointer<class OTR_r_t> ledartIt(OTR_r_t->add->key(fLedaKey)) ;
//   std::cout << "LedaPointer<class OTR_r_t> created ... " << flush 
//        << ledartIt->n_rt << std::endl ;
//   //ledartIt->itr = fLedaKey ;
//   ledartIt->pitch = fEPitch ;
//   HepVector tpar = fTParization->GetPars() ;
//   ledartIt->n_rt = tpar.num_row() ;
//   for(int i=0; i<tpar.num_row(); i++) ledartIt->par_rt[i] = tpar[i] ;
//   ledartIt->par_rt[19] = fTParization->GetDegree() ;
//   HepVector spar = fSParization->GetPars() ;
//   ledartIt->n_res = spar.num_row() ;
//   for(int i=0; i<spar.num_row(); i++) ledartIt->par_res[i] = spar[i] ;
//   ledartIt->par_res[19] = fSParization->GetDegree() ;
//   std::cout << "End of waxRTRelation::WriteDB()" << std::endl ;
// }

void
waxRTRelation::Read(std::istream& ifs) 
{
  //std::ifstream ifs(filename.c_str()) ;
  if(ifs) {
    int format ;
    ifs >> format ;
    //std::cout << "Reading from file:" << filename << " " << format << std::endl ;
    switch(format) { 
    case 0:
      float t0;
      ifs >> fTMin >> fTMax >> t0;
      if(fTMax>0.4) fEPitch=kPitchTen;
      else          fEPitch=kPitchFive;
      break;
    case 1:
    case 2:
      ifs >> fLedaKey >> fEPitch ;
      fTMin = fgRMin[fEPitch] ; fTMax = fgRMax[fEPitch] ;
      break;
    case 3:
      ifs >> fLedaKey >> fEPitch >> fEParType ;
      ifs >> fTMin >> fTMax ;
      break;
    case 4:
      ifs >> fLedaKey >> fEPitch >> fEParType ;
      break;
    default:
      std::cout << "waxRTRelation::Read(...): " 
	   << "file format "<< format << " not supported !" << std::endl ;  
      break;
    }
    if(format <=3) {
      fTParization->Read(fTMin,fTMax,format,ifs) ;
      fSParization->Read(fTMin,fTMax,format,ifs) ;
    } else {
      fTParization->Read(format,ifs) ;
      fSParization->Read(format,ifs) ;
      fTMin = fTParization->GetXMin() ;
      fTMax = fTParization->GetXMax() ;
    }
//     std::cout << fLedaKey << " " << fEPitch << " " << fTMin << " "<< fTMax << std::endl 
// 	 << "rtknots: " << fTParization->GetKnots()
//     	 << "rtpars: " << fTParization->GetPars() << std::endl ;
  }
  FillRTTable() ;
}

void
waxRTRelation::Write(int format, std::ostream& ofs) const 
{
  //std::cout << "waxRTRelation::Write " << format << " " << filename << std::endl ;
  //std::ofstream ofs(filename.c_str()) ;
  switch(format) { 
  case 0:
    ofs << fTMin << " " << fTMax << " " << 0 << std::endl ;
    break ;
  case 1:
  case 2:
    ofs << format  << " " << fLedaKey << " " << fEPitch << std::endl ;
    break;
  case 3:
    ofs << format  << " " << fLedaKey << " " << fEPitch << " " 
	<< fEParType << " " 
	<< setw(11) << setprecision(4) << GetTMin() 
	<< setw(11) << setprecision(4) << GetTMax() << std::endl ;
    break;
  case 4:
    ofs << format  << " " << fLedaKey << " " << fEPitch << " " 
	<< fEParType << std::endl ;
    break ;
  default:
    break ;
  }
  fTParization->Write(format,ofs) ;
  fSParization->Write(format,ofs) ;
}

waxRTRelation::RTResult
waxRTRelation:: CalcRTResultFromT(Time t) const 
{
  RTResult rt;
  waxFunctionValue fv = fTParization->CalcFunction(t) ;
  rt.fR      = fv.fY ;
  rt.fT      = fv.fX ;
  rt.fdTdR   = 1/fv.fdYdX ;
  rt.fSigmaR = sqrt(fSParization->CalcValue(t)) ;
  return rt ;
}

waxRTRelation::RTResult
waxRTRelation::CalcRTResultFromR(Radius r) const 
{
  RTResult rt;
  float precision = 0.1 ; // channel
  waxFunctionValue fv = fTParization->CalcInverse(r,precision) ;
  rt.fR      = fv.fY ;
  rt.fT      = fv.fX ;
  rt.fdTdR   = 1/fv.fdYdX ;
  rt.fSigmaR = sqrt(fSParization->CalcValue(fv.fX)) ;
  return rt ;
}

void waxRTRelation::FillRTTable()
{  
  //std::cout << "waxRTRelation::FillRTTable()" << std::endl ;
  ftRTTable.resize(GetNTableBins()) ;
  ftMapTable.resize(GetNTableBins()) ;

  // Fill table rtresults 
  for(int xbin=0; xbin<GetNTableBins() ; xbin++) {
    Time t = GetTMin() + xbin*(GetTMax()-GetTMin())/GetNTableBins() ;
    ftRTTable[xbin] = CalcRTResultFromT(t) ;
  } 
  
  // Fill map table
  fRMax = ftRTTable.front().fR ;
  fRMin = ftRTTable.back().fR ;
  
  if(fRMax < fRMin) swap(fRMax,fRMin) ;
  float deltaR = (fRMax-fRMin)/fgNTableBins ;
  // too slow ? .... use sign deltaT !
  float precision = fabs(0.1*deltaR) ;
  for(int biny(0); biny<fgNTableBins; biny++) {
    float r = fRMin + biny*deltaR ;
    float t = fTParization->CalcInverse(r,precision).fX ;
    ftMapTable[biny] = GetBinFromT(t) ;
  }
  //std::cout << "End of waxRTRelation::FillRTTable()" << std::endl ;
}

float waxRTRelation::GetAverageSigmaR() const
{
  float min,max ;
  min=max=ftRTTable[0].fSigmaR ;  
  float sum(0) ;
  for(int xbin=0; xbin<GetNTableBins() ; xbin++) {
    float sigma= ftRTTable[xbin].fSigmaR ;
    sum += sigma ;
    if     (min<sigma ) min = sigma ;
    else if(max>sigma ) max = sigma ;
  }
  return sum/GetNTableBins() ;
  //return 0.5*(min+max) ;
}
